/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
'use strict';

/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
class NavigationController {
  static preprocessPath(path) {
    return path.replace(NavigationController.REPLACE_REGEX, '$1/$4').replace('//', '/');
  }
  checkArgument(condition, message) {
    if (!condition) {
      throw Error('IllegalArgumentException: ' + (message || ''));
    }
  }
  getPathItems(path, navigationItems) {
    return navigationItems.reduce((acc, item) => {
      const windowPath = NavigationController.preprocessPath(path);
      const itemPath = NavigationController.preprocessPath(item.path);
      if (item.children) {
        const childItems = this.getPathItems(path, item.children);
        if (childItems.length > 0) {
          acc.push(item, ...childItems);
        }
        else if ((!!item.jsFiles || !!item.htmlFile) && windowPath.startsWith(itemPath)) {
          acc.push(item);
        }
      }
      else if (windowPath.startsWith(itemPath)) {
        acc.push(item);
      }
      return acc;
    }, []);
  }
  getSelectedNavigationItem(path, navigationItems) {
    const items = this.getPathItems(path, navigationItems);
    if (items.length > 0) {
      return items[items.length - 1];
    }
    return null;
  }
  setActiveNavigationByPath(path, navigationItems) {
    this.checkArgument(path.startsWith('/'), 'Path must start with /');
    this.deactivateNavigationItems(navigationItems);
    const items = this.getPathItems(path, navigationItems);
    if (items.length > 0) {
      items.forEach((item, index) => {
        if (index === items.length - 1) {
          item.active = true;
        }
        else {
          item.expanded = true;
        }
      });
      return items[items.length - 1];
    }
    return null;
  }
  getTranslatedPath(path, navigationItems) {
    const items = this.getPathItems(path, navigationItems);
    return items.map((item) => item.title);
  }
  deactivateNavigationItems(navigationItems = []) {
    navigationItems.forEach((navigationItem) => {
      var _a;
      navigationItem.active = false;
      this.deactivateNavigationItems((_a = navigationItem.children) !== null && _a !== void 0 ? _a : []);
    });
  }
}
NavigationController.REPLACE_REGEX = /^(((?!\?).)+)(\?.+)?(#((?!\?).)*)(\?.*)?$/;

exports.NavigationController = NavigationController;

