/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { Component, h, Prop, State, Watch } from '@stencil/core';
export class CardText {
  constructor() {
    /** Styles the text element like a title. By default set to false. */
    this.isTitle = false;
    /** Styles the text element for large content. By default set to false. */
    this.largeText = false;
    /** Placeholder for the text element. By default empty string. */
    this.label = '';
    /** Value of the text inside the text element. By default empty string. */
    this.value = '';
    this.tileText = {
      isTitle: false,
      label: '',
      value: '',
      largeText: false,
    };
  }
  watchIsTitleHandler(newValue) {
    this.tileText = Object.assign(Object.assign({}, this.tileText), { isTitle: newValue });
  }
  watchLabelHandler(newValue) {
    this.tileText = Object.assign(Object.assign({}, this.tileText), { label: newValue });
  }
  watchValueHandler(newValue) {
    this.tileText = Object.assign(Object.assign({}, this.tileText), { value: newValue });
  }
  watchLargeTextHandler(newValue) {
    this.tileText = Object.assign(Object.assign({}, this.tileText), { largeText: newValue });
  }
  componentWillLoad() {
    this.tileText = {
      value: this.value,
      label: this.label,
      isTitle: this.isTitle,
      largeText: this.largeText,
    };
  }
  render() {
    if (this.tileText.isTitle) {
      return (h("div", { class: "title" },
        h("span", { "data-test": "webcore.webcomponents.card-text.bci-card-text.title", class: "placeholder" }, this.tileText.label),
        h("span", null, this.tileText.value)));
    }
    else if (this.tileText.largeText) {
      return (h("div", { class: "large-text" },
        h("span", { "data-test": "webcore.webcomponents.card-text.bci-card-text.large-title", class: "placeholder" }, this.tileText.label),
        h("span", null, this.tileText.value)));
    }
    else {
      return (h("div", { class: "normal-text" },
        h("div", null,
          h("span", { "data-test": "webcore.webcomponents.card-text.bci-card-text.normal-title", class: "placeholder" }, this.tileText.label),
          h("span", null, this.tileText.value))));
    }
  }
  static get is() { return "bci-card-text"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["card-text.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["card-text.css"]
  }; }
  static get properties() { return {
    "isTitle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Styles the text element like a title. By default set to false."
      },
      "attribute": "is-title",
      "reflect": false,
      "defaultValue": "false"
    },
    "largeText": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Styles the text element for large content. By default set to false."
      },
      "attribute": "large-text",
      "reflect": false,
      "defaultValue": "false"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Placeholder for the text element. By default empty string."
      },
      "attribute": "label",
      "reflect": false,
      "defaultValue": "''"
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Value of the text inside the text element. By default empty string."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "''"
    }
  }; }
  static get states() { return {
    "tileText": {}
  }; }
  static get watchers() { return [{
      "propName": "isTitle",
      "methodName": "watchIsTitleHandler"
    }, {
      "propName": "label",
      "methodName": "watchLabelHandler"
    }, {
      "propName": "value",
      "methodName": "watchValueHandler"
    }, {
      "propName": "largeText",
      "methodName": "watchLargeTextHandler"
    }]; }
}

