/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, h, State, Element, Listen } from '@stencil/core';
import { MDCMenu } from '@material/menu';
export class ContextMenu {
  constructor() {
    this.showMenu = false;
    this.actionList = [];
    this.closeMenuHandler = () => {
      this.showMenu = false;
      window.removeEventListener('click', this.closeMenuHandler);
      window.removeEventListener('wheel', this.closeMenuHandler);
    };
  }
  showContextMenu(event) {
    this.actionList = event.detail.actions;
    this.contextParamValue = event.detail.contextParams;
    this.showMenu = true;
  }
  closeContextMenu() {
    if (this.showMenu) {
      this.showMenu = false;
      window.removeEventListener('click', this.closeMenuHandler);
      window.removeEventListener('wheel', this.closeMenuHandler);
    }
  }
  preventWheelDefault(event) {
    event.stopPropagation();
  }
  componentDidRender() {
    if (this.showMenu) {
      this.menu = new MDCMenu(this.el.querySelector('.mdc-menu'));
      this.menu.open = true;
      this.menu.setAnchorMargin({ top: 8 });
      window.addEventListener('click', this.closeMenuHandler);
      window.addEventListener('wheel', this.closeMenuHandler);
    }
    window.dispatchEvent(new CustomEvent('bci.portal.contextmenu.toggle', { detail: { showMenu: this.showMenu } }));
  }
  goToAction(action) {
    const contextParam = this.contextParamValue;
    window.dispatchEvent(new CustomEvent('bci.portal.navigation.actionItemClicked', { detail: Object.assign({ action }, contextParam) }));
  }
  render() {
    if (this.showMenu) {
      return (h("div", { class: "mdc-menu-surface--anchor" },
        h("div", { class: "bci-link-list-context-menu mdc-menu mdc-menu-surface" },
          h("ul", { class: "mdc-list", role: "menu", "aria-hidden": "true", "aria-orientation": "vertical", tabindex: "-1" }, this.actionList.map((action, index) => (h("li", { "data-test": "webcore.webcomponents.bci-context-menu.mdc-list-item", class: "mdc-list-item", "data-value": action, key: index, "aria-selected": "false", role: "action", onClick: () => this.goToAction(action) },
            h("span", { class: "mdc-list-item__text" }, action.title))))))));
    }
  }
  static get is() { return "bci-context-menu"; }
  static get originalStyleUrls() { return {
    "$": ["context-menu.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["context-menu.css"]
  }; }
  static get states() { return {
    "showMenu": {},
    "actionList": {},
    "contextParamValue": {}
  }; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "bci.portal.showContextActions",
      "method": "showContextMenu",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "bci.portal.closeContextMenu",
      "method": "closeContextMenu",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "wheel",
      "method": "preventWheelDefault",
      "target": undefined,
      "capture": false,
      "passive": true
    }]; }
}

