/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, h, Prop } from '@stencil/core';
import { endOfMonth, getDaysInMonth, getISODay } from 'date-fns';
import state, { setLocale } from '../../../utils/locale-store';
import { createDate } from '../../../utils/utils';
import './month-header/month-header';
import './week-header/week-header';
import './weekdays/weekdays';
export class Datepicker {
  async componentWillLoad() {
    await setLocale();
    if (!this.formatDate) {
      this.formatDate = state.dateFormat;
    }
  }
  render() {
    const currentMonth = this.selectedMonth;
    const isoWeekday = getISODay(createDate(new Date(), this.selectedYear, currentMonth, 0));
    const offset = this.sundayFirst ? isoWeekday + 1 : isoWeekday;
    const minimumDate = new Date(this.minDate);
    const maximumDate = new Date(this.maxDate);
    const dateRestrictionObj = {
      minimumDate,
      maximumDate,
    };
    return (h("div", { class: "bci-core-datepicker" },
      h("div", { class: "bci-core-datepicker-container" },
        h("div", { class: "bci-core-month-container" },
          h("bci-week-header", { "data-test": "webcore.webcomponents.bci-datepicker.sunday-first", "sunday-first": this.sundayFirst }),
          h("bci-weekdays", { "data-test": "webcore.webcomponents.bci-datepicker.select-date", datesObj: this.datesObj, daysInMonth: getDaysInMonth(createDate(new Date(), this.selectedYear, currentMonth)), lastDayOfMonth: endOfMonth(createDate(new Date(), this.selectedYear, this.selectedMonth, 0)), dateRestrictionObj: dateRestrictionObj, month: currentMonth, rangeDate: this.rangeDate, offset: offset, selectedDate: this.selectedDate, todaysDate: this.today, year: this.selectedYear, formatDate: this.formatDate, timeZone: this.timeZone })))));
  }
  static get is() { return "bci-datepicker"; }
  static get originalStyleUrls() { return {
    "$": ["datepicker.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["datepicker.css"]
  }; }
  static get properties() { return {
    "minDate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "min-date",
      "reflect": false
    },
    "maxDate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "max-date",
      "reflect": false
    },
    "rangeDate": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "range-date",
      "reflect": false
    },
    "sundayFirst": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "sunday-first",
      "reflect": false
    },
    "bciDatepickerId": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "bci-datepicker-id",
      "reflect": false
    },
    "formatDate": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "format-date",
      "reflect": false
    },
    "datePickerInputObj": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "date-picker-input-obj",
      "reflect": false
    },
    "timeZone": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "time-zone",
      "reflect": false
    },
    "today": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The current day"
      },
      "attribute": "today",
      "reflect": false
    },
    "selectedDate": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Date that is currently selected"
      },
      "attribute": "selected-date",
      "reflect": false
    },
    "selectedMonth": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Month which the calendar should display."
      },
      "attribute": "selected-month",
      "reflect": false
    },
    "selectedYear": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Year which the calendar should display."
      },
      "attribute": "selected-year",
      "reflect": false
    },
    "datesObj": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "DatesObject",
        "resolved": "DatesObject",
        "references": {
          "DatesObject": {
            "location": "import",
            "path": "./datepicker.models"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Object to keep track of the currently selected date range"
      }
    }
  }; }
}

