/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Host, h, Prop, Event, State } from '@stencil/core';
import { translate } from '../../../utils/utils';
export class SearchInput {
  constructor() {
    this.searchString = '';
    this.isColumnExpanded = false;
    this.darkBackground = false;
    this.focussed = false;
    this.checkedFacilities = [];
  }
  setSearchInput(input) {
    this.search.emit(input);
  }
  handleOnFocus() {
    this.focussed = true;
    this.isColumnExpanded = true;
    this.columnInputExpanded.emit(this.isColumnExpanded);
  }
  handleOnBlur() {
    this.focussed = false;
  }
  handleBreadcrumbItemClick(level) {
    this.breadcrumbItemClicked.emit(level);
  }
  toggleInput() {
    this.isColumnExpanded = !this.isColumnExpanded;
    this.columnInputExpanded.emit(this.isColumnExpanded);
  }
  showSearchInput() {
    return this.getSelectedItemsPerLevel().length < 4;
  }
  getSelectedItemsPerLevel() {
    return this.facilitySelectorController.selectionState.selectedItemPerLevel || [];
  }
  updateCheckedFacilities() {
    var _a, _b;
    const checkedItems = ((_b = (_a = this.facilitySelectorController) === null || _a === void 0 ? void 0 : _a.selectionState) === null || _b === void 0 ? void 0 : _b.checkedItemPerLevel) || [];
    if (checkedItems.length === 0) {
      this.checkedFacilities = [];
      return;
    }
    const allCheckedItems = [...checkedItems[checkedItems.length - 1]];
    if (allCheckedItems[0] === undefined) {
      return;
    }
    const addParentItemsToAllCheckedItems = (facility) => {
      if (!facility.parentLocationId && JSON.stringify(this.checkedFacilities) !== JSON.stringify(allCheckedItems)) {
        this.checkedFacilities = allCheckedItems;
        return;
      }
      this.facilitySelectorController.getParent(facility).then((parent) => {
        if (parent) {
          allCheckedItems.unshift(parent);
          addParentItemsToAllCheckedItems(parent);
        }
      });
    };
    addParentItemsToAllCheckedItems(allCheckedItems[0]);
  }
  getCheckedItemsPerLevel() {
    const checkedItemsByLevel = [];
    for (let i = 0; i < this.facilitySelectorController.selectionState.checkedItemPerLevel.length; i++) {
      checkedItemsByLevel.push(this.facilitySelectorController.selectionState.checkedItemPerLevel[i] || []);
    }
    return checkedItemsByLevel;
  }
  getNumberOfCheckedItems() {
    return this.facilitySelectorController.selectionState.numberOfCheckedItems;
  }
  render() {
    this.updateCheckedFacilities();
    return (h(Host, { class: "facility-search-container" },
      h("div", { class: `form-group ${this.darkBackground ? 'dark-background' : ''}` },
        h("div", { class: `facility-search-path-item ${this.focussed ? 'focussed' : ''}` }, this.isColumnExpanded ? this.renderPath(this.getSelectedItemsPerLevel()) : this.renderCheckedItemsText()),
        h("input", { value: this.searchString, onFocus: () => this.handleOnFocus(), onBlur: () => this.handleOnBlur(), onInput: (event) => this.setSearchInput(event.target.value), "data-test": "webcore.webcomponents.search-input.search.input" }),
        h("span", { onClick: () => this.toggleInput(), class: "bosch-ic bosch-ic-clickable bosch-ic-search", "data-test": "webcore.webcomponents.search-input.toggle.span" }),
        h("label", { class: "label-top" }, translate('faclities')))));
  }
  renderPath(items, showLastConnector = true) {
    return items.map((item, i) => {
      if (item) {
        return [
          h("div", { "data-test": "webcore.webcomponents.search-input.breadcrum-item-click", class: "selection-path-breadcrumb", title: item.name, onClick: () => this.handleBreadcrumbItemClick(i) }, item.name),
          (i < items.length - 1 || showLastConnector) && h("span", { class: `bosch-ic bosch-ic-small bosch-ic-forward-right` }),
        ];
      }
    });
  }
  renderCheckedItemsText() {
    // If only one item is checked, then show the whole selection path
    if (this.getNumberOfCheckedItems() === 1) {
      return this.renderPath(this.checkedFacilities, false);
    }
    else {
      return (h("div", null, this.getCheckedItemsPerLevel().map((entry, index) => (h("div", null,
        index !== 0 && h("span", { class: "bosch-ic bosch-ic-small bosch-ic-forward-right" }),
        entry.length > 0 ? entry.map((res) => res.name).join(', ') : '...')))));
    }
  }
  static get is() { return "bci-facility-search-input"; }
  static get originalStyleUrls() { return {
    "$": ["search-input.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["search-input.css"]
  }; }
  static get properties() { return {
    "searchString": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "search-string",
      "reflect": false,
      "defaultValue": "''"
    },
    "facilitySelectorController": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "FacilitySelectorController",
        "resolved": "FacilitySelectorController",
        "references": {
          "FacilitySelectorController": {
            "location": "import",
            "path": "../../.."
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "isColumnExpanded": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "is-column-expanded",
      "reflect": false,
      "defaultValue": "false"
    },
    "darkBackground": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "dark-background",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "focussed": {},
    "checkedFacilities": {}
  }; }
  static get events() { return [{
      "method": "search",
      "name": "search",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }, {
      "method": "columnInputExpanded",
      "name": "columnInputExpanded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }, {
      "method": "breadcrumbItemClicked",
      "name": "breadcrumbItemClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
}

