/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/**
 * Copyright (C) 2020. Robert Bosch GmbH Copyright (C) 2020. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Host, h, Prop, Event, Method, Element } from '@stencil/core';
import { flattenArray, translate } from '../../../utils/utils';
export class SearchResultList {
  constructor() {
    this.searchString = '';
  }
  async markCheckedItems() {
    if (!this.searchResults || Object.keys(this.searchResults).length === 0) {
      return;
    }
    flattenArray(Object.values(this.searchResults)).forEach((item) => {
      const checkbox = this.el.querySelector('#checkbox-item-' + item.id);
      if (!checkbox) {
        return;
      }
      if (this.facilitySelectorController.selectionState.isChecked(item)) {
        checkbox.checked = true;
      }
      else if (this.facilitySelectorController.selectionState.isIndeterminate(item)) {
        checkbox.indeterminate = true;
      }
      else {
        checkbox.checked = false;
        checkbox.indeterminate = false;
      }
    });
  }
  highLightLabel(searchResult) {
    const label = searchResult.name;
    if (!this.searchString) {
      return label;
    }
    const indexSearch = label.toLowerCase().indexOf(this.searchString.toLowerCase());
    const highlight = label.substring(indexSearch, indexSearch + this.searchString.length);
    if (indexSearch === -1) {
      return label;
    }
    return [label.substring(0, indexSearch), h("strong", null, highlight), label.substring(indexSearch + this.searchString.length)];
  }
  handleSearchResultSelection(searchItem) {
    this.searchResultSelected.emit({ item: searchItem, isSelectedOrChecked: true });
  }
  handleSearchResultChecked({ detail: checked }, searchItem) {
    this.searchResultChecked.emit({ item: searchItem, isSelectedOrChecked: checked });
  }
  isEmptySearchResult() {
    return this.searchString && Object.keys(this.searchResults || {}).length === 0;
  }
  render() {
    return (h(Host, { class: `search-result-container ${this.isEmptySearchResult() ? 'empty' : ''}` },
      h("div", { class: 'bci-portal-dialog-content-empty-hint' }, translate('emptyMessageSearchFacility')),
      Object.keys(this.searchResults || {}).map((key) => (h("div", null,
        h("span", { class: "facility-type" }, key),
        h("ul", { class: "mdc-list", role: "listbox", "aria-label": "List with checkbox items" }, this.searchResults[key].map((searchResult, index) => (h("li", { "data-test": "webcore.webcomponents.search-result.result-list-item", class: "mdc-list-item", id: 'li-' + searchResult.id, role: "checkbox", "aria-checked": "false", tabindex: index, onClick: () => this.handleSearchResultSelection(searchResult) },
          h("span", { class: "mdc-list-item__graphic" },
            h("checkbox-input", { "data-test": "webcore.webcomponents.search-result.search-result-checked", checkboxId: 'checkbox-item-' + searchResult.id, onCheckedChange: (checked) => this.handleSearchResultChecked(checked, searchResult) })),
          h("label", { class: "mdc-list-item__text" }, this.highLightLabel(searchResult)),
          searchResult.hasChildren ? h("span", { class: "bosch-ic bosch-ic-forward-right bosch-ic-clickable" }) : '')))))))));
  }
  static get is() { return "bci-facility-search-result"; }
  static get originalStyleUrls() { return {
    "$": ["search-result.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["search-result.css"]
  }; }
  static get properties() { return {
    "searchString": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "search-string",
      "reflect": false,
      "defaultValue": "''"
    },
    "searchResults": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Record<string, Facility[]>",
        "resolved": "{ [x: string]: Facility[]; }",
        "references": {
          "Record": {
            "location": "global"
          },
          "Facility": {
            "location": "import",
            "path": "../facility.interface"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "facilitySelectorController": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "FacilitySelectorController",
        "resolved": "FacilitySelectorController",
        "references": {
          "FacilitySelectorController": {
            "location": "import",
            "path": "../../.."
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    }
  }; }
  static get events() { return [{
      "method": "searchResultSelected",
      "name": "searchResultSelected",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "ColumnItemSelectedOrChecked",
        "resolved": "ColumnItemSelectedOrChecked",
        "references": {
          "ColumnItemSelectedOrChecked": {
            "location": "import",
            "path": "../facility.interface"
          }
        }
      }
    }, {
      "method": "searchResultChecked",
      "name": "searchResultChecked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "ColumnItemSelectedOrChecked",
        "resolved": "ColumnItemSelectedOrChecked",
        "references": {
          "ColumnItemSelectedOrChecked": {
            "location": "import",
            "path": "../facility.interface"
          }
        }
      }
    }]; }
  static get methods() { return {
    "markCheckedItems": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
}

