/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { Component, h, Listen, State } from '@stencil/core';
import { NotificationClass } from '../header-message-icon/header-message-icon.interface';
import { translate } from '../../utils/utils';
export class HeaderBanner {
  constructor() {
    this.multiBanners = [];
    this.showBanner = false;
  }
  showBannerMessage(event) {
    this.showBanner = false;
    if (!event.detail) {
      return;
    }
    this.bannerMessage = {
      text: event.detail.text || '',
      bannerClass: event.detail.bannerClass || '',
      buttons: event.detail.buttons || [],
      showDismissButton: !!event.detail.showDismissButton || (event.detail.buttons || []).length === 0,
      bannerId: event.detail.bannerId || '',
    };
    this.multiBanners.push(this.bannerMessage);
    if (this.bannerMessage.buttons.length > 1 && this.bannerMessage.showDismissButton) {
      this.bannerMessage.buttons = this.bannerMessage.buttons.slice(0, 1);
      console.error('The banner component can only display a maximum of two buttons.');
    }
    if (this.bannerMessage.buttons.length > 2 && !this.bannerMessage.showDismissButton) {
      this.bannerMessage.buttons = this.bannerMessage.buttons.slice(0, 2);
      console.error('The banner component can only display a maximum of two buttons.');
    }
    this.showBanner = true;
  }
  hideBanner(event) {
    if (!event.detail) {
      return;
    }
    this.dismiss(event.detail.bannerId);
  }
  dismiss(bannerId) {
    this.showBanner = false;
    setTimeout(() => {
      this.multiBanners = this.multiBanners.filter((item) => {
        return item.bannerId !== bannerId;
      });
      this.showBanner = true;
    }, 800);
  }
  goToButtonUrl(link) {
    window.location.href = link;
  }
  getClassIcon(bannerClass) {
    const icon = 'bosch-ic bosch-ic-medium bosch-ic';
    switch (bannerClass) {
      case NotificationClass.Error:
        return `${icon}-alert-error-filled bci-portal-icon-red`;
      case NotificationClass.Success:
        return `${icon}-alert-success-filled bci-portal-icon-green`;
      case NotificationClass.Warning:
        return `${icon}-alert-warning-filled bci-portal-icon-yellow`;
      case NotificationClass.Information:
        return `${icon}-info-i bci-portal-icon-blue`;
      default:
        return ``;
    }
  }
  render() {
    if (this.bannerMessage) {
      return (h("div", null, this.multiBanners.map((item) => (h("div", { class: `bci-portal-banner ${this.showBanner ? '' : 'hide'}`, id: item.bannerId },
        h("div", { class: "bci-portal-banner-text-container" },
          h("div", { class: this.getClassIcon(item.bannerClass) }),
          h("div", null,
            " ",
            item.text)),
        h("div", { class: "bci-portal-button-container" },
          item.buttons && item.buttons.length > 0
            ? item.buttons.map((button) => (h("button", { "data-test": "webcore.webcomponents.header-banner.bci-portal-banner.button-link", class: "button-link no-arrow", onClick: () => {
                if (button.clickHandler) {
                  button.clickHandler();
                  this.dismiss(item.bannerId);
                }
                else {
                  this.goToButtonUrl(button.link);
                }
              } }, button.text)))
            : '',
          item.showDismissButton ? (h("button", { class: "button-link no-arrow", onClick: () => this.dismiss(item.bannerId) }, translate('dismiss'))) : ('')))))[this.multiBanners.length - 1]));
    }
  }
  static get is() { return "bci-portal-header-banner"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["header-banner.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["header-banner.css"]
  }; }
  static get states() { return {
    "multiBanners": {},
    "showBanner": {}
  }; }
  static get listeners() { return [{
      "name": "bci.portal.banner.show",
      "method": "showBannerMessage",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "bci.portal.banner.hide",
      "method": "hideBanner",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}

