/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Prop, h, Event, Element, State } from '@stencil/core';
import { MDCMenu } from '@material/menu';
import { MDCMenuSurface } from '@material/menu-surface';
export class HeaderIconComponent {
  constructor() {
    this.url = '';
    this.menuOpen = false;
    this.hasMenuItems = false;
  }
  componentDidRender() {
    this.menu = new MDCMenu(this.el.querySelector('.mdc-menu'));
    this.menu.setAnchorMargin({ top: 62, left: -8, right: -8 });
    this.menuSurface = new MDCMenuSurface(this.el.querySelector('.mdc-menu-surface'));
    this.menuSurface.listen('MDCMenuSurface:closed', () => {
      this.menuOpen = false;
    });
    this.menuSurface.listen('MDCMenuSurface:opened', () => {
      this.menuOpen = true;
    });
    this.hasMenuItems = this.menu.items.length > 0;
  }
  toggleMenu(event) {
    if (this.menu.items.length > 0) {
      this.menu.open = !this.menu.open;
      // Workaround for 'greedy' body click handler bug in MDCMenu
      // https://github.com/material-components/material-components-web/issues/6188
      document.body.addEventListener('click', (e) => {
        if (event.target === e.target) {
          e.stopPropagation();
        }
      }, { capture: true, once: true });
    }
    else {
      this.headerNavItemClicked.emit(this.url);
    }
  }
  render() {
    return (h("div", { class: "bci-portal-header-icon mdc-menu-surface--anchor" },
      h("span", { "data-test": "webcore.webcomponents.header-icon.bci-portal-header.menu-item", class: 'mdc-button bosch-ic bosch-ic-medium bosch-ic-clickable ' + this.iconName + (this.menuOpen ? ' menu-open' : ''), onClick: (e) => this.toggleMenu(e) },
        this.text || '',
        this.hasMenuItems ? h("span", { class: "bci-portal-navigation-item-icon-expand bosch-ic Bosch-Ic-arrow-expand-right" }) : ''),
      h("div", { class: "mdc-menu mdc-menu-surface" },
        h("div", { class: "mdc-list bci-portal-header-menu-list", role: "menu", "aria-hidden": "true", "aria-orientation": "vertical", tabindex: "-1" },
          h("slot", null)))));
  }
  static get is() { return "bci-portal-header-icon"; }
  static get originalStyleUrls() { return {
    "$": ["header-icon.component.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["header-icon.component.css"]
  }; }
  static get properties() { return {
    "iconName": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "icon-name",
      "reflect": false
    },
    "url": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "url",
      "reflect": false,
      "defaultValue": "''"
    },
    "text": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "text",
      "reflect": false
    }
  }; }
  static get states() { return {
    "menuOpen": {},
    "hasMenuItems": {}
  }; }
  static get events() { return [{
      "method": "headerNavItemClicked",
      "name": "bci.portal.header.navItemClicked",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
}

