/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Element, h, Prop, State, Watch } from '@stencil/core';
import { MDCMenu } from '@material/menu';
export class HeaderUserIconComponent {
  constructor() {
    this.hasMenuItems = false;
    this.showAccountMenu = true;
  }
  componentDidRender() {
    this.menu = new MDCMenu(this.el.querySelector('.mdc-menu'));
    this.hasMenuItems = true;
  }
  onSearchableItemsChanged(items) {
    this.el.querySelectorAll('bci-portal-header-search-icon').forEach((searchIcon) => (searchIcon.navigationItems = items));
  }
  getAvatarLabel() {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    if (((_a = this.userInfo) === null || _a === void 0 ? void 0 : _a.givenName) || ((_b = this.userInfo) === null || _b === void 0 ? void 0 : _b.familyName)) {
      let label = '';
      label += ((_c = this.userInfo) === null || _c === void 0 ? void 0 : _c.givenName) ? (_d = this.userInfo) === null || _d === void 0 ? void 0 : _d.givenName.substring(0, 1).toUpperCase() : '';
      label += ((_e = this.userInfo) === null || _e === void 0 ? void 0 : _e.familyName) ? (_f = this.userInfo) === null || _f === void 0 ? void 0 : _f.familyName.substring(0, 1).toUpperCase() : '';
      return label;
    }
    return ((_g = this.userInfo) === null || _g === void 0 ? void 0 : _g.userName) ? (_h = this.userInfo) === null || _h === void 0 ? void 0 : _h.userName.substring(0, 1).toUpperCase() : 'n/n';
  }
  getNameLabel() {
    if (!this.userInfo) {
      return 'n/n';
    }
    return this.userInfo.givenName || this.userInfo.familyName
      ? (this.userInfo.givenName + ' ' + this.userInfo.familyName).trim()
      : this.userInfo.userName;
  }
  toggleMenu(anchor, top, left, right) {
    const hostEl = this.el;
    const anchorEl = hostEl.querySelector(anchor);
    this.menu.setAnchorElement(anchorEl);
    this.menu.setAbsolutePosition(0, 0);
    this.menu.setAnchorMargin({ top, left, right });
    this.menu.open = !this.menu.open;
    // Workaround for 'greedy' body click handler bug in MDCMenu
    // https://github.com/material-components/material-components-web/issues/6188
    const triggerElements = hostEl.querySelectorAll('.user-avatar, .user-avatar .avatar, .user-avatar .user-avatar-info, .bci-portal-header-small-icon');
    const userInfo = hostEl.querySelector('.user-info');
    document.body.addEventListener('click', (e) => clickEvent(e), { capture: true, once: true });
    function clickEvent(e) {
      if (userInfo.contains(e.target)) {
        document.body.addEventListener('click', (e) => clickEvent(e), { capture: true, once: true });
      }
      else if (Array.from(triggerElements).includes(e.target)) {
        e.stopPropagation();
      }
    }
  }
  toggleAccountMenu() {
    this.showAccountMenu = !this.showAccountMenu;
  }
  copyTenantIdClick() {
    var _a;
    navigator.clipboard.writeText((_a = this.userInfo) === null || _a === void 0 ? void 0 : _a.tenantId);
    this.menu.open = !this.menu.open;
  }
  render() {
    var _a, _b, _c;
    return (h("div", { class: "bci-portal-header-user-icon mdc-menu-surface--anchor" },
      h("span", { class: "forVp2and3" },
        h("bci-portal-header-search-icon", { navigationItems: this.searchableItems })),
      h("span", { class: "bci-portal-header-small-icon bosch-ic bosch-ic-medium bosch-ic-my-brand-frame bosch-ic-clickable", "data-test": "webcore.webcomponents.header-user-icon.menu.span", onClick: () => this.toggleMenu('.bci-portal-header-small-icon', 55, 150, -23) }),
      h("div", { class: "user-avatar", "data-test": "webcore.webcomponents.header-user-icon.user-avatar", onClick: () => this.toggleMenu('.user-avatar .avatar', 48, 150, -40) },
        h("div", { class: "avatar" }, this.getAvatarLabel()),
        h("div", { class: "user-avatar-info" },
          h("span", { class: "user-name" }, this.getNameLabel()))),
      h("div", { class: "mdc-menu mdc-menu-surface" },
        h("span", { class: "user-info" },
          h("span", { class: "avatar" },
            h("span", { class: "avatar-label", "data-test": "webcore.webcomponents.header-user-icon.user-avatar.span" },
              h("span", null, this.getAvatarLabel()))),
          h("span", { class: "details" },
            h("span", { class: "user-name" }, this.getNameLabel()),
            h("span", { class: "email" }, (_a = this.userInfo) === null || _a === void 0 ? void 0 : _a.email),
            h("span", { class: "tenant" },
              h("span", { class: "tenant-name" }, (_b = this.userInfo) === null || _b === void 0 ? void 0 : _b.tenantName),
              h("span", { class: "copy-tenant-id bosch-ic-small bosch-ic-clickable bosch-ic-copy", title: 'Copy Org. ID: ' + ((_c = this.userInfo) === null || _c === void 0 ? void 0 : _c.tenantId), onClick: () => this.copyTenantIdClick() })))),
        h("span", { class: "forVp1" },
          h("bci-portal-header-search-icon", { navigationItems: this.searchableItems })),
        h("div", { "data-test": "webcore.webcomponents.header-user-icon.component.menu.menuitems", class: 'bci-portal-header-user-icon-menu mdc-list ' + (this.showAccountMenu ? 'show' : 'hide'), role: "menu", "aria-hidden": "true", "aria-orientation": "vertical", tabindex: "-1" },
          this.hasMenuItems ? (h("span", { class: "account bosch-ic bosch-ic-medium bosch-ic-my-brand-frame bosch-ic-clickable", onClick: () => this.toggleAccountMenu() },
            this.text,
            h("span", { class: "bci-portal-navigation-item-icon-expand bosch-ic Bosch-Ic-arrow-expand-right" }))) : (''),
          h("slot", null)))));
  }
  static get is() { return "bci-portal-header-user-icon"; }
  static get originalStyleUrls() { return {
    "$": ["header-user-icon.component.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["header-user-icon.component.css"]
  }; }
  static get properties() { return {
    "text": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "text",
      "reflect": false
    },
    "userInfo": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "User",
        "resolved": "User",
        "references": {
          "User": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "searchableItems": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "Array<SidebarNavItem>",
        "resolved": "SidebarNavItem[]",
        "references": {
          "Array": {
            "location": "global"
          },
          "SidebarNavItem": {
            "location": "import",
            "path": "../../interfaces"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    }
  }; }
  static get states() { return {
    "hasMenuItems": {},
    "showAccountMenu": {}
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "searchableItems",
      "methodName": "onSearchableItemsChanged"
    }]; }
}

