/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2021 Robert Bosch GmbH Copyright (C) 2021 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, h, Host, Prop, Watch, Event } from '@stencil/core';
import { translate } from '../../../../utils/utils';
export class MultiSelectorLevel {
  onLevelChange(level) {
    this.level = level;
  }
  render() {
    var _a, _b;
    return (h(Host, null,
      this.backButtonLabel ? (h("button", { class: "back-button button-primary", onClick: () => this.navigateBack.emit() },
        h("bci-icon", { name: "back-left" }),
        h("span", { class: "back-button-label" }, this.backButtonLabel))) : (''), (_b = (_a = this.level) === null || _a === void 0 ? void 0 : _a.sections) === null || _b === void 0 ? void 0 :
      _b.map((section) => (h("bci-multi-selector-level-section", { section: section, highlight: this.highlight }))),
      !this.hasSectionsWithItems() && (h("div", { class: this.level.sections[0].items.length === 0 ? 'empty-state-first-column' : 'empty-state' }, translate('noFilterResults')))));
  }
  /**
   * Function to check if a section has any items
   * returns true if any section of the level has an item
   * returns false if no section of the level has an item
   *
   * @returns boolean
   */
  hasSectionsWithItems() {
    let hasItems = true;
    this.level.sections.forEach((section) => {
      section.items.length === 0 ? (hasItems = false) : (hasItems = true);
    });
    return hasItems;
  }
  static get is() { return "bci-multi-selector-level"; }
  static get originalStyleUrls() { return {
    "$": ["multi-selector-level.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["multi-selector-level.css"]
  }; }
  static get properties() { return {
    "level": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Level",
        "resolved": "Level",
        "references": {
          "Level": {
            "location": "import",
            "path": "./model/multi-selector-level"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "nodes that gonna be shown within one level depth of the tree"
      }
    },
    "backButtonLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label of the back button"
      },
      "attribute": "back-button-label",
      "reflect": false
    },
    "highlight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "This string is highlighted in the labels"
      },
      "attribute": "highlight",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "navigateBack",
      "name": "navigateBack",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Event to notify, when back button was clicked"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get watchers() { return [{
      "propName": "level",
      "methodName": "onLevelChange"
    }]; }
}

