/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2021 Robert Bosch GmbH Copyright (C) 2021 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, Element, Event, h, Host, Prop, Watch } from '@stencil/core';
export class MultiSelectorTreeComponent {
  constructor() {
    /** text that needs to be highlighted in the result list*/
    this.filterText = '';
    /** rendered/shown levels on tree */
    this.activeLevels = [];
    this.newLevelHasBeenAdded = false;
    this.levelElement = ({ level, isLast }) => {
      var _a;
      return (h("bci-multi-selector-level", { level: level, backButtonLabel: (_a = level.parentNode) === null || _a === void 0 ? void 0 : _a.label, onNavigateBack: (event) => this.handleNavigateBack(event), highlight: isLast ? this.filterText : '' }));
    };
  }
  handleActiveLevelsChange(newLevels, oldLevels) {
    if (oldLevels.length < newLevels.length) {
      this.newLevelHasBeenAdded = true;
    }
  }
  componentDidRender() {
    if (this.newLevelHasBeenAdded) {
      this.newLevelHasBeenAdded = false;
      this.scrollToEnd();
    }
  }
  render() {
    var _a;
    const lastLevel = this.activeLevels ? this.activeLevels[this.activeLevels.length - 1] : undefined;
    return (h(Host, null, (_a = this.activeLevels) === null || _a === void 0 ? void 0 :
      _a.map((level) => (h(this.levelElement, { level: level, isLast: level === lastLevel }))),
      this.isLoading ? this.renderLoadingIndicator(this.isLoading) : ''));
  }
  renderLoadingIndicator(isLoading) {
    return (h("div", { class: "loading-indicator-icon" },
      h("bci-loading-indicator", { isLoading: isLoading })));
  }
  /**
   * closes the current active level
   */
  handleNavigateBack(event) {
    event.stopPropagation();
    this.scrollTo(-2, -1);
    setTimeout(() => {
      this.navigateBack.emit();
    }, 500);
  }
  scrollToEnd() {
    this.scrollTo(-1);
  }
  scrollTo(start, end) {
    Array.from(this.host.querySelectorAll('bci-multi-selector-level'))
      .slice(start, end)
      .forEach((element) => element.scrollIntoView({ behavior: 'smooth', block: 'nearest', inline: 'center' }));
  }
  static get is() { return "bci-multi-selector-tree"; }
  static get originalStyleUrls() { return {
    "$": ["multi-selector-tree.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["multi-selector-tree.css"]
  }; }
  static get properties() { return {
    "filterText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "text that needs to be highlighted in the result list"
      },
      "attribute": "filter-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "activeLevels": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Level[]",
        "resolved": "Level[]",
        "references": {
          "Level": {
            "location": "import",
            "path": "../multi-selector-level/model/multi-selector-level"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "rendered/shown levels on tree"
      },
      "defaultValue": "[]"
    },
    "isLoading": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "show a loading spinner"
      },
      "attribute": "is-loading",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "navigateBack",
      "name": "navigateBack",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "triggered on backwards navigation (1 level backwards)"
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }]; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "activeLevels",
      "methodName": "handleActiveLevelsChange"
    }]; }
}

