/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2021 Robert Bosch GmbH Copyright (C) 2021 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Component, h, Host, Prop, State, Watch } from '@stencil/core';
import { setLocale } from '../../../utils/locale-store';
export class MultiSelectorSearchLabel {
  constructor() {
    this.breadcrumbTrails = [];
  }
  async componentWillLoad() {
    await setLocale();
    this.computeBreadcrumbs(this.selections);
  }
  computeBreadcrumbs(selections) {
    const selectionsByRootNodeId = selections.reduce((acc, selection) => {
      const key = '' + selection[0].id;
      acc[key] = acc[key] || [];
      acc[key].push(selection);
      return acc;
    }, {});
    this.breadcrumbTrails = Object.values(selectionsByRootNodeId).map((selections) => {
      // find longest common prefix
      const [firstSelection, ...otherSelections] = selections;
      const labels = firstSelection
        .map((node, index) => otherSelections.every((selection) => selection.length > index && selection[index].id === node.id) ? node.label : null)
        .filter((label) => !!label);
      // if this longest common prefix breadcrumb trail represents more than one selected element, add the number of elements represented to the end
      if (selections.length > 1) {
        labels[labels.length - 1] += ` (${selections.length})`;
      }
      return labels;
    });
  }
  render() {
    return (h(Host, null, this.breadcrumbTrails
      .map((breadcrumbs) => this.renderBreadcrumbs(breadcrumbs))
      .map((element, index) => [index > 0 ? h("span", null, ",\u00A0") : '', element])));
  }
  renderBreadcrumb(breadcrumb, index) {
    return (h("span", { class: "breadcrumb" },
      index > 0 ? (h("span", { class: "breadcrumb-separator" },
        h("bci-icon", { name: "forward-right" }),
        "\u2009")) : (''),
      breadcrumb));
  }
  renderBreadcrumbs(breadcrumbs) {
    const [first, ...rest] = breadcrumbs;
    const last = rest.pop();
    return (h("span", { class: "breadcrumbs" },
      this.renderBreadcrumb(first, 0),
      rest.map((breadcrumb, index) => this.renderBreadcrumb(breadcrumb, index + 1)),
      !!last ? this.renderBreadcrumb(last, breadcrumbs.length - 1) : ''));
  }
  static get is() { return "bci-multi-selector-search-label"; }
  static get originalStyleUrls() { return {
    "$": ["multi-selector-search-label.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["multi-selector-search-label.css"]
  }; }
  static get properties() { return {
    "selections": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "NodePath[]",
        "resolved": "NodePath[]",
        "references": {
          "NodePath": {
            "location": "import",
            "path": "../model"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The selected Node Path is containing all nodes(starting from rootNode) leading to the selected Node(leaf)"
      }
    }
  }; }
  static get states() { return {
    "breadcrumbTrails": {}
  }; }
  static get watchers() { return [{
      "propName": "selections",
      "methodName": "computeBreadcrumbs"
    }]; }
}

