/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2021 Robert Bosch GmbH Copyright (C) 2021 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import { Key } from 'ts-key-enum';
export var KeyboardAdditionalActionKeyCodes;
(function (KeyboardAdditionalActionKeyCodes) {
  KeyboardAdditionalActionKeyCodes["Space"] = "Space";
})(KeyboardAdditionalActionKeyCodes || (KeyboardAdditionalActionKeyCodes = {}));
export var MouseActionKeyCodes;
(function (MouseActionKeyCodes) {
  MouseActionKeyCodes["MouseKey"] = "MouseKey";
})(MouseActionKeyCodes || (MouseActionKeyCodes = {}));
export class KeyEventListener {
  constructor(documentElement) {
    this.allEventKeys = Object.assign(Object.assign(Object.assign({}, Key), KeyboardAdditionalActionKeyCodes), MouseActionKeyCodes);
    this.registeredEvents = new Map();
    this.document = document;
    this.active = false;
    if (documentElement) {
      this.document = documentElement;
    }
  }
  startEventListeners(eventMap) {
    eventMap.forEach((_, eventType) => {
      this.document.addEventListener(eventType, (event) => {
        if (event instanceof KeyboardEvent && this.allEventKeys[event.code]) {
          this.notify(eventType, this.allEventKeys[event.code]);
        }
        else if (event instanceof MouseEvent) {
          this.notify(eventType, this.allEventKeys[MouseActionKeyCodes.MouseKey]);
        }
      });
    });
  }
  addListener(eventType, callback, listenerKey) {
    if (!listenerKey || eventType === 'mousedown' || eventType === 'mouseup') {
      this.addMouseEventListener(eventType, callback);
    }
    else {
      this.addKeyboardEventListener(eventType, callback, listenerKey);
    }
    return this;
  }
  removeListener(eventType, listenerKey) {
    const targetEvent = this.registeredEvents.get(eventType);
    if (targetEvent) {
      targetEvent.delete(listenerKey);
    }
    return this;
  }
  getOrCreateEventByEventType(eventType) {
    const targetEvent = this.getTargetEvent(eventType);
    return targetEvent ? targetEvent : this.createTargetEvent(eventType);
  }
  getTargetEvent(eventType) {
    return this.registeredEvents.get(eventType);
  }
  createTargetEvent(eventType) {
    const newTargetEvent = new Map();
    this.registeredEvents.set(eventType, newTargetEvent);
    this.startEventListeners(this.registeredEvents);
    return newTargetEvent;
  }
  addKeyboardEventListener(eventType, callback, listenerKey) {
    const targetEvent = this.getOrCreateEventByEventType(eventType);
    targetEvent.set(listenerKey, callback);
  }
  addMouseEventListener(eventType, callback) {
    const targetEvent = this.getOrCreateEventByEventType(eventType);
    targetEvent.set(MouseActionKeyCodes.MouseKey, callback);
  }
  notify(eventType, key) {
    var _a;
    if (!this.active) {
      return;
    }
    const targetEvent = this.registeredEvents.get(eventType);
    // calls the function belonging to the used key
    (_a = targetEvent === null || targetEvent === void 0 ? void 0 : targetEvent.get(key)) === null || _a === void 0 ? void 0 : _a();
  }
}

