/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/**
 * uniform function to create a node for e2e testing
 * @param props
 * @returns
 */
export function createNode(props = {}) {
  const timestamp = String(Date.now());
  return Object.assign({ id: timestamp, label: 'Label ' + timestamp, active: false, focused: false, path: [], selectionState: 'unchecked', children: [], hasChildren: true, level: 0, parent: null }, props);
}
/**
 * uniform function to create a node for e2e testing
 * @param props
 * @returns
 */
export function createLevelSection(props = {}) {
  return {
    items: props.items ? props.items : [this.createNode()],
    label: props.label ? props.label : '',
  };
}
/**
 * uniform function to create a node for e2e testing
 * @param props
 * @returns
 */
export function createLevel(props = {}) {
  return {
    parentNode: props.parentNode ? props.parentNode : null,
    sections: props.sections ? props.sections : [],
  };
}
export function createDataSourceMock() {
  const getChildren = createGetChildrenMock();
  const getRoots = createGetRootMock();
  const dataSource = {
    getChildren,
    getRoots,
  };
  return {
    dataSource,
    getChildren,
    getRoots,
  };
}
export function createGetChildrenMock() {
  return mockFunction();
}
export function createGetRootMock() {
  return mockFunction();
}
export function mockFunction(_fn) {
  return jest.fn();
}
export function populateDataSourceMock(getRootsMock, getChildrenMock, data) {
  getRootsMock.mockImplementation(() => Promise.resolve(getRoots(data)));
  getChildrenMock.mockImplementation((nodeId) => Promise.resolve(getChildren(nodeId, data)));
}
export function extractResolveFromPromise() {
  let extractedResolve;
  const promise = new Promise((resolve) => {
    extractedResolve = resolve;
  });
  return { promise, resolve: extractedResolve };
}
export function getRoots(tree) {
  return [...tree].map(({ id, label }) => ({
    id,
    label,
  }));
}
export function getChildren(parentId, tree) {
  if (!parentId) {
    return [];
  }
  // doing some breadth-first search starting with the root nodes
  const workingSet = [...tree];
  while (workingSet.length) {
    const node = workingSet.shift();
    if (node.id === parentId) {
      return getRoots(node.children || []);
    }
    else {
      workingSet.push(...(node.children || []));
    }
  }
  return [];
}
export const nodesPerLevel = 3;
/**
 * was used to generate the test data set. the parameter was [3,1,2]
 * @param depths
 * @returns
 */
export function generateMockData(depths) {
  const labelTexts = ['first', 'second', 'third'];
  const createNode = (index, depth, parentId, parentLabel) => {
    const nodeId = String(index + 1);
    const nodeLabel = labelTexts[index];
    const id = parentId ? `${parentId}-${nodeId}` : nodeId;
    const label = parentLabel ? `${parentLabel}-${nodeLabel}` : nodeLabel;
    const children = depth > 0 ? [0, 1, 2].reduce((acc, i) => [...acc, createNode(i, depth - 1, id, label)], []) : undefined;
    return {
      id,
      label,
      children,
    };
  };
  return depths
    .filter((_, index) => index < nodesPerLevel)
    .reduce((acc, depth, index) => [...acc, createNode(index, depth)], []);
}
export function getMockData() {
  return [
    {
      id: '1',
      label: 'first',
      children: [
        {
          id: '1-1',
          label: 'first-first',
          children: [
            {
              id: '1-1-1',
              label: 'first-first-first',
              children: [
                { id: '1-1-1-1', label: 'first-first-first-first' },
                { id: '1-1-1-2', label: 'first-first-first-second' },
                { id: '1-1-1-3', label: 'first-first-first-third' },
              ],
            },
            {
              id: '1-1-2',
              label: 'first-first-second',
              children: [
                { id: '1-1-2-1', label: 'first-first-second-first' },
                { id: '1-1-2-2', label: 'first-first-second-second' },
                { id: '1-1-2-3', label: 'first-first-second-third' },
              ],
            },
            {
              id: '1-1-3',
              label: 'first-first-third',
              children: [
                { id: '1-1-3-1', label: 'first-first-third-first' },
                { id: '1-1-3-2', label: 'first-first-third-second' },
                { id: '1-1-3-3', label: 'first-first-third-third' },
              ],
            },
          ],
        },
        {
          id: '1-2',
          label: 'first-second',
          children: [
            {
              id: '1-2-1',
              label: 'first-second-first',
              children: [
                { id: '1-2-1-1', label: 'first-second-first-first' },
                { id: '1-2-1-2', label: 'first-second-first-second' },
                { id: '1-2-1-3', label: 'first-second-first-third' },
              ],
            },
            {
              id: '1-2-2',
              label: 'first-second-second',
              children: [
                { id: '1-2-2-1', label: 'first-second-second-first' },
                { id: '1-2-2-2', label: 'first-second-second-second' },
                { id: '1-2-2-3', label: 'first-second-second-third' },
              ],
            },
            {
              id: '1-2-3',
              label: 'first-second-third',
              children: [
                { id: '1-2-3-1', label: 'first-second-third-first' },
                { id: '1-2-3-2', label: 'first-second-third-second' },
                { id: '1-2-3-3', label: 'first-second-third-third' },
              ],
            },
          ],
        },
        {
          id: '1-3',
          label: 'first-third',
          children: [
            {
              id: '1-3-1',
              label: 'first-third-first',
              children: [
                { id: '1-3-1-1', label: 'first-third-first-first' },
                { id: '1-3-1-2', label: 'first-third-first-second' },
                { id: '1-3-1-3', label: 'first-third-first-third' },
              ],
            },
            {
              id: '1-3-2',
              label: 'first-third-second',
              children: [
                { id: '1-3-2-1', label: 'first-third-second-first' },
                { id: '1-3-2-2', label: 'first-third-second-second' },
                { id: '1-3-2-3', label: 'first-third-second-third' },
              ],
            },
            {
              id: '1-3-3',
              label: 'first-third-third',
              children: [
                { id: '1-3-3-1', label: 'first-third-third-first' },
                { id: '1-3-3-2', label: 'first-third-third-second' },
                { id: '1-3-3-3', label: 'first-third-third-third' },
              ],
            },
          ],
        },
      ],
    },
    {
      id: '2',
      label: 'second',
      children: [
        { id: '2-1', label: 'second-first' },
        { id: '2-2', label: 'second-second' },
        { id: '2-3', label: 'second-third' },
      ],
    },
    {
      id: '3',
      label: 'third',
      children: [
        {
          id: '3-1',
          label: 'third-first',
          children: [
            { id: '3-1-1', label: 'third-first-first' },
            { id: '3-1-2', label: 'third-first-second' },
            { id: '3-1-3', label: 'third-first-third' },
          ],
        },
        {
          id: '3-2',
          label: 'third-second',
          children: [
            { id: '3-2-1', label: 'third-second-first' },
            { id: '3-2-2', label: 'third-second-second' },
            { id: '3-2-3', label: 'third-second-third' },
          ],
        },
        {
          id: '3-3',
          label: 'third-third',
          children: [
            { id: '3-3-1', label: 'third-third-first' },
            { id: '3-3-2', label: 'third-third-second' },
            { id: '3-3-3', label: 'third-third-third' },
          ],
        },
      ],
    },
  ];
}

