/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { Component, Element, Event, h, Host, Prop } from '@stencil/core';
export class TableCard {
  componentWillLoad() {
    this.hasActions = !!this.tableCardElement.querySelector('[slot="actions"]');
  }
  checkboxSelectionChangeMethod(state) {
    this.checkboxSelectionChange.emit(state);
  }
  render() {
    return (h(Host, null,
      h("div", { class: 'bci-table-card' + (this.selected ? ' selected' : '') + (!this.selected && this.disabled ? ' disabled' : '') },
        h("div", { class: "bci-table-columns-values" },
          h("div", { class: "bci-table-column" },
            h("slot", { name: "content" })),
          this.isMultipleSelection ? (h("div", { class: "bci-table-checkbox" },
            h("checkbox-input", { "data-test": "webcore.webcomponents.table-card.checkbox", onIndicationChange: (event) => this.checkboxSelectionChangeMethod(event.detail) }))) : (h("div", null))),
        this.hasActions ? (h("div", { class: "bci-table-row-actions" },
          h("slot", { name: "actions" }))) : (h("div", null)))));
  }
  static get is() { return "bci-table-card"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["table-card.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["table-card.css"]
  }; }
  static get properties() { return {
    "selected": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "prop for setting selection/focus state"
      },
      "attribute": "selected",
      "reflect": false
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "prop for setting disabled state"
      },
      "attribute": "disabled",
      "reflect": false
    },
    "isMultipleSelection": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "prop for multiple selection"
      },
      "attribute": "is-multiple-selection",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "checkboxSelectionChange",
      "name": "checkboxSelectionChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Event emitted if the checkbox input changes its value"
      },
      "complexType": {
        "original": "CheckboxCheckedIndication",
        "resolved": "\"checked\" | \"unchecked\"",
        "references": {
          "CheckboxCheckedIndication": {
            "location": "import",
            "path": "../checkbox-input/checkbox-input.interface"
          }
        }
      }
    }]; }
  static get elementRef() { return "tableCardElement"; }
}

