/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
/*
 * Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.
 */
import lscache from 'lscache';
import { NotificationClass } from '../components/header-message-icon/header-message-icon.interface';
export class MessagesController {
  constructor() {
    this.hour = 3600000;
    this.msgValidationInterval = this.hour * 1;
    this.maxMessageSize = 50;
    this.defaultExpiryTime = this.hour * 8;
    lscache.setExpiryMilliseconds(this.defaultExpiryTime);
    this.startDeleteOldMessagesTimer();
  }
  getMessages(localStorageKey) {
    return localStorageKey ? lscache.get(localStorageKey) || [] : [];
  }
  updateMessages(messages, localStorageKey) {
    if (localStorageKey) {
      lscache.set(localStorageKey, messages);
    }
    window.dispatchEvent(new Event('bci.portal.message.updated'));
  }
  deleteMessages(localStorageKey) {
    if (localStorageKey) {
      lscache.set(localStorageKey, []);
    }
    window.dispatchEvent(new Event('bci.portal.message.updated'));
  }
  startDeleteOldMessagesTimer() {
    setInterval(() => {
      let messages = this.getMessages();
      const now = Date.now();
      const currentSizeOrg = messages.length;
      const filteredMessages = [...messages]
        .reverse()
        .filter((message, index) => {
        return index < this.maxMessageSize && message.timeStamp + this.defaultExpiryTime > now;
      })
        .reverse();
      if (currentSizeOrg < messages.length) {
        filteredMessages.push(...messages.slice(currentSizeOrg - 1));
      }
      messages = filteredMessages;
      this.updateMessages(filteredMessages);
    }, this.msgValidationInterval);
  }
  determineClassColor(message) {
    if (!!message.isRead) {
      return '';
    }
    switch (message.class) {
      case NotificationClass.Error:
        return `bci-portal-border-red`;
      case NotificationClass.Success:
        return `bci-portal-border-green`;
      case NotificationClass.Warning:
        return `bci-portal-border-yellow`;
      default:
        return `bci-portal-border-blue`;
    }
  }
  determineClassIcon(message) {
    const icon = 'bosch-ic bosch-ic-medium bosch-ic';
    switch (message.class) {
      case NotificationClass.Error:
        return `${icon}-alert-error-filled bci-portal-icon-red`;
      case NotificationClass.Success:
        return `${icon}-alert-success-filled bci-portal-icon-green`;
      case NotificationClass.Warning:
        return `${icon}-alert-warning-filled bci-portal-icon-yellow`;
      default:
        return `${icon}-info-i bci-portal-icon-blue`;
    }
  }
}

