/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { EventEmitter } from '../../stencil-public-runtime';
import './datepicker/datepicker';
import './timepicker/timepicker';
import { DateTime, Template, TemplateGroup, TimeStamp } from './datetime-picker.interface';
import { DatePickerConfig, TimePickerConfig, TimePickerTimeFormat } from './model';
export declare class DatetimePicker {
  /** Label of the datepicker input field. */
  label: string;
  /** Placeholder of the datepicker input field. */
  placeholder: string;
  /** Id of the datepicker. */
  dateTimePickerId: string;
  /** Format of the date. */
  formatDate: string;
  /** The max date that should be selectable. */
  maxDate: number;
  /** The min date that should be selectable. */
  minDate: number;
  /**
   * Whether the timepicker should be hidden.
   * @deprecated Use hide property instead. */
  hideTime: boolean;
  /**
   * Which parts of the datepicker should be hidden
   * @type 'calendar' | 'time'
   *  */
  hide?: 'calendar' | 'time';
  /** A list of templates that are selectable. If there are no templates provided, the "Templates"-Button will not be shown. */
  templatesList: Array<Template | TemplateGroup>;
  /** Whether the "Templates"-Button should be disabled. */
  templatesDisabled: boolean;
  /** Whether should be the first day of the week in the calendar. */
  sundayFirst: boolean;
  /** Whether a range should be selectable. */
  rangeDateTime: boolean;
  /** Whether the datepicker should be disabled. */
  disabled: boolean;
  darkBackground: boolean;
  /** please use timeFormat 'H:mm'(hours and minutes) or 'hh:mm a'(hours and minutes with AM/PM) or 'hh:mm:ss a(hours,minutes,seconds with AM/PM)',for the date range picker  */
  timeFormat: TimePickerTimeFormat;
  timeZone: string;
  dateTime: DateTime;
  timeStamp: TimeStamp;
  dateTimePickerInputObj: DateTime;
  formatTime: string;
  inputPlaceholder: string;
  datePickerConfig: DatePickerConfig;
  timePickerConfig: TimePickerConfig;
  hideSeconds: boolean;
  /** Reactive state for UI changes depening on overlay opening state */
  private overlayVisible;
  /**
   * Event emitted when the datetime picker dropdown is closed via the enter key,
   * tab key on last input field.
   * Value emitted is the string from the main input field
   */
  newDateSelected: EventEmitter<DateTime>;
  private dateTimePickerInput;
  private datepickerRef;
  private overlayHandler;
  private formatService;
  private dateService;
  private showTemplates;
  private selectedTemplateId;
  componentWillLoad(): Promise<void>;
  componentDidLoad(): void;
  onDateTimeChanged(dateTime: DateTime): void;
  onTimeStampChanged(timeStamp: TimeStamp): void;
  onPlaceholderChanged(): void;
  /**
   * - listens to the 'OverlayOpenedEvent' event which is triggered when another datetimepicker on the page is opened (see overlay component)
   * - if this is the case and the overlay of this datetimepicker is open (which closes through the event) the placeholder text needs to be updated
   * - therefore the placeholder is shown again after the user opens the datetimepicker and closes it through the opening of another datetimepicker on the page
   */
  updatePlaceholderOnOverlayOpenedEvent(): void;
  handleMouseDown(event: any): void;
  handleTouchStart(event: any): void;
  private handleClickOutside;
  handleEscape(ev: KeyboardEvent): void;
  handleTab(ev: KeyboardEvent): void;
  handleEnter(ev: KeyboardEvent): void;
  toggle(): Promise<void>;
  showPickerContainer(): void;
  private getStartupDateTime;
  private updateDatePickerInput;
  /** Function to reset the datepicker's selection */
  resetSelection(): Promise<void>;
  private getInputValue;
  private get openingIcon();
  private updateDatePickerComponent;
  /** Corrects data based on configuration */
  private correctData;
  private setInputPlaceholderText;
  private save;
  private isNull;
  private handleSelectionChanged;
  private handleDateChange;
  private handleTimeChange;
  private handleRangeChange;
  private hideOverlay;
  render(): any;
}

