/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { DateFactoryConfig, InputData, InputDataValidity } from '../model';
import { FormattingService } from './formatting-service';
/**
 * This config is used when no config was provided to the dateServiceFactory function
 */
export declare const defaultDateConfig: DateFactoryConfig;
/**
 * Uses the defaultDateConfig if no value is provided
 * @param config
 * @returns
 */
export declare function dateServiceFactory(config?: Partial<DateFactoryConfig>): DateService;
/**
 * Used to manipulate date values
 */
export declare class DateService {
  hideSeconds: boolean;
  rangeDateTime: boolean;
  minDate: number;
  maxDate: number;
  formatService: FormattingService;
  constructor(hideSeconds: boolean, rangeDateTime: boolean, minDate: number, maxDate: number, formatService: FormattingService);
  getTodayValue(timeZone?: string): Date;
  getDataFromInput(input: string): InputData;
  /**
   * check whether the input contains valid date and time values and that they are also in the correct time order.
   * @param input
   * @returns
   */
  validateInput(input: Omit<InputData, 'valid'>): InputDataValidity;
  /**
   * Corrects date values, puts them within allowed ranges and swaps values if necessary.
   * @param dateObj dateStart, dateEnd, timeStart, timeEnd
   * @returns corrected dateStart, dateEnd, timeStart, timeEnd
   */
  correctInputData(dateObj: Omit<InputData, 'valid'>): Omit<InputData, 'valid'>;
  private dateWithinAllowedRange;
  /**
   * Creates valid InputData object from two dates.
   * @param dateStart Date to infer dateStart and timeStart from
   * @param dateEnd Date to infer dateEnd and timeEnd from
   */
  private createInputData;
  /**
   * Returns utc timestamp created by timezone from property time-zone or with local browser timezone
   * @param date string parsed with formatDate
   * @param time string in format 00:00
   */
  toSelectionTimestamp(date: string | null, time: string | null): number | undefined;
  private getDateFromDateInput;
  private getTimeFromDateInput;
  private getPartFromDateInput;
  private get DATE_FORMATS();
  private get TIME_FORMATS();
}

