/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { FormattingFactoryConfig } from '../model';
/**
 * is used in the formattingServiceFactory if no valid is provided
 */
export declare const defaultFactoryConfig: FormattingFactoryConfig;
/**
 * Uses the defaultFactoryConfig if nothing was provided
 * @param config
 * @returns
 */
export declare function formattingServiceFactory(config?: Partial<FormattingFactoryConfig>): FormattingService;
/**
 * Used to format date values
 */
export declare class FormattingService {
  dateFormat: string;
  timeFormat: string;
  timeZone?: string;
  constructor(dateFormat: string, timeFormat: string, timeZone?: string);
  /**
   * Returns date in any format with utc timezone set by property time-zone or with local browser timezone
   * @param date
   * @returns
   */
  formatDate(date: Date | string | number | undefined): string;
  formatDateToDate(date: string): Date;
  /**
   * Returns time in any format with utc timezone set by property time-zone or with local browser timezone
   * @param time
   * @returns
   */
  formatTime(time: Date | number | undefined): string;
  /**
   * Formats a date/time range for display
   * @param {string} [dateStart] - The start date
   * @param {string} [dateEnd] - The end date
   * @param {string} [timeStart] - The start time
   * @param {string} [timeEnd] - The end time
   * @returns {string} The formatted date/time range string
   */
  formatRange(dateStart?: string, dateEnd?: string, timeStart?: string, timeEnd?: string): string;
  formatDateWithOptionalTime(date: string, time?: string | null): Date;
  timestampToZonedTime(timestamp: number): Date;
  private formatNumber;
}

