/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { EventEmitter } from '../../stencil-public-runtime';
import { SidebarNavItem } from '../../interfaces/navigation.interface';
import { HeaderSearchItem } from './header-search-icon.interface';
export declare class HeaderIconSearch {
  searchList: HeaderSearchItem[];
  searchInput: HTMLInputElement;
  searchOptions: {
    key: string;
    allowTypo: boolean;
    limit: number;
  };
  thresholdScoreLimit: number;
  hasTypo: boolean;
  navigationItems: SidebarNavItem[];
  showSearchInput: boolean;
  searchString: string;
  searchResults: HeaderSearchItem[];
  searchResultItemClicked: EventEmitter;
  componentWillLoad(): Promise<void>;
  handleKeyDown(ev: KeyboardEvent): void;
  /**
   * Searches through the search list to find a match
   * @param searchStr
   */
  search(searchStr: string): void;
  /**
   * Gets an array of searchable titles and tags based on the views array
   * @param navigationItems
   */
  getSearchList(navigationItems: SidebarNavItem[], parentItem?: SidebarNavItem): HeaderSearchItem[];
  /**
   * Returns individual list item for title and tags.
   * @param navigationItem
   */
  getSearchListItems(navigationItem: SidebarNavItem, parentItem?: SidebarNavItem): HeaderSearchItem[];
  /**
   * Method to highlight search text in result
   * @param text
   * @param searchString
   */
  getSearchResultText(searchItem: HeaderSearchItem, searchString: string): string;
  /**
   * Called on click of a search result
   * Navigates to desired view
   * @param path
   */
  navigateToItem(detail: HeaderSearchItem): void;
  /**
   * Called when user clicks on the search icon in the header
   */
  showSearch(): void;
  /**
   * Called when the user closes the search input
   */
  closeSearch(): void;
  /**
   * Called when the user clears the search input
   */
  clearSearch(): void;
  render(): any;
}

