/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { CheckedState } from '.';
export declare type MultiSelectorDataSource = RootNode[] | RootDataSource;
export interface RootDataSource {
  /** returns the children of a given node element */
  getChildren(nodeId: Node['id']): Promise<Node[]>;
  /** returns the root level(all root nodes) of the selector */
  getRoots(): Promise<Node[]>;
}
/**
 * You need to use a RootNode if you want to provide multiple tree structures at once in the multi selector.
 * The user can then select which tree structure he wants to use and based on that the dataSource is selected.
 */
export interface RootNode {
  id: string;
  /**
   * Represents an element within the filter dropdown
   * Used for the UI and needs to be language specific (Search Type)
   * */
  label: string;
  dataSource: RootDataSource;
}
/**
 * Node is the equivalent for a selectable row in the multi selector column
 */
export interface Node {
  id: string;
  /**
   * Needs to be language specific
   */
  label: string;
  /**
   * True is assumed if its not defined
   */
  hasChildren?: boolean;
  /**
   * Needs to be language specific
   */
  typeLabel?: string;
}
/**
 * Version 2:
 * Types of nodes that represents a node grouping on the UI
 *
 * Is needed for the "filter by" functionality and contains all node grouping names (sections of a column)
 */
export declare type NodeLink = Pick<Node, 'id' | 'label'>;
/**
 * [0] level 0 (root)
 * [1] level 1
 * ...
 * [length - 1] self
 */
export declare type NodePath = NodeLink[];
/**
 * Whenever the selection is finished, i.e. the selector is closed.
 */
export interface SelectionFinishedEvent {
  /**
   * All currently selected nodes
   */
  selections: NodePath[];
  /**
   * root node id is optional, because there may not be a RootNode object
   */
  rootId?: string;
}
/**
 * Whenever the selection is save, this event will be emitted
 */
export declare type SelectionSaveEvent = SelectionFinishedEvent;
export declare type SelectionChangedReason = CheckedState | 'reset';
/**
 * Whenever the user selects/unselects a node, this event will be emitted
 */
export interface SelectionChangedEvent extends SelectionSaveEvent {
  /**
   * The affeced node
   */
  changedNode: NodePath;
  /**
   * What happened to the element
   */
  reason: SelectionChangedReason;
}
/**
 * Emitted when a initial selection was provided, but the values do no longer exist.
 */
export interface InvalidPreselectionErrorEvent {
  /** These nodes are now removed from the selection as they are invalid */
  invalidNodes: NodePath[];
}

