/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { NodePath } from '../../../../model';
import { LevelSection } from '../../../multi-selector-level-section/model';
import { Level } from '../../../multi-selector-level/model';
import { MultiSelectorTree, MultiSelectorTreeNode } from '../../model';
/** Stateless Service to handle the Multi-Selector Tree */
/**
 * returns an array of the nodes along the given nodePath and given node link ids as a flat array
 *
 * @param tree
 * @param nodePath
 * @returns
 * @example
 * Data set (tree structure):
 * BaP
 * - A100
 * -- L1c
 * -- L1b
 * - A200
 * -- L2a
 * -- L2b
 * AnP
 * - A300
 * -- L3a
 * -- L3c
 * - A400
 * -- L4b
 * -- L4c
 *
 * Path: BaP, A200, L2a
 *
 * Result (array): [
 * {
 * BaP
 * - A100
 * -- L1c
 * -- L1b
 * - A200
 * -- L2a
 * -- L2b},
 * {
 * - A200
 * -- L2a
 * -- L2},
 * {
 * -- L2}
 * ]
 *
 * The nodes are returned in an array form, but they retain there references to there children
 */
export declare function getNodesFromTreeByNodePath(tree: MultiSelectorTree, nodePath: NodePath): MultiSelectorTreeNode[];
/**
 * sets all nodes and subNodes inactive in the given tree (deep)
 * @param tree
 * @returns
 */
export declare function setAllNodesInTreeToInactive(tree: MultiSelectorTree): MultiSelectorTree;
/**
 * clones the given node and sets also the proper connection to the parent Node (if exists)
 * !!No deep cloning!!
 * @param node
 * @returns
 */
export declare function cloneNode(node: MultiSelectorTreeNode): MultiSelectorTreeNode;
/**
 * returns all active levels of the tree (deep)
 * @param tree
 * @returns
 */
export declare function getActiveLevels(tree: MultiSelectorTree, previousLevelActiveNode?: MultiSelectorTreeNode): Level[];
export declare function getNodeFromTreeByNodePath(tree: MultiSelectorTree, nodePath: NodePath | undefined): MultiSelectorTreeNode;
/**
 * create a level with sections
 * @param nodes
 * @returns
 */
export declare function createLevelByNodes(nodes: MultiSelectorTreeNode[], previousLevelActiveNode?: MultiSelectorTreeNode): Level;
/**
 * create a level with sections
 * @param nodes
 * @returns
 */
export declare function createLevelSectionsByNodes(nodes: MultiSelectorTreeNode[]): LevelSection[];
export declare function getNodesByLevel(level: Level): any[];
export declare function sortByLabelAsc(items: MultiSelectorTreeNode[]): MultiSelectorTreeNode[];

