/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { EventEmitter } from '../../stencil-public-runtime';
import { InvalidPreselectionErrorEvent, MultiSelectorDataSource, NodePath, SelectionChangedEvent, SelectionFinishedEvent } from './model';
import { SelectionMode } from './model';
export declare class MultiSelector {
  /**
   * Mark this element to be shown on dark background (it will adapt styling). Default: false.
   */
  darkBackground: boolean;
  /** DataSource that provides function implementations to receive/call data content */
  dataSource: MultiSelectorDataSource;
  /** defines the mode of selection - if single, then is only 1 selection at a time possible (default multi) */
  selectionMode: SelectionMode;
  /** pre selection */
  initialSelection: NodePath[];
  /** Enforce single column mode, normally only used for smaller screens / selectors. */
  singleColumnMode: boolean;
  /** Prevents going into fullscreen / responsive mode on low resolutions. */
  preventFullscreen: boolean;
  /** state that keeps the focus as long as the user interacts with any component part of this multi selector (including overlay) */
  focused: boolean;
  /** Emits the final selection. It is triggered whenever user closes selector. */
  selectionFinished: EventEmitter<SelectionFinishedEvent>;
  /** Emits all selected elements and emits when any element selection was added/removed/changed  */
  selectionChanged: EventEmitter<SelectionChangedEvent>;
  /** Emits all invalid elements when they are detected */
  invalidSelection: EventEmitter<InvalidPreselectionErrorEvent>;
  private selectedDataSource;
  private searchValue;
  private activeLevels;
  private isLoading;
  private enforceOverlayViewport;
  private inputComponents;
  private readonly showSave;
  private readonly overlayVP1Config;
  private readonly overlayVP2And3Config;
  private currentOverlayConfig;
  private _dataSources;
  private overlayHandler;
  private multiSelectorManager;
  private widthObserver;
  private overlayKeyEventListener;
  private inputKeyObserver;
  hideDialog(): void;
  onDataSourceUpdate(): void;
  onSelectionModeChange(): void;
  onSearchChange(newSearch: string): void;
  onInitialSelectionChange(newInitialSelection: NodePath[]): void;
  componentDidLoad(): void;
  componentWillLoad(): Promise<void>;
  render(): any;
  /**
   * @returns the standard Input component
   */
  private renderStandardInput;
  /**
   * @returns a rendered Input dedicated for the overlay component
   */
  private renderOverlayInput;
  private renderInput;
  private initiateTreeManager;
  private setupHideOverlayListeners;
  private setupWidthObserver;
  private setupKeyListeners;
  private setupSearchInputKeyListener;
  private onMultiSelectorInputArrowDown;
  private onArrowUp;
  private setupOverlayKeyListener;
  private handleResetSelection;
  private handleNodeActiveChange;
  private handleNodeSelectionChange;
  private handleNavigateBack;
  private handleOnSelectSearchType;
  private handleClearSearch;
  private handleOnFinish;
  private handleOnMultiSelectorInputFocus;
  private selectionUpdateObserverFunction;
  private onActiveChangeObserverFunction;
}

