/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { Key } from 'ts-key-enum';
export declare type KeyboardKeyCodes = keyof (typeof Key & typeof KeyboardAdditionalActionKeyCodes);
export declare type KeyboardEventType = 'keydown' | 'keypress' | 'keyup';
export declare enum KeyboardAdditionalActionKeyCodes {
  Space = "Space"
}
export declare type MouseEventType = 'mousedown' | 'mouseup';
export declare enum MouseActionKeyCodes {
  MouseKey = "MouseKey"
}
export declare type EventType = KeyboardEventType | MouseEventType;
export declare type KeyCodes = KeyboardKeyCodes | keyof typeof MouseActionKeyCodes;
export declare type ListenerOfKnownKeyCodes = Map<KeyCodes, Callback>;
declare type Callback = () => void;
export declare class KeyEventListener {
  private allEventKeys;
  private registeredEvents;
  private document;
  active: boolean;
  private startEventListeners;
  constructor(documentElement?: Document);
  /**
   * adds a listener of a specific eventType
   * Overload for Keyboard and Mouse Listener
   * @param eventType
   * @param listenerKey
   * @param callback
   * @returns
   */
  addListener(eventType: MouseEventType, callback: Callback): this;
  addListener(eventType: KeyboardEventType, callback: Callback, listenerKey: KeyboardKeyCodes): this;
  removeListener(eventType: MouseEventType): this;
  removeListener(eventType: KeyboardEventType, listenerKey: KeyboardKeyCodes): this;
  private getOrCreateEventByEventType;
  private getTargetEvent;
  private createTargetEvent;
  private addKeyboardEventListener;
  private addMouseEventListener;
  private notify;
}
export {};

