/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { BannerButton } from '../components/header-banner/header-banner.interface';
import { NotificationClass } from '../components/header-message-icon/header-message-icon.interface';
import { SidebarNavItemAction } from './navigation.interface';
export interface UserSettings {
  language: string;
  timezone: string;
  dateFormat: string;
  timeFormat: string;
}
export interface User {
  tenant: string;
  tenantId: string;
  tenantName: string;
  name: string;
  email: string;
  userId: string;
  userName: string;
  givenName: string;
  familyName: string;
}
export declare type Context = ContextWithContribution | ContextLinkWithContribution | ContextWithContributionRequest | ContextDialogViewWithContribution;
export declare type ContextBase = {
  contributionId: string;
  contextName: string;
  contextVersion: string;
  contributorApplicationId: string;
};
export declare type ContextWithContribution = ContextBase & {
  contextType: 'DATA' | 'ACTION';
  contribution: Record<string, string>;
};
export declare type ContextLinkWithContribution = ContextBase & {
  contribution: LinkContribution;
  contextType: 'LINK';
};
export declare type ContextDialogViewWithContribution = ContextBase & {
  contribution: ContributionRequest;
  contextType: 'DIALOG_VIEW';
};
export declare type ContextWithContributionRequest = ContextBase & {
  contextType: 'DATA';
  contributionRequest: Record<string, ContributionRequest>;
};
export declare type ContributionRequest = {
  url: string;
};
export interface Contribution {
  urlKpiMeta: string;
  urlKpiValues: string;
}
export interface LinkContribution {
  path?: string;
  navigationPath: string[];
  urlKpiMeta?: string;
  urlKpiValues?: string;
}
export interface CookieSettings {
  marketing: boolean;
  comfort: boolean;
  processing: string;
}
export interface PortalContext {
  getUserSettings(): UserSettings;
  getSCSUserSetting(key: string): Promise<any>;
  getUser(): User;
  getToken(): string;
  dispatchMessage(messageClass: NotificationClass, title: string, description?: string, link?: string): any;
  showBanner(bannerClass: NotificationClass, text: string, buttons: BannerButton, showDismissButton: boolean): any;
  getContextContributions(type: string, name: string, version: string): Promise<Context[]>;
  getContextContributionById(contributionId: string): Promise<Context | undefined>;
  getDataForContextContribution<T>(contributionId: string, urlSelector: string, placeholderParams: Record<string, string>): Promise<T>;
  showContextMenu(targetElement: any, context: string, contextParams: any): SidebarNavItemAction[];
  hasUserPermission(privilege: string, applicationId: string, resourceOwningTenantId: string, resourceType: string, resourceId: string): Promise<boolean>;
  getActions(context: string): SidebarNavItemAction[];
  getCookieSettings(): CookieSettings;
  logout(): void;
  enableBrowserFullscreen(): boolean;
  disableBrowserFullscreen(): boolean;
  enablePortalFullscreen(): boolean;
  disablePortalFullscreen(): boolean;
}
export interface Logger {
  error(msg: string, object?: any): any;
  info(msg: string, object?: any): any;
  warn(msg: string, object?: any): any;
}
export declare type MaximizedMode = 'None' | 'Maximized' | 'MaximizedMenuOpen';

