/* Copyright (C) 2023. Robert Bosch GmbH Copyright (C) 2023. Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved. */
import { p as proxyCustomElement, H, c as createEvent, h, b as Host } from './p-6fc5b736.js';
import { n as normalizeKey, K as KEY, C as Corner, s as strings$4, a as strings$5, M as MDCMenu } from './p-a0abfb33.js';
import { t as translate } from './p-9dead869.js';
import { d as setLocale } from './p-d4c86b19.js';
import { _ as __extends, a as __assign, b as __values } from './p-8b4a0103.js';
import { M as MDCFoundation, a as MDCComponent, e as estimateScrollWidth } from './p-52859ac2.js';
import { M as MDCRipple, b as MDCRippleFoundation } from './p-ab919fb6.js';

/**
 * @license
 * Copyright 2016 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var cssClasses$4 = {
    LABEL_FLOAT_ABOVE: 'mdc-floating-label--float-above',
    LABEL_REQUIRED: 'mdc-floating-label--required',
    LABEL_SHAKE: 'mdc-floating-label--shake',
    ROOT: 'mdc-floating-label',
};

/**
 * @license
 * Copyright 2016 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCFloatingLabelFoundation = /** @class */ (function (_super) {
    __extends(MDCFloatingLabelFoundation, _super);
    function MDCFloatingLabelFoundation(adapter) {
        var _this = _super.call(this, __assign(__assign({}, MDCFloatingLabelFoundation.defaultAdapter), adapter)) || this;
        _this.shakeAnimationEndHandler = function () {
            _this.handleShakeAnimationEnd();
        };
        return _this;
    }
    Object.defineProperty(MDCFloatingLabelFoundation, "cssClasses", {
        get: function () {
            return cssClasses$4;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCFloatingLabelFoundation, "defaultAdapter", {
        /**
         * See {@link MDCFloatingLabelAdapter} for typing information on parameters and return types.
         */
        get: function () {
            // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
            return {
                addClass: function () { return undefined; },
                removeClass: function () { return undefined; },
                getWidth: function () { return 0; },
                registerInteractionHandler: function () { return undefined; },
                deregisterInteractionHandler: function () { return undefined; },
            };
            // tslint:enable:object-literal-sort-keys
        },
        enumerable: false,
        configurable: true
    });
    MDCFloatingLabelFoundation.prototype.init = function () {
        this.adapter.registerInteractionHandler('animationend', this.shakeAnimationEndHandler);
    };
    MDCFloatingLabelFoundation.prototype.destroy = function () {
        this.adapter.deregisterInteractionHandler('animationend', this.shakeAnimationEndHandler);
    };
    /**
     * Returns the width of the label element.
     */
    MDCFloatingLabelFoundation.prototype.getWidth = function () {
        return this.adapter.getWidth();
    };
    /**
     * Styles the label to produce a shake animation to indicate an error.
     * @param shouldShake If true, adds the shake CSS class; otherwise, removes shake class.
     */
    MDCFloatingLabelFoundation.prototype.shake = function (shouldShake) {
        var LABEL_SHAKE = MDCFloatingLabelFoundation.cssClasses.LABEL_SHAKE;
        if (shouldShake) {
            this.adapter.addClass(LABEL_SHAKE);
        }
        else {
            this.adapter.removeClass(LABEL_SHAKE);
        }
    };
    /**
     * Styles the label to float or dock.
     * @param shouldFloat If true, adds the float CSS class; otherwise, removes float and shake classes to dock the label.
     */
    MDCFloatingLabelFoundation.prototype.float = function (shouldFloat) {
        var _a = MDCFloatingLabelFoundation.cssClasses, LABEL_FLOAT_ABOVE = _a.LABEL_FLOAT_ABOVE, LABEL_SHAKE = _a.LABEL_SHAKE;
        if (shouldFloat) {
            this.adapter.addClass(LABEL_FLOAT_ABOVE);
        }
        else {
            this.adapter.removeClass(LABEL_FLOAT_ABOVE);
            this.adapter.removeClass(LABEL_SHAKE);
        }
    };
    /**
     * Styles the label as required.
     * @param isRequired If true, adds an asterisk to the label, indicating that it is required.
     */
    MDCFloatingLabelFoundation.prototype.setRequired = function (isRequired) {
        var LABEL_REQUIRED = MDCFloatingLabelFoundation.cssClasses.LABEL_REQUIRED;
        if (isRequired) {
            this.adapter.addClass(LABEL_REQUIRED);
        }
        else {
            this.adapter.removeClass(LABEL_REQUIRED);
        }
    };
    MDCFloatingLabelFoundation.prototype.handleShakeAnimationEnd = function () {
        var LABEL_SHAKE = MDCFloatingLabelFoundation.cssClasses.LABEL_SHAKE;
        this.adapter.removeClass(LABEL_SHAKE);
    };
    return MDCFloatingLabelFoundation;
}(MDCFoundation));

/**
 * @license
 * Copyright 2016 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCFloatingLabel = /** @class */ (function (_super) {
    __extends(MDCFloatingLabel, _super);
    function MDCFloatingLabel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCFloatingLabel.attachTo = function (root) {
        return new MDCFloatingLabel(root);
    };
    /**
     * Styles the label to produce the label shake for errors.
     * @param shouldShake If true, shakes the label by adding a CSS class; otherwise, stops shaking by removing the class.
     */
    MDCFloatingLabel.prototype.shake = function (shouldShake) {
        this.foundation.shake(shouldShake);
    };
    /**
     * Styles the label to float/dock.
     * @param shouldFloat If true, floats the label by adding a CSS class; otherwise, docks it by removing the class.
     */
    MDCFloatingLabel.prototype.float = function (shouldFloat) {
        this.foundation.float(shouldFloat);
    };
    /**
     * Styles the label as required.
     * @param isRequired If true, adds an asterisk to the label, indicating that it is required.
     */
    MDCFloatingLabel.prototype.setRequired = function (isRequired) {
        this.foundation.setRequired(isRequired);
    };
    MDCFloatingLabel.prototype.getWidth = function () {
        return this.foundation.getWidth();
    };
    MDCFloatingLabel.prototype.getDefaultFoundation = function () {
        var _this = this;
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        var adapter = {
            addClass: function (className) { return _this.root.classList.add(className); },
            removeClass: function (className) { return _this.root.classList.remove(className); },
            getWidth: function () { return estimateScrollWidth(_this.root); },
            registerInteractionHandler: function (evtType, handler) {
                return _this.listen(evtType, handler);
            },
            deregisterInteractionHandler: function (evtType, handler) {
                return _this.unlisten(evtType, handler);
            },
        };
        // tslint:enable:object-literal-sort-keys
        return new MDCFloatingLabelFoundation(adapter);
    };
    return MDCFloatingLabel;
}(MDCComponent));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var cssClasses$3 = {
    LINE_RIPPLE_ACTIVE: 'mdc-line-ripple--active',
    LINE_RIPPLE_DEACTIVATING: 'mdc-line-ripple--deactivating',
};

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCLineRippleFoundation = /** @class */ (function (_super) {
    __extends(MDCLineRippleFoundation, _super);
    function MDCLineRippleFoundation(adapter) {
        var _this = _super.call(this, __assign(__assign({}, MDCLineRippleFoundation.defaultAdapter), adapter)) || this;
        _this.transitionEndHandler = function (evt) {
            _this.handleTransitionEnd(evt);
        };
        return _this;
    }
    Object.defineProperty(MDCLineRippleFoundation, "cssClasses", {
        get: function () {
            return cssClasses$3;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCLineRippleFoundation, "defaultAdapter", {
        /**
         * See {@link MDCLineRippleAdapter} for typing information on parameters and return types.
         */
        get: function () {
            // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
            return {
                addClass: function () { return undefined; },
                removeClass: function () { return undefined; },
                hasClass: function () { return false; },
                setStyle: function () { return undefined; },
                registerEventHandler: function () { return undefined; },
                deregisterEventHandler: function () { return undefined; },
            };
            // tslint:enable:object-literal-sort-keys
        },
        enumerable: false,
        configurable: true
    });
    MDCLineRippleFoundation.prototype.init = function () {
        this.adapter.registerEventHandler('transitionend', this.transitionEndHandler);
    };
    MDCLineRippleFoundation.prototype.destroy = function () {
        this.adapter.deregisterEventHandler('transitionend', this.transitionEndHandler);
    };
    MDCLineRippleFoundation.prototype.activate = function () {
        this.adapter.removeClass(cssClasses$3.LINE_RIPPLE_DEACTIVATING);
        this.adapter.addClass(cssClasses$3.LINE_RIPPLE_ACTIVE);
    };
    MDCLineRippleFoundation.prototype.setRippleCenter = function (xCoordinate) {
        this.adapter.setStyle('transform-origin', xCoordinate + "px center");
    };
    MDCLineRippleFoundation.prototype.deactivate = function () {
        this.adapter.addClass(cssClasses$3.LINE_RIPPLE_DEACTIVATING);
    };
    MDCLineRippleFoundation.prototype.handleTransitionEnd = function (evt) {
        // Wait for the line ripple to be either transparent or opaque
        // before emitting the animation end event
        var isDeactivating = this.adapter.hasClass(cssClasses$3.LINE_RIPPLE_DEACTIVATING);
        if (evt.propertyName === 'opacity') {
            if (isDeactivating) {
                this.adapter.removeClass(cssClasses$3.LINE_RIPPLE_ACTIVE);
                this.adapter.removeClass(cssClasses$3.LINE_RIPPLE_DEACTIVATING);
            }
        }
    };
    return MDCLineRippleFoundation;
}(MDCFoundation));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCLineRipple = /** @class */ (function (_super) {
    __extends(MDCLineRipple, _super);
    function MDCLineRipple() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCLineRipple.attachTo = function (root) {
        return new MDCLineRipple(root);
    };
    /**
     * Activates the line ripple
     */
    MDCLineRipple.prototype.activate = function () {
        this.foundation.activate();
    };
    /**
     * Deactivates the line ripple
     */
    MDCLineRipple.prototype.deactivate = function () {
        this.foundation.deactivate();
    };
    /**
     * Sets the transform origin given a user's click location.
     * The `rippleCenter` is the x-coordinate of the middle of the ripple.
     */
    MDCLineRipple.prototype.setRippleCenter = function (xCoordinate) {
        this.foundation.setRippleCenter(xCoordinate);
    };
    MDCLineRipple.prototype.getDefaultFoundation = function () {
        var _this = this;
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        var adapter = {
            addClass: function (className) { return _this.root.classList.add(className); },
            removeClass: function (className) { return _this.root.classList.remove(className); },
            hasClass: function (className) { return _this.root.classList.contains(className); },
            setStyle: function (propertyName, value) { return _this.root.style.setProperty(propertyName, value); },
            registerEventHandler: function (evtType, handler) { return _this.listen(evtType, handler); },
            deregisterEventHandler: function (evtType, handler) { return _this.unlisten(evtType, handler); },
        };
        // tslint:enable:object-literal-sort-keys
        return new MDCLineRippleFoundation(adapter);
    };
    return MDCLineRipple;
}(MDCComponent));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var strings$3 = {
    NOTCH_ELEMENT_SELECTOR: '.mdc-notched-outline__notch',
};
var numbers$1 = {
    // This should stay in sync with $mdc-notched-outline-padding * 2.
    NOTCH_ELEMENT_PADDING: 8,
};
var cssClasses$2 = {
    NO_LABEL: 'mdc-notched-outline--no-label',
    OUTLINE_NOTCHED: 'mdc-notched-outline--notched',
    OUTLINE_UPGRADED: 'mdc-notched-outline--upgraded',
};

/**
 * @license
 * Copyright 2017 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCNotchedOutlineFoundation = /** @class */ (function (_super) {
    __extends(MDCNotchedOutlineFoundation, _super);
    function MDCNotchedOutlineFoundation(adapter) {
        return _super.call(this, __assign(__assign({}, MDCNotchedOutlineFoundation.defaultAdapter), adapter)) || this;
    }
    Object.defineProperty(MDCNotchedOutlineFoundation, "strings", {
        get: function () {
            return strings$3;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCNotchedOutlineFoundation, "cssClasses", {
        get: function () {
            return cssClasses$2;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCNotchedOutlineFoundation, "numbers", {
        get: function () {
            return numbers$1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCNotchedOutlineFoundation, "defaultAdapter", {
        /**
         * See {@link MDCNotchedOutlineAdapter} for typing information on parameters and return types.
         */
        get: function () {
            // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
            return {
                addClass: function () { return undefined; },
                removeClass: function () { return undefined; },
                setNotchWidthProperty: function () { return undefined; },
                removeNotchWidthProperty: function () { return undefined; },
            };
            // tslint:enable:object-literal-sort-keys
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Adds the outline notched selector and updates the notch width calculated based off of notchWidth.
     */
    MDCNotchedOutlineFoundation.prototype.notch = function (notchWidth) {
        var OUTLINE_NOTCHED = MDCNotchedOutlineFoundation.cssClasses.OUTLINE_NOTCHED;
        if (notchWidth > 0) {
            notchWidth += numbers$1.NOTCH_ELEMENT_PADDING; // Add padding from left/right.
        }
        this.adapter.setNotchWidthProperty(notchWidth);
        this.adapter.addClass(OUTLINE_NOTCHED);
    };
    /**
     * Removes notched outline selector to close the notch in the outline.
     */
    MDCNotchedOutlineFoundation.prototype.closeNotch = function () {
        var OUTLINE_NOTCHED = MDCNotchedOutlineFoundation.cssClasses.OUTLINE_NOTCHED;
        this.adapter.removeClass(OUTLINE_NOTCHED);
        this.adapter.removeNotchWidthProperty();
    };
    return MDCNotchedOutlineFoundation;
}(MDCFoundation));

/**
 * @license
 * Copyright 2017 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCNotchedOutline = /** @class */ (function (_super) {
    __extends(MDCNotchedOutline, _super);
    function MDCNotchedOutline() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCNotchedOutline.attachTo = function (root) {
        return new MDCNotchedOutline(root);
    };
    MDCNotchedOutline.prototype.initialSyncWithDOM = function () {
        this.notchElement =
            this.root.querySelector(strings$3.NOTCH_ELEMENT_SELECTOR);
        var label = this.root.querySelector('.' + MDCFloatingLabelFoundation.cssClasses.ROOT);
        if (label) {
            label.style.transitionDuration = '0s';
            this.root.classList.add(cssClasses$2.OUTLINE_UPGRADED);
            requestAnimationFrame(function () {
                label.style.transitionDuration = '';
            });
        }
        else {
            this.root.classList.add(cssClasses$2.NO_LABEL);
        }
    };
    /**
     * Updates classes and styles to open the notch to the specified width.
     * @param notchWidth The notch width in the outline.
     */
    MDCNotchedOutline.prototype.notch = function (notchWidth) {
        this.foundation.notch(notchWidth);
    };
    /**
     * Updates classes and styles to close the notch.
     */
    MDCNotchedOutline.prototype.closeNotch = function () {
        this.foundation.closeNotch();
    };
    MDCNotchedOutline.prototype.getDefaultFoundation = function () {
        var _this = this;
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        var adapter = {
            addClass: function (className) { return _this.root.classList.add(className); },
            removeClass: function (className) { return _this.root.classList.remove(className); },
            setNotchWidthProperty: function (width) {
                _this.notchElement.style.setProperty('width', width + 'px');
            },
            removeNotchWidthProperty: function () {
                _this.notchElement.style.removeProperty('width');
            },
        };
        // tslint:enable:object-literal-sort-keys
        return new MDCNotchedOutlineFoundation(adapter);
    };
    return MDCNotchedOutline;
}(MDCComponent));

/**
 * @license
 * Copyright 2016 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var cssClasses$1 = {
    ACTIVATED: 'mdc-select--activated',
    DISABLED: 'mdc-select--disabled',
    FOCUSED: 'mdc-select--focused',
    INVALID: 'mdc-select--invalid',
    MENU_INVALID: 'mdc-select__menu--invalid',
    OUTLINED: 'mdc-select--outlined',
    REQUIRED: 'mdc-select--required',
    ROOT: 'mdc-select',
    WITH_LEADING_ICON: 'mdc-select--with-leading-icon',
};
var strings$2 = {
    ARIA_CONTROLS: 'aria-controls',
    ARIA_DESCRIBEDBY: 'aria-describedby',
    ARIA_SELECTED_ATTR: 'aria-selected',
    CHANGE_EVENT: 'MDCSelect:change',
    HIDDEN_INPUT_SELECTOR: 'input[type="hidden"]',
    LABEL_SELECTOR: '.mdc-floating-label',
    LEADING_ICON_SELECTOR: '.mdc-select__icon',
    LINE_RIPPLE_SELECTOR: '.mdc-line-ripple',
    MENU_SELECTOR: '.mdc-select__menu',
    OUTLINE_SELECTOR: '.mdc-notched-outline',
    SELECTED_TEXT_SELECTOR: '.mdc-select__selected-text',
    SELECT_ANCHOR_SELECTOR: '.mdc-select__anchor',
    VALUE_ATTR: 'data-value',
};
var numbers = {
    LABEL_SCALE: 0.75,
    UNSET_INDEX: -1,
    CLICK_DEBOUNCE_TIMEOUT_MS: 330,
};

/**
 * @license
 * Copyright 2016 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCSelectFoundation = /** @class */ (function (_super) {
    __extends(MDCSelectFoundation, _super);
    /* istanbul ignore next: optional argument is not a branch statement */
    /**
     * @param adapter
     * @param foundationMap Map from subcomponent names to their subfoundations.
     */
    function MDCSelectFoundation(adapter, foundationMap) {
        if (foundationMap === void 0) { foundationMap = {}; }
        var _this = _super.call(this, __assign(__assign({}, MDCSelectFoundation.defaultAdapter), adapter)) || this;
        // Disabled state
        _this.disabled = false;
        // isMenuOpen is used to track the state of the menu by listening to the
        // MDCMenuSurface:closed event For reference, menu.open will return false if
        // the menu is still closing, but isMenuOpen returns false only after the menu
        // has closed
        _this.isMenuOpen = false;
        // By default, select is invalid if it is required but no value is selected.
        _this.useDefaultValidation = true;
        _this.customValidity = true;
        _this.lastSelectedIndex = numbers.UNSET_INDEX;
        _this.clickDebounceTimeout = 0;
        _this.recentlyClicked = false;
        _this.leadingIcon = foundationMap.leadingIcon;
        _this.helperText = foundationMap.helperText;
        return _this;
    }
    Object.defineProperty(MDCSelectFoundation, "cssClasses", {
        get: function () {
            return cssClasses$1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelectFoundation, "numbers", {
        get: function () {
            return numbers;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelectFoundation, "strings", {
        get: function () {
            return strings$2;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelectFoundation, "defaultAdapter", {
        /**
         * See {@link MDCSelectAdapter} for typing information on parameters and return types.
         */
        get: function () {
            // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
            return {
                addClass: function () { return undefined; },
                removeClass: function () { return undefined; },
                hasClass: function () { return false; },
                activateBottomLine: function () { return undefined; },
                deactivateBottomLine: function () { return undefined; },
                getSelectedIndex: function () { return -1; },
                setSelectedIndex: function () { return undefined; },
                hasLabel: function () { return false; },
                floatLabel: function () { return undefined; },
                getLabelWidth: function () { return 0; },
                setLabelRequired: function () { return undefined; },
                hasOutline: function () { return false; },
                notchOutline: function () { return undefined; },
                closeOutline: function () { return undefined; },
                setRippleCenter: function () { return undefined; },
                notifyChange: function () { return undefined; },
                setSelectedText: function () { return undefined; },
                isSelectAnchorFocused: function () { return false; },
                getSelectAnchorAttr: function () { return ''; },
                setSelectAnchorAttr: function () { return undefined; },
                removeSelectAnchorAttr: function () { return undefined; },
                addMenuClass: function () { return undefined; },
                removeMenuClass: function () { return undefined; },
                openMenu: function () { return undefined; },
                closeMenu: function () { return undefined; },
                getAnchorElement: function () { return null; },
                setMenuAnchorElement: function () { return undefined; },
                setMenuAnchorCorner: function () { return undefined; },
                setMenuWrapFocus: function () { return undefined; },
                focusMenuItemAtIndex: function () { return undefined; },
                getMenuItemCount: function () { return 0; },
                getMenuItemValues: function () { return []; },
                getMenuItemTextAtIndex: function () { return ''; },
                isTypeaheadInProgress: function () { return false; },
                typeaheadMatchItem: function () { return -1; },
            };
            // tslint:enable:object-literal-sort-keys
        },
        enumerable: false,
        configurable: true
    });
    /** Returns the index of the currently selected menu item, or -1 if none. */
    MDCSelectFoundation.prototype.getSelectedIndex = function () {
        return this.adapter.getSelectedIndex();
    };
    MDCSelectFoundation.prototype.setSelectedIndex = function (index, closeMenu, skipNotify) {
        if (closeMenu === void 0) { closeMenu = false; }
        if (skipNotify === void 0) { skipNotify = false; }
        if (index >= this.adapter.getMenuItemCount()) {
            return;
        }
        if (index === numbers.UNSET_INDEX) {
            this.adapter.setSelectedText('');
        }
        else {
            this.adapter.setSelectedText(this.adapter.getMenuItemTextAtIndex(index).trim());
        }
        this.adapter.setSelectedIndex(index);
        if (closeMenu) {
            this.adapter.closeMenu();
        }
        if (!skipNotify && this.lastSelectedIndex !== index) {
            this.handleChange();
        }
        this.lastSelectedIndex = index;
    };
    MDCSelectFoundation.prototype.setValue = function (value, skipNotify) {
        if (skipNotify === void 0) { skipNotify = false; }
        var index = this.adapter.getMenuItemValues().indexOf(value);
        this.setSelectedIndex(index, /** closeMenu */ false, skipNotify);
    };
    MDCSelectFoundation.prototype.getValue = function () {
        var index = this.adapter.getSelectedIndex();
        var menuItemValues = this.adapter.getMenuItemValues();
        return index !== numbers.UNSET_INDEX ? menuItemValues[index] : '';
    };
    MDCSelectFoundation.prototype.getDisabled = function () {
        return this.disabled;
    };
    MDCSelectFoundation.prototype.setDisabled = function (isDisabled) {
        this.disabled = isDisabled;
        if (this.disabled) {
            this.adapter.addClass(cssClasses$1.DISABLED);
            this.adapter.closeMenu();
        }
        else {
            this.adapter.removeClass(cssClasses$1.DISABLED);
        }
        if (this.leadingIcon) {
            this.leadingIcon.setDisabled(this.disabled);
        }
        if (this.disabled) {
            // Prevent click events from focusing select. Simply pointer-events: none
            // is not enough since screenreader clicks may bypass this.
            this.adapter.removeSelectAnchorAttr('tabindex');
        }
        else {
            this.adapter.setSelectAnchorAttr('tabindex', '0');
        }
        this.adapter.setSelectAnchorAttr('aria-disabled', this.disabled.toString());
    };
    /** Opens the menu. */
    MDCSelectFoundation.prototype.openMenu = function () {
        this.adapter.addClass(cssClasses$1.ACTIVATED);
        this.adapter.openMenu();
        this.isMenuOpen = true;
        this.adapter.setSelectAnchorAttr('aria-expanded', 'true');
    };
    /**
     * @param content Sets the content of the helper text.
     */
    MDCSelectFoundation.prototype.setHelperTextContent = function (content) {
        if (this.helperText) {
            this.helperText.setContent(content);
        }
    };
    /**
     * Re-calculates if the notched outline should be notched and if the label
     * should float.
     */
    MDCSelectFoundation.prototype.layout = function () {
        if (this.adapter.hasLabel()) {
            var optionHasValue = this.getValue().length > 0;
            var isFocused = this.adapter.hasClass(cssClasses$1.FOCUSED);
            var shouldFloatAndNotch = optionHasValue || isFocused;
            var isRequired = this.adapter.hasClass(cssClasses$1.REQUIRED);
            this.notchOutline(shouldFloatAndNotch);
            this.adapter.floatLabel(shouldFloatAndNotch);
            this.adapter.setLabelRequired(isRequired);
        }
    };
    /**
     * Synchronizes the list of options with the state of the foundation. Call
     * this whenever menu options are dynamically updated.
     */
    MDCSelectFoundation.prototype.layoutOptions = function () {
        var menuItemValues = this.adapter.getMenuItemValues();
        var selectedIndex = menuItemValues.indexOf(this.getValue());
        this.setSelectedIndex(selectedIndex, /** closeMenu */ false, /** skipNotify */ true);
    };
    MDCSelectFoundation.prototype.handleMenuOpened = function () {
        if (this.adapter.getMenuItemValues().length === 0) {
            return;
        }
        // Menu should open to the last selected element, should open to first menu item otherwise.
        var selectedIndex = this.getSelectedIndex();
        var focusItemIndex = selectedIndex >= 0 ? selectedIndex : 0;
        this.adapter.focusMenuItemAtIndex(focusItemIndex);
    };
    MDCSelectFoundation.prototype.handleMenuClosing = function () {
        this.adapter.setSelectAnchorAttr('aria-expanded', 'false');
    };
    MDCSelectFoundation.prototype.handleMenuClosed = function () {
        this.adapter.removeClass(cssClasses$1.ACTIVATED);
        this.isMenuOpen = false;
        // Unfocus the select if menu is closed without a selection
        if (!this.adapter.isSelectAnchorFocused()) {
            this.blur();
        }
    };
    /**
     * Handles value changes, via change event or programmatic updates.
     */
    MDCSelectFoundation.prototype.handleChange = function () {
        this.layout();
        this.adapter.notifyChange(this.getValue());
        var isRequired = this.adapter.hasClass(cssClasses$1.REQUIRED);
        if (isRequired && this.useDefaultValidation) {
            this.setValid(this.isValid());
        }
    };
    MDCSelectFoundation.prototype.handleMenuItemAction = function (index) {
        this.setSelectedIndex(index, /** closeMenu */ true);
    };
    /**
     * Handles focus events from select element.
     */
    MDCSelectFoundation.prototype.handleFocus = function () {
        this.adapter.addClass(cssClasses$1.FOCUSED);
        this.layout();
        this.adapter.activateBottomLine();
    };
    /**
     * Handles blur events from select element.
     */
    MDCSelectFoundation.prototype.handleBlur = function () {
        if (this.isMenuOpen) {
            return;
        }
        this.blur();
    };
    MDCSelectFoundation.prototype.handleClick = function (normalizedX) {
        if (this.disabled || this.recentlyClicked) {
            return;
        }
        this.setClickDebounceTimeout();
        if (this.isMenuOpen) {
            this.adapter.closeMenu();
            return;
        }
        this.adapter.setRippleCenter(normalizedX);
        this.openMenu();
    };
    /**
     * Handles keydown events on select element. Depending on the type of
     * character typed, does typeahead matching or opens menu.
     */
    MDCSelectFoundation.prototype.handleKeydown = function (event) {
        if (this.isMenuOpen || !this.adapter.hasClass(cssClasses$1.FOCUSED)) {
            return;
        }
        var isEnter = normalizeKey(event) === KEY.ENTER;
        var isSpace = normalizeKey(event) === KEY.SPACEBAR;
        var arrowUp = normalizeKey(event) === KEY.ARROW_UP;
        var arrowDown = normalizeKey(event) === KEY.ARROW_DOWN;
        var isModifier = event.ctrlKey || event.metaKey;
        // Typeahead
        if (!isModifier &&
            (!isSpace && event.key && event.key.length === 1 ||
                isSpace && this.adapter.isTypeaheadInProgress())) {
            var key = isSpace ? ' ' : event.key;
            var typeaheadNextIndex = this.adapter.typeaheadMatchItem(key, this.getSelectedIndex());
            if (typeaheadNextIndex >= 0) {
                this.setSelectedIndex(typeaheadNextIndex);
            }
            event.preventDefault();
            return;
        }
        if (!isEnter && !isSpace && !arrowUp && !arrowDown) {
            return;
        }
        this.openMenu();
        event.preventDefault();
    };
    /**
     * Opens/closes the notched outline.
     */
    MDCSelectFoundation.prototype.notchOutline = function (openNotch) {
        if (!this.adapter.hasOutline()) {
            return;
        }
        var isFocused = this.adapter.hasClass(cssClasses$1.FOCUSED);
        if (openNotch) {
            var labelScale = numbers.LABEL_SCALE;
            var labelWidth = this.adapter.getLabelWidth() * labelScale;
            this.adapter.notchOutline(labelWidth);
        }
        else if (!isFocused) {
            this.adapter.closeOutline();
        }
    };
    /**
     * Sets the aria label of the leading icon.
     */
    MDCSelectFoundation.prototype.setLeadingIconAriaLabel = function (label) {
        if (this.leadingIcon) {
            this.leadingIcon.setAriaLabel(label);
        }
    };
    /**
     * Sets the text content of the leading icon.
     */
    MDCSelectFoundation.prototype.setLeadingIconContent = function (content) {
        if (this.leadingIcon) {
            this.leadingIcon.setContent(content);
        }
    };
    MDCSelectFoundation.prototype.getUseDefaultValidation = function () {
        return this.useDefaultValidation;
    };
    MDCSelectFoundation.prototype.setUseDefaultValidation = function (useDefaultValidation) {
        this.useDefaultValidation = useDefaultValidation;
    };
    MDCSelectFoundation.prototype.setValid = function (isValid) {
        if (!this.useDefaultValidation) {
            this.customValidity = isValid;
        }
        this.adapter.setSelectAnchorAttr('aria-invalid', (!isValid).toString());
        if (isValid) {
            this.adapter.removeClass(cssClasses$1.INVALID);
            this.adapter.removeMenuClass(cssClasses$1.MENU_INVALID);
        }
        else {
            this.adapter.addClass(cssClasses$1.INVALID);
            this.adapter.addMenuClass(cssClasses$1.MENU_INVALID);
        }
        this.syncHelperTextValidity(isValid);
    };
    MDCSelectFoundation.prototype.isValid = function () {
        if (this.useDefaultValidation &&
            this.adapter.hasClass(cssClasses$1.REQUIRED) &&
            !this.adapter.hasClass(cssClasses$1.DISABLED)) {
            // See notes for required attribute under https://www.w3.org/TR/html52/sec-forms.html#the-select-element
            // TL;DR: Invalid if no index is selected, or if the first index is selected and has an empty value.
            return this.getSelectedIndex() !== numbers.UNSET_INDEX &&
                (this.getSelectedIndex() !== 0 || Boolean(this.getValue()));
        }
        return this.customValidity;
    };
    MDCSelectFoundation.prototype.setRequired = function (isRequired) {
        if (isRequired) {
            this.adapter.addClass(cssClasses$1.REQUIRED);
        }
        else {
            this.adapter.removeClass(cssClasses$1.REQUIRED);
        }
        this.adapter.setSelectAnchorAttr('aria-required', isRequired.toString());
        this.adapter.setLabelRequired(isRequired);
    };
    MDCSelectFoundation.prototype.getRequired = function () {
        return this.adapter.getSelectAnchorAttr('aria-required') === 'true';
    };
    MDCSelectFoundation.prototype.init = function () {
        var anchorEl = this.adapter.getAnchorElement();
        if (anchorEl) {
            this.adapter.setMenuAnchorElement(anchorEl);
            this.adapter.setMenuAnchorCorner(Corner.BOTTOM_START);
        }
        this.adapter.setMenuWrapFocus(false);
        this.setDisabled(this.adapter.hasClass(cssClasses$1.DISABLED));
        this.syncHelperTextValidity(!this.adapter.hasClass(cssClasses$1.INVALID));
        this.layout();
        this.layoutOptions();
    };
    /**
     * Unfocuses the select component.
     */
    MDCSelectFoundation.prototype.blur = function () {
        this.adapter.removeClass(cssClasses$1.FOCUSED);
        this.layout();
        this.adapter.deactivateBottomLine();
        var isRequired = this.adapter.hasClass(cssClasses$1.REQUIRED);
        if (isRequired && this.useDefaultValidation) {
            this.setValid(this.isValid());
        }
    };
    MDCSelectFoundation.prototype.syncHelperTextValidity = function (isValid) {
        if (!this.helperText) {
            return;
        }
        this.helperText.setValidity(isValid);
        var helperTextVisible = this.helperText.isVisible();
        var helperTextId = this.helperText.getId();
        if (helperTextVisible && helperTextId) {
            this.adapter.setSelectAnchorAttr(strings$2.ARIA_DESCRIBEDBY, helperTextId);
        }
        else {
            // Needed because screenreaders will read labels pointed to by
            // `aria-describedby` even if they are `aria-hidden`.
            this.adapter.removeSelectAnchorAttr(strings$2.ARIA_DESCRIBEDBY);
        }
    };
    MDCSelectFoundation.prototype.setClickDebounceTimeout = function () {
        var _this = this;
        clearTimeout(this.clickDebounceTimeout);
        this.clickDebounceTimeout = setTimeout(function () {
            _this.recentlyClicked = false;
        }, numbers.CLICK_DEBOUNCE_TIMEOUT_MS);
        this.recentlyClicked = true;
    };
    return MDCSelectFoundation;
}(MDCFoundation));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var strings$1 = {
    ARIA_HIDDEN: 'aria-hidden',
    ROLE: 'role',
};
var cssClasses = {
    HELPER_TEXT_VALIDATION_MSG: 'mdc-select-helper-text--validation-msg',
    HELPER_TEXT_VALIDATION_MSG_PERSISTENT: 'mdc-select-helper-text--validation-msg-persistent',
};

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCSelectHelperTextFoundation = /** @class */ (function (_super) {
    __extends(MDCSelectHelperTextFoundation, _super);
    function MDCSelectHelperTextFoundation(adapter) {
        return _super.call(this, __assign(__assign({}, MDCSelectHelperTextFoundation.defaultAdapter), adapter)) || this;
    }
    Object.defineProperty(MDCSelectHelperTextFoundation, "cssClasses", {
        get: function () {
            return cssClasses;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelectHelperTextFoundation, "strings", {
        get: function () {
            return strings$1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelectHelperTextFoundation, "defaultAdapter", {
        /**
         * See {@link MDCSelectHelperTextAdapter} for typing information on parameters and return types.
         */
        get: function () {
            // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
            return {
                addClass: function () { return undefined; },
                removeClass: function () { return undefined; },
                hasClass: function () { return false; },
                setAttr: function () { return undefined; },
                getAttr: function () { return null; },
                removeAttr: function () { return undefined; },
                setContent: function () { return undefined; },
            };
            // tslint:enable:object-literal-sort-keys
        },
        enumerable: false,
        configurable: true
    });
    /**
     * @return The ID of the helper text, or null if none is set.
     */
    MDCSelectHelperTextFoundation.prototype.getId = function () {
        return this.adapter.getAttr('id');
    };
    /**
     * @return Whether the helper text is currently visible.
     */
    MDCSelectHelperTextFoundation.prototype.isVisible = function () {
        return this.adapter.getAttr(strings$1.ARIA_HIDDEN) !== 'true';
    };
    /**
     * Sets the content of the helper text field.
     */
    MDCSelectHelperTextFoundation.prototype.setContent = function (content) {
        this.adapter.setContent(content);
    };
    /**
     * Sets the helper text to act as a validation message.
     * By default, validation messages are hidden when the select is valid and
     * visible when the select is invalid.
     *
     * @param isValidation True to make the helper text act as an error validation
     *     message.
     */
    MDCSelectHelperTextFoundation.prototype.setValidation = function (isValidation) {
        if (isValidation) {
            this.adapter.addClass(cssClasses.HELPER_TEXT_VALIDATION_MSG);
        }
        else {
            this.adapter.removeClass(cssClasses.HELPER_TEXT_VALIDATION_MSG);
        }
    };
    /**
     * Sets the persistency of the validation helper text.
     * This keeps the validation message visible even if the select is valid,
     * though it will be displayed in the normal (grey) color.
     */
    MDCSelectHelperTextFoundation.prototype.setValidationMsgPersistent = function (isPersistent) {
        if (isPersistent) {
            this.adapter.addClass(cssClasses.HELPER_TEXT_VALIDATION_MSG_PERSISTENT);
        }
        else {
            this.adapter.removeClass(cssClasses.HELPER_TEXT_VALIDATION_MSG_PERSISTENT);
        }
    };
    /**
     * @return Whether the helper text acts as a validation message.
     * By default, validation messages are hidden when the select is valid and
     * visible when the select is invalid.
     */
    MDCSelectHelperTextFoundation.prototype.getIsValidation = function () {
        return this.adapter.hasClass(cssClasses.HELPER_TEXT_VALIDATION_MSG);
    };
    /**
     * @return Whether the validation helper text persists even if the input is
     * valid. If it is, it will be displayed in the normal (grey) color.
     */
    MDCSelectHelperTextFoundation.prototype.getIsValidationMsgPersistent = function () {
        return this.adapter.hasClass(cssClasses.HELPER_TEXT_VALIDATION_MSG_PERSISTENT);
    };
    /**
     * When acting as a validation message, shows/hides the helper text and
     * triggers alerts as necessary based on the select's validity.
     */
    MDCSelectHelperTextFoundation.prototype.setValidity = function (selectIsValid) {
        var isValidationMsg = this.adapter.hasClass(cssClasses.HELPER_TEXT_VALIDATION_MSG);
        if (!isValidationMsg) {
            // Non-validating helper-text is always displayed and does not participate
            // in validation logic.
            return;
        }
        var isPersistentValidationMsg = this.adapter.hasClass(cssClasses.HELPER_TEXT_VALIDATION_MSG_PERSISTENT);
        // Validating helper text is displayed if select is invalid, unless it is
        // set as persistent, in which case it always displays.
        var msgShouldDisplay = !selectIsValid || isPersistentValidationMsg;
        if (msgShouldDisplay) {
            this.showToScreenReader();
            // In addition to displaying, also trigger an alert if the select
            // has become invalid.
            if (!selectIsValid) {
                this.adapter.setAttr(strings$1.ROLE, 'alert');
            }
            else {
                this.adapter.removeAttr(strings$1.ROLE);
            }
            return;
        }
        // Hide everything.
        this.adapter.removeAttr(strings$1.ROLE);
        this.hide();
    };
    /**
     * Makes the helper text visible to screen readers.
     */
    MDCSelectHelperTextFoundation.prototype.showToScreenReader = function () {
        this.adapter.removeAttr(strings$1.ARIA_HIDDEN);
    };
    /**
     * Hides the help text from screen readers.
     */
    MDCSelectHelperTextFoundation.prototype.hide = function () {
        this.adapter.setAttr(strings$1.ARIA_HIDDEN, 'true');
    };
    return MDCSelectHelperTextFoundation;
}(MDCFoundation));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCSelectHelperText = /** @class */ (function (_super) {
    __extends(MDCSelectHelperText, _super);
    function MDCSelectHelperText() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCSelectHelperText.attachTo = function (root) {
        return new MDCSelectHelperText(root);
    };
    Object.defineProperty(MDCSelectHelperText.prototype, "foundationForSelect", {
        // Provided for access by MDCSelect component
        get: function () {
            return this.foundation;
        },
        enumerable: false,
        configurable: true
    });
    MDCSelectHelperText.prototype.getDefaultFoundation = function () {
        var _this = this;
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        var adapter = {
            addClass: function (className) { return _this.root.classList.add(className); },
            removeClass: function (className) { return _this.root.classList.remove(className); },
            hasClass: function (className) { return _this.root.classList.contains(className); },
            getAttr: function (attr) { return _this.root.getAttribute(attr); },
            setAttr: function (attr, value) { return _this.root.setAttribute(attr, value); },
            removeAttr: function (attr) { return _this.root.removeAttribute(attr); },
            setContent: function (content) {
                _this.root.textContent = content;
            },
        };
        // tslint:enable:object-literal-sort-keys
        return new MDCSelectHelperTextFoundation(adapter);
    };
    return MDCSelectHelperText;
}(MDCComponent));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var strings = {
    ICON_EVENT: 'MDCSelect:icon',
    ICON_ROLE: 'button',
};

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var INTERACTION_EVENTS = ['click', 'keydown'];
var MDCSelectIconFoundation = /** @class */ (function (_super) {
    __extends(MDCSelectIconFoundation, _super);
    function MDCSelectIconFoundation(adapter) {
        var _this = _super.call(this, __assign(__assign({}, MDCSelectIconFoundation.defaultAdapter), adapter)) || this;
        _this.savedTabIndex = null;
        _this.interactionHandler = function (evt) {
            _this.handleInteraction(evt);
        };
        return _this;
    }
    Object.defineProperty(MDCSelectIconFoundation, "strings", {
        get: function () {
            return strings;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelectIconFoundation, "defaultAdapter", {
        /**
         * See {@link MDCSelectIconAdapter} for typing information on parameters and return types.
         */
        get: function () {
            // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
            return {
                getAttr: function () { return null; },
                setAttr: function () { return undefined; },
                removeAttr: function () { return undefined; },
                setContent: function () { return undefined; },
                registerInteractionHandler: function () { return undefined; },
                deregisterInteractionHandler: function () { return undefined; },
                notifyIconAction: function () { return undefined; },
            };
            // tslint:enable:object-literal-sort-keys
        },
        enumerable: false,
        configurable: true
    });
    MDCSelectIconFoundation.prototype.init = function () {
        var e_1, _a;
        this.savedTabIndex = this.adapter.getAttr('tabindex');
        try {
            for (var INTERACTION_EVENTS_1 = __values(INTERACTION_EVENTS), INTERACTION_EVENTS_1_1 = INTERACTION_EVENTS_1.next(); !INTERACTION_EVENTS_1_1.done; INTERACTION_EVENTS_1_1 = INTERACTION_EVENTS_1.next()) {
                var evtType = INTERACTION_EVENTS_1_1.value;
                this.adapter.registerInteractionHandler(evtType, this.interactionHandler);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (INTERACTION_EVENTS_1_1 && !INTERACTION_EVENTS_1_1.done && (_a = INTERACTION_EVENTS_1.return)) _a.call(INTERACTION_EVENTS_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    MDCSelectIconFoundation.prototype.destroy = function () {
        var e_2, _a;
        try {
            for (var INTERACTION_EVENTS_2 = __values(INTERACTION_EVENTS), INTERACTION_EVENTS_2_1 = INTERACTION_EVENTS_2.next(); !INTERACTION_EVENTS_2_1.done; INTERACTION_EVENTS_2_1 = INTERACTION_EVENTS_2.next()) {
                var evtType = INTERACTION_EVENTS_2_1.value;
                this.adapter.deregisterInteractionHandler(evtType, this.interactionHandler);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (INTERACTION_EVENTS_2_1 && !INTERACTION_EVENTS_2_1.done && (_a = INTERACTION_EVENTS_2.return)) _a.call(INTERACTION_EVENTS_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    MDCSelectIconFoundation.prototype.setDisabled = function (disabled) {
        if (!this.savedTabIndex) {
            return;
        }
        if (disabled) {
            this.adapter.setAttr('tabindex', '-1');
            this.adapter.removeAttr('role');
        }
        else {
            this.adapter.setAttr('tabindex', this.savedTabIndex);
            this.adapter.setAttr('role', strings.ICON_ROLE);
        }
    };
    MDCSelectIconFoundation.prototype.setAriaLabel = function (label) {
        this.adapter.setAttr('aria-label', label);
    };
    MDCSelectIconFoundation.prototype.setContent = function (content) {
        this.adapter.setContent(content);
    };
    MDCSelectIconFoundation.prototype.handleInteraction = function (evt) {
        var isEnterKey = evt.key === 'Enter' || evt.keyCode === 13;
        if (evt.type === 'click' || isEnterKey) {
            this.adapter.notifyIconAction();
        }
    };
    return MDCSelectIconFoundation;
}(MDCFoundation));

/**
 * @license
 * Copyright 2018 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCSelectIcon = /** @class */ (function (_super) {
    __extends(MDCSelectIcon, _super);
    function MDCSelectIcon() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCSelectIcon.attachTo = function (root) {
        return new MDCSelectIcon(root);
    };
    Object.defineProperty(MDCSelectIcon.prototype, "foundationForSelect", {
        // Provided for access by MDCSelect component
        get: function () {
            return this.foundation;
        },
        enumerable: false,
        configurable: true
    });
    MDCSelectIcon.prototype.getDefaultFoundation = function () {
        var _this = this;
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        var adapter = {
            getAttr: function (attr) { return _this.root.getAttribute(attr); },
            setAttr: function (attr, value) { return _this.root.setAttribute(attr, value); },
            removeAttr: function (attr) { return _this.root.removeAttribute(attr); },
            setContent: function (content) {
                _this.root.textContent = content;
            },
            registerInteractionHandler: function (evtType, handler) {
                return _this.listen(evtType, handler);
            },
            deregisterInteractionHandler: function (evtType, handler) {
                return _this.unlisten(evtType, handler);
            },
            notifyIconAction: function () { return _this.emit(MDCSelectIconFoundation.strings.ICON_EVENT, {} /* evtData */, true /* shouldBubble */); },
        };
        // tslint:enable:object-literal-sort-keys
        return new MDCSelectIconFoundation(adapter);
    };
    return MDCSelectIcon;
}(MDCComponent));

/**
 * @license
 * Copyright 2016 Google Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
var MDCSelect = /** @class */ (function (_super) {
    __extends(MDCSelect, _super);
    function MDCSelect() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MDCSelect.attachTo = function (root) {
        return new MDCSelect(root);
    };
    MDCSelect.prototype.initialize = function (labelFactory, lineRippleFactory, outlineFactory, menuFactory, iconFactory, helperTextFactory) {
        if (labelFactory === void 0) { labelFactory = function (el) { return new MDCFloatingLabel(el); }; }
        if (lineRippleFactory === void 0) { lineRippleFactory = function (el) { return new MDCLineRipple(el); }; }
        if (outlineFactory === void 0) { outlineFactory = function (el) { return new MDCNotchedOutline(el); }; }
        if (menuFactory === void 0) { menuFactory = function (el) { return new MDCMenu(el); }; }
        if (iconFactory === void 0) { iconFactory = function (el) { return new MDCSelectIcon(el); }; }
        if (helperTextFactory === void 0) { helperTextFactory = function (el) { return new MDCSelectHelperText(el); }; }
        this.selectAnchor =
            this.root.querySelector(strings$2.SELECT_ANCHOR_SELECTOR);
        this.selectedText =
            this.root.querySelector(strings$2.SELECTED_TEXT_SELECTOR);
        this.hiddenInput = this.root.querySelector(strings$2.HIDDEN_INPUT_SELECTOR);
        if (!this.selectedText) {
            throw new Error('MDCSelect: Missing required element: The following selector must be present: ' +
                ("'" + strings$2.SELECTED_TEXT_SELECTOR + "'"));
        }
        if (this.selectAnchor.hasAttribute(strings$2.ARIA_CONTROLS)) {
            var helperTextElement = document.getElementById(this.selectAnchor.getAttribute(strings$2.ARIA_CONTROLS));
            if (helperTextElement) {
                this.helperText = helperTextFactory(helperTextElement);
            }
        }
        this.menuSetup(menuFactory);
        var labelElement = this.root.querySelector(strings$2.LABEL_SELECTOR);
        this.label = labelElement ? labelFactory(labelElement) : null;
        var lineRippleElement = this.root.querySelector(strings$2.LINE_RIPPLE_SELECTOR);
        this.lineRipple =
            lineRippleElement ? lineRippleFactory(lineRippleElement) : null;
        var outlineElement = this.root.querySelector(strings$2.OUTLINE_SELECTOR);
        this.outline = outlineElement ? outlineFactory(outlineElement) : null;
        var leadingIcon = this.root.querySelector(strings$2.LEADING_ICON_SELECTOR);
        if (leadingIcon) {
            this.leadingIcon = iconFactory(leadingIcon);
        }
        if (!this.root.classList.contains(cssClasses$1.OUTLINED)) {
            this.ripple = this.createRipple();
        }
    };
    /**
     * Initializes the select's event listeners and internal state based
     * on the environment's state.
     */
    MDCSelect.prototype.initialSyncWithDOM = function () {
        var _this = this;
        this.handleFocus = function () {
            _this.foundation.handleFocus();
        };
        this.handleBlur = function () {
            _this.foundation.handleBlur();
        };
        this.handleClick = function (evt) {
            _this.selectAnchor.focus();
            _this.foundation.handleClick(_this.getNormalizedXCoordinate(evt));
        };
        this.handleKeydown = function (evt) {
            _this.foundation.handleKeydown(evt);
        };
        this.handleMenuItemAction = function (evt) {
            _this.foundation.handleMenuItemAction(evt.detail.index);
        };
        this.handleMenuOpened = function () {
            _this.foundation.handleMenuOpened();
        };
        this.handleMenuClosed = function () {
            _this.foundation.handleMenuClosed();
        };
        this.handleMenuClosing = function () {
            _this.foundation.handleMenuClosing();
        };
        this.selectAnchor.addEventListener('focus', this.handleFocus);
        this.selectAnchor.addEventListener('blur', this.handleBlur);
        this.selectAnchor.addEventListener('click', this.handleClick);
        this.selectAnchor.addEventListener('keydown', this.handleKeydown);
        this.menu.listen(strings$4.CLOSED_EVENT, this.handleMenuClosed);
        this.menu.listen(strings$4.CLOSING_EVENT, this.handleMenuClosing);
        this.menu.listen(strings$4.OPENED_EVENT, this.handleMenuOpened);
        this.menu.listen(strings$5.SELECTED_EVENT, this.handleMenuItemAction);
        if (this.hiddenInput) {
            if (this.hiddenInput.value) {
                // If the hidden input already has a value, use it to restore the
                // select's value. This can happen e.g. if the user goes back or (in
                // some browsers) refreshes the page.
                this.foundation.setValue(this.hiddenInput.value, /** skipNotify */ true);
                this.foundation.layout();
                return;
            }
            this.hiddenInput.value = this.value;
        }
    };
    MDCSelect.prototype.destroy = function () {
        this.selectAnchor.removeEventListener('focus', this.handleFocus);
        this.selectAnchor.removeEventListener('blur', this.handleBlur);
        this.selectAnchor.removeEventListener('keydown', this.handleKeydown);
        this.selectAnchor.removeEventListener('click', this.handleClick);
        this.menu.unlisten(strings$4.CLOSED_EVENT, this.handleMenuClosed);
        this.menu.unlisten(strings$4.OPENED_EVENT, this.handleMenuOpened);
        this.menu.unlisten(strings$5.SELECTED_EVENT, this.handleMenuItemAction);
        this.menu.destroy();
        if (this.ripple) {
            this.ripple.destroy();
        }
        if (this.outline) {
            this.outline.destroy();
        }
        if (this.leadingIcon) {
            this.leadingIcon.destroy();
        }
        if (this.helperText) {
            this.helperText.destroy();
        }
        _super.prototype.destroy.call(this);
    };
    Object.defineProperty(MDCSelect.prototype, "value", {
        get: function () {
            return this.foundation.getValue();
        },
        set: function (value) {
            this.foundation.setValue(value);
        },
        enumerable: false,
        configurable: true
    });
    MDCSelect.prototype.setValue = function (value, skipNotify) {
        if (skipNotify === void 0) { skipNotify = false; }
        this.foundation.setValue(value, skipNotify);
    };
    Object.defineProperty(MDCSelect.prototype, "selectedIndex", {
        get: function () {
            return this.foundation.getSelectedIndex();
        },
        set: function (selectedIndex) {
            this.foundation.setSelectedIndex(selectedIndex, /* closeMenu */ true);
        },
        enumerable: false,
        configurable: true
    });
    MDCSelect.prototype.setSelectedIndex = function (selectedIndex, skipNotify) {
        if (skipNotify === void 0) { skipNotify = false; }
        this.foundation.setSelectedIndex(selectedIndex, /* closeMenu */ true, skipNotify);
    };
    Object.defineProperty(MDCSelect.prototype, "disabled", {
        get: function () {
            return this.foundation.getDisabled();
        },
        set: function (disabled) {
            this.foundation.setDisabled(disabled);
            if (this.hiddenInput) {
                this.hiddenInput.disabled = disabled;
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelect.prototype, "leadingIconAriaLabel", {
        set: function (label) {
            this.foundation.setLeadingIconAriaLabel(label);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelect.prototype, "leadingIconContent", {
        /**
         * Sets the text content of the leading icon.
         */
        set: function (content) {
            this.foundation.setLeadingIconContent(content);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelect.prototype, "helperTextContent", {
        /**
         * Sets the text content of the helper text.
         */
        set: function (content) {
            this.foundation.setHelperTextContent(content);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelect.prototype, "useDefaultValidation", {
        /**
         * Enables or disables the default validation scheme where a required select
         * must be non-empty. Set to false for custom validation.
         * @param useDefaultValidation Set this to false to ignore default
         *     validation scheme.
         */
        set: function (useDefaultValidation) {
            this.foundation.setUseDefaultValidation(useDefaultValidation);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelect.prototype, "valid", {
        /**
         * Checks if the select is in a valid state.
         */
        get: function () {
            return this.foundation.isValid();
        },
        /**
         * Sets the current invalid state of the select.
         */
        set: function (isValid) {
            this.foundation.setValid(isValid);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MDCSelect.prototype, "required", {
        /**
         * Returns whether the select is required.
         */
        get: function () {
            return this.foundation.getRequired();
        },
        /**
         * Sets the control to the required state.
         */
        set: function (isRequired) {
            this.foundation.setRequired(isRequired);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Re-calculates if the notched outline should be notched and if the label
     * should float.
     */
    MDCSelect.prototype.layout = function () {
        this.foundation.layout();
    };
    /**
     * Synchronizes the list of options with the state of the foundation. Call
     * this whenever menu options are dynamically updated.
     */
    MDCSelect.prototype.layoutOptions = function () {
        this.foundation.layoutOptions();
        this.menu.layout();
        // Update cached menuItemValues for adapter.
        this.menuItemValues =
            this.menu.items.map(function (el) { return el.getAttribute(strings$2.VALUE_ATTR) || ''; });
        if (this.hiddenInput) {
            this.hiddenInput.value = this.value;
        }
    };
    MDCSelect.prototype.getDefaultFoundation = function () {
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        var adapter = __assign(__assign(__assign(__assign({}, this.getSelectAdapterMethods()), this.getCommonAdapterMethods()), this.getOutlineAdapterMethods()), this.getLabelAdapterMethods());
        return new MDCSelectFoundation(adapter, this.getFoundationMap());
    };
    /**
     * Handles setup for the menu.
     */
    MDCSelect.prototype.menuSetup = function (menuFactory) {
        this.menuElement = this.root.querySelector(strings$2.MENU_SELECTOR);
        this.menu = menuFactory(this.menuElement);
        this.menu.hasTypeahead = true;
        this.menu.singleSelection = true;
        this.menuItemValues =
            this.menu.items.map(function (el) { return el.getAttribute(strings$2.VALUE_ATTR) || ''; });
    };
    MDCSelect.prototype.createRipple = function () {
        var _this = this;
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        var adapter = __assign(__assign({}, MDCRipple.createAdapter({ root: this.selectAnchor })), { registerInteractionHandler: function (evtType, handler) {
                _this.selectAnchor.addEventListener(evtType, handler);
            }, deregisterInteractionHandler: function (evtType, handler) {
                _this.selectAnchor.removeEventListener(evtType, handler);
            } });
        // tslint:enable:object-literal-sort-keys
        return new MDCRipple(this.selectAnchor, new MDCRippleFoundation(adapter));
    };
    MDCSelect.prototype.getSelectAdapterMethods = function () {
        var _this = this;
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        return {
            getMenuItemAttr: function (menuItem, attr) {
                return menuItem.getAttribute(attr);
            },
            setSelectedText: function (text) {
                _this.selectedText.textContent = text;
            },
            isSelectAnchorFocused: function () { return document.activeElement === _this.selectAnchor; },
            getSelectAnchorAttr: function (attr) {
                return _this.selectAnchor.getAttribute(attr);
            },
            setSelectAnchorAttr: function (attr, value) {
                _this.selectAnchor.setAttribute(attr, value);
            },
            removeSelectAnchorAttr: function (attr) {
                _this.selectAnchor.removeAttribute(attr);
            },
            addMenuClass: function (className) {
                _this.menuElement.classList.add(className);
            },
            removeMenuClass: function (className) {
                _this.menuElement.classList.remove(className);
            },
            openMenu: function () {
                _this.menu.open = true;
            },
            closeMenu: function () {
                _this.menu.open = false;
            },
            getAnchorElement: function () {
                return _this.root.querySelector(strings$2.SELECT_ANCHOR_SELECTOR);
            },
            setMenuAnchorElement: function (anchorEl) {
                _this.menu.setAnchorElement(anchorEl);
            },
            setMenuAnchorCorner: function (anchorCorner) {
                _this.menu.setAnchorCorner(anchorCorner);
            },
            setMenuWrapFocus: function (wrapFocus) {
                _this.menu.wrapFocus = wrapFocus;
            },
            getSelectedIndex: function () {
                var index = _this.menu.selectedIndex;
                return index instanceof Array ? index[0] : index;
            },
            setSelectedIndex: function (index) {
                _this.menu.selectedIndex = index;
            },
            focusMenuItemAtIndex: function (index) {
                _this.menu.items[index].focus();
            },
            getMenuItemCount: function () { return _this.menu.items.length; },
            // Cache menu item values. layoutOptions() updates this cache.
            getMenuItemValues: function () { return _this.menuItemValues; },
            getMenuItemTextAtIndex: function (index) {
                return _this.menu.getPrimaryTextAtIndex(index);
            },
            isTypeaheadInProgress: function () { return _this.menu.typeaheadInProgress; },
            typeaheadMatchItem: function (nextChar, startingIndex) {
                return _this.menu.typeaheadMatchItem(nextChar, startingIndex);
            },
        };
        // tslint:enable:object-literal-sort-keys
    };
    MDCSelect.prototype.getCommonAdapterMethods = function () {
        var _this = this;
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        return {
            addClass: function (className) {
                _this.root.classList.add(className);
            },
            removeClass: function (className) {
                _this.root.classList.remove(className);
            },
            hasClass: function (className) { return _this.root.classList.contains(className); },
            setRippleCenter: function (normalizedX) {
                _this.lineRipple && _this.lineRipple.setRippleCenter(normalizedX);
            },
            activateBottomLine: function () {
                _this.lineRipple && _this.lineRipple.activate();
            },
            deactivateBottomLine: function () {
                _this.lineRipple && _this.lineRipple.deactivate();
            },
            notifyChange: function (value) {
                if (_this.hiddenInput) {
                    _this.hiddenInput.value = value;
                }
                var index = _this.selectedIndex;
                _this.emit(strings$2.CHANGE_EVENT, { value: value, index: index }, true /* shouldBubble  */);
            },
        };
        // tslint:enable:object-literal-sort-keys
    };
    MDCSelect.prototype.getOutlineAdapterMethods = function () {
        var _this = this;
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        return {
            hasOutline: function () { return Boolean(_this.outline); },
            notchOutline: function (labelWidth) {
                _this.outline && _this.outline.notch(labelWidth);
            },
            closeOutline: function () {
                _this.outline && _this.outline.closeNotch();
            },
        };
        // tslint:enable:object-literal-sort-keys
    };
    MDCSelect.prototype.getLabelAdapterMethods = function () {
        var _this = this;
        // tslint:disable:object-literal-sort-keys Methods should be in the same order as the adapter interface.
        return {
            hasLabel: function () { return !!_this.label; },
            floatLabel: function (shouldFloat) {
                _this.label && _this.label.float(shouldFloat);
            },
            getLabelWidth: function () { return _this.label ? _this.label.getWidth() : 0; },
            setLabelRequired: function (isRequired) {
                _this.label && _this.label.setRequired(isRequired);
            },
        };
        // tslint:enable:object-literal-sort-keys
    };
    /**
     * Calculates where the line ripple should start based on the x coordinate within the component.
     */
    MDCSelect.prototype.getNormalizedXCoordinate = function (evt) {
        var targetClientRect = evt.target.getBoundingClientRect();
        var xCoordinate = this.isTouchEvent(evt) ? evt.touches[0].clientX : evt.clientX;
        return xCoordinate - targetClientRect.left;
    };
    MDCSelect.prototype.isTouchEvent = function (evt) {
        return Boolean(evt.touches);
    };
    /**
     * Returns a map of all subcomponents to subfoundations.
     */
    MDCSelect.prototype.getFoundationMap = function () {
        return {
            helperText: this.helperText ? this.helperText.foundationForSelect :
                undefined,
            leadingIcon: this.leadingIcon ? this.leadingIcon.foundationForSelect :
                undefined,
        };
    };
    return MDCSelect;
}(MDCComponent));

const paginatorComponentCss = "@charset \"UTF-8\";.mdc-deprecated-list{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);line-height:1.5rem;margin:0;padding:8px 0;list-style-type:none;color:rgba(0, 0, 0, 0.87);color:var(--mdc-theme-text-primary-on-background, rgba(0, 0, 0, 0.87))}.mdc-deprecated-list:focus{outline:none}.mdc-deprecated-list-item{height:48px}.mdc-deprecated-list-item__secondary-text{color:rgba(0, 0, 0, 0.54);color:var(--mdc-theme-text-secondary-on-background, rgba(0, 0, 0, 0.54))}.mdc-deprecated-list-item__graphic{background-color:transparent}.mdc-deprecated-list-item__graphic{color:rgba(0, 0, 0, 0.38);color:var(--mdc-theme-text-icon-on-background, rgba(0, 0, 0, 0.38))}.mdc-deprecated-list-item__meta{color:rgba(0, 0, 0, 0.38);color:var(--mdc-theme-text-hint-on-background, rgba(0, 0, 0, 0.38))}.mdc-deprecated-list-group__subheader{color:rgba(0, 0, 0, 0.87);color:var(--mdc-theme-text-primary-on-background, rgba(0, 0, 0, 0.87))}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__text{opacity:0.38}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__text,.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__primary-text,.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__secondary-text{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-deprecated-list-item--selected,.mdc-deprecated-list-item--activated{color:#6200ee;color:var(--mdc-theme-primary, #6200ee)}.mdc-deprecated-list-item--selected .mdc-deprecated-list-item__graphic,.mdc-deprecated-list-item--activated .mdc-deprecated-list-item__graphic{color:#6200ee;color:var(--mdc-theme-primary, #6200ee)}.mdc-deprecated-list--dense{padding-top:4px;padding-bottom:4px;font-size:0.812rem}.mdc-deprecated-list-item__wrapper{display:block}.mdc-deprecated-list-item{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;padding-left:16px;padding-right:16px;height:48px}.mdc-deprecated-list-item:focus{outline:none}.mdc-deprecated-list-item:not(.mdc-deprecated-list-item--selected):focus::before,.mdc-deprecated-list-item.mdc-ripple-upgraded--background-focused::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid transparent;border-radius:inherit;content:\"\";pointer-events:none}@media screen and (forced-colors: active){.mdc-deprecated-list-item:not(.mdc-deprecated-list-item--selected):focus::before,.mdc-deprecated-list-item.mdc-ripple-upgraded--background-focused::before{border-color:CanvasText}}.mdc-deprecated-list-item.mdc-deprecated-list-item--selected::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:3px double transparent;border-radius:inherit;content:\"\";pointer-events:none}@media screen and (forced-colors: active){.mdc-deprecated-list-item.mdc-deprecated-list-item--selected::before{border-color:CanvasText}}[dir=rtl] .mdc-deprecated-list-item,.mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-deprecated-list--icon-list .mdc-deprecated-list-item{padding-left:16px;padding-right:16px;height:56px}[dir=rtl] .mdc-deprecated-list--icon-list .mdc-deprecated-list-item,.mdc-deprecated-list--icon-list .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item{padding-left:16px;padding-right:16px;height:56px}[dir=rtl] .mdc-deprecated-list--avatar-list .mdc-deprecated-list-item,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item{padding-left:16px;padding-right:16px;height:56px}[dir=rtl] .mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-deprecated-list--image-list .mdc-deprecated-list-item{padding-left:16px;padding-right:16px;height:72px}[dir=rtl] .mdc-deprecated-list--image-list .mdc-deprecated-list-item,.mdc-deprecated-list--image-list .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-deprecated-list--video-list .mdc-deprecated-list-item{padding-left:0px;padding-right:16px;height:72px}[dir=rtl] .mdc-deprecated-list--video-list .mdc-deprecated-list-item,.mdc-deprecated-list--video-list .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:0px;}.mdc-deprecated-list--dense .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:16px;width:20px;height:20px}[dir=rtl] .mdc-deprecated-list--dense .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--dense .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:16px;margin-right:0;}.mdc-deprecated-list-item__graphic{flex-shrink:0;align-items:center;justify-content:center;fill:currentColor;object-fit:cover;margin-left:0;margin-right:32px;width:24px;height:24px}[dir=rtl] .mdc-deprecated-list-item__graphic,.mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:32px;margin-right:0;}.mdc-deprecated-list--icon-list .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:32px;width:24px;height:24px}[dir=rtl] .mdc-deprecated-list--icon-list .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--icon-list .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:32px;margin-right:0;}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:16px;width:40px;height:40px;border-radius:50%}[dir=rtl] .mdc-deprecated-list--avatar-list .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:16px;margin-right:0;}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:16px;width:40px;height:40px}[dir=rtl] .mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:16px;margin-right:0;}.mdc-deprecated-list--image-list .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:16px;width:56px;height:56px}[dir=rtl] .mdc-deprecated-list--image-list .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--image-list .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:16px;margin-right:0;}.mdc-deprecated-list--video-list .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:16px;width:100px;height:56px}[dir=rtl] .mdc-deprecated-list--video-list .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--video-list .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:16px;margin-right:0;}.mdc-deprecated-list .mdc-deprecated-list-item__graphic{display:inline-flex}.mdc-deprecated-list-item__meta{margin-left:auto;margin-right:0}.mdc-deprecated-list-item__meta:not(.material-icons){-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-caption-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.75rem;font-size:var(--mdc-typography-caption-font-size, 0.75rem);line-height:1.25rem;line-height:var(--mdc-typography-caption-line-height, 1.25rem);font-weight:400;font-weight:var(--mdc-typography-caption-font-weight, 400);letter-spacing:0.0333333333em;letter-spacing:var(--mdc-typography-caption-letter-spacing, 0.0333333333em);text-decoration:inherit;text-decoration:var(--mdc-typography-caption-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-caption-text-transform, inherit)}.mdc-deprecated-list-item[dir=rtl] .mdc-deprecated-list-item__meta,[dir=rtl] .mdc-deprecated-list-item .mdc-deprecated-list-item__meta{margin-left:0;margin-right:auto}.mdc-deprecated-list-item__text{text-overflow:ellipsis;white-space:nowrap;overflow:hidden}.mdc-deprecated-list-item__text[for]{pointer-events:none}.mdc-deprecated-list-item__primary-text{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-deprecated-list-item__primary-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-deprecated-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-deprecated-list--video-list .mdc-deprecated-list-item__primary-text,.mdc-deprecated-list--image-list .mdc-deprecated-list-item__primary-text,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item__primary-text,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item__primary-text,.mdc-deprecated-list--icon-list .mdc-deprecated-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-deprecated-list--video-list .mdc-deprecated-list-item__primary-text::before,.mdc-deprecated-list--image-list .mdc-deprecated-list-item__primary-text::before,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item__primary-text::before,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item__primary-text::before,.mdc-deprecated-list--icon-list .mdc-deprecated-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-deprecated-list--video-list .mdc-deprecated-list-item__primary-text::after,.mdc-deprecated-list--image-list .mdc-deprecated-list-item__primary-text::after,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item__primary-text::after,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item__primary-text::after,.mdc-deprecated-list--icon-list .mdc-deprecated-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-deprecated-list--dense .mdc-deprecated-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-deprecated-list--dense .mdc-deprecated-list-item__primary-text::before{display:inline-block;width:0;height:24px;content:\"\";vertical-align:0}.mdc-deprecated-list--dense .mdc-deprecated-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-deprecated-list-item__secondary-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-body2-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.875rem;font-size:var(--mdc-typography-body2-font-size, 0.875rem);line-height:1.25rem;line-height:var(--mdc-typography-body2-line-height, 1.25rem);font-weight:400;font-weight:var(--mdc-typography-body2-font-weight, 400);letter-spacing:0.0178571429em;letter-spacing:var(--mdc-typography-body2-letter-spacing, 0.0178571429em);text-decoration:inherit;text-decoration:var(--mdc-typography-body2-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-body2-text-transform, inherit);text-overflow:ellipsis;white-space:nowrap;overflow:hidden;display:block;margin-top:0;line-height:normal}.mdc-deprecated-list-item__secondary-text::before{display:inline-block;width:0;height:20px;content:\"\";vertical-align:0}.mdc-deprecated-list--dense .mdc-deprecated-list-item__secondary-text{font-size:inherit}.mdc-deprecated-list--dense .mdc-deprecated-list-item{height:40px}.mdc-deprecated-list--two-line .mdc-deprecated-list-item__text{align-self:flex-start}.mdc-deprecated-list--two-line .mdc-deprecated-list-item{height:64px}.mdc-deprecated-list--two-line.mdc-deprecated-list--video-list .mdc-deprecated-list-item,.mdc-deprecated-list--two-line.mdc-deprecated-list--image-list .mdc-deprecated-list-item,.mdc-deprecated-list--two-line.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-item,.mdc-deprecated-list--two-line.mdc-deprecated-list--avatar-list .mdc-deprecated-list-item,.mdc-deprecated-list--two-line.mdc-deprecated-list--icon-list .mdc-deprecated-list-item{height:72px}.mdc-deprecated-list--two-line.mdc-deprecated-list--icon-list .mdc-deprecated-list-item__graphic{align-self:flex-start;margin-top:16px}.mdc-deprecated-list--two-line.mdc-deprecated-list--dense .mdc-deprecated-list-item,.mdc-deprecated-list--avatar-list.mdc-deprecated-list--dense .mdc-deprecated-list-item{height:60px}.mdc-deprecated-list--avatar-list.mdc-deprecated-list--dense .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:16px;width:36px;height:36px}[dir=rtl] .mdc-deprecated-list--avatar-list.mdc-deprecated-list--dense .mdc-deprecated-list-item__graphic,.mdc-deprecated-list--avatar-list.mdc-deprecated-list--dense .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:16px;margin-right:0;}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item{cursor:pointer}a.mdc-deprecated-list-item{color:inherit;text-decoration:none}.mdc-deprecated-list-divider{height:0;margin:0;border:none;border-bottom-width:1px;border-bottom-style:solid}.mdc-deprecated-list-divider{border-bottom-color:rgba(0, 0, 0, 0.12)}.mdc-deprecated-list-divider--padded{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list-divider--padded,.mdc-deprecated-list-divider--padded[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list-divider--inset{margin-left:72px;margin-right:0;width:calc(100% - 72px)}[dir=rtl] .mdc-deprecated-list-divider--inset,.mdc-deprecated-list-divider--inset[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list-divider--inset.mdc-deprecated-list-divider--padded{margin-left:72px;margin-right:0;width:calc(100% - 88px)}[dir=rtl] .mdc-deprecated-list-divider--inset.mdc-deprecated-list-divider--padded,.mdc-deprecated-list-divider--inset.mdc-deprecated-list-divider--padded[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading{margin-left:16px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading,.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list .mdc-deprecated-list-divider--inset-trailing{width:calc(100% - 16px)}.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing,.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding{margin-left:16px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding,.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding,.mdc-deprecated-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading{margin-left:72px;margin-right:0;width:calc(100% - 72px)}[dir=rtl] .mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading,.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-trailing{width:calc(100% - 16px)}.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing{margin-left:72px;margin-right:0;width:calc(100% - 88px)}[dir=rtl] .mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing,.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding{margin-left:16px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding,.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding,.mdc-deprecated-list--icon-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading{margin-left:72px;margin-right:0;width:calc(100% - 72px)}[dir=rtl] .mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-trailing{width:calc(100% - 16px)}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing{margin-left:72px;margin-right:0;width:calc(100% - 88px)}[dir=rtl] .mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding{margin-left:16px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding,.mdc-deprecated-list--avatar-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading{margin-left:72px;margin-right:0;width:calc(100% - 72px)}[dir=rtl] .mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-trailing{width:calc(100% - 16px)}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing{margin-left:72px;margin-right:0;width:calc(100% - 88px)}[dir=rtl] .mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing[dir=rtl]{margin-left:0;margin-right:72px;}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding{margin-left:16px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding,.mdc-deprecated-list--thumbnail-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading{margin-left:88px;margin-right:0;width:calc(100% - 88px)}[dir=rtl] .mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading,.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading[dir=rtl]{margin-left:0;margin-right:88px;}.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-trailing{width:calc(100% - 16px)}.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing{margin-left:88px;margin-right:0;width:calc(100% - 104px)}[dir=rtl] .mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing,.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing[dir=rtl]{margin-left:0;margin-right:88px;}.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding{margin-left:16px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding,.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding{margin-left:16px;margin-right:0;width:calc(100% - 32px)}[dir=rtl] .mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding,.mdc-deprecated-list--image-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding[dir=rtl]{margin-left:0;margin-right:16px;}.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading{margin-left:116px;margin-right:0;width:calc(100% - 116px)}[dir=rtl] .mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading,.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading[dir=rtl]{margin-left:0;margin-right:116px;}.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-trailing{width:calc(100% - 16px)}.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing{margin-left:116px;margin-right:0;width:calc(100% - 132px)}[dir=rtl] .mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing,.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing[dir=rtl]{margin-left:0;margin-right:116px;}.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding{margin-left:0px;margin-right:0;width:calc(100% - 0px)}[dir=rtl] .mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding,.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--padding[dir=rtl]{margin-left:0;margin-right:0px;}.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding{margin-left:0px;margin-right:0;width:calc(100% - 16px)}[dir=rtl] .mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding,.mdc-deprecated-list--video-list .mdc-deprecated-list-divider--inset-leading.mdc-deprecated-list-divider--inset-trailing.mdc-deprecated-list-divider--inset-padding[dir=rtl]{margin-left:0;margin-right:0px;}.mdc-deprecated-list-group .mdc-deprecated-list{padding:0}.mdc-deprecated-list-group__subheader{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);margin:calc((3rem - 1.5rem) / 2) 16px}.mdc-list-item__primary-text{color:rgba(0, 0, 0, 0.87);color:var(--mdc-theme-text-primary-on-background, rgba(0, 0, 0, 0.87))}.mdc-list-item__secondary-text{color:rgba(0, 0, 0, 0.54);color:var(--mdc-theme-text-secondary-on-background, rgba(0, 0, 0, 0.54))}.mdc-list-item__overline-text{color:rgba(0, 0, 0, 0.38);color:var(--mdc-theme-text-hint-on-background, rgba(0, 0, 0, 0.38))}.mdc-list-item--with-leading-icon .mdc-list-item__start,.mdc-list-item--with-trailing-icon .mdc-list-item__end{background-color:transparent}.mdc-list-item--with-leading-icon .mdc-list-item__start,.mdc-list-item--with-trailing-icon .mdc-list-item__end{color:rgba(0, 0, 0, 0.38);color:var(--mdc-theme-text-icon-on-background, rgba(0, 0, 0, 0.38))}.mdc-list-item__end{color:rgba(0, 0, 0, 0.38);color:var(--mdc-theme-text-hint-on-background, rgba(0, 0, 0, 0.38))}.mdc-list-item--disabled .mdc-list-item__start,.mdc-list-item--disabled .mdc-list-item__content,.mdc-list-item--disabled .mdc-list-item__end{opacity:0.38}.mdc-list-item--disabled .mdc-list-item__primary-text{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-list-item--disabled .mdc-list-item__secondary-text{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-list-item--disabled .mdc-list-item__overline-text{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-list-item--disabled.mdc-list-item--with-leading-icon .mdc-list-item__start{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-list-item--disabled.mdc-list-item--with-trailing-icon .mdc-list-item__end{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-list-item--disabled.mdc-list-item--with-trailing-meta .mdc-list-item__end{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-list-item--selected .mdc-list-item__primary-text,.mdc-list-item--activated .mdc-list-item__primary-text{color:#6200ee;color:var(--mdc-theme-primary, #6200ee)}.mdc-list-item--selected.mdc-list-item--with-leading-icon .mdc-list-item__start,.mdc-list-item--activated.mdc-list-item--with-leading-icon .mdc-list-item__start{color:#6200ee;color:var(--mdc-theme-primary, #6200ee)}.mdc-deprecated-list-group__subheader{color:rgba(0, 0, 0, 0.87);color:var(--mdc-theme-text-primary-on-background, rgba(0, 0, 0, 0.87))}@media screen and (forced-colors: active), (-ms-high-contrast: active){.mdc-list-divider::after{content:\"\";display:block;border-bottom-width:1px;border-bottom-style:solid;border-bottom-color:white}}.mdc-list{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);line-height:1.5rem;margin:0;padding:8px 0;list-style-type:none}.mdc-list:focus{outline:none}.mdc-list-item__wrapper{display:block}.mdc-list-item{display:flex;position:relative;align-items:center;justify-content:flex-start;overflow:hidden;padding:0;align-items:stretch;cursor:pointer}.mdc-list-item:focus{outline:none}.mdc-list-item.mdc-list-item--with-one-line{height:48px}.mdc-list-item.mdc-list-item--with-two-lines{height:64px}.mdc-list-item.mdc-list-item--with-three-lines{height:88px}.mdc-list-item.mdc-list-item--with-one-line .mdc-list-item__start{align-self:center;margin-top:0}.mdc-list-item.mdc-list-item--with-two-lines .mdc-list-item__start{align-self:flex-start;margin-top:16px}.mdc-list-item.mdc-list-item--with-three-lines .mdc-list-item__start{align-self:flex-start;margin-top:16px}.mdc-list-item.mdc-list-item--with-one-line .mdc-list-item__end{align-self:center;margin-top:0}.mdc-list-item.mdc-list-item--with-two-lines .mdc-list-item__end{align-self:center;margin-top:0}.mdc-list-item.mdc-list-item--with-three-lines .mdc-list-item__end{align-self:flex-start;margin-top:16px}.mdc-list-item.mdc-list-item--disabled,.mdc-list-item.mdc-list-item--non-interactive{cursor:auto}.mdc-list-item:not(.mdc-list-item--selected):focus::before,.mdc-list-item.mdc-ripple-upgraded--background-focused::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid transparent;border-radius:inherit;content:\"\";pointer-events:none}@media screen and (forced-colors: active){.mdc-list-item:not(.mdc-list-item--selected):focus::before,.mdc-list-item.mdc-ripple-upgraded--background-focused::before{border-color:CanvasText}}.mdc-list-item.mdc-list-item--selected::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:3px double transparent;border-radius:inherit;content:\"\";pointer-events:none}@media screen and (forced-colors: active){.mdc-list-item.mdc-list-item--selected::before{border-color:CanvasText}}.mdc-list-item.mdc-list-item--selected:focus::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:3px solid transparent;border-radius:inherit;content:\"\";pointer-events:none}@media screen and (forced-colors: active){.mdc-list-item.mdc-list-item--selected:focus::before{border-color:CanvasText}}a.mdc-list-item{color:inherit;text-decoration:none}.mdc-list-item__start{fill:currentColor;flex-shrink:0;pointer-events:none}.mdc-list-item__end{flex-shrink:0;pointer-events:none}.mdc-list-item__content{text-overflow:ellipsis;white-space:nowrap;overflow:hidden;align-self:center;flex:1;pointer-events:none}.mdc-list-item--with-two-lines .mdc-list-item__content,.mdc-list-item--with-three-lines .mdc-list-item__content{align-self:stretch}.mdc-list-item__content[for]{pointer-events:none}.mdc-list-item__primary-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);text-overflow:ellipsis;white-space:nowrap;overflow:hidden}.mdc-list-item--with-two-lines .mdc-list-item__primary-text,.mdc-list-item--with-three-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before,.mdc-list-item--with-three-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after,.mdc-list-item--with-three-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item__secondary-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-body2-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.875rem;font-size:var(--mdc-typography-body2-font-size, 0.875rem);line-height:1.25rem;line-height:var(--mdc-typography-body2-line-height, 1.25rem);font-weight:400;font-weight:var(--mdc-typography-body2-font-weight, 400);letter-spacing:0.0178571429em;letter-spacing:var(--mdc-typography-body2-letter-spacing, 0.0178571429em);text-decoration:inherit;text-decoration:var(--mdc-typography-body2-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-body2-text-transform, inherit);text-overflow:ellipsis;white-space:nowrap;overflow:hidden;display:block;margin-top:0;line-height:normal}.mdc-list-item__secondary-text::before{display:inline-block;width:0;height:20px;content:\"\";vertical-align:0}.mdc-list-item--with-three-lines .mdc-list-item__secondary-text{white-space:normal;line-height:20px}.mdc-list-item--with-overline .mdc-list-item__secondary-text{white-space:nowrap;line-height:auto}.mdc-list-item__overline-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-overline-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.75rem;font-size:var(--mdc-typography-overline-font-size, 0.75rem);line-height:2rem;line-height:var(--mdc-typography-overline-line-height, 2rem);font-weight:500;font-weight:var(--mdc-typography-overline-font-weight, 500);letter-spacing:0.1666666667em;letter-spacing:var(--mdc-typography-overline-letter-spacing, 0.1666666667em);text-decoration:none;text-decoration:var(--mdc-typography-overline-text-decoration, none);text-transform:uppercase;text-transform:var(--mdc-typography-overline-text-transform, uppercase);text-overflow:ellipsis;white-space:nowrap;overflow:hidden}.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:24px;content:\"\";vertical-align:0}.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-three-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-three-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-three-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-avatar.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-avatar.mdc-list-item,.mdc-list-item--with-leading-avatar.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-avatar .mdc-list-item__start{margin-left:16px;margin-right:16px}[dir=rtl] .mdc-list-item--with-leading-avatar .mdc-list-item__start,.mdc-list-item--with-leading-avatar .mdc-list-item__start[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-list-item--with-leading-avatar .mdc-list-item__start{width:40px;height:40px}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-avatar.mdc-list-item--with-one-line{height:56px}.mdc-list-item--with-leading-avatar.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-avatar .mdc-list-item__start{border-radius:50%}.mdc-list-item--with-leading-icon .mdc-list-item__start{width:24px;height:24px}.mdc-list-item--with-leading-icon.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-icon.mdc-list-item,.mdc-list-item--with-leading-icon.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-icon .mdc-list-item__start{margin-left:16px;margin-right:32px}[dir=rtl] .mdc-list-item--with-leading-icon .mdc-list-item__start,.mdc-list-item--with-leading-icon .mdc-list-item__start[dir=rtl]{margin-left:32px;margin-right:16px;}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-icon.mdc-list-item--with-one-line{height:56px}.mdc-list-item--with-leading-icon.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-thumbnail.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-thumbnail.mdc-list-item,.mdc-list-item--with-leading-thumbnail.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-thumbnail .mdc-list-item__start{margin-left:16px;margin-right:16px}[dir=rtl] .mdc-list-item--with-leading-thumbnail .mdc-list-item__start,.mdc-list-item--with-leading-thumbnail .mdc-list-item__start[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-list-item--with-leading-thumbnail .mdc-list-item__start{width:40px;height:40px}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-one-line{height:56px}.mdc-list-item--with-leading-thumbnail.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-image.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-image.mdc-list-item,.mdc-list-item--with-leading-image.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-image .mdc-list-item__start{margin-left:16px;margin-right:16px}[dir=rtl] .mdc-list-item--with-leading-image .mdc-list-item__start,.mdc-list-item--with-leading-image .mdc-list-item__start[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-list-item--with-leading-image .mdc-list-item__start{width:56px;height:56px}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-image.mdc-list-item--with-one-line{height:72px}.mdc-list-item--with-leading-image.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__start{align-self:flex-start;margin-top:8px}.mdc-list-item--with-leading-video.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-video.mdc-list-item,.mdc-list-item--with-leading-video.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-video .mdc-list-item__start{margin-left:0;margin-right:16px}[dir=rtl] .mdc-list-item--with-leading-video .mdc-list-item__start,.mdc-list-item--with-leading-video .mdc-list-item__start[dir=rtl]{margin-left:16px;margin-right:0;}.mdc-list-item--with-leading-video .mdc-list-item__start{width:100px;height:56px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-video.mdc-list-item--with-one-line{height:72px}.mdc-list-item--with-leading-video.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-checkbox.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-checkbox.mdc-list-item,.mdc-list-item--with-leading-checkbox.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-checkbox .mdc-list-item__start{margin-left:8px;margin-right:24px}[dir=rtl] .mdc-list-item--with-leading-checkbox .mdc-list-item__start,.mdc-list-item--with-leading-checkbox .mdc-list-item__start[dir=rtl]{margin-left:24px;margin-right:8px;}.mdc-list-item--with-leading-checkbox .mdc-list-item__start{width:40px;height:40px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__start{align-self:flex-start;margin-top:8px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-one-line{height:56px}.mdc-list-item--with-leading-checkbox.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-radio.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-radio.mdc-list-item,.mdc-list-item--with-leading-radio.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-radio .mdc-list-item__start{margin-left:8px;margin-right:24px}[dir=rtl] .mdc-list-item--with-leading-radio .mdc-list-item__start,.mdc-list-item--with-leading-radio .mdc-list-item__start[dir=rtl]{margin-left:24px;margin-right:8px;}.mdc-list-item--with-leading-radio .mdc-list-item__start{width:40px;height:40px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__start{align-self:flex-start;margin-top:8px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-radio.mdc-list-item--with-one-line{height:56px}.mdc-list-item--with-leading-radio.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-leading-switch.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-list-item--with-leading-switch.mdc-list-item,.mdc-list-item--with-leading-switch.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-list-item--with-leading-switch .mdc-list-item__start{margin-left:16px;margin-right:16px}[dir=rtl] .mdc-list-item--with-leading-switch .mdc-list-item__start,.mdc-list-item--with-leading-switch .mdc-list-item__start[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-list-item--with-leading-switch .mdc-list-item__start{width:36px;height:20px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__start{align-self:flex-start;margin-top:16px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__overline-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__overline-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines .mdc-list-item__overline-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:32px;content:\"\";vertical-align:0}.mdc-list-item--with-leading-switch.mdc-list-item--with-one-line{height:56px}.mdc-list-item--with-leading-switch.mdc-list-item--with-two-lines{height:72px}.mdc-list-item--with-trailing-icon.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-list-item--with-trailing-icon.mdc-list-item,.mdc-list-item--with-trailing-icon.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-list-item--with-trailing-icon .mdc-list-item__end{margin-left:16px;margin-right:16px}[dir=rtl] .mdc-list-item--with-trailing-icon .mdc-list-item__end,.mdc-list-item--with-trailing-icon .mdc-list-item__end[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-list-item--with-trailing-icon .mdc-list-item__end{width:24px;height:24px}.mdc-list-item--with-trailing-meta.mdc-list-item--with-two-lines .mdc-list-item__end{align-self:flex-start;margin-top:0}.mdc-list-item--with-trailing-meta.mdc-list-item--with-three-lines .mdc-list-item__end{align-self:flex-start;margin-top:0}.mdc-list-item--with-trailing-meta.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-list-item--with-trailing-meta.mdc-list-item,.mdc-list-item--with-trailing-meta.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-list-item--with-trailing-meta .mdc-list-item__end{margin-left:28px;margin-right:16px}[dir=rtl] .mdc-list-item--with-trailing-meta .mdc-list-item__end,.mdc-list-item--with-trailing-meta .mdc-list-item__end[dir=rtl]{margin-left:16px;margin-right:28px;}.mdc-list-item--with-trailing-meta.mdc-list-item--with-two-lines .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-trailing-meta.mdc-list-item--with-two-lines .mdc-list-item__end::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-trailing-meta.mdc-list-item--with-three-lines .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-trailing-meta.mdc-list-item--with-three-lines .mdc-list-item__end::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-list-item--with-trailing-meta .mdc-list-item__end{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-caption-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.75rem;font-size:var(--mdc-typography-caption-font-size, 0.75rem);line-height:1.25rem;line-height:var(--mdc-typography-caption-line-height, 1.25rem);font-weight:400;font-weight:var(--mdc-typography-caption-font-weight, 400);letter-spacing:0.0333333333em;letter-spacing:var(--mdc-typography-caption-letter-spacing, 0.0333333333em);text-decoration:inherit;text-decoration:var(--mdc-typography-caption-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-caption-text-transform, inherit)}.mdc-list-item--with-trailing-checkbox.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-list-item--with-trailing-checkbox.mdc-list-item,.mdc-list-item--with-trailing-checkbox.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-list-item--with-trailing-checkbox .mdc-list-item__end{margin-left:24px;margin-right:8px}[dir=rtl] .mdc-list-item--with-trailing-checkbox .mdc-list-item__end,.mdc-list-item--with-trailing-checkbox .mdc-list-item__end[dir=rtl]{margin-left:8px;margin-right:24px;}.mdc-list-item--with-trailing-checkbox .mdc-list-item__end{width:40px;height:40px}.mdc-list-item--with-trailing-checkbox.mdc-list-item--with-three-lines .mdc-list-item__end{align-self:flex-start;margin-top:8px}.mdc-list-item--with-trailing-radio.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-list-item--with-trailing-radio.mdc-list-item,.mdc-list-item--with-trailing-radio.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-list-item--with-trailing-radio .mdc-list-item__end{margin-left:24px;margin-right:8px}[dir=rtl] .mdc-list-item--with-trailing-radio .mdc-list-item__end,.mdc-list-item--with-trailing-radio .mdc-list-item__end[dir=rtl]{margin-left:8px;margin-right:24px;}.mdc-list-item--with-trailing-radio .mdc-list-item__end{width:40px;height:40px}.mdc-list-item--with-trailing-radio.mdc-list-item--with-three-lines .mdc-list-item__end{align-self:flex-start;margin-top:8px}.mdc-list-item--with-trailing-switch.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-list-item--with-trailing-switch.mdc-list-item,.mdc-list-item--with-trailing-switch.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-list-item--with-trailing-switch .mdc-list-item__end{margin-left:16px;margin-right:16px}[dir=rtl] .mdc-list-item--with-trailing-switch .mdc-list-item__end,.mdc-list-item--with-trailing-switch .mdc-list-item__end[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-list-item--with-trailing-switch .mdc-list-item__end{width:36px;height:20px}.mdc-list-item--with-trailing-switch.mdc-list-item--with-three-lines .mdc-list-item__end{align-self:flex-start;margin-top:16px}.mdc-list-item--with-overline.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-overline.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:20px;content:\"\";vertical-align:0}.mdc-list-item--with-overline.mdc-list-item--with-three-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal}.mdc-list-item--with-overline.mdc-list-item--with-three-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:20px;content:\"\";vertical-align:0}.mdc-list-item{padding-left:16px;padding-right:16px}[dir=rtl] .mdc-list-item,.mdc-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-list-group .mdc-deprecated-list{padding:0}.mdc-list-group__subheader{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);margin:calc((3rem - 1.5rem) / 2) 16px}.mdc-list-divider{background-color:rgba(0, 0, 0, 0.12)}.mdc-list-divider{height:1px}.mdc-list-divider{padding:0;background-clip:content-box}.mdc-list-divider.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-text.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-icon.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-image.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-thumbnail.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-avatar.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-checkbox.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-switch.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-radio.mdc-list-divider--with-leading-inset{padding-left:16px;padding-right:auto}[dir=rtl] .mdc-list-divider.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-text.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-icon.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-image.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-thumbnail.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-avatar.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-checkbox.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-switch.mdc-list-divider--with-leading-inset,[dir=rtl] .mdc-list-divider--with-leading-radio.mdc-list-divider--with-leading-inset,.mdc-list-divider.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-text.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-icon.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-image.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-thumbnail.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-avatar.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-checkbox.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-switch.mdc-list-divider--with-leading-inset[dir=rtl],.mdc-list-divider--with-leading-radio.mdc-list-divider--with-leading-inset[dir=rtl]{padding-left:auto;padding-right:16px;}.mdc-list-divider.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-text.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-icon.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-image.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-thumbnail.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-avatar.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-checkbox.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-switch.mdc-list-divider--with-trailing-inset,.mdc-list-divider--with-leading-radio.mdc-list-divider--with-trailing-inset{padding-left:auto;padding-right:16px}[dir=rtl] .mdc-list-divider.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-text.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-icon.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-image.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-thumbnail.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-avatar.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-checkbox.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-switch.mdc-list-divider--with-trailing-inset,[dir=rtl] .mdc-list-divider--with-leading-radio.mdc-list-divider--with-trailing-inset,.mdc-list-divider.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-text.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-icon.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-image.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-thumbnail.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-avatar.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-checkbox.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-switch.mdc-list-divider--with-trailing-inset[dir=rtl],.mdc-list-divider--with-leading-radio.mdc-list-divider--with-trailing-inset[dir=rtl]{padding-left:16px;padding-right:auto;}.mdc-list-divider--with-leading-video.mdc-list-divider--with-leading-inset{padding-left:0px;padding-right:auto}[dir=rtl] .mdc-list-divider--with-leading-video.mdc-list-divider--with-leading-inset,.mdc-list-divider--with-leading-video.mdc-list-divider--with-leading-inset[dir=rtl]{padding-left:auto;padding-right:0px;}[dir=rtl] .mdc-list-divider,.mdc-list-divider[dir=rtl]{padding:0;}@keyframes mdc-ripple-fg-radius-in{from{animation-timing-function:cubic-bezier(0.4, 0, 0.2, 1);transform:translate(var(--mdc-ripple-fg-translate-start, 0)) scale(1)}to{transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}}@keyframes mdc-ripple-fg-opacity-in{from{animation-timing-function:linear;opacity:0}to{opacity:var(--mdc-ripple-fg-opacity, 0)}}@keyframes mdc-ripple-fg-opacity-out{from{animation-timing-function:linear;opacity:var(--mdc-ripple-fg-opacity, 0)}to{opacity:0}}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity;--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded .mdc-deprecated-list-item__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded .mdc-deprecated-list-item__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--unbounded .mdc-deprecated-list-item__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--foreground-activation .mdc-deprecated-list-item__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--foreground-deactivation .mdc-deprecated-list-item__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded .mdc-list-item__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded .mdc-list-item__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--unbounded .mdc-list-item__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--foreground-activation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--foreground-deactivation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded .mdc-deprecated-list-item__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded .mdc-list-item__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, #000)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:hover .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-surface--hover .mdc-deprecated-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--background-focused .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:not(.mdc-ripple-upgraded):focus .mdc-deprecated-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:not(.mdc-ripple-upgraded) .mdc-deprecated-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:not(.mdc-ripple-upgraded):active .mdc-deprecated-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, #000)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:hover .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated .mdc-deprecated-list-item__ripple::before{opacity:0.12;opacity:var(--mdc-ripple-activated-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated .mdc-deprecated-list-item__ripple::after{background-color:#6200ee;background-color:var(--mdc-ripple-color, var(--mdc-theme-primary, #6200ee))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:hover .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated.mdc-ripple-surface--hover .mdc-deprecated-list-item__ripple::before{opacity:0.16;opacity:var(--mdc-ripple-hover-opacity, 0.16)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated.mdc-ripple-upgraded--background-focused .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:not(.mdc-ripple-upgraded):focus .mdc-deprecated-list-item__ripple::before{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-ripple-focus-opacity, 0.24)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:not(.mdc-ripple-upgraded) .mdc-deprecated-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:not(.mdc-ripple-upgraded):active .mdc-deprecated-list-item__ripple::after{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-ripple-press-opacity, 0.24)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.24)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated .mdc-list-item__ripple::before{opacity:0.12;opacity:var(--mdc-ripple-activated-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated .mdc-list-item__ripple::after{background-color:#6200ee;background-color:var(--mdc-ripple-color, var(--mdc-theme-primary, #6200ee))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:hover .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.16;opacity:var(--mdc-ripple-hover-opacity, 0.16)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-ripple-focus-opacity, 0.24)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-ripple-press-opacity, 0.24)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--activated.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.24)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::before{opacity:0.08;opacity:var(--mdc-ripple-selected-opacity, 0.08)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::after{background-color:#6200ee;background-color:var(--mdc-ripple-color, var(--mdc-theme-primary, #6200ee))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:hover .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected.mdc-ripple-surface--hover .mdc-deprecated-list-item__ripple::before{opacity:0.12;opacity:var(--mdc-ripple-hover-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-deprecated-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-deprecated-list-item__ripple::before{transition-duration:75ms;opacity:0.2;opacity:var(--mdc-ripple-focus-opacity, 0.2)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded) .mdc-deprecated-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-deprecated-list-item__ripple::after{transition-duration:75ms;opacity:0.2;opacity:var(--mdc-ripple-press-opacity, 0.2)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.2)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected .mdc-list-item__ripple::before{opacity:0.08;opacity:var(--mdc-ripple-selected-opacity, 0.08)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected .mdc-list-item__ripple::after{background-color:#6200ee;background-color:var(--mdc-ripple-color, var(--mdc-theme-primary, #6200ee))}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:hover .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.12;opacity:var(--mdc-ripple-hover-opacity, 0.12)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.2;opacity:var(--mdc-ripple-focus-opacity, 0.2)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.2;opacity:var(--mdc-ripple-press-opacity, 0.2)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.2)}:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-deprecated-list-item__ripple,:not(.mdc-deprecated-list-item--disabled).mdc-deprecated-list-item .mdc-list-item__ripple{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none}.mdc-deprecated-list-item--disabled{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity;--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::before,.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded .mdc-deprecated-list-item__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded .mdc-deprecated-list-item__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--unbounded .mdc-deprecated-list-item__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--foreground-activation .mdc-deprecated-list-item__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--foreground-deactivation .mdc-deprecated-list-item__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::before,.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded .mdc-list-item__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded .mdc-list-item__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--unbounded .mdc-list-item__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--foreground-activation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--foreground-deactivation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::before,.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded .mdc-deprecated-list-item__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::before,.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded .mdc-list-item__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::before,.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, #000)}.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::before,.mdc-deprecated-list-item--disabled .mdc-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, #000)}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--background-focused .mdc-deprecated-list-item__ripple::before,.mdc-deprecated-list-item--disabled:not(.mdc-ripple-upgraded):focus .mdc-deprecated-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-deprecated-list-item--disabled.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,.mdc-deprecated-list-item--disabled:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-deprecated-list-item--disabled .mdc-deprecated-list-item__ripple,.mdc-deprecated-list-item--disabled .mdc-list-item__ripple{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none}:not(.mdc-list-item--disabled).mdc-list-item{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded .mdc-list-item__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded .mdc-list-item__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded--unbounded .mdc-list-item__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded--foreground-activation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded--foreground-deactivation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded .mdc-list-item__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, #000)}:not(.mdc-list-item--disabled).mdc-list-item:hover .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}:not(.mdc-list-item--disabled).mdc-list-item:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-list-item--disabled).mdc-list-item:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}:not(.mdc-list-item--disabled).mdc-list-item.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}:not(.mdc-list-item--disabled).mdc-list-item--activated .mdc-list-item__ripple::before{opacity:0.12;opacity:var(--mdc-ripple-activated-opacity, 0.12)}:not(.mdc-list-item--disabled).mdc-list-item--activated .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item--activated .mdc-list-item__ripple::after{background-color:#6200ee;background-color:var(--mdc-ripple-color, var(--mdc-theme-primary, #6200ee))}:not(.mdc-list-item--disabled).mdc-list-item--activated:hover .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item--activated.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.16;opacity:var(--mdc-ripple-hover-opacity, 0.16)}:not(.mdc-list-item--disabled).mdc-list-item--activated.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item--activated:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-ripple-focus-opacity, 0.24)}:not(.mdc-list-item--disabled).mdc-list-item--activated:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-list-item--disabled).mdc-list-item--activated:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-ripple-press-opacity, 0.24)}:not(.mdc-list-item--disabled).mdc-list-item--activated.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.24)}:not(.mdc-list-item--disabled).mdc-list-item--selected .mdc-list-item__ripple::before{opacity:0.08;opacity:var(--mdc-ripple-selected-opacity, 0.08)}:not(.mdc-list-item--disabled).mdc-list-item--selected .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item--selected .mdc-list-item__ripple::after{background-color:#6200ee;background-color:var(--mdc-ripple-color, var(--mdc-theme-primary, #6200ee))}:not(.mdc-list-item--disabled).mdc-list-item--selected:hover .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item--selected.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.12;opacity:var(--mdc-ripple-hover-opacity, 0.12)}:not(.mdc-list-item--disabled).mdc-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,:not(.mdc-list-item--disabled).mdc-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.2;opacity:var(--mdc-ripple-focus-opacity, 0.2)}:not(.mdc-list-item--disabled).mdc-list-item--selected:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}:not(.mdc-list-item--disabled).mdc-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.2;opacity:var(--mdc-ripple-press-opacity, 0.2)}:not(.mdc-list-item--disabled).mdc-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.2)}:not(.mdc-list-item--disabled).mdc-list-item .mdc-list-item__ripple{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none}.mdc-list-item--disabled{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}.mdc-list-item--disabled .mdc-list-item__ripple::before,.mdc-list-item--disabled .mdc-list-item__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}.mdc-list-item--disabled .mdc-list-item__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}.mdc-list-item--disabled .mdc-list-item__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}.mdc-list-item--disabled.mdc-ripple-upgraded .mdc-list-item__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}.mdc-list-item--disabled.mdc-ripple-upgraded .mdc-list-item__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}.mdc-list-item--disabled.mdc-ripple-upgraded--unbounded .mdc-list-item__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}.mdc-list-item--disabled.mdc-ripple-upgraded--foreground-activation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}.mdc-list-item--disabled.mdc-ripple-upgraded--foreground-deactivation .mdc-list-item__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}.mdc-list-item--disabled .mdc-list-item__ripple::before,.mdc-list-item--disabled .mdc-list-item__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}.mdc-list-item--disabled.mdc-ripple-upgraded .mdc-list-item__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}.mdc-list-item--disabled .mdc-list-item__ripple::before,.mdc-list-item--disabled .mdc-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, #000)}.mdc-list-item--disabled.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,.mdc-list-item--disabled:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-list-item--disabled .mdc-list-item__ripple{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none}.mdc-menu-surface{display:none;position:absolute;box-sizing:border-box;max-width:calc(100vw - 32px);max-width:var(--mdc-menu-max-width, calc(100vw - 32px));max-height:calc(100vh - 32px);max-height:var(--mdc-menu-max-height, calc(100vh - 32px));margin:0;padding:0;transform:scale(1);transform-origin:top left;opacity:0;overflow:auto;will-change:transform, opacity;z-index:8;transition:opacity 0.03s linear, transform 0.12s cubic-bezier(0, 0, 0.2, 1), height 250ms cubic-bezier(0, 0, 0.2, 1);box-shadow:0px 5px 5px -3px rgba(0, 0, 0, 0.2), 0px 8px 10px 1px rgba(0, 0, 0, 0.14), 0px 3px 14px 2px rgba(0, 0, 0, 0.12);background-color:#fff;background-color:var(--mdc-theme-surface, #fff);color:#000;color:var(--mdc-theme-on-surface, #000);border-radius:4px;border-radius:var(--mdc-shape-medium, 4px);transform-origin-left:top left;transform-origin-right:top right}.mdc-menu-surface:focus{outline:none}.mdc-menu-surface--animating-open{display:inline-block;transform:scale(0.8);opacity:0}.mdc-menu-surface--open{display:inline-block;transform:scale(1);opacity:1}.mdc-menu-surface--animating-closed{display:inline-block;opacity:0;transition:opacity 0.075s linear}[dir=rtl] .mdc-menu-surface,.mdc-menu-surface[dir=rtl]{transform-origin-left:top right;transform-origin-right:top left;}.mdc-menu-surface--anchor{position:relative;overflow:visible}.mdc-menu-surface--fixed{position:fixed}.mdc-menu-surface--fullwidth{width:100%}.mdc-elevation-overlay{position:absolute;border-radius:inherit;pointer-events:none;opacity:0;opacity:var(--mdc-elevation-overlay-opacity, 0);transition:opacity 280ms cubic-bezier(0.4, 0, 0.2, 1);background-color:#fff;background-color:var(--mdc-elevation-overlay-color, #fff)}.mdc-menu{min-width:112px;min-width:var(--mdc-menu-min-width, 112px)}.mdc-menu .mdc-deprecated-list-item__meta{color:rgba(0, 0, 0, 0.87)}.mdc-menu .mdc-deprecated-list-item__graphic{color:rgba(0, 0, 0, 0.87)}.mdc-menu .mdc-menu-item--submenu-open .mdc-deprecated-list-item__ripple::before{opacity:0.04}.mdc-menu .mdc-menu-item--submenu-open .mdc-list-item__ripple::before{opacity:0.04}.mdc-menu .mdc-deprecated-list{color:rgba(0, 0, 0, 0.87)}.mdc-menu .mdc-deprecated-list,.mdc-menu .mdc-list{position:relative}.mdc-menu .mdc-deprecated-list .mdc-elevation-overlay,.mdc-menu .mdc-list .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}.mdc-menu .mdc-deprecated-list-divider{margin:8px 0}.mdc-menu .mdc-deprecated-list-item{user-select:none}.mdc-menu .mdc-deprecated-list-item--disabled{cursor:auto}.mdc-menu a.mdc-deprecated-list-item .mdc-deprecated-list-item__text,.mdc-menu a.mdc-deprecated-list-item .mdc-deprecated-list-item__graphic{pointer-events:none}.mdc-menu__selection-group{padding:0;fill:currentColor}.mdc-menu__selection-group .mdc-deprecated-list-item{padding-left:56px;padding-right:16px}[dir=rtl] .mdc-menu__selection-group .mdc-deprecated-list-item,.mdc-menu__selection-group .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:56px;}.mdc-menu__selection-group .mdc-menu__selection-group-icon{left:16px;right:initial;display:none;position:absolute;top:50%;transform:translateY(-50%)}[dir=rtl] .mdc-menu__selection-group .mdc-menu__selection-group-icon,.mdc-menu__selection-group .mdc-menu__selection-group-icon[dir=rtl]{left:initial;right:16px;}.mdc-menu-item--selected .mdc-menu__selection-group-icon{display:inline}.mdc-floating-label{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);position:absolute;left:0;-webkit-transform-origin:left top;transform-origin:left top;line-height:1.15rem;text-align:left;text-overflow:ellipsis;white-space:nowrap;cursor:text;overflow:hidden;will-change:transform;transition:transform 150ms cubic-bezier(0.4, 0, 0.2, 1), color 150ms cubic-bezier(0.4, 0, 0.2, 1)}[dir=rtl] .mdc-floating-label,.mdc-floating-label[dir=rtl]{right:0;left:auto;-webkit-transform-origin:right top;transform-origin:right top;text-align:right;}.mdc-floating-label--float-above{cursor:auto}.mdc-floating-label--required::after{margin-left:1px;margin-right:0px;content:\"*\"}[dir=rtl] .mdc-floating-label--required,.mdc-floating-label--required[dir=rtl]{}[dir=rtl] .mdc-floating-label--required::after,.mdc-floating-label--required[dir=rtl]::after{margin-left:0;margin-right:1px}.mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-standard 250ms 1}@keyframes mdc-floating-label-shake-float-above-standard{0%{transform:translateX(calc(0 - 0%)) translateY(-106%) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(-106%) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(-106%) scale(0.75)}100%{transform:translateX(calc(0 - 0%)) translateY(-106%) scale(0.75)}}.mdc-line-ripple::before,.mdc-line-ripple::after{position:absolute;bottom:0;left:0;width:100%;border-bottom-style:solid;content:\"\"}.mdc-line-ripple::before{border-bottom-width:1px}.mdc-line-ripple::before{z-index:1}.mdc-line-ripple::after{transform:scaleX(0);border-bottom-width:2px;opacity:0;z-index:2}.mdc-line-ripple::after{transition:transform 180ms cubic-bezier(0.4, 0, 0.2, 1), opacity 180ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-line-ripple--active::after{transform:scaleX(1);opacity:1}.mdc-line-ripple--deactivating::after{opacity:0}.mdc-notched-outline{display:flex;position:absolute;top:0;right:0;left:0;box-sizing:border-box;width:100%;max-width:100%;height:100%;text-align:left;pointer-events:none}[dir=rtl] .mdc-notched-outline,.mdc-notched-outline[dir=rtl]{text-align:right;}.mdc-notched-outline__leading,.mdc-notched-outline__notch,.mdc-notched-outline__trailing{box-sizing:border-box;height:100%;border-top:1px solid;border-bottom:1px solid;pointer-events:none}.mdc-notched-outline__leading{border-left:1px solid;border-right:none;width:12px}[dir=rtl] .mdc-notched-outline__leading,.mdc-notched-outline__leading[dir=rtl]{border-left:none;border-right:1px solid;}.mdc-notched-outline__trailing{border-left:none;border-right:1px solid;flex-grow:1}[dir=rtl] .mdc-notched-outline__trailing,.mdc-notched-outline__trailing[dir=rtl]{border-left:1px solid;border-right:none;}.mdc-notched-outline__notch{flex:0 0 auto;width:auto;max-width:calc(100% - 12px * 2)}.mdc-notched-outline .mdc-floating-label{display:inline-block;position:relative;max-width:100%}.mdc-notched-outline .mdc-floating-label--float-above{text-overflow:clip}.mdc-notched-outline--upgraded .mdc-floating-label--float-above{max-width:133.3333333333%}.mdc-notched-outline--notched .mdc-notched-outline__notch{padding-left:0;padding-right:8px;border-top:none}[dir=rtl] .mdc-notched-outline--notched .mdc-notched-outline__notch,.mdc-notched-outline--notched .mdc-notched-outline__notch[dir=rtl]{padding-left:8px;padding-right:0;}.mdc-notched-outline--no-label .mdc-notched-outline__notch{display:none}.mdc-select{display:inline-flex;position:relative}.mdc-select:not(.mdc-select--disabled) .mdc-select__selected-text{color:rgba(0, 0, 0, 0.87)}.mdc-select.mdc-select--disabled .mdc-select__selected-text{color:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled) .mdc-floating-label{color:rgba(0, 0, 0, 0.6)}.mdc-select:not(.mdc-select--disabled).mdc-select--focused .mdc-floating-label{color:rgba(98, 0, 238, 0.87)}.mdc-select.mdc-select--disabled .mdc-floating-label{color:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled) .mdc-select__dropdown-icon{fill:rgba(0, 0, 0, 0.54)}.mdc-select:not(.mdc-select--disabled).mdc-select--focused .mdc-select__dropdown-icon{fill:#6200ee;fill:var(--mdc-theme-primary, #6200ee)}.mdc-select.mdc-select--disabled .mdc-select__dropdown-icon{fill:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled)+.mdc-select-helper-text{color:rgba(0, 0, 0, 0.6)}.mdc-select.mdc-select--disabled+.mdc-select-helper-text{color:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled) .mdc-select__icon{color:rgba(0, 0, 0, 0.54)}.mdc-select.mdc-select--disabled .mdc-select__icon{color:rgba(0, 0, 0, 0.38)}@media screen and (forced-colors: active), (-ms-high-contrast: active){.mdc-select.mdc-select--disabled .mdc-select__selected-text{color:GrayText}.mdc-select.mdc-select--disabled .mdc-select__dropdown-icon{fill:red}.mdc-select.mdc-select--disabled .mdc-floating-label{color:GrayText}.mdc-select.mdc-select--disabled .mdc-line-ripple::before{border-bottom-color:GrayText}.mdc-select.mdc-select--disabled .mdc-notched-outline__leading,.mdc-select.mdc-select--disabled .mdc-notched-outline__notch,.mdc-select.mdc-select--disabled .mdc-notched-outline__trailing{border-color:GrayText}.mdc-select.mdc-select--disabled .mdc-select__icon{color:GrayText}.mdc-select.mdc-select--disabled+.mdc-select-helper-text{color:GrayText}}.mdc-select .mdc-floating-label{top:50%;transform:translateY(-50%);pointer-events:none}.mdc-select .mdc-select__anchor{padding-left:16px;padding-right:0}[dir=rtl] .mdc-select .mdc-select__anchor,.mdc-select .mdc-select__anchor[dir=rtl]{padding-left:0;padding-right:16px;}.mdc-select.mdc-select--with-leading-icon .mdc-select__anchor{padding-left:0;padding-right:0}[dir=rtl] .mdc-select.mdc-select--with-leading-icon .mdc-select__anchor,.mdc-select.mdc-select--with-leading-icon .mdc-select__anchor[dir=rtl]{padding-left:0;padding-right:0;}.mdc-select .mdc-select__icon{width:24px;height:24px;font-size:24px}.mdc-select .mdc-select__dropdown-icon{width:24px;height:24px}.mdc-select .mdc-select__menu .mdc-deprecated-list-item{padding-left:16px;padding-right:16px}[dir=rtl] .mdc-select .mdc-select__menu .mdc-deprecated-list-item,.mdc-select .mdc-select__menu .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-select .mdc-select__menu .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:12px}[dir=rtl] .mdc-select .mdc-select__menu .mdc-deprecated-list-item__graphic,.mdc-select .mdc-select__menu .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:12px;margin-right:0;}.mdc-select__dropdown-icon{margin-left:12px;margin-right:12px;display:inline-flex;position:relative;align-self:center;align-items:center;justify-content:center;flex-shrink:0;pointer-events:none}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-active,.mdc-select__dropdown-icon .mdc-select__dropdown-icon-inactive{position:absolute;top:0;left:0}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-graphic{width:41.6666666667%;height:20.8333333333%}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-inactive{opacity:1;transition:opacity 75ms linear 75ms}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-active{opacity:0;transition:opacity 75ms linear}[dir=rtl] .mdc-select__dropdown-icon,.mdc-select__dropdown-icon[dir=rtl]{margin-left:12px;margin-right:12px;}.mdc-select--activated .mdc-select__dropdown-icon .mdc-select__dropdown-icon-inactive{opacity:0;transition:opacity 49.5ms linear}.mdc-select--activated .mdc-select__dropdown-icon .mdc-select__dropdown-icon-active{opacity:1;transition:opacity 100.5ms linear 49.5ms}.mdc-select__anchor{width:200px;min-width:0;flex:1 1 auto;position:relative;box-sizing:border-box;overflow:hidden;outline:none;cursor:pointer}.mdc-select__anchor .mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-select__selected-text-container{display:flex;appearance:none;pointer-events:none;box-sizing:border-box;width:auto;min-width:0;flex-grow:1;height:28px;border:none;outline:none;padding:0;background-color:transparent;color:inherit}.mdc-select__selected-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);text-overflow:ellipsis;white-space:nowrap;overflow:hidden;display:block;width:100%;text-align:left}[dir=rtl] .mdc-select__selected-text,.mdc-select__selected-text[dir=rtl]{text-align:right;}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-floating-label{color:#b00020;color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-floating-label{color:#b00020;color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--invalid+.mdc-select-helper-text--validation-msg{color:#b00020;color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-select__dropdown-icon{fill:#b00020;fill:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-select__dropdown-icon{fill:#b00020;fill:var(--mdc-theme-error, #b00020)}.mdc-select--disabled{cursor:default;pointer-events:none}.mdc-select--with-leading-icon .mdc-select__menu .mdc-deprecated-list-item{padding-left:12px;padding-right:12px}[dir=rtl] .mdc-select--with-leading-icon .mdc-select__menu .mdc-deprecated-list-item,.mdc-select--with-leading-icon .mdc-select__menu .mdc-deprecated-list-item[dir=rtl]{padding-left:12px;padding-right:12px;}@media screen and (forced-colors: active), (-ms-high-contrast: active){.mdc-select__menu::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid transparent;border-radius:inherit;content:\"\";pointer-events:none}}@media screen and (forced-colors: active) and (forced-colors: active), screen and (-ms-high-contrast: active) and (forced-colors: active){.mdc-select__menu::before{border-color:CanvasText}}.mdc-select__menu .mdc-deprecated-list .mdc-select__icon,.mdc-select__menu .mdc-list .mdc-select__icon{margin-left:0;margin-right:0}[dir=rtl] .mdc-select__menu .mdc-deprecated-list .mdc-select__icon,[dir=rtl] .mdc-select__menu .mdc-list .mdc-select__icon,.mdc-select__menu .mdc-deprecated-list .mdc-select__icon[dir=rtl],.mdc-select__menu .mdc-list .mdc-select__icon[dir=rtl]{margin-left:0;margin-right:0;}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--activated,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--selected,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--activated{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__graphic,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--activated .mdc-deprecated-list-item__graphic,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__graphic,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--activated .mdc-deprecated-list-item__graphic{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-select__menu .mdc-list-item__start{display:inline-flex;align-items:center}.mdc-select__option{padding-left:16px;padding-right:16px}[dir=rtl] .mdc-select__option,.mdc-select__option[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-select__one-line-option.mdc-list-item--with-one-line{height:48px}.mdc-select__two-line-option.mdc-list-item--with-two-lines{height:64px}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__start{margin-top:20px}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-select__two-line-option.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-select__two-line-option.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:36px;content:\"\";vertical-align:0}.mdc-select__option-with-leading-content{padding-left:0;padding-right:12px}.mdc-select__option-with-leading-content.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-select__option-with-leading-content.mdc-list-item,.mdc-select__option-with-leading-content.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-select__option-with-leading-content .mdc-list-item__start{margin-left:12px;margin-right:0}[dir=rtl] .mdc-select__option-with-leading-content .mdc-list-item__start,.mdc-select__option-with-leading-content .mdc-list-item__start[dir=rtl]{margin-left:0;margin-right:12px;}.mdc-select__option-with-leading-content .mdc-list-item__start{width:36px;height:24px}[dir=rtl] .mdc-select__option-with-leading-content,.mdc-select__option-with-leading-content[dir=rtl]{padding-left:12px;padding-right:0;}.mdc-select__option-with-meta.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-select__option-with-meta.mdc-list-item,.mdc-select__option-with-meta.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-select__option-with-meta .mdc-list-item__end{margin-left:12px;margin-right:12px}[dir=rtl] .mdc-select__option-with-meta .mdc-list-item__end,.mdc-select__option-with-meta .mdc-list-item__end[dir=rtl]{margin-left:12px;margin-right:12px;}.mdc-select--filled .mdc-select__anchor{height:56px;display:flex;align-items:baseline}.mdc-select--filled .mdc-select__anchor::before{display:inline-block;width:0;height:40px;content:\"\";vertical-align:0}.mdc-select--filled.mdc-select--no-label .mdc-select__anchor .mdc-select__selected-text::before{content:\"​\"}.mdc-select--filled.mdc-select--no-label .mdc-select__anchor .mdc-select__selected-text-container{height:100%;display:inline-flex;align-items:center}.mdc-select--filled.mdc-select--no-label .mdc-select__anchor::before{display:none}.mdc-select--filled .mdc-select__anchor{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:0;border-bottom-left-radius:0}.mdc-select--filled:not(.mdc-select--disabled) .mdc-select__anchor{background-color:whitesmoke}.mdc-select--filled.mdc-select--disabled .mdc-select__anchor{background-color:#fafafa}.mdc-select--filled:not(.mdc-select--disabled) .mdc-line-ripple::before{border-bottom-color:rgba(0, 0, 0, 0.42)}.mdc-select--filled:not(.mdc-select--disabled):hover .mdc-line-ripple::before{border-bottom-color:rgba(0, 0, 0, 0.87)}.mdc-select--filled:not(.mdc-select--disabled) .mdc-line-ripple::after{border-bottom-color:#6200ee;border-bottom-color:var(--mdc-theme-primary, #6200ee)}.mdc-select--filled.mdc-select--disabled .mdc-line-ripple::before{border-bottom-color:rgba(0, 0, 0, 0.06)}.mdc-select--filled .mdc-floating-label{max-width:calc(100% - 64px)}.mdc-select--filled .mdc-floating-label--float-above{max-width:calc(100% / 0.75 - 64px / 0.75)}.mdc-select--filled .mdc-menu-surface--is-open-below{border-top-left-radius:0px;border-top-right-radius:0px}.mdc-select--filled.mdc-select--focused.mdc-line-ripple::after{transform:scale(1, 2);opacity:1}.mdc-select--filled .mdc-floating-label{left:16px;right:initial}[dir=rtl] .mdc-select--filled .mdc-floating-label,.mdc-select--filled .mdc-floating-label[dir=rtl]{left:initial;right:16px;}.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label{left:48px;right:initial}[dir=rtl] .mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label,.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label[dir=rtl]{left:initial;right:48px;}.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label{max-width:calc(100% - 96px)}.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label--float-above{max-width:calc(100% / 0.75 - 96px / 0.75)}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-line-ripple::before{border-bottom-color:#b00020;border-bottom-color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled):hover .mdc-line-ripple::before{border-bottom-color:#b00020;border-bottom-color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-line-ripple::after{border-bottom-color:#b00020;border-bottom-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined{border:none}.mdc-select--outlined .mdc-select__anchor{height:56px}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{transform:translateY(-37.25px) scale(1)}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{font-size:0.75rem}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) scale(0.75)}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined-56px 250ms 1}@keyframes mdc-floating-label-shake-float-above-select-outlined-56px{0%{transform:translateX(calc(0 - 0%)) translateY(-34.75px) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(-34.75px) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(-34.75px) scale(0.75)}100%{transform:translateX(calc(0 - 0%)) translateY(-34.75px) scale(0.75)}}.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px)}[dir=rtl] .mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading[dir=rtl]{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0;}@supports (top: max(0%)){.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading{width:max(12px, var(--mdc-shape-small, 4px))}}@supports (top: max(0%)){.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__notch{max-width:calc(100% - max(12px, var(--mdc-shape-small, 4px)) * 2)}}.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__trailing{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0}[dir=rtl] .mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__trailing,.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__trailing[dir=rtl]{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px);}@supports (top: max(0%)){.mdc-select--outlined .mdc-select__anchor{padding-left:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}[dir=rtl] .mdc-select--outlined .mdc-select__anchor,.mdc-select--outlined .mdc-select__anchor[dir=rtl]{padding-left:0;}@supports (top: max(0%)){[dir=rtl] .mdc-select--outlined .mdc-select__anchor,.mdc-select--outlined .mdc-select__anchor[dir=rtl]{padding-right:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}@supports (top: max(0%)){.mdc-select--outlined+.mdc-select-helper-text{margin-left:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}[dir=rtl] .mdc-select--outlined+.mdc-select-helper-text,.mdc-select--outlined+.mdc-select-helper-text[dir=rtl]{margin-left:0;}@supports (top: max(0%)){[dir=rtl] .mdc-select--outlined+.mdc-select-helper-text,.mdc-select--outlined+.mdc-select-helper-text[dir=rtl]{margin-right:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}.mdc-select--outlined:not(.mdc-select--disabled) .mdc-select__anchor{background-color:transparent}.mdc-select--outlined.mdc-select--disabled .mdc-select__anchor{background-color:transparent}.mdc-select--outlined:not(.mdc-select--disabled) .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled) .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled) .mdc-notched-outline__trailing{border-color:rgba(0, 0, 0, 0.38)}.mdc-select--outlined:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:rgba(0, 0, 0, 0.87)}.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-width:2px}.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-color:#6200ee;border-color:var(--mdc-theme-primary, #6200ee)}.mdc-select--outlined.mdc-select--disabled .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--disabled .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--disabled .mdc-notched-outline__trailing{border-color:rgba(0, 0, 0, 0.06)}.mdc-select--outlined .mdc-select__anchor :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mdc-select--outlined .mdc-select__anchor{display:flex;align-items:baseline;overflow:visible}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined 250ms 1}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{transform:translateY(-37.25px) scale(1)}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{font-size:0.75rem}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) scale(0.75)}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:1px}.mdc-select--outlined .mdc-select__anchor .mdc-select__selected-text::before{content:\"​\"}.mdc-select--outlined .mdc-select__anchor .mdc-select__selected-text-container{height:100%;display:inline-flex;align-items:center}.mdc-select--outlined .mdc-select__anchor::before{display:none}.mdc-select--outlined .mdc-select__selected-text-container{display:flex;border:none;z-index:1;background-color:transparent}.mdc-select--outlined .mdc-select__icon{z-index:2}.mdc-select--outlined .mdc-floating-label{line-height:1.15rem;left:4px;right:initial}[dir=rtl] .mdc-select--outlined .mdc-floating-label,.mdc-select--outlined .mdc-floating-label[dir=rtl]{left:initial;right:4px;}.mdc-select--outlined.mdc-select--focused .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:2px}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled) .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled) .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled) .mdc-notched-outline__trailing{border-color:#b00020;border-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:#b00020;border-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-width:2px}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-color:#b00020;border-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label{left:36px;right:initial}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label[dir=rtl]{left:initial;right:36px;}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above{transform:translateY(-37.25px) translateX(-32px) scale(1)}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-37.25px) translateX(32px) scale(1);}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above{font-size:0.75rem}.mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) translateX(-32px) scale(0.75)}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above,[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl],.mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-34.75px) translateX(32px) scale(0.75);}.mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px 250ms 1}@keyframes mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px{0%{transform:translateX(calc(0 - 32px)) translateY(-34.75px) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 32px)) translateY(-34.75px) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 32px)) translateY(-34.75px) scale(0.75)}100%{transform:translateX(calc(0 - 32px)) translateY(-34.75px) scale(0.75)}}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon,.mdc-select--outlined.mdc-select--with-leading-icon[dir=rtl]{}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--shake,.mdc-select--outlined.mdc-select--with-leading-icon[dir=rtl] .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px 250ms 1}@keyframes mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px-rtl{0%{transform:translateX(calc(0 - -32px)) translateY(-34.75px) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - -32px)) translateY(-34.75px) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - -32px)) translateY(-34.75px) scale(0.75)}100%{transform:translateX(calc(0 - -32px)) translateY(-34.75px) scale(0.75)}}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-select__anchor :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 96px)}.mdc-select--outlined .mdc-menu-surface{margin-bottom:8px}.mdc-select--outlined.mdc-select--no-label .mdc-menu-surface,.mdc-select--outlined .mdc-menu-surface--is-open-below{margin-bottom:0}.mdc-select__anchor{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}.mdc-select__anchor .mdc-select__ripple::before,.mdc-select__anchor .mdc-select__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}.mdc-select__anchor .mdc-select__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}.mdc-select__anchor .mdc-select__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}.mdc-select__anchor.mdc-ripple-upgraded .mdc-select__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}.mdc-select__anchor.mdc-ripple-upgraded .mdc-select__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}.mdc-select__anchor.mdc-ripple-upgraded--unbounded .mdc-select__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}.mdc-select__anchor.mdc-ripple-upgraded--foreground-activation .mdc-select__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}.mdc-select__anchor.mdc-ripple-upgraded--foreground-deactivation .mdc-select__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}.mdc-select__anchor .mdc-select__ripple::before,.mdc-select__anchor .mdc-select__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}.mdc-select__anchor.mdc-ripple-upgraded .mdc-select__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}.mdc-select__anchor .mdc-select__ripple::before,.mdc-select__anchor .mdc-select__ripple::after{background-color:rgba(0, 0, 0, 0.87);background-color:var(--mdc-ripple-color, rgba(0, 0, 0, 0.87))}.mdc-select__anchor:hover .mdc-select__ripple::before,.mdc-select__anchor.mdc-ripple-surface--hover .mdc-select__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}.mdc-select__anchor.mdc-ripple-upgraded--background-focused .mdc-select__ripple::before,.mdc-select__anchor:not(.mdc-ripple-upgraded):focus .mdc-select__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-select__anchor .mdc-select__ripple{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, var(--mdc-theme-on-surface, #000))}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:hover .mdc-deprecated-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-surface--hover .mdc-deprecated-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-deprecated-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-deprecated-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded) .mdc-deprecated-list-item__ripple::after{transition:opacity 150ms linear}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-deprecated-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, var(--mdc-theme-on-surface, #000))}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:hover .mdc-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select-helper-text{margin:0;margin-left:16px;margin-right:16px;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-caption-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.75rem;font-size:var(--mdc-typography-caption-font-size, 0.75rem);line-height:1.25rem;line-height:var(--mdc-typography-caption-line-height, 1.25rem);font-weight:400;font-weight:var(--mdc-typography-caption-font-weight, 400);letter-spacing:0.0333333333em;letter-spacing:var(--mdc-typography-caption-letter-spacing, 0.0333333333em);text-decoration:inherit;text-decoration:var(--mdc-typography-caption-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-caption-text-transform, inherit);display:block;margin-top:0;line-height:normal}[dir=rtl] .mdc-select-helper-text,.mdc-select-helper-text[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-select-helper-text::before{display:inline-block;width:0;height:16px;content:\"\";vertical-align:0}.mdc-select-helper-text--validation-msg{opacity:0;transition:opacity 180ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-select--invalid+.mdc-select-helper-text--validation-msg,.mdc-select-helper-text--validation-msg-persistent{opacity:1}.mdc-select--with-leading-icon .mdc-select__icon{display:inline-block;box-sizing:border-box;border:none;text-decoration:none;cursor:pointer;user-select:none;flex-shrink:0;align-self:center;background-color:transparent;fill:currentColor}.mdc-select--with-leading-icon .mdc-select__icon{margin-left:12px;margin-right:12px}[dir=rtl] .mdc-select--with-leading-icon .mdc-select__icon,.mdc-select--with-leading-icon .mdc-select__icon[dir=rtl]{margin-left:12px;margin-right:12px;}.mdc-select__icon:not([tabindex]),.mdc-select__icon[tabindex=\"-1\"]{cursor:default;pointer-events:none}.mdc-floating-label{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);position:absolute;left:0;-webkit-transform-origin:left top;transform-origin:left top;line-height:1.15rem;text-align:left;text-overflow:ellipsis;white-space:nowrap;cursor:text;overflow:hidden;will-change:transform;transition:transform 150ms cubic-bezier(0.4, 0, 0.2, 1), color 150ms cubic-bezier(0.4, 0, 0.2, 1)}[dir=rtl] .mdc-floating-label,.mdc-floating-label[dir=rtl]{right:0;left:auto;-webkit-transform-origin:right top;transform-origin:right top;text-align:right;}.mdc-floating-label--float-above{cursor:auto}.mdc-floating-label--required::after{margin-left:1px;margin-right:0px;content:\"*\"}[dir=rtl] .mdc-floating-label--required,.mdc-floating-label--required[dir=rtl]{}[dir=rtl] .mdc-floating-label--required::after,.mdc-floating-label--required[dir=rtl]::after{margin-left:0;margin-right:1px}.mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-standard 250ms 1}@keyframes mdc-floating-label-shake-float-above-standard{0%{transform:translateX(calc(0 - 0%)) translateY(-106%) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(-106%) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(-106%) scale(0.75)}100%{transform:translateX(calc(0 - 0%)) translateY(-106%) scale(0.75)}}.mdc-line-ripple::before,.mdc-line-ripple::after{position:absolute;bottom:0;left:0;width:100%;border-bottom-style:solid;content:\"\"}.mdc-line-ripple::before{border-bottom-width:1px}.mdc-line-ripple::before{z-index:1}.mdc-line-ripple::after{transform:scaleX(0);border-bottom-width:2px;opacity:0;z-index:2}.mdc-line-ripple::after{transition:transform 180ms cubic-bezier(0.4, 0, 0.2, 1), opacity 180ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-line-ripple--active::after{transform:scaleX(1);opacity:1}.mdc-line-ripple--deactivating::after{opacity:0}.mdc-notched-outline{display:flex;position:absolute;top:0;right:0;left:0;box-sizing:border-box;width:100%;max-width:100%;height:100%;text-align:left;pointer-events:none}[dir=rtl] .mdc-notched-outline,.mdc-notched-outline[dir=rtl]{text-align:right;}.mdc-notched-outline__leading,.mdc-notched-outline__notch,.mdc-notched-outline__trailing{box-sizing:border-box;height:100%;border-top:1px solid;border-bottom:1px solid;pointer-events:none}.mdc-notched-outline__leading{border-left:1px solid;border-right:none;width:12px}[dir=rtl] .mdc-notched-outline__leading,.mdc-notched-outline__leading[dir=rtl]{border-left:none;border-right:1px solid;}.mdc-notched-outline__trailing{border-left:none;border-right:1px solid;flex-grow:1}[dir=rtl] .mdc-notched-outline__trailing,.mdc-notched-outline__trailing[dir=rtl]{border-left:1px solid;border-right:none;}.mdc-notched-outline__notch{flex:0 0 auto;width:auto;max-width:calc(100% - 12px * 2)}.mdc-notched-outline .mdc-floating-label{display:inline-block;position:relative;max-width:100%}.mdc-notched-outline .mdc-floating-label--float-above{text-overflow:clip}.mdc-notched-outline--upgraded .mdc-floating-label--float-above{max-width:133.3333333333%}.mdc-notched-outline--notched .mdc-notched-outline__notch{padding-left:0;padding-right:8px;border-top:none}[dir=rtl] .mdc-notched-outline--notched .mdc-notched-outline__notch,.mdc-notched-outline--notched .mdc-notched-outline__notch[dir=rtl]{padding-left:8px;padding-right:0;}.mdc-notched-outline--no-label .mdc-notched-outline__notch{display:none}.mdc-select{display:inline-flex;position:relative}.mdc-select:not(.mdc-select--disabled) .mdc-select__selected-text{color:rgba(0, 0, 0, 0.87)}.mdc-select.mdc-select--disabled .mdc-select__selected-text{color:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled) .mdc-floating-label{color:rgba(0, 0, 0, 0.6)}.mdc-select:not(.mdc-select--disabled).mdc-select--focused .mdc-floating-label{color:rgba(98, 0, 238, 0.87)}.mdc-select.mdc-select--disabled .mdc-floating-label{color:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled) .mdc-select__dropdown-icon{fill:rgba(0, 0, 0, 0.54)}.mdc-select:not(.mdc-select--disabled).mdc-select--focused .mdc-select__dropdown-icon{fill:#6200ee;fill:var(--mdc-theme-primary, #6200ee)}.mdc-select.mdc-select--disabled .mdc-select__dropdown-icon{fill:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled)+.mdc-select-helper-text{color:rgba(0, 0, 0, 0.6)}.mdc-select.mdc-select--disabled+.mdc-select-helper-text{color:rgba(0, 0, 0, 0.38)}.mdc-select:not(.mdc-select--disabled) .mdc-select__icon{color:rgba(0, 0, 0, 0.54)}.mdc-select.mdc-select--disabled .mdc-select__icon{color:rgba(0, 0, 0, 0.38)}@media screen and (forced-colors: active), (-ms-high-contrast: active){.mdc-select.mdc-select--disabled .mdc-select__selected-text{color:GrayText}.mdc-select.mdc-select--disabled .mdc-select__dropdown-icon{fill:red}.mdc-select.mdc-select--disabled .mdc-floating-label{color:GrayText}.mdc-select.mdc-select--disabled .mdc-line-ripple::before{border-bottom-color:GrayText}.mdc-select.mdc-select--disabled .mdc-notched-outline__leading,.mdc-select.mdc-select--disabled .mdc-notched-outline__notch,.mdc-select.mdc-select--disabled .mdc-notched-outline__trailing{border-color:GrayText}.mdc-select.mdc-select--disabled .mdc-select__icon{color:GrayText}.mdc-select.mdc-select--disabled+.mdc-select-helper-text{color:GrayText}}.mdc-select .mdc-floating-label{top:50%;transform:translateY(-50%);pointer-events:none}.mdc-select .mdc-select__anchor{padding-left:16px;padding-right:0}[dir=rtl] .mdc-select .mdc-select__anchor,.mdc-select .mdc-select__anchor[dir=rtl]{padding-left:0;padding-right:16px;}.mdc-select.mdc-select--with-leading-icon .mdc-select__anchor{padding-left:0;padding-right:0}[dir=rtl] .mdc-select.mdc-select--with-leading-icon .mdc-select__anchor,.mdc-select.mdc-select--with-leading-icon .mdc-select__anchor[dir=rtl]{padding-left:0;padding-right:0;}.mdc-select .mdc-select__icon{width:24px;height:24px;font-size:24px}.mdc-select .mdc-select__dropdown-icon{width:24px;height:24px}.mdc-select .mdc-select__menu .mdc-deprecated-list-item{padding-left:16px;padding-right:16px}[dir=rtl] .mdc-select .mdc-select__menu .mdc-deprecated-list-item,.mdc-select .mdc-select__menu .mdc-deprecated-list-item[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-select .mdc-select__menu .mdc-deprecated-list-item__graphic{margin-left:0;margin-right:12px}[dir=rtl] .mdc-select .mdc-select__menu .mdc-deprecated-list-item__graphic,.mdc-select .mdc-select__menu .mdc-deprecated-list-item__graphic[dir=rtl]{margin-left:12px;margin-right:0;}.mdc-select__dropdown-icon{margin-left:12px;margin-right:12px;display:inline-flex;position:relative;align-self:center;align-items:center;justify-content:center;flex-shrink:0;pointer-events:none}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-active,.mdc-select__dropdown-icon .mdc-select__dropdown-icon-inactive{position:absolute;top:0;left:0}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-graphic{width:41.6666666667%;height:20.8333333333%}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-inactive{opacity:1;transition:opacity 75ms linear 75ms}.mdc-select__dropdown-icon .mdc-select__dropdown-icon-active{opacity:0;transition:opacity 75ms linear}[dir=rtl] .mdc-select__dropdown-icon,.mdc-select__dropdown-icon[dir=rtl]{margin-left:12px;margin-right:12px;}.mdc-select--activated .mdc-select__dropdown-icon .mdc-select__dropdown-icon-inactive{opacity:0;transition:opacity 49.5ms linear}.mdc-select--activated .mdc-select__dropdown-icon .mdc-select__dropdown-icon-active{opacity:1;transition:opacity 100.5ms linear 49.5ms}.mdc-select__anchor{width:200px;min-width:0;flex:1 1 auto;position:relative;box-sizing:border-box;overflow:hidden;outline:none;cursor:pointer}.mdc-select__anchor .mdc-floating-label--float-above{transform:translateY(-106%) scale(0.75)}.mdc-select__selected-text-container{display:flex;appearance:none;pointer-events:none;box-sizing:border-box;width:auto;min-width:0;flex-grow:1;height:28px;border:none;outline:none;padding:0;background-color:transparent;color:inherit}.mdc-select__selected-text{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-subtitle1-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:1rem;font-size:var(--mdc-typography-subtitle1-font-size, 1rem);line-height:1.75rem;line-height:var(--mdc-typography-subtitle1-line-height, 1.75rem);font-weight:400;font-weight:var(--mdc-typography-subtitle1-font-weight, 400);letter-spacing:0.009375em;letter-spacing:var(--mdc-typography-subtitle1-letter-spacing, 0.009375em);text-decoration:inherit;text-decoration:var(--mdc-typography-subtitle1-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-subtitle1-text-transform, inherit);text-overflow:ellipsis;white-space:nowrap;overflow:hidden;display:block;width:100%;text-align:left}[dir=rtl] .mdc-select__selected-text,.mdc-select__selected-text[dir=rtl]{text-align:right;}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-floating-label{color:#b00020;color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-floating-label{color:#b00020;color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--invalid+.mdc-select-helper-text--validation-msg{color:#b00020;color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-select__dropdown-icon{fill:#b00020;fill:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-select__dropdown-icon{fill:#b00020;fill:var(--mdc-theme-error, #b00020)}.mdc-select--disabled{cursor:default;pointer-events:none}.mdc-select--with-leading-icon .mdc-select__menu .mdc-deprecated-list-item{padding-left:12px;padding-right:12px}[dir=rtl] .mdc-select--with-leading-icon .mdc-select__menu .mdc-deprecated-list-item,.mdc-select--with-leading-icon .mdc-select__menu .mdc-deprecated-list-item[dir=rtl]{padding-left:12px;padding-right:12px;}@media screen and (forced-colors: active), (-ms-high-contrast: active){.mdc-select__menu::before{position:absolute;box-sizing:border-box;width:100%;height:100%;top:0;left:0;border:1px solid transparent;border-radius:inherit;content:\"\";pointer-events:none}}@media screen and (forced-colors: active) and (forced-colors: active), screen and (-ms-high-contrast: active) and (forced-colors: active){.mdc-select__menu::before{border-color:CanvasText}}.mdc-select__menu .mdc-deprecated-list .mdc-select__icon,.mdc-select__menu .mdc-list .mdc-select__icon{margin-left:0;margin-right:0}[dir=rtl] .mdc-select__menu .mdc-deprecated-list .mdc-select__icon,[dir=rtl] .mdc-select__menu .mdc-list .mdc-select__icon,.mdc-select__menu .mdc-deprecated-list .mdc-select__icon[dir=rtl],.mdc-select__menu .mdc-list .mdc-select__icon[dir=rtl]{margin-left:0;margin-right:0;}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--activated,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--selected,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--activated{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__graphic,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--activated .mdc-deprecated-list-item__graphic,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__graphic,.mdc-select__menu .mdc-list .mdc-deprecated-list-item--activated .mdc-deprecated-list-item__graphic{color:#000;color:var(--mdc-theme-on-surface, #000)}.mdc-select__menu .mdc-list-item__start{display:inline-flex;align-items:center}.mdc-select__option{padding-left:16px;padding-right:16px}[dir=rtl] .mdc-select__option,.mdc-select__option[dir=rtl]{padding-left:16px;padding-right:16px;}.mdc-select__one-line-option.mdc-list-item--with-one-line{height:48px}.mdc-select__two-line-option.mdc-list-item--with-two-lines{height:64px}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__start{margin-top:20px}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__primary-text{display:block;margin-top:0;line-height:normal;margin-bottom:-20px}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__primary-text::before{display:inline-block;width:0;height:28px;content:\"\";vertical-align:0}.mdc-select__two-line-option.mdc-list-item--with-two-lines .mdc-list-item__primary-text::after{display:inline-block;width:0;height:20px;content:\"\";vertical-align:-20px}.mdc-select__two-line-option.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end{display:block;margin-top:0;line-height:normal}.mdc-select__two-line-option.mdc-list-item--with-two-lines.mdc-list-item--with-trailing-meta .mdc-list-item__end::before{display:inline-block;width:0;height:36px;content:\"\";vertical-align:0}.mdc-select__option-with-leading-content{padding-left:0;padding-right:12px}.mdc-select__option-with-leading-content.mdc-list-item{padding-left:0;padding-right:auto}[dir=rtl] .mdc-select__option-with-leading-content.mdc-list-item,.mdc-select__option-with-leading-content.mdc-list-item[dir=rtl]{padding-left:auto;padding-right:0;}.mdc-select__option-with-leading-content .mdc-list-item__start{margin-left:12px;margin-right:0}[dir=rtl] .mdc-select__option-with-leading-content .mdc-list-item__start,.mdc-select__option-with-leading-content .mdc-list-item__start[dir=rtl]{margin-left:0;margin-right:12px;}.mdc-select__option-with-leading-content .mdc-list-item__start{width:36px;height:24px}[dir=rtl] .mdc-select__option-with-leading-content,.mdc-select__option-with-leading-content[dir=rtl]{padding-left:12px;padding-right:0;}.mdc-select__option-with-meta.mdc-list-item{padding-left:auto;padding-right:0}[dir=rtl] .mdc-select__option-with-meta.mdc-list-item,.mdc-select__option-with-meta.mdc-list-item[dir=rtl]{padding-left:0;padding-right:auto;}.mdc-select__option-with-meta .mdc-list-item__end{margin-left:12px;margin-right:12px}[dir=rtl] .mdc-select__option-with-meta .mdc-list-item__end,.mdc-select__option-with-meta .mdc-list-item__end[dir=rtl]{margin-left:12px;margin-right:12px;}.mdc-select--filled .mdc-select__anchor{height:56px;display:flex;align-items:baseline}.mdc-select--filled .mdc-select__anchor::before{display:inline-block;width:0;height:40px;content:\"\";vertical-align:0}.mdc-select--filled.mdc-select--no-label .mdc-select__anchor .mdc-select__selected-text::before{content:\"​\"}.mdc-select--filled.mdc-select--no-label .mdc-select__anchor .mdc-select__selected-text-container{height:100%;display:inline-flex;align-items:center}.mdc-select--filled.mdc-select--no-label .mdc-select__anchor::before{display:none}.mdc-select--filled .mdc-select__anchor{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:0;border-bottom-left-radius:0}.mdc-select--filled:not(.mdc-select--disabled) .mdc-select__anchor{background-color:whitesmoke}.mdc-select--filled.mdc-select--disabled .mdc-select__anchor{background-color:#fafafa}.mdc-select--filled:not(.mdc-select--disabled) .mdc-line-ripple::before{border-bottom-color:rgba(0, 0, 0, 0.42)}.mdc-select--filled:not(.mdc-select--disabled):hover .mdc-line-ripple::before{border-bottom-color:rgba(0, 0, 0, 0.87)}.mdc-select--filled:not(.mdc-select--disabled) .mdc-line-ripple::after{border-bottom-color:#6200ee;border-bottom-color:var(--mdc-theme-primary, #6200ee)}.mdc-select--filled.mdc-select--disabled .mdc-line-ripple::before{border-bottom-color:rgba(0, 0, 0, 0.06)}.mdc-select--filled .mdc-floating-label{max-width:calc(100% - 64px)}.mdc-select--filled .mdc-floating-label--float-above{max-width:calc(100% / 0.75 - 64px / 0.75)}.mdc-select--filled .mdc-menu-surface--is-open-below{border-top-left-radius:0px;border-top-right-radius:0px}.mdc-select--filled.mdc-select--focused.mdc-line-ripple::after{transform:scale(1, 2);opacity:1}.mdc-select--filled .mdc-floating-label{left:16px;right:initial}[dir=rtl] .mdc-select--filled .mdc-floating-label,.mdc-select--filled .mdc-floating-label[dir=rtl]{left:initial;right:16px;}.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label{left:48px;right:initial}[dir=rtl] .mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label,.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label[dir=rtl]{left:initial;right:48px;}.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label{max-width:calc(100% - 96px)}.mdc-select--filled.mdc-select--with-leading-icon .mdc-floating-label--float-above{max-width:calc(100% / 0.75 - 96px / 0.75)}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-line-ripple::before{border-bottom-color:#b00020;border-bottom-color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled):hover .mdc-line-ripple::before{border-bottom-color:#b00020;border-bottom-color:var(--mdc-theme-error, #b00020)}.mdc-select--invalid:not(.mdc-select--disabled) .mdc-line-ripple::after{border-bottom-color:#b00020;border-bottom-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined{border:none}.mdc-select--outlined .mdc-select__anchor{height:56px}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{transform:translateY(-37.25px) scale(1)}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{font-size:0.75rem}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) scale(0.75)}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined-56px 250ms 1}@keyframes mdc-floating-label-shake-float-above-select-outlined-56px{0%{transform:translateX(calc(0 - 0%)) translateY(-34.75px) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 0%)) translateY(-34.75px) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 0%)) translateY(-34.75px) scale(0.75)}100%{transform:translateX(calc(0 - 0%)) translateY(-34.75px) scale(0.75)}}.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px)}[dir=rtl] .mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading[dir=rtl]{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0;}@supports (top: max(0%)){.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__leading{width:max(12px, var(--mdc-shape-small, 4px))}}@supports (top: max(0%)){.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__notch{max-width:calc(100% - max(12px, var(--mdc-shape-small, 4px)) * 2)}}.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__trailing{border-top-left-radius:0;border-top-right-radius:4px;border-top-right-radius:var(--mdc-shape-small, 4px);border-bottom-right-radius:4px;border-bottom-right-radius:var(--mdc-shape-small, 4px);border-bottom-left-radius:0}[dir=rtl] .mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__trailing,.mdc-select--outlined .mdc-notched-outline .mdc-notched-outline__trailing[dir=rtl]{border-top-left-radius:4px;border-top-left-radius:var(--mdc-shape-small, 4px);border-top-right-radius:0;border-bottom-right-radius:0;border-bottom-left-radius:4px;border-bottom-left-radius:var(--mdc-shape-small, 4px);}@supports (top: max(0%)){.mdc-select--outlined .mdc-select__anchor{padding-left:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}[dir=rtl] .mdc-select--outlined .mdc-select__anchor,.mdc-select--outlined .mdc-select__anchor[dir=rtl]{padding-left:0;}@supports (top: max(0%)){[dir=rtl] .mdc-select--outlined .mdc-select__anchor,.mdc-select--outlined .mdc-select__anchor[dir=rtl]{padding-right:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}@supports (top: max(0%)){.mdc-select--outlined+.mdc-select-helper-text{margin-left:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}[dir=rtl] .mdc-select--outlined+.mdc-select-helper-text,.mdc-select--outlined+.mdc-select-helper-text[dir=rtl]{margin-left:0;}@supports (top: max(0%)){[dir=rtl] .mdc-select--outlined+.mdc-select-helper-text,.mdc-select--outlined+.mdc-select-helper-text[dir=rtl]{margin-right:max(16px, calc(var(--mdc-shape-small, 4px) + 4px))}}.mdc-select--outlined:not(.mdc-select--disabled) .mdc-select__anchor{background-color:transparent}.mdc-select--outlined.mdc-select--disabled .mdc-select__anchor{background-color:transparent}.mdc-select--outlined:not(.mdc-select--disabled) .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled) .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled) .mdc-notched-outline__trailing{border-color:rgba(0, 0, 0, 0.38)}.mdc-select--outlined:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:rgba(0, 0, 0, 0.87)}.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-width:2px}.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-color:#6200ee;border-color:var(--mdc-theme-primary, #6200ee)}.mdc-select--outlined.mdc-select--disabled .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--disabled .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--disabled .mdc-notched-outline__trailing{border-color:rgba(0, 0, 0, 0.06)}.mdc-select--outlined .mdc-select__anchor :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 60px)}.mdc-select--outlined .mdc-select__anchor{display:flex;align-items:baseline;overflow:visible}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined 250ms 1}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{transform:translateY(-37.25px) scale(1)}.mdc-select--outlined .mdc-select__anchor .mdc-floating-label--float-above{font-size:0.75rem}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) scale(0.75)}.mdc-select--outlined .mdc-select__anchor.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-select--outlined .mdc-select__anchor .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:1px}.mdc-select--outlined .mdc-select__anchor .mdc-select__selected-text::before{content:\"​\"}.mdc-select--outlined .mdc-select__anchor .mdc-select__selected-text-container{height:100%;display:inline-flex;align-items:center}.mdc-select--outlined .mdc-select__anchor::before{display:none}.mdc-select--outlined .mdc-select__selected-text-container{display:flex;border:none;z-index:1;background-color:transparent}.mdc-select--outlined .mdc-select__icon{z-index:2}.mdc-select--outlined .mdc-floating-label{line-height:1.15rem;left:4px;right:initial}[dir=rtl] .mdc-select--outlined .mdc-floating-label,.mdc-select--outlined .mdc-floating-label[dir=rtl]{left:initial;right:4px;}.mdc-select--outlined.mdc-select--focused .mdc-notched-outline--notched .mdc-notched-outline__notch{padding-top:2px}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled) .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled) .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled) .mdc-notched-outline__trailing{border-color:#b00020;border-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled):not(.mdc-select--focused) .mdc-select__anchor:hover .mdc-notched-outline .mdc-notched-outline__trailing{border-color:#b00020;border-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-width:2px}.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__leading,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__notch,.mdc-select--outlined.mdc-select--invalid:not(.mdc-select--disabled).mdc-select--focused .mdc-notched-outline .mdc-notched-outline__trailing{border-color:#b00020;border-color:var(--mdc-theme-error, #b00020)}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label{left:36px;right:initial}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label[dir=rtl]{left:initial;right:36px;}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above{transform:translateY(-37.25px) translateX(-32px) scale(1)}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-37.25px) translateX(32px) scale(1);}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--float-above{font-size:0.75rem}.mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above{transform:translateY(-34.75px) translateX(-32px) scale(0.75)}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above,[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl],.mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above[dir=rtl]{transform:translateY(-34.75px) translateX(32px) scale(0.75);}.mdc-select--outlined.mdc-select--with-leading-icon.mdc-notched-outline--upgraded .mdc-floating-label--float-above,.mdc-select--outlined.mdc-select--with-leading-icon .mdc-notched-outline--upgraded .mdc-floating-label--float-above{font-size:1rem}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px 250ms 1}@keyframes mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px{0%{transform:translateX(calc(0 - 32px)) translateY(-34.75px) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - 32px)) translateY(-34.75px) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - 32px)) translateY(-34.75px) scale(0.75)}100%{transform:translateX(calc(0 - 32px)) translateY(-34.75px) scale(0.75)}}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon,.mdc-select--outlined.mdc-select--with-leading-icon[dir=rtl]{}[dir=rtl] .mdc-select--outlined.mdc-select--with-leading-icon .mdc-floating-label--shake,.mdc-select--outlined.mdc-select--with-leading-icon[dir=rtl] .mdc-floating-label--shake{animation:mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px 250ms 1}@keyframes mdc-floating-label-shake-float-above-select-outlined-leading-icon-56px-rtl{0%{transform:translateX(calc(0 - -32px)) translateY(-34.75px) scale(0.75)}33%{animation-timing-function:cubic-bezier(0.5, 0, 0.701732, 0.495819);transform:translateX(calc(4% - -32px)) translateY(-34.75px) scale(0.75)}66%{animation-timing-function:cubic-bezier(0.302435, 0.381352, 0.55, 0.956352);transform:translateX(calc(-4% - -32px)) translateY(-34.75px) scale(0.75)}100%{transform:translateX(calc(0 - -32px)) translateY(-34.75px) scale(0.75)}}.mdc-select--outlined.mdc-select--with-leading-icon .mdc-select__anchor :not(.mdc-notched-outline--notched) .mdc-notched-outline__notch{max-width:calc(100% - 96px)}.mdc-select--outlined .mdc-menu-surface{margin-bottom:8px}.mdc-select--outlined.mdc-select--no-label .mdc-menu-surface,.mdc-select--outlined .mdc-menu-surface--is-open-below{margin-bottom:0}.mdc-select__anchor{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}.mdc-select__anchor .mdc-select__ripple::before,.mdc-select__anchor .mdc-select__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}.mdc-select__anchor .mdc-select__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}.mdc-select__anchor .mdc-select__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}.mdc-select__anchor.mdc-ripple-upgraded .mdc-select__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}.mdc-select__anchor.mdc-ripple-upgraded .mdc-select__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}.mdc-select__anchor.mdc-ripple-upgraded--unbounded .mdc-select__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}.mdc-select__anchor.mdc-ripple-upgraded--foreground-activation .mdc-select__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}.mdc-select__anchor.mdc-ripple-upgraded--foreground-deactivation .mdc-select__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}.mdc-select__anchor .mdc-select__ripple::before,.mdc-select__anchor .mdc-select__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}.mdc-select__anchor.mdc-ripple-upgraded .mdc-select__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}.mdc-select__anchor .mdc-select__ripple::before,.mdc-select__anchor .mdc-select__ripple::after{background-color:rgba(0, 0, 0, 0.87);background-color:var(--mdc-ripple-color, rgba(0, 0, 0, 0.87))}.mdc-select__anchor:hover .mdc-select__ripple::before,.mdc-select__anchor.mdc-ripple-surface--hover .mdc-select__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}.mdc-select__anchor.mdc-ripple-upgraded--background-focused .mdc-select__ripple::before,.mdc-select__anchor:not(.mdc-ripple-upgraded):focus .mdc-select__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-select__anchor .mdc-select__ripple{position:absolute;top:0;left:0;width:100%;height:100%;pointer-events:none}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-deprecated-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, var(--mdc-theme-on-surface, #000))}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:hover .mdc-deprecated-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-surface--hover .mdc-deprecated-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-deprecated-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-deprecated-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded) .mdc-deprecated-list-item__ripple::after{transition:opacity 150ms linear}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-deprecated-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected .mdc-list-item__ripple::after{background-color:#000;background-color:var(--mdc-ripple-color, var(--mdc-theme-on-surface, #000))}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:hover .mdc-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-surface--hover .mdc-list-item__ripple::before{opacity:0.04;opacity:var(--mdc-ripple-hover-opacity, 0.04)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded--background-focused .mdc-list-item__ripple::before,.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):focus .mdc-list-item__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-focus-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded) .mdc-list-item__ripple::after{transition:opacity 150ms linear}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected:not(.mdc-ripple-upgraded):active .mdc-list-item__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select__menu .mdc-deprecated-list .mdc-deprecated-list-item--selected.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-ripple-press-opacity, 0.12)}.mdc-select-helper-text{margin:0;margin-left:16px;margin-right:16px;-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-caption-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));font-size:0.75rem;font-size:var(--mdc-typography-caption-font-size, 0.75rem);line-height:1.25rem;line-height:var(--mdc-typography-caption-line-height, 1.25rem);font-weight:400;font-weight:var(--mdc-typography-caption-font-weight, 400);letter-spacing:0.0333333333em;letter-spacing:var(--mdc-typography-caption-letter-spacing, 0.0333333333em);text-decoration:inherit;text-decoration:var(--mdc-typography-caption-text-decoration, inherit);text-transform:inherit;text-transform:var(--mdc-typography-caption-text-transform, inherit);display:block;margin-top:0;line-height:normal}[dir=rtl] .mdc-select-helper-text,.mdc-select-helper-text[dir=rtl]{margin-left:16px;margin-right:16px;}.mdc-select-helper-text::before{display:inline-block;width:0;height:16px;content:\"\";vertical-align:0}.mdc-select-helper-text--validation-msg{opacity:0;transition:opacity 180ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-select--invalid+.mdc-select-helper-text--validation-msg,.mdc-select-helper-text--validation-msg-persistent{opacity:1}.mdc-select--with-leading-icon .mdc-select__icon{display:inline-block;box-sizing:border-box;border:none;text-decoration:none;cursor:pointer;user-select:none;flex-shrink:0;align-self:center;background-color:transparent;fill:currentColor}.mdc-select--with-leading-icon .mdc-select__icon{margin-left:12px;margin-right:12px}[dir=rtl] .mdc-select--with-leading-icon .mdc-select__icon,.mdc-select--with-leading-icon .mdc-select__icon[dir=rtl]{margin-left:12px;margin-right:12px;}.mdc-select__icon:not([tabindex]),.mdc-select__icon[tabindex=\"-1\"]{cursor:default;pointer-events:none}.mdc-touch-target-wrapper{display:inline}.mdc-button{position:relative;display:inline-flex;align-items:center;justify-content:center;box-sizing:border-box;min-width:64px;border:none;outline:none;line-height:inherit;user-select:none;-webkit-appearance:none;overflow:visible;vertical-align:middle;background:transparent}.mdc-button .mdc-elevation-overlay{width:100%;height:100%;top:0;left:0}.mdc-button::-moz-focus-inner{padding:0;border:0}.mdc-button:active{outline:none}.mdc-button:hover{cursor:pointer}.mdc-button:disabled{cursor:default;pointer-events:none}.mdc-button .mdc-button__icon{margin-left:0;margin-right:8px;display:inline-block;position:relative;vertical-align:top}[dir=rtl] .mdc-button .mdc-button__icon,.mdc-button .mdc-button__icon[dir=rtl]{margin-left:8px;margin-right:0;}.mdc-button .mdc-button__label{position:relative}.mdc-button .mdc-button__focus-ring{display:none}@media screen and (forced-colors: active){.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__focus-ring,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__focus-ring{pointer-events:none;border:2px solid transparent;border-radius:6px;box-sizing:content-box;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(\n      100% + 4px\n    );width:calc(\n      100% + 4px\n    );display:block}}@media screen and (forced-colors: active) and (forced-colors: active){.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__focus-ring,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__focus-ring{border-color:CanvasText}}@media screen and (forced-colors: active){.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__focus-ring::after,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__focus-ring::after{content:\"\";border:2px solid transparent;border-radius:8px;display:block;position:absolute;top:50%;left:50%;transform:translate(-50%, -50%);height:calc(100% + 4px);width:calc(100% + 4px)}}@media screen and (forced-colors: active) and (forced-colors: active){.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__focus-ring::after,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__focus-ring::after{border-color:CanvasText}}.mdc-button .mdc-button__touch{position:absolute;top:50%;height:48px;left:0;right:0;transform:translateY(-50%)}.mdc-button__label+.mdc-button__icon{margin-left:8px;margin-right:0}[dir=rtl] .mdc-button__label+.mdc-button__icon,.mdc-button__label+.mdc-button__icon[dir=rtl]{margin-left:0;margin-right:8px;}svg.mdc-button__icon{fill:currentColor}.mdc-button--touch{margin-top:6px;margin-bottom:6px}.mdc-button{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;font-family:Roboto, sans-serif;font-family:var(--mdc-typography-button-font-family, var(--mdc-typography-font-family, Roboto, sans-serif));text-decoration:none;text-decoration:var(--mdc-typography-button-text-decoration, none)}.mdc-button{padding:0 8px 0 8px}.mdc-button--unelevated{transition:box-shadow 280ms cubic-bezier(0.4, 0, 0.2, 1);padding:0 16px 0 16px}.mdc-button--unelevated.mdc-button--icon-trailing{padding:0 12px 0 16px}.mdc-button--unelevated.mdc-button--icon-leading{padding:0 16px 0 12px}.mdc-button--raised{transition:box-shadow 280ms cubic-bezier(0.4, 0, 0.2, 1);padding:0 16px 0 16px}.mdc-button--raised.mdc-button--icon-trailing{padding:0 12px 0 16px}.mdc-button--raised.mdc-button--icon-leading{padding:0 16px 0 12px}.mdc-button--outlined{border-style:solid;transition:border 280ms cubic-bezier(0.4, 0, 0.2, 1)}.mdc-button--outlined .mdc-button__ripple{border-style:solid;border-color:transparent}.mdc-button{--mdc-ripple-fg-size:0;--mdc-ripple-left:0;--mdc-ripple-top:0;--mdc-ripple-fg-scale:1;--mdc-ripple-fg-translate-end:0;--mdc-ripple-fg-translate-start:0;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);will-change:transform, opacity}.mdc-button .mdc-button__ripple::before,.mdc-button .mdc-button__ripple::after{position:absolute;border-radius:50%;opacity:0;pointer-events:none;content:\"\"}.mdc-button .mdc-button__ripple::before{transition:opacity 15ms linear, background-color 15ms linear;z-index:1;z-index:var(--mdc-ripple-z-index, 1)}.mdc-button .mdc-button__ripple::after{z-index:0;z-index:var(--mdc-ripple-z-index, 0)}.mdc-button.mdc-ripple-upgraded .mdc-button__ripple::before{transform:scale(var(--mdc-ripple-fg-scale, 1))}.mdc-button.mdc-ripple-upgraded .mdc-button__ripple::after{top:0;left:0;transform:scale(0);transform-origin:center center}.mdc-button.mdc-ripple-upgraded--unbounded .mdc-button__ripple::after{top:var(--mdc-ripple-top, 0);left:var(--mdc-ripple-left, 0)}.mdc-button.mdc-ripple-upgraded--foreground-activation .mdc-button__ripple::after{animation:mdc-ripple-fg-radius-in 225ms forwards, mdc-ripple-fg-opacity-in 75ms forwards}.mdc-button.mdc-ripple-upgraded--foreground-deactivation .mdc-button__ripple::after{animation:mdc-ripple-fg-opacity-out 150ms;transform:translate(var(--mdc-ripple-fg-translate-end, 0)) scale(var(--mdc-ripple-fg-scale, 1))}.mdc-button .mdc-button__ripple::before,.mdc-button .mdc-button__ripple::after{top:calc(50% - 100%);left:calc(50% - 100%);width:200%;height:200%}.mdc-button.mdc-ripple-upgraded .mdc-button__ripple::after{width:var(--mdc-ripple-fg-size, 100%);height:var(--mdc-ripple-fg-size, 100%)}.mdc-button__ripple{position:absolute;box-sizing:content-box;overflow:hidden;z-index:0;top:0;left:0;bottom:0;right:0}.mdc-button{font-family:Roboto, sans-serif;font-family:var(--mdc-text-button-label-text-font, var(--mdc-typography-button-font-family, var(--mdc-typography-font-family, Roboto, sans-serif)));font-size:0.875rem;font-size:var(--mdc-text-button-label-text-size, var(--mdc-typography-button-font-size, 0.875rem));letter-spacing:0.0892857143em;letter-spacing:var(--mdc-text-button-label-text-tracking, var(--mdc-typography-button-letter-spacing, 0.0892857143em));font-weight:500;font-weight:var(--mdc-text-button-label-text-weight, var(--mdc-typography-button-font-weight, 500));text-transform:uppercase;text-transform:var(--mdc-text-button-label-text-transform, var(--mdc-typography-button-text-transform, uppercase));height:36px;height:var(--mdc-text-button-container-height, 36px);border-radius:4px;border-radius:var(--mdc-text-button-container-shape, var(--mdc-shape-small, 4px))}.mdc-button:not(:disabled){color:#6200ee;color:var(--mdc-text-button-label-text-color, var(--mdc-theme-primary, #6200ee))}.mdc-button:disabled{color:rgba(0, 0, 0, 0.38);color:var(--mdc-text-button-disabled-label-text-color, rgba(0, 0, 0, 0.38))}.mdc-button .mdc-button__icon{font-size:1.125rem;font-size:var(--mdc-text-button-with-icon-icon-size, 1.125rem);width:1.125rem;width:var(--mdc-text-button-with-icon-icon-size, 1.125rem);height:1.125rem;height:var(--mdc-text-button-with-icon-icon-size, 1.125rem)}.mdc-button .mdc-button__ripple::before,.mdc-button .mdc-button__ripple::after{background-color:#6200ee;background-color:var(--mdc-text-button-hover-state-layer-color, var(--mdc-theme-primary, #6200ee))}.mdc-button:hover .mdc-button__ripple::before,.mdc-button.mdc-ripple-surface--hover .mdc-button__ripple::before{opacity:0.04;opacity:var(--mdc-text-button-hover-state-layer-opacity, 0.04)}.mdc-button.mdc-ripple-upgraded--background-focused .mdc-button__ripple::before,.mdc-button:not(.mdc-ripple-upgraded):focus .mdc-button__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-text-button-focus-state-layer-opacity, 0.12)}.mdc-button:not(.mdc-ripple-upgraded) .mdc-button__ripple::after{transition:opacity 150ms linear}.mdc-button:not(.mdc-ripple-upgraded):active .mdc-button__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-text-button-pressed-state-layer-opacity, 0.12)}.mdc-button.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-text-button-pressed-state-layer-opacity, 0.12)}.mdc-button .mdc-button__ripple{border-radius:4px;border-radius:var(--mdc-text-button-container-shape, var(--mdc-shape-small, 4px))}.mdc-button--unelevated{font-family:Roboto, sans-serif;font-family:var(--mdc-filled-button-label-text-font, var(--mdc-typography-button-font-family, var(--mdc-typography-font-family, Roboto, sans-serif)));font-size:0.875rem;font-size:var(--mdc-filled-button-label-text-size, var(--mdc-typography-button-font-size, 0.875rem));letter-spacing:0.0892857143em;letter-spacing:var(--mdc-filled-button-label-text-tracking, var(--mdc-typography-button-letter-spacing, 0.0892857143em));font-weight:500;font-weight:var(--mdc-filled-button-label-text-weight, var(--mdc-typography-button-font-weight, 500));text-transform:uppercase;text-transform:var(--mdc-filled-button-label-text-transform, var(--mdc-typography-button-text-transform, uppercase));height:36px;height:var(--mdc-filled-button-container-height, 36px);border-radius:4px;border-radius:var(--mdc-filled-button-container-shape, var(--mdc-shape-small, 4px))}.mdc-button--unelevated:not(:disabled){background-color:#6200ee;background-color:var(--mdc-filled-button-container-color, var(--mdc-theme-primary, #6200ee))}.mdc-button--unelevated:disabled{background-color:rgba(0, 0, 0, 0.12);background-color:var(--mdc-filled-button-disabled-container-color, rgba(0, 0, 0, 0.12))}.mdc-button--unelevated:not(:disabled){color:#fff;color:var(--mdc-filled-button-label-text-color, var(--mdc-theme-on-primary, #fff))}.mdc-button--unelevated:disabled{color:rgba(0, 0, 0, 0.38);color:var(--mdc-filled-button-disabled-label-text-color, rgba(0, 0, 0, 0.38))}.mdc-button--unelevated .mdc-button__icon{font-size:1.125rem;font-size:var(--mdc-filled-button-with-icon-icon-size, 1.125rem);width:1.125rem;width:var(--mdc-filled-button-with-icon-icon-size, 1.125rem);height:1.125rem;height:var(--mdc-filled-button-with-icon-icon-size, 1.125rem)}.mdc-button--unelevated .mdc-button__ripple::before,.mdc-button--unelevated .mdc-button__ripple::after{background-color:#fff;background-color:var(--mdc-filled-button-hover-state-layer-color, var(--mdc-theme-on-primary, #fff))}.mdc-button--unelevated:hover .mdc-button__ripple::before,.mdc-button--unelevated.mdc-ripple-surface--hover .mdc-button__ripple::before{opacity:0.08;opacity:var(--mdc-filled-button-hover-state-layer-opacity, 0.08)}.mdc-button--unelevated.mdc-ripple-upgraded--background-focused .mdc-button__ripple::before,.mdc-button--unelevated:not(.mdc-ripple-upgraded):focus .mdc-button__ripple::before{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-filled-button-focus-state-layer-opacity, 0.24)}.mdc-button--unelevated:not(.mdc-ripple-upgraded) .mdc-button__ripple::after{transition:opacity 150ms linear}.mdc-button--unelevated:not(.mdc-ripple-upgraded):active .mdc-button__ripple::after{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-filled-button-pressed-state-layer-opacity, 0.24)}.mdc-button--unelevated.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-filled-button-pressed-state-layer-opacity, 0.24)}.mdc-button--unelevated .mdc-button__ripple{border-radius:4px;border-radius:var(--mdc-filled-button-container-shape, var(--mdc-shape-small, 4px))}.mdc-button--raised{font-family:Roboto, sans-serif;font-family:var(--mdc-protected-button-label-text-font, var(--mdc-typography-button-font-family, var(--mdc-typography-font-family, Roboto, sans-serif)));font-size:0.875rem;font-size:var(--mdc-protected-button-label-text-size, var(--mdc-typography-button-font-size, 0.875rem));letter-spacing:0.0892857143em;letter-spacing:var(--mdc-protected-button-label-text-tracking, var(--mdc-typography-button-letter-spacing, 0.0892857143em));font-weight:500;font-weight:var(--mdc-protected-button-label-text-weight, var(--mdc-typography-button-font-weight, 500));text-transform:uppercase;text-transform:var(--mdc-protected-button-label-text-transform, var(--mdc-typography-button-text-transform, uppercase));height:36px;height:var(--mdc-protected-button-container-height, 36px);border-radius:4px;border-radius:var(--mdc-protected-button-container-shape, var(--mdc-shape-small, 4px));--mdc-elevation-box-shadow-for-gss:0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12);box-shadow:0px 3px 1px -2px rgba(0, 0, 0, 0.2), 0px 2px 2px 0px rgba(0, 0, 0, 0.14), 0px 1px 5px 0px rgba(0, 0, 0, 0.12);box-shadow:var(--mdc-protected-button-container-elevation, var(--mdc-elevation-box-shadow-for-gss))}.mdc-button--raised:not(:disabled){background-color:#6200ee;background-color:var(--mdc-protected-button-container-color, var(--mdc-theme-primary, #6200ee))}.mdc-button--raised:disabled{background-color:rgba(0, 0, 0, 0.12);background-color:var(--mdc-protected-button-disabled-container-color, rgba(0, 0, 0, 0.12))}.mdc-button--raised:not(:disabled){color:#fff;color:var(--mdc-protected-button-label-text-color, var(--mdc-theme-on-primary, #fff))}.mdc-button--raised:disabled{color:rgba(0, 0, 0, 0.38);color:var(--mdc-protected-button-disabled-label-text-color, rgba(0, 0, 0, 0.38))}.mdc-button--raised .mdc-button__icon{font-size:1.125rem;font-size:var(--mdc-protected-button-with-icon-icon-size, 1.125rem);width:1.125rem;width:var(--mdc-protected-button-with-icon-icon-size, 1.125rem);height:1.125rem;height:var(--mdc-protected-button-with-icon-icon-size, 1.125rem)}.mdc-button--raised .mdc-button__ripple::before,.mdc-button--raised .mdc-button__ripple::after{background-color:#fff;background-color:var(--mdc-protected-button-hover-state-layer-color, var(--mdc-theme-on-primary, #fff))}.mdc-button--raised:hover .mdc-button__ripple::before,.mdc-button--raised.mdc-ripple-surface--hover .mdc-button__ripple::before{opacity:0.08;opacity:var(--mdc-protected-button-hover-state-layer-opacity, 0.08)}.mdc-button--raised.mdc-ripple-upgraded--background-focused .mdc-button__ripple::before,.mdc-button--raised:not(.mdc-ripple-upgraded):focus .mdc-button__ripple::before{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-protected-button-focus-state-layer-opacity, 0.24)}.mdc-button--raised:not(.mdc-ripple-upgraded) .mdc-button__ripple::after{transition:opacity 150ms linear}.mdc-button--raised:not(.mdc-ripple-upgraded):active .mdc-button__ripple::after{transition-duration:75ms;opacity:0.24;opacity:var(--mdc-protected-button-pressed-state-layer-opacity, 0.24)}.mdc-button--raised.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-protected-button-pressed-state-layer-opacity, 0.24)}.mdc-button--raised .mdc-button__ripple{border-radius:4px;border-radius:var(--mdc-protected-button-container-shape, var(--mdc-shape-small, 4px))}.mdc-button--raised.mdc-ripple-upgraded--background-focused,.mdc-button--raised:not(.mdc-ripple-upgraded):focus{--mdc-elevation-box-shadow-for-gss:0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12);box-shadow:0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12);box-shadow:var(--mdc-protected-button-focus-container-elevation, var(--mdc-elevation-box-shadow-for-gss))}.mdc-button--raised:hover{--mdc-elevation-box-shadow-for-gss:0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12);box-shadow:0px 2px 4px -1px rgba(0, 0, 0, 0.2), 0px 4px 5px 0px rgba(0, 0, 0, 0.14), 0px 1px 10px 0px rgba(0, 0, 0, 0.12);box-shadow:var(--mdc-protected-button-hover-container-elevation, var(--mdc-elevation-box-shadow-for-gss))}.mdc-button--raised:not(:disabled):active{--mdc-elevation-box-shadow-for-gss:0px 5px 5px -3px rgba(0, 0, 0, 0.2), 0px 8px 10px 1px rgba(0, 0, 0, 0.14), 0px 3px 14px 2px rgba(0, 0, 0, 0.12);box-shadow:0px 5px 5px -3px rgba(0, 0, 0, 0.2), 0px 8px 10px 1px rgba(0, 0, 0, 0.14), 0px 3px 14px 2px rgba(0, 0, 0, 0.12);box-shadow:var(--mdc-protected-button-pressed-container-elevation, var(--mdc-elevation-box-shadow-for-gss))}.mdc-button--raised:disabled{--mdc-elevation-box-shadow-for-gss:0px 0px 0px 0px rgba(0, 0, 0, 0.2), 0px 0px 0px 0px rgba(0, 0, 0, 0.14), 0px 0px 0px 0px rgba(0, 0, 0, 0.12);box-shadow:0px 0px 0px 0px rgba(0, 0, 0, 0.2), 0px 0px 0px 0px rgba(0, 0, 0, 0.14), 0px 0px 0px 0px rgba(0, 0, 0, 0.12);box-shadow:var(--mdc-protected-button-disabled-container-elevation, var(--mdc-elevation-box-shadow-for-gss))}.mdc-button--outlined{font-family:Roboto, sans-serif;font-family:var(--mdc-outlined-button-label-text-font, var(--mdc-typography-button-font-family, var(--mdc-typography-font-family, Roboto, sans-serif)));font-size:0.875rem;font-size:var(--mdc-outlined-button-label-text-size, var(--mdc-typography-button-font-size, 0.875rem));letter-spacing:0.0892857143em;letter-spacing:var(--mdc-outlined-button-label-text-tracking, var(--mdc-typography-button-letter-spacing, 0.0892857143em));font-weight:500;font-weight:var(--mdc-outlined-button-label-text-weight, var(--mdc-typography-button-font-weight, 500));text-transform:uppercase;text-transform:var(--mdc-outlined-button-label-text-transform, var(--mdc-typography-button-text-transform, uppercase));height:36px;height:var(--mdc-outlined-button-container-height, 36px);border-radius:4px;border-radius:var(--mdc-outlined-button-container-shape, var(--mdc-shape-small, 4px));padding:0 15px 0 15px;border-width:1px;border-width:var(--mdc-outlined-button-outline-width, 1px)}.mdc-button--outlined:not(:disabled){color:#6200ee;color:var(--mdc-outlined-button-label-text-color, var(--mdc-theme-primary, #6200ee))}.mdc-button--outlined:disabled{color:rgba(0, 0, 0, 0.38);color:var(--mdc-outlined-button-disabled-label-text-color, rgba(0, 0, 0, 0.38))}.mdc-button--outlined .mdc-button__icon{font-size:1.125rem;font-size:var(--mdc-outlined-button-with-icon-icon-size, 1.125rem);width:1.125rem;width:var(--mdc-outlined-button-with-icon-icon-size, 1.125rem);height:1.125rem;height:var(--mdc-outlined-button-with-icon-icon-size, 1.125rem)}.mdc-button--outlined .mdc-button__ripple::before,.mdc-button--outlined .mdc-button__ripple::after{background-color:#6200ee;background-color:var(--mdc-outlined-button-hover-state-layer-color, var(--mdc-theme-primary, #6200ee))}.mdc-button--outlined:hover .mdc-button__ripple::before,.mdc-button--outlined.mdc-ripple-surface--hover .mdc-button__ripple::before{opacity:0.04;opacity:var(--mdc-outlined-button-hover-state-layer-opacity, 0.04)}.mdc-button--outlined.mdc-ripple-upgraded--background-focused .mdc-button__ripple::before,.mdc-button--outlined:not(.mdc-ripple-upgraded):focus .mdc-button__ripple::before{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-outlined-button-focus-state-layer-opacity, 0.12)}.mdc-button--outlined:not(.mdc-ripple-upgraded) .mdc-button__ripple::after{transition:opacity 150ms linear}.mdc-button--outlined:not(.mdc-ripple-upgraded):active .mdc-button__ripple::after{transition-duration:75ms;opacity:0.12;opacity:var(--mdc-outlined-button-pressed-state-layer-opacity, 0.12)}.mdc-button--outlined.mdc-ripple-upgraded{--mdc-ripple-fg-opacity:var(--mdc-outlined-button-pressed-state-layer-opacity, 0.12)}.mdc-button--outlined .mdc-button__ripple{border-radius:4px;border-radius:var(--mdc-outlined-button-container-shape, var(--mdc-shape-small, 4px))}.mdc-button--outlined:not(:disabled){border-color:rgba(0, 0, 0, 0.12);border-color:var(--mdc-outlined-button-outline-color, rgba(0, 0, 0, 0.12))}.mdc-button--outlined:disabled{border-color:rgba(0, 0, 0, 0.12);border-color:var(--mdc-outlined-button-disabled-outline-color, rgba(0, 0, 0, 0.12))}.mdc-button--outlined.mdc-button--icon-trailing{padding:0 11px 0 15px}.mdc-button--outlined.mdc-button--icon-leading{padding:0 15px 0 11px}.mdc-button--outlined .mdc-button__ripple{top:-1px;left:-1px;bottom:-1px;right:-1px;border-width:1px;border-width:var(--mdc-outlined-button-outline-width, 1px)}.mdc-button--outlined .mdc-button__touch{left:calc(-1 * 1px);left:calc(-1 * var(--mdc-outlined-button-outline-width, 1px));width:calc(100% + 2 * 1px);width:calc(100% + 2 * var(--mdc-outlined-button-outline-width, 1px))}.mdc-button--raised .mdc-button__icon,.mdc-button--unelevated .mdc-button__icon,.mdc-button--outlined .mdc-button__icon{margin-left:-4px;margin-right:8px}[dir=rtl] .mdc-button--raised .mdc-button__icon,[dir=rtl] .mdc-button--unelevated .mdc-button__icon,[dir=rtl] .mdc-button--outlined .mdc-button__icon,.mdc-button--raised .mdc-button__icon[dir=rtl],.mdc-button--unelevated .mdc-button__icon[dir=rtl],.mdc-button--outlined .mdc-button__icon[dir=rtl]{margin-left:8px;margin-right:-4px;}.mdc-button--raised .mdc-button__label+.mdc-button__icon,.mdc-button--unelevated .mdc-button__label+.mdc-button__icon,.mdc-button--outlined .mdc-button__label+.mdc-button__icon{margin-left:8px;margin-right:-4px}[dir=rtl] .mdc-button--raised .mdc-button__label+.mdc-button__icon,[dir=rtl] .mdc-button--unelevated .mdc-button__label+.mdc-button__icon,[dir=rtl] .mdc-button--outlined .mdc-button__label+.mdc-button__icon,.mdc-button--raised .mdc-button__label+.mdc-button__icon[dir=rtl],.mdc-button--unelevated .mdc-button__label+.mdc-button__icon[dir=rtl],.mdc-button--outlined .mdc-button__label+.mdc-button__icon[dir=rtl]{margin-left:-4px;margin-right:8px;}/*!\n *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */@font-face{font-family:var(--bci-global-font-family);font-weight:400;src:url(\"..//fonts/BoschSans-Regular.eot\");src:url(\"..//fonts/BoschSans-Regular.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Regular.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Regular.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-RegularItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-RegularItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-RegularItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Regular.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:300;src:url(\"..//fonts/BoschSans-Light.eot\");src:url(\"..//fonts/BoschSans-Light.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Light.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Light.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Light.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-LightItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-LightItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-LightItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Light.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:500;src:url(\"..//fonts/BoschSans-Medium.eot\");src:url(\"..//fonts/BoschSans-Medium.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Medium.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Medium.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Medium.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-MediumItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-MediumItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-MediumItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Medium.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:700;src:url(\"..//fonts/BoschSans-Bold.eot\");src:url(\"..//fonts/BoschSans-Bold.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Bold.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Bold.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BoldItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BoldItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BoldItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Bold.svg#svgFontName\") format(\"svg\");}@font-face{font-family:var(--bci-global-font-family);font-weight:900;src:url(\"..//fonts/BoschSans-Black.eot\");src:url(\"..//fonts/BoschSans-Black.eot?#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/BoschSans-Black.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-Black.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-Black.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-BlackItalic.woff2\") format(\"woff2\"), url(\"..//fonts/BoschSans-BlackItalic.woff\") format(\"woff\"), url(\"..//fonts/BoschSans-BlackItalic.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSans-Black.svg#svgFontName\") format(\"svg\");}@font-face{font-family:\"Bosch-Ic\";font-style:normal;font-stretch:normal;font-weight:normal;src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9\");src:url(\"..//fonts/Bosch-Icon.eot?mh5qa9#iefix\") format(\"embedded-opentype\"), url(\"..//fonts/Bosch-Icon.ttf?mh5qa9\") format(\"truetype\"), url(\"..//fonts/Bosch-Icon.woff?mh5qa9\") format(\"woff\"), url(\"..//fonts/Bosch-Icon.svg?mh5qa9#Bosch-Icon\") format(\"svg\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:300;src:url(\"..//fonts/BoschSansCond-Regular.otf\"), url(\"..//fonts/BoschSansCondensed-Regular.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-RegularItalic.ttf\") format(\"truetype\")}@font-face{font-family:\"Bosch-Sans-Condensed\";font-weight:700;src:url(\"..//fonts/BoschSansCond-Bold.otf\"), url(\"..//fonts/BoschSansCondensed-Bold.ttf\") format(\"truetype\"), url(\"..//fonts/BoschSansCondensed-BoldItalic.ttf\") format(\"truetype\")}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2018 Robert Bosch GmbH Copyright (C) 2018 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2020 Robert Bosch GmbH Copyright (C) 2020 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2022 Robert Bosch GmbH Copyright (C) 2022 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2023 Robert Bosch GmbH Copyright (C) 2023 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.flex-column{display:flex;flex-direction:column;box-sizing:border-box}.flex-row{display:flex;flex-direction:row;box-sizing:border-box}.flex-none{flex:0 0 auto}.flex-grow{flex:1 1 100%;height:100%}.flex-layout-start-strech{place-content:stretch flex-start;align-items:stretch}.flex-layout-center-center{place-content:center;align-items:center}.flex-fill{width:100%;box-sizing:border-box;max-width:100%}.flex-row-wrap{flex-flow:row wrap;box-sizing:border-box;display:flex}.flex-row-wrap.grow{flex:1 1 100%;max-width:100%}.flex-center{display:flex;justify-content:center;align-items:center}.flex-space-between{display:flex;justify-content:space-between;align-items:center}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */body{margin:0;display:flex;flex-direction:column}main,*{font-family:var(--bci-global-font-family)}::selection,::-moz-selection{background-color:#007bc0}a::-moz-selection{color:#ffffff}.lead{margin-bottom:24px;font-size:18px;font-weight:300;line-height:1.4}@media (min-width: 768px){.lead{font-size:24px}}/*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.bosch-ic,.Bosch-Ic{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic:before,.Bosch-Ic:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-large,.Bosch-Ic-Large{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-large:before,.Bosch-Ic-Large:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:28px;vertical-align:middle}.bosch-ic-medium,.Bosch-Ic-Medium{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-medium:before,.Bosch-Ic-Medium:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;vertical-align:middle}.bosch-ic-normal,.Bosch-Ic-Normal{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-normal:before,.Bosch-Ic-Normal:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:20px;vertical-align:middle}.bosch-ic-small,.Bosch-Ic-Small{line-height:1;vertical-align:middle;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.bosch-ic-small:before,.Bosch-Ic-Small:before{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:16px;vertical-align:middle}.Bosch-Ic-Disabled,.bosch-ic-disabled,.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic[disabled],.bosch-ic[disabled],.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover{cursor:not-allowed;color:#d0d4d8}.Bosch-Ic-Disabled:hover,.bosch-ic-disabled:hover,.Bosch-Ic-Disabled:hover:hover,.bosch-ic-disabled:hover:hover,.Bosch-Ic[disabled]:hover,.bosch-ic[disabled]:hover,.Bosch-Ic[disabled]:hover:hover,.bosch-ic[disabled]:hover:hover{color:#d0d4d8}.bosch-ic-clickable,.Bosch-Ic-Clickable{cursor:pointer}.bosch-ic-clickable:disabled,.bosch-ic-clickable.disabled,.Bosch-Ic-Clickable:disabled,.Bosch-Ic-Clickable.disabled{color:#a4abb3;cursor:not-allowed}.bosch-ic-activated,.Bosch-Ic-Activated{color:#007bc0}.bosch-ic-3d-print:before{content:\"\\e085\"}.bosch-ic-a-to-b:before{content:\"\\e086\"}.bosch-ic-add:before{content:\"\\e087\"}.bosch-ic-address-consumer-data:before{content:\"\\e088\"}.bosch-ic-adress-book:before{content:\"\\e089\"}.bosch-ic-agility:before{content:\"\\e08a\"}.bosch-ic-alarm-clock:before{content:\"\\e08b\"}.bosch-ic-ambulance:before{content:\"\\e08c\"}.bosch-ic-analysis:before{content:\"\\e08d\"}.bosch-ic-anchor:before{content:\"\\e08e\"}.bosch-ic-arrow-down:before{content:\"\\e08f\"}.bosch-ic-arrow-left:before{content:\"\\e090\"}.bosch-ic-arrow-left-down:before{content:\"\\e091\"}.bosch-ic-arrow-left-up:before{content:\"\\e092\"}.bosch-ic-arrow-right:before{content:\"\\e093\"}.bosch-ic-arrow-right-down:before{content:\"\\e094\"}.bosch-ic-arrow-right-up:before{content:\"\\e095\"}.bosch-ic-arrow-turn-left:before{content:\"\\e096\"}.bosch-ic-arrow-turn-right:before{content:\"\\e097\"}.bosch-ic-arrow-up:before{content:\"\\e098\"}.bosch-ic-arrows:before{content:\"\\e099\"}.bosch-ic-arrows-x-y-z:before{content:\"\\e09a\"}.bosch-ic-artificial-intelligence:before{content:\"\\e09b\"}.bosch-ic-assembly-line:before{content:\"\\e09c\"}.bosch-ic-atom:before{content:\"\\e09d\"}.bosch-ic-augmented-reality:before{content:\"\\e09e\"}.bosch-ic-autobod:before{content:\"\\e09f\"}.bosch-ic-back-left:before{content:\"\\e0a0\"}.bosch-ic-back-left-double:before{content:\"\\e0a1\"}.bosch-ic-bar-chart:before{content:\"\\e0a2\"}.bosch-ic-bar-chart-double:before{content:\"\\e0a3\"}.bosch-ic-battery-0:before{content:\"\\e0a4\"}.bosch-ic-battery-1:before{content:\"\\e0a5\"}.bosch-ic-battery-2:before{content:\"\\e0a6\"}.bosch-ic-battery-3:before{content:\"\\e0a7\"}.bosch-ic-battery-4:before{content:\"\\e0a8\"}.bosch-ic-battery-fail:before{content:\"\\e0a9\"}.bosch-ic-battery-li-ion:before{content:\"\\e0aa\"}.bosch-ic-battery-loading:before{content:\"\\e0ab\"}.bosch-ic-bed:before{content:\"\\e0ac\"}.bosch-ic-bicycle:before{content:\"\\e0ad\"}.bosch-ic-bicycle-e:before{content:\"\\e0ae\"}.bosch-ic-blog:before{content:\"\\e0af\"}.bosch-ic-bluetooth:before{content:\"\\e0b0\"}.bosch-ic-bluetooth-le:before{content:\"\\e0b1\"}.bosch-ic-board-speaker:before{content:\"\\e0b2\"}.bosch-ic-boiler-connected:before{content:\"\\e0b3\"}.bosch-ic-book:before{content:\"\\e0b4\"}.bosch-ic-bookmark:before{content:\"\\e0b5\"}.bosch-ic-bookmark-add:before{content:\"\\e0b6\"}.bosch-ic-bookmark-check:before{content:\"\\e0b7\"}.bosch-ic-bookmark-delete:before{content:\"\\e0b8\"}.bosch-ic-books:before{content:\"\\e0b9\"}.bosch-ic-bookshelf:before{content:\"\\e0ba\"}.bosch-ic-box:before{content:\"\\e0bb\"}.bosch-ic-box-closed:before{content:\"\\e0bc\"}.bosch-ic-box-delivery:before{content:\"\\e0bd\"}.bosch-ic-box-delivery-connected:before{content:\"\\e0be\"}.bosch-ic-brake-disk:before{content:\"\\e0bf\"}.bosch-ic-briefcase:before{content:\"\\e0c0\"}.bosch-ic-broom:before{content:\"\\e0c1\"}.bosch-ic-broom-cloud-dirt:before{content:\"\\e0c2\"}.bosch-ic-brush:before{content:\"\\e0c3\"}.bosch-ic-bug:before{content:\"\\e0c4\"}.bosch-ic-building:before{content:\"\\e0c5\"}.bosch-ic-cactus:before{content:\"\\e0c6\"}.bosch-ic-calculate:before{content:\"\\e0c7\"}.bosch-ic-calendar:before{content:\"\\e0c8\"}.bosch-ic-calendar-add:before{content:\"\\e0c9\"}.bosch-ic-calendar-settings:before{content:\"\\e0ca\"}.bosch-ic-call:before{content:\"\\e0cb\"}.bosch-ic-call-add:before{content:\"\\e0cc\"}.bosch-ic-call-remove:before{content:\"\\e0cd\"}.bosch-ic-call-sos:before{content:\"\\e0ce\"}.bosch-ic-call-team:before{content:\"\\e0cf\"}.bosch-ic-call-wifi:before{content:\"\\e0d0\"}.bosch-ic-camera:before{content:\"\\e0d1\"}.bosch-ic-camera-switch:before{content:\"\\e0d2\"}.bosch-ic-car:before{content:\"\\e0d3\"}.bosch-ic-car-clock:before{content:\"\\e0d4\"}.bosch-ic-car-connected:before{content:\"\\e0d5\"}.bosch-ic-car-locator:before{content:\"\\e0d6\"}.bosch-ic-car-mechanic:before{content:\"\\e0d7\"}.bosch-ic-car-rental:before{content:\"\\e0d8\"}.bosch-ic-car-seat-connected:before{content:\"\\e0d9\"}.bosch-ic-car-side:before{content:\"\\e0da\"}.bosch-ic-car-side-user:before{content:\"\\e0db\"}.bosch-ic-cart:before{content:\"\\e0dc\"}.bosch-ic-certificate:before{content:\"\\e0dd\"}.bosch-ic-certificate-pki:before{content:\"\\e0de\"}.bosch-ic-change:before{content:\"\\e0df\"}.bosch-ic-chart-bar:before{content:\"\\e0e0\"}.bosch-ic-chart-check:before{content:\"\\e0e1\"}.bosch-ic-chart-dummy:before{content:\"\\e0e2\"}.bosch-ic-chart-line:before{content:\"\\e0e3\"}.bosch-ic-chat:before{content:\"\\e0e4\"}.bosch-ic-chat-add:before{content:\"\\e0e5\"}.bosch-ic-chat-question-answer:before{content:\"\\e0e6\"}.bosch-ic-chat-remove:before{content:\"\\e0e7\"}.bosch-ic-checklist:before{content:\"\\e0e8\"}.bosch-ic-checkmark:before{content:\"\\e0e9\"}.bosch-ic-chip:before{content:\"\\e0ea\"}.bosch-ic-circle-segment:before{content:\"\\e0eb\"}.bosch-ic-clear-all:before{content:\"\\e0ec\"}.bosch-ic-clipboard:before{content:\"\\e0ed\"}.bosch-ic-clipboard-list:before{content:\"\\e0ee\"}.bosch-ic-clock:before{content:\"\\e0ef\"}.bosch-ic-close:before{content:\"\\e0f0\"}.bosch-ic-close-all:before{content:\"\\e0f1\"}.bosch-ic-cloud:before{content:\"\\e0f2\"}.bosch-ic-cloud-co2:before{content:\"\\e0f3\"}.bosch-ic-cloud-download:before{content:\"\\e0f4\"}.bosch-ic-cloud-nox:before{content:\"\\e0f5\"}.bosch-ic-cloud-upload:before{content:\"\\e0f6\"}.bosch-ic-club-arm-chair:before{content:\"\\e0f7\"}.bosch-ic-coffee-break:before{content:\"\\e0f8\"}.bosch-ic-coin-currency:before{content:\"\\e0f9\"}.bosch-ic-coin-dollar:before{content:\"\\e0fa\"}.bosch-ic-colorpicker:before{content:\"\\e0fb\"}.bosch-ic-colors:before{content:\"\\e0fc\"}.bosch-ic-command:before{content:\"\\e0fd\"}.bosch-ic-communicator:before{content:\"\\e0fe\"}.bosch-ic-compare:before{content:\"\\e0ff\"}.bosch-ic-components:before{content:\"\\e100\"}.bosch-ic-components-available:before{content:\"\\e101\"}.bosch-ic-components-reservation:before{content:\"\\e102\"}.bosch-ic-concierge-bell:before{content:\"\\e103\"}.bosch-ic-configuration:before{content:\"\\e104\"}.bosch-ic-confirmation:before{content:\"\\e105\"}.bosch-ic-connection-off:before{content:\"\\e106\"}.bosch-ic-connection-on:before{content:\"\\e107\"}.bosch-ic-connectivity:before{content:\"\\e108\"}.bosch-ic-construction:before{content:\"\\e109\"}.bosch-ic-consumer-goods:before{content:\"\\e10a\"}.bosch-ic-contract:before{content:\"\\e10b\"}.bosch-ic-copy:before{content:\"\\e10c\"}.bosch-ic-cube:before{content:\"\\e10d\"}.bosch-ic-cube-arrows:before{content:\"\\e10e\"}.bosch-ic-cube-filled:before{content:\"\\e10f\"}.bosch-ic-cube-shock:before{content:\"\\e110\"}.bosch-ic-cube-stacked:before{content:\"\\e111\"}.bosch-ic-customerservice:before{content:\"\\e112\"}.bosch-ic-cutlery:before{content:\"\\e113\"}.bosch-ic-damper:before{content:\"\\e114\"}.bosch-ic-danger-flash:before{content:\"\\e115\"}.bosch-ic-data-loss:before{content:\"\\e116\"}.bosch-ic-dealer-details:before{content:\"\\e117\"}.bosch-ic-delete:before{content:\"\\e118\"}.bosch-ic-delivery:before{content:\"\\e119\"}.bosch-ic-denied:before{content:\"\\e11a\"}.bosch-ic-desktop:before{content:\"\\e11b\"}.bosch-ic-desktop-application-2d:before{content:\"\\e11c\"}.bosch-ic-desktop-application-3d:before{content:\"\\e11d\"}.bosch-ic-development:before{content:\"\\e11e\"}.bosch-ic-devices:before{content:\"\\e11f\"}.bosch-ic-devices-mobile:before{content:\"\\e120\"}.bosch-ic-directory:before{content:\"\\e121\"}.bosch-ic-dna:before{content:\"\\e122\"}.bosch-ic-do-not-disturb:before{content:\"\\e123\"}.bosch-ic-document:before{content:\"\\e124\"}.bosch-ic-document-add:before{content:\"\\e125\"}.bosch-ic-document-arrow-down:before{content:\"\\e126\"}.bosch-ic-document-arrow-up:before{content:\"\\e127\"}.bosch-ic-document-audio:before{content:\"\\e128\"}.bosch-ic-document-check:before{content:\"\\e129\"}.bosch-ic-document-cloud:before{content:\"\\e12a\"}.bosch-ic-document-copy:before{content:\"\\e12b\"}.bosch-ic-document-delete:before{content:\"\\e12c\"}.bosch-ic-document-doc:before{content:\"\\e12d\"}.bosch-ic-document-edit:before{content:\"\\e12e\"}.bosch-ic-document-error:before{content:\"\\e12f\"}.bosch-ic-document-locked:before{content:\"\\e130\"}.bosch-ic-document-log:before{content:\"\\e131\"}.bosch-ic-document-one:before{content:\"\\e132\"}.bosch-ic-document-pdf:before{content:\"\\e133\"}.bosch-ic-document-plain:before{content:\"\\e134\"}.bosch-ic-document-plain-add:before{content:\"\\e135\"}.bosch-ic-document-plain-delete:before{content:\"\\e136\"}.bosch-ic-document-ppt:before{content:\"\\e137\"}.bosch-ic-document-pub:before{content:\"\\e138\"}.bosch-ic-document-rtf:before{content:\"\\e139\"}.bosch-ic-document-save-to:before{content:\"\\e13a\"}.bosch-ic-document-search:before{content:\"\\e13b\"}.bosch-ic-document-settings:before{content:\"\\e13c\"}.bosch-ic-document-test:before{content:\"\\e13d\"}.bosch-ic-document-text:before{content:\"\\e13e\"}.bosch-ic-document-txt:before{content:\"\\e13f\"}.bosch-ic-document-vsd:before{content:\"\\e140\"}.bosch-ic-document-xls:before{content:\"\\e141\"}.bosch-ic-document-xml:before{content:\"\\e142\"}.bosch-ic-document-zip:before{content:\"\\e143\"}.bosch-ic-dome:before{content:\"\\e144\"}.bosch-ic-door:before{content:\"\\e145\"}.bosch-ic-door-sensor:before{content:\"\\e146\"}.bosch-ic-down:before{content:\"\\e147\"}.bosch-ic-down-double:before{content:\"\\e148\"}.bosch-ic-download:before{content:\"\\e149\"}.bosch-ic-drag-handle:before{content:\"\\e14a\"}.bosch-ic-dragdrop:before{content:\"\\e14b\"}.bosch-ic-drop:before{content:\"\\e14c\"}.bosch-ic-e-charging:before{content:\"\\e14d\"}.bosch-ic-e-mobility:before{content:\"\\e14e\"}.bosch-ic-ear-off-disabled-light:before{content:\"\\e14f\"}.bosch-ic-ear-on:before{content:\"\\e150\"}.bosch-ic-eco-system:before{content:\"\\e151\"}.bosch-ic-edit:before{content:\"\\e152\"}.bosch-ic-education:before{content:\"\\e153\"}.bosch-ic-efficiency:before{content:\"\\e154\"}.bosch-ic-elevator:before{content:\"\\e155\"}.bosch-ic-elevator-alarm:before{content:\"\\e156\"}.bosch-ic-elevator-cloud:before{content:\"\\e157\"}.bosch-ic-elevator-headset:before{content:\"\\e158\"}.bosch-ic-elevator-service:before{content:\"\\e159\"}.bosch-ic-emergency-exit:before{content:\"\\e15a\"}.bosch-ic-emoji-happy:before{content:\"\\e15b\"}.bosch-ic-emoji-neutral:before{content:\"\\e15c\"}.bosch-ic-emoji-sad:before{content:\"\\e15d\"}.bosch-ic-emoji-super-happy:before{content:\"\\e15e\"}.bosch-ic-emoji-very-sad:before{content:\"\\e15f\"}.bosch-ic-eu-energy-label:before{content:\"\\e160\"}.bosch-ic-excavator:before{content:\"\\e161\"}.bosch-ic-exit:before{content:\"\\e163\"}.bosch-ic-expansion-arrows:before{content:\"\\e164\"}.bosch-ic-explosion:before{content:\"\\e165\"}.bosch-ic-export:before{content:\"\\e166\"}.bosch-ic-externallink:before{content:\"\\e167\"}.bosch-ic-fast-forward:before{content:\"\\e168\"}.bosch-ic-faucet:before{content:\"\\e169\"}.bosch-ic-favorites:before{content:\"\\e16a\"}.bosch-ic-fax:before{content:\"\\e16b\"}.bosch-ic-female:before{content:\"\\e16c\"}.bosch-ic-film:before{content:\"\\e16d\"}.bosch-ic-filter:before{content:\"\\e16e\"}.bosch-ic-filter-success:before{content:\"\\e16f\"}.bosch-ic-fingerprint:before{content:\"\\e170\"}.bosch-ic-fire:before{content:\"\\e171\"}.bosch-ic-fire-emergency:before{content:\"\\e172\"}.bosch-ic-fireworks:before{content:\"\\e173\"}.bosch-ic-first-aid:before{content:\"\\e174\"}.bosch-ic-first-aid-cross:before{content:\"\\e175\"}.bosch-ic-fit-to-sceen:before{content:\"\\e176\"}.bosch-ic-flag:before{content:\"\\e177\"}.bosch-ic-flash:before{content:\"\\e178\"}.bosch-ic-flask:before{content:\"\\e179\"}.bosch-ic-flexpress:before{content:\"\\e17a\"}.bosch-ic-folder:before{content:\"\\e17b\"}.bosch-ic-folder-download:before{content:\"\\e17c\"}.bosch-ic-folder-open:before{content:\"\\e17d\"}.bosch-ic-folder-upload:before{content:\"\\e17e\"}.bosch-ic-fork-lift:before{content:\"\\e17f\"}.bosch-ic-fork-lift-locator:before{content:\"\\e180\"}.bosch-ic-forward-right:before{content:\"\\e181\"}.bosch-ic-forward-right-double:before{content:\"\\e182\"}.bosch-ic-full-empty:before{content:\"\\e183\"}.bosch-ic-fullscreen:before{content:\"\\e184\"}.bosch-ic-fullscreen-arrows:before{content:\"\\e185\"}.bosch-ic-fullscreen-exit:before{content:\"\\e186\"}.bosch-ic-g-arrow-down:before{content:\"\\e187\"}.bosch-ic-g-arrow-up:before{content:\"\\e188\"}.bosch-ic-generator:before{content:\"\\e189\"}.bosch-ic-gift:before{content:\"\\e18a\"}.bosch-ic-glance:before{content:\"\\e18b\"}.bosch-ic-glas-plate:before{content:\"\\e18c\"}.bosch-ic-globe:before{content:\"\\e18d\"}.bosch-ic-globe-arrow:before{content:\"\\e18e\"}.bosch-ic-glossary:before{content:\"\\e18f\"}.bosch-ic-grid-view:before{content:\"\\e190\"}.bosch-ic-hammer:before{content:\"\\e191\"}.bosch-ic-hand:before{content:\"\\e192\"}.bosch-ic-hand-motion:before{content:\"\\e193\"}.bosch-ic-hand-motion-off-disabled-light:before{content:\"\\e194\"}.bosch-ic-handlewithcare:before{content:\"\\e195\"}.bosch-ic-handover-add:before{content:\"\\e196\"}.bosch-ic-handover-check:before{content:\"\\e197\"}.bosch-ic-handover-remove:before{content:\"\\e198\"}.bosch-ic-hanger:before{content:\"\\e199\"}.bosch-ic-health:before{content:\"\\e19a\"}.bosch-ic-heart:before{content:\"\\e19b\"}.bosch-ic-history:before{content:\"\\e19c\"}.bosch-ic-home:before{content:\"\\e19d\"}.bosch-ic-home-locator:before{content:\"\\e19e\"}.bosch-ic-hourglass:before{content:\"\\e19f\"}.bosch-ic-hydrant:before{content:\"\\e1a0\"}.bosch-ic-ice:before{content:\"\\e1a1\"}.bosch-ic-imagery:before{content:\"\\e1a2\"}.bosch-ic-imagery-add:before{content:\"\\e1a3\"}.bosch-ic-imagery-remove:before{content:\"\\e1a4\"}.bosch-ic-import:before{content:\"\\e1a5\"}.bosch-ic-imprint:before{content:\"\\e1a6\"}.bosch-ic-impulse:before{content:\"\\e1a7\"}.bosch-ic-industry:before{content:\"\\e1a8\"}.bosch-ic-industry-clock:before{content:\"\\e1a9\"}.bosch-ic-industry-connected:before{content:\"\\e1aa\"}.bosch-ic-info:before{content:\"\\e1ab\"}.bosch-ic-info-i:before{content:\"\\e1ac\"}.bosch-ic-interval:before{content:\"\\e1ad\"}.bosch-ic-it-device:before{content:\"\\e1ae\"}.bosch-ic-jewel:before{content:\"\\e1af\"}.bosch-ic-keyboard:before{content:\"\\e1b0\"}.bosch-ic-label:before{content:\"\\e1b1\"}.bosch-ic-laptop:before{content:\"\\e1b2\"}.bosch-ic-laser:before{content:\"\\e1b3\"}.bosch-ic-layout:before{content:\"\\e1b4\"}.bosch-ic-leaf:before{content:\"\\e1b5\"}.bosch-ic-led:before{content:\"\\e1b6\"}.bosch-ic-brick:before{content:\"\\e1b7\"}.bosch-ic-brick-slanting-1:before{content:\"\\e1b8\"}.bosch-ic-brick-slanting-2:before{content:\"\\e1b9\"}.bosch-ic-less-minimize:before{content:\"\\e1ba\"}.bosch-ic-lightbulb:before{content:\"\\e1bb\"}.bosch-ic-lightbulb-off:before{content:\"\\e1bc\"}.bosch-ic-line-chart:before{content:\"\\e1bd\"}.bosch-ic-link:before{content:\"\\e1be\"}.bosch-ic-link-broken:before{content:\"\\e1bf\"}.bosch-ic-list-view:before{content:\"\\e1c0\"}.bosch-ic-list-view-mobile:before{content:\"\\e1c1\"}.bosch-ic-livechat:before{content:\"\\e1c2\"}.bosch-ic-locator:before{content:\"\\e1c3\"}.bosch-ic-locator-ip-connected:before{content:\"\\e1c4\"}.bosch-ic-locator-ip-disconnected:before{content:\"\\e1c5\"}.bosch-ic-locator-spot:before{content:\"\\e1c6\"}.bosch-ic-locator-spot-check:before{content:\"\\e1c7\"}.bosch-ic-lock-closed:before{content:\"\\e1c8\"}.bosch-ic-lock-open:before{content:\"\\e1c9\"}.bosch-ic-login:before{content:\"\\e1ca\"}.bosch-ic-logistics:before{content:\"\\e1cb\"}.bosch-ic-logout:before{content:\"\\e1cc\"}.bosch-ic-machine:before{content:\"\\e1cd\"}.bosch-ic-magnet:before{content:\"\\e1ce\"}.bosch-ic-mail:before{content:\"\\e1cf\"}.bosch-ic-mail-forward:before{content:\"\\e1d0\"}.bosch-ic-mail-open:before{content:\"\\e1d1\"}.bosch-ic-mail-out:before{content:\"\\e1d2\"}.bosch-ic-mail-reply:before{content:\"\\e1d3\"}.bosch-ic-male:before{content:\"\\e1d4\"}.bosch-ic-map:before{content:\"\\e1d5\"}.bosch-ic-material-add:before{content:\"\\e1d6\"}.bosch-ic-material-check:before{content:\"\\e1d7\"}.bosch-ic-material-remove:before{content:\"\\e1d8\"}.bosch-ic-maximize:before{content:\"\\e1d9\"}.bosch-ic-mechanic:before{content:\"\\e1da\"}.bosch-ic-megaphone:before{content:\"\\e1db\"}.bosch-ic-message:before{content:\"\\e1dc\"}.bosch-ic-mic:before{content:\"\\e1dd\"}.bosch-ic-microphone:before{content:\"\\e1de\"}.bosch-ic-microphone-classic:before{content:\"\\e1df\"}.bosch-ic-minimize:before{content:\"\\e1e0\"}.bosch-ic-money-currency:before{content:\"\\e1e1\"}.bosch-ic-money-dollar:before{content:\"\\e1e2\"}.bosch-ic-moneybag-currency:before{content:\"\\e1e3\"}.bosch-ic-moneybag-dollar:before{content:\"\\e1e4\"}.bosch-ic-monument:before{content:\"\\e1e5\"}.bosch-ic-moon:before{content:\"\\e1e6\"}.bosch-ic-motorcycle:before{content:\"\\e1e7\"}.bosch-ic-motorcycle-side-car:before{content:\"\\e1e8\"}.bosch-ic-motorcycle-side:before{content:\"\\e1e9\"}.bosch-ic-mouse:before{content:\"\\e1ea\"}.bosch-ic-mouse-left:before{content:\"\\e1eb\"}.bosch-ic-mouse-right:before{content:\"\\e1ec\"}.bosch-ic-mouth:before{content:\"\\e1ed\"}.bosch-ic-my-product:before{content:\"\\e1ee\"}.bosch-ic-new-way-work:before{content:\"\\e1ef\"}.bosch-ic-newsletter:before{content:\"\\e1f0\"}.bosch-ic-newspaper:before{content:\"\\e1f1\"}.bosch-ic-nose:before{content:\"\\e1f2\"}.bosch-ic-notepad:before{content:\"\\e1f3\"}.bosch-ic-notepad-edit:before{content:\"\\e1f4\"}.bosch-ic-notification:before{content:\"\\e1f5\"}.bosch-ic-notification-active:before{content:\"\\e1f6\"}.bosch-ic-notification-add:before{content:\"\\e1f7\"}.bosch-ic-notification-remove:before{content:\"\\e1f8\"}.bosch-ic-oil-car:before{content:\"\\e1f9\"}.bosch-ic-omnichannel:before{content:\"\\e1fa\"}.bosch-ic-options:before{content:\"\\e1fb\"}.bosch-ic-origami-boat:before{content:\"\\e1fc\"}.bosch-ic-pallete-car:before{content:\"\\e1fd\"}.bosch-ic-pallete-car-connected:before{content:\"\\e1fe\"}.bosch-ic-panel:before{content:\"\\e1ff\"}.bosch-ic-paperclip:before{content:\"\\e200\"}.bosch-ic-paperplane:before{content:\"\\e201\"}.bosch-ic-parking:before{content:\"\\e202\"}.bosch-ic-pause:before{content:\"\\e203\"}.bosch-ic-payment:before{content:\"\\e204\"}.bosch-ic-people:before{content:\"\\e205\"}.bosch-ic-petrol-station:before{content:\"\\e206\"}.bosch-ic-piggybank:before{content:\"\\e207\"}.bosch-ic-pin-classic:before{content:\"\\e208\"}.bosch-ic-pin-modern:before{content:\"\\e209\"}.bosch-ic-plane-side:before{content:\"\\e20a\"}.bosch-ic-play:before{content:\"\\e20b\"}.bosch-ic-plug-12v:before{content:\"\\e20c\"}.bosch-ic-podium-speaker:before{content:\"\\e20d\"}.bosch-ic-police:before{content:\"\\e20e\"}.bosch-ic-poop:before{content:\"\\e20f\"}.bosch-ic-post-it:before{content:\"\\e210\"}.bosch-ic-postal-code:before{content:\"\\e211\"}.bosch-ic-power:before{content:\"\\e212\"}.bosch-ic-print:before{content:\"\\e213\"}.bosch-ic-prototyping:before{content:\"\\e214\"}.bosch-ic-puzzle:before{content:\"\\e215\"}.bosch-ic-quad:before{content:\"\\e216\"}.bosch-ic-question:before{content:\"\\e217\"}.bosch-ic-radar:before{content:\"\\e218\"}.bosch-ic-radio:before{content:\"\\e219\"}.bosch-ic-radiotower:before{content:\"\\e21a\"}.bosch-ic-redirect:before{content:\"\\e21b\"}.bosch-ic-redo:before{content:\"\\e21c\"}.bosch-ic-reference:before{content:\"\\e21d\"}.bosch-ic-refresh:before{content:\"\\e21e\"}.bosch-ic-refresh-cloud:before{content:\"\\e21f\"}.bosch-ic-registration:before{content:\"\\e220\"}.bosch-ic-remote:before{content:\"\\e221\"}.bosch-ic-reset:before{content:\"\\e222\"}.bosch-ic-resolution:before{content:\"\\e223\"}.bosch-ic-robot:before{content:\"\\e224\"}.bosch-ic-robot-connected:before{content:\"\\e225\"}.bosch-ic-robothead:before{content:\"\\e226\"}.bosch-ic-rocket:before{content:\"\\e227\"}.bosch-ic-rotation:before{content:\"\\e228\"}.bosch-ic-rotation-360:before{content:\"\\e229\"}.bosch-ic-rotation-x-left:before{content:\"\\e22a\"}.bosch-ic-rotation-x-right:before{content:\"\\e22b\"}.bosch-ic-rotation-y-down:before{content:\"\\e22c\"}.bosch-ic-rotation-y-up:before{content:\"\\e22d\"}.bosch-ic-route:before{content:\"\\e22e\"}.bosch-ic-ruler:before{content:\"\\e22f\"}.bosch-ic-ruler-pen:before{content:\"\\e230\"}.bosch-ic-satellite:before{content:\"\\e231\"}.bosch-ic-save:before{content:\"\\e232\"}.bosch-ic-scale:before{content:\"\\e233\"}.bosch-ic-scissors:before{content:\"\\e234\"}.bosch-ic-search:before{content:\"\\e235\"}.bosch-ic-security:before{content:\"\\e236\"}.bosch-ic-security-check:before{content:\"\\e237\"}.bosch-ic-security-user:before{content:\"\\e238\"}.bosch-ic-sensor:before{content:\"\\e239\"}.bosch-ic-server:before{content:\"\\e23a\"}.bosch-ic-server-expansion:before{content:\"\\e23b\"}.bosch-ic-server-rate:before{content:\"\\e23c\"}.bosch-ic-service-agent:before{content:\"\\e23d\"}.bosch-ic-service-time:before{content:\"\\e23e\"}.bosch-ic-settings:before{content:\"\\e23f\"}.bosch-ic-settings-arrows:before{content:\"\\e240\"}.bosch-ic-settings-connected:before{content:\"\\e241\"}.bosch-ic-share:before{content:\"\\e242\"}.bosch-ic-shield-stop:before{content:\"\\e243\"}.bosch-ic-ship:before{content:\"\\e244\"}.bosch-ic-ship-side:before{content:\"\\e245\"}.bosch-ic-shoppingcart:before{content:\"\\e246\"}.bosch-ic-shoppingcart-add:before{content:\"\\e247\"}.bosch-ic-shoppingcart-switch:before{content:\"\\e248\"}.bosch-ic-signal-full:before{content:\"\\e249\"}.bosch-ic-signal-half:before{content:\"\\e24a\"}.bosch-ic-signal-lost:before{content:\"\\e24b\"}.bosch-ic-signpost:before{content:\"\\e24c\"}.bosch-ic-simcard:before{content:\"\\e24d\"}.bosch-ic-skyscraper:before{content:\"\\e24e\"}.bosch-ic-smartcity:before{content:\"\\e24f\"}.bosch-ic-smarthome:before{content:\"\\e250\"}.bosch-ic-smartphone:before{content:\"\\e251\"}.bosch-ic-smartphone-acoustic:before{content:\"\\e252\"}.bosch-ic-smartphone-arrow-right:before{content:\"\\e253\"}.bosch-ic-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.bosch-ic-smartphone-clock:before{content:\"\\e255\"}.bosch-ic-smartphone-cloud:before{content:\"\\e256\"}.bosch-ic-smartphone-loading-wheel:before{content:\"\\e257\"}.bosch-ic-smartphone-smashed:before{content:\"\\e258\"}.bosch-ic-smartphone-vibration:before{content:\"\\e259\"}.bosch-ic-smartphone-z:before{content:\"\\e25a\"}.bosch-ic-smartwatch-connected:before{content:\"\\e25b\"}.bosch-ic-spare-parts:before{content:\"\\e25c\"}.bosch-ic-spark-plug:before{content:\"\\e25d\"}.bosch-ic-speaker-acoustic:before{content:\"\\e25e\"}.bosch-ic-speedometer:before{content:\"\\e25f\"}.bosch-ic-stadium:before{content:\"\\e260\"}.bosch-ic-stairs:before{content:\"\\e261\"}.bosch-ic-stamp:before{content:\"\\e262\"}.bosch-ic-standby:before{content:\"\\e263\"}.bosch-ic-steering-wheel:before{content:\"\\e264\"}.bosch-ic-steering-wheel-connected:before{content:\"\\e265\"}.bosch-ic-steering-wheel-hands:before{content:\"\\e266\"}.bosch-ic-steps:before{content:\"\\e267\"}.bosch-ic-stethoscope:before{content:\"\\e268\"}.bosch-ic-stop:before{content:\"\\e269\"}.bosch-ic-stopwatch:before{content:\"\\e26a\"}.bosch-ic-storage-local:before{content:\"\\e26b\"}.bosch-ic-structure:before{content:\"\\e26c\"}.bosch-ic-subtitles:before{content:\"\\e26d\"}.bosch-ic-subtitles-off-light:before{content:\"\\e26e\"}.bosch-ic-summary:before{content:\"\\e26f\"}.bosch-ic-sun:before{content:\"\\e270\"}.bosch-ic-sun-half-filled:before{content:\"\\e271\"}.bosch-ic-sunshade-table:before{content:\"\\e272\"}.bosch-ic-surveillance:before{content:\"\\e273\"}.bosch-ic-table-chairs:before{content:\"\\e274\"}.bosch-ic-tablet:before{content:\"\\e275\"}.bosch-ic-team:before{content:\"\\e276\"}.bosch-ic-team-lightbulb:before{content:\"\\e277\"}.bosch-ic-technical-service:before{content:\"\\e278\"}.bosch-ic-temperature:before{content:\"\\e279\"}.bosch-ic-temperature-high:before{content:\"\\e27a\"}.bosch-ic-temperature-low:before{content:\"\\e27b\"}.bosch-ic-theater:before{content:\"\\e27c\"}.bosch-ic-thumb-down:before{content:\"\\e27d\"}.bosch-ic-thumb-up:before{content:\"\\e27e\"}.bosch-ic-ticket:before{content:\"\\e27f\"}.bosch-ic-touch:before{content:\"\\e280\"}.bosch-ic-touch-sos:before{content:\"\\e281\"}.bosch-ic-towing-truck:before{content:\"\\e282\"}.bosch-ic-train:before{content:\"\\e283\"}.bosch-ic-train-side:before{content:\"\\e284\"}.bosch-ic-transport-movements:before{content:\"\\e285\"}.bosch-ic-transscript:before{content:\"\\e286\"}.bosch-ic-transscript-off:before{content:\"\\e287\"}.bosch-ic-transscript-on:before{content:\"\\e288\"}.bosch-ic-tree:before{content:\"\\e289\"}.bosch-ic-tricycle:before{content:\"\\e28a\"}.bosch-ic-trolley-empty:before{content:\"\\e28b\"}.bosch-ic-trolley-filled-box:before{content:\"\\e28c\"}.bosch-ic-trolly-filled-files:before{content:\"\\e28d\"}.bosch-ic-trophy:before{content:\"\\e28e\"}.bosch-ic-truck:before{content:\"\\e28f\"}.bosch-ic-tuktuk:before{content:\"\\e290\"}.bosch-ic-tune:before{content:\"\\e291\"}.bosch-ic-typography:before{content:\"\\e292\"}.bosch-ic-ui:before{content:\"\\e293\"}.bosch-ic-umbrella:before{content:\"\\e294\"}.bosch-ic-undo:before{content:\"\\e295\"}.bosch-ic-up:before{content:\"\\e296\"}.bosch-ic-up-double:before{content:\"\\e297\"}.bosch-ic-up-down:before{content:\"\\e298\"}.bosch-ic-upload:before{content:\"\\e299\"}.bosch-ic-usb:before{content:\"\\e29a\"}.bosch-ic-user:before{content:\"\\e29b\"}.bosch-ic-user-advanced:before{content:\"\\e29c\"}.bosch-ic-user-beginner:before{content:\"\\e29d\"}.bosch-ic-user-desktop:before{content:\"\\e29e\"}.bosch-ic-user-down:before{content:\"\\e29f\"}.bosch-ic-user-falling:before{content:\"\\e2a0\"}.bosch-ic-user-hand:before{content:\"\\e2a1\"}.bosch-ic-user-hand-stop:before{content:\"\\e2a2\"}.bosch-ic-user-professional:before{content:\"\\e2a3\"}.bosch-ic-user-run:before{content:\"\\e2a4\"}.bosch-ic-user-share:before{content:\"\\e2a5\"}.bosch-ic-user-voice:before{content:\"\\e2a6\"}.bosch-ic-user-walk:before{content:\"\\e2a7\"}.bosch-ic-video:before{content:\"\\e2a8\"}.bosch-ic-video-disabled-light:before{content:\"\\e2a9\"}.bosch-ic-video-record:before{content:\"\\e2aa\"}.bosch-ic-virtual-reality:before{content:\"\\e2ab\"}.bosch-ic-volume-disabled:before{content:\"\\e2ac\"}.bosch-ic-volume-high:before{content:\"\\e2ad\"}.bosch-ic-volume-low:before{content:\"\\e2ae\"}.bosch-ic-volume-off-light:before{content:\"\\e2af\"}.bosch-ic-wand:before{content:\"\\e2b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-watch-off-disabled-light:before{content:\"\\e2b2\"}.bosch-ic-watch-on:before{content:\"\\e2b3\"}.bosch-ic-water-shower:before{content:\"\\e2b4\"}.bosch-ic-welcome:before{content:\"\\e2b5\"}.bosch-ic-wellness:before{content:\"\\e2b6\"}.bosch-ic-wheelchair:before{content:\"\\e2b7\"}.bosch-ic-whistle:before{content:\"\\e2b8\"}.bosch-ic-wide-angled-arrow:before{content:\"\\e2b9\"}.bosch-ic-wifi:before{content:\"\\e2ba\"}.bosch-ic-window-analysis:before{content:\"\\e2bb\"}.bosch-ic-window-browser:before{content:\"\\e2bc\"}.bosch-ic-window-console:before{content:\"\\e2bd\"}.bosch-ic-window-gaussian-view:before{content:\"\\e2be\"}.bosch-ic-window-info:before{content:\"\\e2bf\"}.bosch-ic-window-new:before{content:\"\\e2c0\"}.bosch-ic-window-resize:before{content:\"\\e2c1\"}.bosch-ic-window-scaling-view:before{content:\"\\e2c2\"}.bosch-ic-window-shuffle:before{content:\"\\e2c3\"}.bosch-ic-window-statistic:before{content:\"\\e2c4\"}.bosch-ic-wishlist:before{content:\"\\e2c5\"}.bosch-ic-work-order:before{content:\"\\e2c6\"}.bosch-ic-worldwideweb:before{content:\"\\e2c7\"}.bosch-ic-wrench:before{content:\"\\e2c8\"}.bosch-ic-zoom-in:before{content:\"\\e2c9\"}.bosch-ic-zoom-out:before{content:\"\\e2ca\"}.bosch-ic-scooter:before{content:\"\\e2cb\"}.bosch-ic-cars-traffic:before{content:\"\\e2cc\"}.bosch-ic-arm-chair:before{content:\"\\e2cd\"}.bosch-ic-car-phone:before{content:\"\\e2ce\"}.bosch-ic-box-lightbulb:before{content:\"\\e2cf\"}.bosch-ic-chair-officedesk:before{content:\"\\e2d0\"}.bosch-ic-calendar-clock:before{content:\"\\e2d1\"}.bosch-ic-document-cv:before{content:\"\\e2d2\"}.bosch-ic-user-run-bag:before{content:\"\\e2d3\"}.bosch-ic-brake-disk-spray:before{content:\"\\e2d4\"}.bosch-ic-circle-abs:before{content:\"\\e2d5\"}.bosch-ic-bright-down:before{content:\"\\e2d6\"}.bosch-ic-bright-up:before{content:\"\\e2d7\"}.bosch-ic-fan:before{content:\"\\e2d8\"}.bosch-ic-sun-moon:before{content:\"\\e2d9\"}.bosch-ic-lightbulb-attention:before{content:\"\\e2da\"}.bosch-ic-settings-attention:before{content:\"\\e2db\"}.bosch-ic-danger-hazards:before{content:\"\\e2dc\"}.bosch-ic-windshield-spray:before{content:\"\\e2dd\"}.bosch-ic-windshield-heat:before{content:\"\\e2de\"}.bosch-ic-light-drop:before{content:\"\\e2df\"}.bosch-ic-starter:before{content:\"\\e2e0\"}.bosch-ic-turbo:before{content:\"\\e2e1\"}.bosch-ic-settings-lock:before{content:\"\\e2e2\"}.bosch-ic-settings-n:before{content:\"\\e2e3\"}.bosch-ic-light-beam-high:before{content:\"\\e2e4\"}.bosch-ic-light-beam-low:before{content:\"\\e2e5\"}.bosch-ic-glas:before{content:\"\\e2e6\"}.bosch-ic-shirt-ice:before{content:\"\\e2e7\"}.bosch-ic-helmet-fan:before{content:\"\\e2e8\"}.bosch-ic-notification-arrow:before{content:\"\\e2e9\"}.bosch-ic-steering-wheel-arrow:before{content:\"\\e2ea\"}.bosch-ic-circle-arrow:before{content:\"\\e2eb\"}.bosch-ic-tempomat-pit:before{content:\"\\e2ec\"}.bosch-ic-radiator-fan-spray:before{content:\"\\e2ed\"}.bosch-ic-boost-down:before{content:\"\\e2ee\"}.bosch-ic-boost-up:before{content:\"\\e2ef\"}.bosch-ic-circle-lock:before{content:\"\\e2f0\"}.bosch-ic-mp-down:before{content:\"\\e2f1\"}.bosch-ic-mp-up:before{content:\"\\e2f2\"}.bosch-ic-tc-down:before{content:\"\\e2f3\"}.bosch-ic-tc-up:before{content:\"\\e2f4\"}.bosch-ic-danger-arrow-attention:before{content:\"\\e2f5\"}.bosch-ic-user-remove:before{content:\"\\e2f6\"}.bosch-ic-user-add:before{content:\"\\e2f7\"}.bosch-ic-post-it-collection:before{content:\"\\e2f8\"}.bosch-ic-horn:before{content:\"\\e2f9\"}.bosch-ic-desktop-notification:before{content:\"\\e2fa\"}.bosch-ic-router:before{content:\"\\e2fb\"}.bosch-ic-berry:before{content:\"\\e2fc\"}.bosch-ic-chat-language:before{content:\"\\e2fd\"}.bosch-ic-game-controller:before{content:\"\\e2fe\"}.bosch-ic-user-brain:before{content:\"\\e2ff\"}.bosch-ic-mouth-finger:before{content:\"\\e300\"}.bosch-ic-excavator-shovel:before{content:\"\\e301\"}.bosch-ic-ear-in:before{content:\"\\e302\"}.bosch-ic-ear-out:before{content:\"\\e303\"}.bosch-ic-lion:before{content:\"\\e304\"}.bosch-ic-car-convertible:before{content:\"\\e305\"}.bosch-ic-car-sports:before{content:\"\\e306\"}.bosch-ic-bear:before{content:\"\\e307\"}.bosch-ic-pot:before{content:\"\\e308\"}.bosch-ic-bed-double:before{content:\"\\e309\"}.bosch-ic-air-hot:before{content:\"\\e30a\"}.bosch-ic-air-ice:before{content:\"\\e30b\"}.bosch-ic-air-leaf:before{content:\"\\e30c\"}.bosch-ic-air-purifier-0:before{content:\"\\e30d\"}.bosch-ic-air-purifier-1:before{content:\"\\e30e\"}.bosch-ic-air-purifier-2:before{content:\"\\e30f\"}.bosch-ic-air-purifier-3:before{content:\"\\e310\"}.bosch-ic-air:before{content:\"\\e311\"}.bosch-ic-fan-off-disabled-bold:before{content:\"\\e312\"}.bosch-ic-ice-off-disabled-bold:before{content:\"\\e313\"}.bosch-ic-faucet-boost:before{content:\"\\e314\"}.bosch-ic-faucet-mode:before{content:\"\\e315\"}.bosch-ic-faucet-off-disabled-bold:before{content:\"\\e316\"}.bosch-ic-faucet-schedule:before{content:\"\\e317\"}.bosch-ic-fuel-cell:before{content:\"\\e318\"}.bosch-ic-heating-floor-off-disabled-bold:before{content:\"\\e319\"}.bosch-ic-heating-floor:before{content:\"\\e31a\"}.bosch-ic-heating-radiator-boost:before{content:\"\\e31b\"}.bosch-ic-heating-radiator-mode:before{content:\"\\e31c\"}.bosch-ic-heating-radiator-off-disabled-bold:before{content:\"\\e31d\"}.bosch-ic-heating-radiator-schedule:before{content:\"\\e31e\"}.bosch-ic-leaf-simple:before{content:\"\\e31f\"}.bosch-ic-oil-barrel:before{content:\"\\e320\"}.bosch-ic-water-bathtub:before{content:\"\\e321\"}.bosch-ic-water-dishwash:before{content:\"\\e322\"}.bosch-ic-water-handwash:before{content:\"\\e323\"}.bosch-ic-water-hot:before{content:\"\\e324\"}.bosch-ic-water-ice:before{content:\"\\e325\"}.bosch-ic-water-leaf:before{content:\"\\e326\"}.bosch-ic-water-ladder:before{content:\"\\e327\"}.bosch-ic-water-purifier-0:before{content:\"\\e328\"}.bosch-ic-water-purifier-1:before{content:\"\\e329\"}.bosch-ic-water-purifier-2:before{content:\"\\e32a\"}.bosch-ic-water-purifier-3:before{content:\"\\e32b\"}.bosch-ic-water-shower-boost:before{content:\"\\e32c\"}.bosch-ic-water-shower-mode:before{content:\"\\e32d\"}.bosch-ic-water-shower-off-disabled-bold:before{content:\"\\e32e\"}.bosch-ic-water-shower-schedule:before{content:\"\\e32f\"}.bosch-ic-water:before{content:\"\\e330\"}.bosch-ic-chart-bar-drop:before{content:\"\\e331\"}.bosch-ic-chart-bar-fire:before{content:\"\\e332\"}.bosch-ic-chart-bar-flash:before{content:\"\\e333\"}.bosch-ic-chart-bar-oil:before{content:\"\\e334\"}.bosch-ic-drop-minus:before{content:\"\\e335\"}.bosch-ic-drop-plus:before{content:\"\\e336\"}.bosch-ic-fan-minus:before{content:\"\\e337\"}.bosch-ic-fan-plus:before{content:\"\\e338\"}.bosch-ic-home-drops-0:before{content:\"\\e339\"}.bosch-ic-home-drops-1:before{content:\"\\e33a\"}.bosch-ic-home-drops-2:before{content:\"\\e33b\"}.bosch-ic-home-drops-3:before{content:\"\\e33c\"}.bosch-ic-home-temperature-in:before{content:\"\\e33d\"}.bosch-ic-home-temperature-out:before{content:\"\\e33e\"}.bosch-ic-home-user-in:before{content:\"\\e33f\"}.bosch-ic-ice-minus:before{content:\"\\e340\"}.bosch-ic-ice-plus:before{content:\"\\e341\"}.bosch-ic-chimney-sweeper:before{content:\"\\e342\"}.bosch-ic-menu-up:before{content:\"\\e343\"}.bosch-ic-security-ice:before{content:\"\\e344\"}.bosch-ic-security-legionella:before{content:\"\\e345\"}.bosch-ic-trolley-case:before{content:\"\\e346\"}.bosch-ic-sun-minus:before{content:\"\\e347\"}.bosch-ic-sun-plug:before{content:\"\\e348\"}.bosch-ic-sun-plus:before{content:\"\\e349\"}.bosch-ic-touch-pit:before{content:\"\\e34a\"}.bosch-ic-temperature-current:before{content:\"\\e34b\"}.bosch-ic-temperature-set:before{content:\"\\e34c\"}.bosch-ic-teddy-bear:before{content:\"\\e34d\"}.bosch-ic-document-svg:before{content:\"\\e34e\"}.bosch-ic-back-menu:before{content:\"\\e34f\"}.bosch-ic-car-arrows:before{content:\"\\e350\"}.bosch-ic-wrench-cloud:before{content:\"\\e351\"}.bosch-ic-weather-cloud-sun:before{content:\"\\e352\"}.bosch-ic-weather-cloudy:before{content:\"\\e353\"}.bosch-ic-weather-gale:before{content:\"\\e354\"}.bosch-ic-weather-mist:before{content:\"\\e355\"}.bosch-ic-weather-moon-cloud:before{content:\"\\e356\"}.bosch-ic-weather-rain-snow:before{content:\"\\e357\"}.bosch-ic-weather-rain-thunder:before{content:\"\\e358\"}.bosch-ic-weather-rain:before{content:\"\\e359\"}.bosch-ic-weather-snow:before{content:\"\\e35a\"}.bosch-ic-weather-thunder:before{content:\"\\e35b\"}.bosch-ic-weather-wind:before{content:\"\\e35c\"}.bosch-ic-plane-start:before{content:\"\\e35d\"}.bosch-ic-plane-land:before{content:\"\\e35e\"}.bosch-ic-graph-power:before{content:\"\\e35f\"}.bosch-ic-floorplan:before{content:\"\\e360\"}.bosch-ic-clock-pause:before{content:\"\\e361\"}.bosch-ic-clock-start:before{content:\"\\e362\"}.bosch-ic-clock-stop:before{content:\"\\e363\"}.bosch-ic-computer-retro:before{content:\"\\e364\"}.bosch-ic-smartphone-retro:before{content:\"\\e365\"}.bosch-ic-wood:before{content:\"\\e366\"}.bosch-ic-solid-fuel:before{content:\"\\e367\"}.bosch-ic-appliance-fan:before{content:\"\\e368\"}.bosch-ic-appliance-flame:before{content:\"\\e369\"}.bosch-ic-appliance-ice:before{content:\"\\e36a\"}.bosch-ic-appliance-oil:before{content:\"\\e36b\"}.bosch-ic-appliance-solid-fuel:before{content:\"\\e36c\"}.bosch-ic-appliance-sun-ice:before{content:\"\\e36d\"}.bosch-ic-appliance-sun:before{content:\"\\e36e\"}.bosch-ic-appliance-wood:before{content:\"\\e36f\"}.bosch-ic-appliance-air-leaf:before{content:\"\\e370\"}.bosch-ic-appliance-water-air:before{content:\"\\e371\"}.bosch-ic-appliance-water-leaf:before{content:\"\\e372\"}.bosch-ic-appliance-water-oil:before{content:\"\\e373\"}.bosch-ic-appliance-water:before{content:\"\\e374\"}.bosch-ic-appliance-heat-pump-air:before{content:\"\\e375\"}.bosch-ic-appliance-heat-pump-ground:before{content:\"\\e376\"}.bosch-ic-appliance-heat-pump-water:before{content:\"\\e377\"}.bosch-ic-appliance-plug-air:before{content:\"\\e378\"}.bosch-ic-appliance-plug-fuel-cell:before{content:\"\\e379\"}.bosch-ic-appliance-plug-leaf:before{content:\"\\e37a\"}.bosch-ic-appliance-plug-water-air:before{content:\"\\e37b\"}.bosch-ic-thermostat-connected-c:before{content:\"\\e37c\"}.bosch-ic-thermostat-connected-f:before{content:\"\\e37d\"}.bosch-ic-image-gallery:before{content:\"\\e37e\"}.bosch-ic-arrows-caution:before{content:\"\\e37f\"}.bosch-ic-arrows-diagonal-disabled-bold:before{content:\"\\e380\"}.bosch-ic-back-left-end:before{content:\"\\e381\"}.bosch-ic-back-right-end:before{content:\"\\e382\"}.bosch-ic-bar-chart-search:before{content:\"\\e383\"}.bosch-ic-batch:before{content:\"\\e384\"}.bosch-ic-calendar-remove:before{content:\"\\e385\"}.bosch-ic-call-bluetooth:before{content:\"\\e386\"}.bosch-ic-call-deny:before{content:\"\\e387\"}.bosch-ic-call-settings:before{content:\"\\e388\"}.bosch-ic-caution-sign-clock:before{content:\"\\e389\"}.bosch-ic-caution-sign-list:before{content:\"\\e38a\"}.bosch-ic-core-data:before{content:\"\\e38b\"}.bosch-ic-cursor-connected:before{content:\"\\e38c\"}.bosch-ic-cursor:before{content:\"\\e38d\"}.bosch-ic-dmc-code:before{content:\"\\e38e\"}.bosch-ic-find-id:before{content:\"\\e38f\"}.bosch-ic-globe-search:before{content:\"\\e390\"}.bosch-ic-interface:before{content:\"\\e391\"}.bosch-ic-material-add-manually:before{content:\"\\e392\"}.bosch-ic-material-list:before{content:\"\\e393\"}.bosch-ic-material-remove-manually:before{content:\"\\e394\"}.bosch-ic-material:before{content:\"\\e395\"}.bosch-ic-menu-checkmark:before{content:\"\\e396\"}.bosch-ic-qr-code:before{content:\"\\e397\"}.bosch-ic-sort-alphabetically:before{content:\"\\e398\"}.bosch-ic-unpin-classic-disabled-bold:before{content:\"\\e399\"}.bosch-ic-unpin-modern-disabled-bold:before{content:\"\\e39a\"}.bosch-ic-window-search:before{content:\"\\e39b\"}.bosch-ic-cloud-clock:before{content:\"\\e39c\"}.bosch-ic-concierge:before{content:\"\\e39d\"}.bosch-ic-heating-radiator:before{content:\"\\e39e\"}.bosch-ic-home-user-out:before{content:\"\\e39f\"}.bosch-ic-hot:before{content:\"\\e3a0\"}.bosch-ic-screen-service:before{content:\"\\e3a1\"}.bosch-ic-sun-ice:before{content:\"\\e3a2\"}.bosch-ic-sun-off-disabled-bold:before{content:\"\\e3a3\"}.bosch-ic-tractor:before{content:\"\\e3a4\"}.bosch-ic-video-recorder-digital:before{content:\"\\e3a5\"}.bosch-ic-windshield-wiper:before{content:\"\\e3a6\"}.bosch-ic-weather-snow-thunder:before{content:\"\\e3a7\"}.bosch-ic-electricity-pylon:before{content:\"\\e3a8\"}.bosch-ic-box-questionmark:before{content:\"\\e3a9\"}.bosch-ic-rainbow:before{content:\"\\e3aa\"}.bosch-ic-medal:before{content:\"\\e3ab\"}.bosch-ic-wheel:before{content:\"\\e3ac\"}.bosch-ic-borescope-connected:before{content:\"\\e3ad\"}.bosch-ic-filter-delete:before{content:\"\\e3ae\"}.bosch-ic-filter-add:before{content:\"\\e3af\"}.bosch-ic-man:before{content:\"\\e3b0\"}.bosch-ic-woman:before{content:\"\\e3b1\"}.bosch-ic-fridge:before{content:\"\\e3b2\"}.bosch-ic-battery-car:before{content:\"\\e3b3\"}.bosch-ic-bag-escalator:before{content:\"\\e3b4\"}.bosch-ic-shopping-bags:before{content:\"\\e3b5\"}.bosch-ic-car-steering-left:before{content:\"\\e3b6\"}.bosch-ic-car-steering-right:before{content:\"\\e3b7\"}.bosch-ic-thumb-up-down:before{content:\"\\e3b8\"}.bosch-ic-user-run-detection:before{content:\"\\e3b9\"}.bosch-ic-bank:before{content:\"\\e3ba\"}.bosch-ic-transportation:before{content:\"\\e3bb\"}.bosch-ic-government:before{content:\"\\e3bc\"}.bosch-ic-user-music:before{content:\"\\e3bd\"}.bosch-ic-bus:before{content:\"\\e3be\"}.bosch-ic-car-euro:before{content:\"\\e3bf\"}.bosch-ic-bag:before{content:\"\\e3c0\"}.bosch-ic-plate-tetrapack:before{content:\"\\e3c1\"}.bosch-ic-hospital:before{content:\"\\e3c2\"}.bosch-ic-desktop-structure:before{content:\"\\e3c3\"}.bosch-ic-frog-head:before{content:\"\\e3c4\"}.bosch-ic-sportboat:before{content:\"\\e3c5\"}.bosch-ic-yacht:before{content:\"\\e3c6\"}.bosch-ic-leaf-tilia:before{content:\"\\e3c7\"}.bosch-ic-music:before{content:\"\\e3c8\"}.bosch-ic-oil-lamp:before{content:\"\\e3c9\"}.bosch-ic-windshield-wiper-rear:before{content:\"\\e3ca\"}.bosch-ic-dongle-connected:before{content:\"\\e3cb\"}.bosch-ic-slate:before{content:\"\\e3cc\"}.bosch-ic-appliance-heat-pump-exhaust:before{content:\"\\e3cd\"}.bosch-ic-alert-list:before{content:\"\\e3ce\"}.bosch-ic-arrow-left-end-frame:before{content:\"\\e3cf\"}.bosch-ic-arrow-right-end-frame:before{content:\"\\e3d0\"}.bosch-ic-shape-ellipse:before{content:\"\\e3d1\"}.bosch-ic-shape-rectangle:before{content:\"\\e3d2\"}.bosch-ic-wishlist-add:before{content:\"\\e3d3\"}.bosch-ic-smartphone-disabled-light:before{content:\"\\e3d4\"}.bosch-ic-airbag:before{content:\"\\e3d5\"}.bosch-ic-compass:before{content:\"\\e3d6\"}.bosch-ic-turbine-wind:before{content:\"\\e3d7\"}.bosch-ic-arrows-diagonal-disabled-light:before{content:\"\\e3d8\"}.bosch-ic-ear-off-disabled-bold:before{content:\"\\e3d9\"}.bosch-ic-fan-off-disabled-light:before{content:\"\\e3da\"}.bosch-ic-faucet-off-disabled-light:before{content:\"\\e3db\"}.bosch-ic-hand-motion-off-disabled-bold:before{content:\"\\e3dc\"}.bosch-ic-heating-floor-off-disabled-light:before{content:\"\\e3dd\"}.bosch-ic-heating-radiator-off-disabled-light:before{content:\"\\e3de\"}.bosch-ic-ice-off-disabled-light:before{content:\"\\e3df\"}.bosch-ic-smartphone-disabled-bold:before{content:\"\\e3e0\"}.bosch-ic-subtitles-off-bold:before{content:\"\\e3e1\"}.bosch-ic-sun-off-disabled-light:before{content:\"\\e3e2\"}.bosch-ic-video-disabled-bold:before{content:\"\\e3e3\"}.bosch-ic-volume-off-bold:before{content:\"\\e3e4\"}.bosch-ic-watch-off-disabled-bold:before{content:\"\\e3e5\"}.bosch-ic-water-shower-off-disabled-light:before{content:\"\\e3e6\"}.bosch-ic-3d-frame-play:before{content:\"\\e3e7\"}.bosch-ic-alert-error-checkmark:before{content:\"\\e3e8\"}.bosch-ic-arrow-up-frame-check:before{content:\"\\e3e9\"}.bosch-ic-arrow-up-frame-error:before{content:\"\\e3ea\"}.bosch-ic-arrows-checkmark:before{content:\"\\e3eb\"}.bosch-ic-assembly-line-settings:before{content:\"\\e3ec\"}.bosch-ic-augmented-reality-player:before{content:\"\\e3ed\"}.bosch-ic-bar-chart-kappa:before{content:\"\\e3ef\"}.bosch-ic-bar-chart-search-detail:before{content:\"\\e3f0\"}.bosch-ic-barcode-checkmark:before{content:\"\\e3f1\"}.bosch-ic-barcode-scan-bulb:before{content:\"\\e3f2\"}.bosch-ic-barcode:before{content:\"\\e3f3\"}.bosch-ic-binary:before{content:\"\\e3f4\"}.bosch-ic-book-lightbulb:before{content:\"\\e3f5\"}.bosch-ic-box-arrow-down:before{content:\"\\e3f6\"}.bosch-ic-box-delivery-checkmark:before{content:\"\\e3f7\"}.bosch-ic-globe-checkmark:before{content:\"\\e3f8\"}.bosch-ic-calendar-boxes:before{content:\"\\e3f9\"}.bosch-ic-calendar-service:before{content:\"\\e3fa\"}.bosch-ic-camera-processor:before{content:\"\\e3fb\"}.bosch-ic-camera-settings:before{content:\"\\e3fc\"}.bosch-ic-chart-line-checkmark:before{content:\"\\e3fd\"}.bosch-ic-checklist-stack:before{content:\"\\e3fe\"}.bosch-ic-checkmark-frame-minimum:before{content:\"\\e3ff\"}.bosch-ic-checkmark-frame-tripple:before{content:\"\\e400\"}.bosch-ic-clipboard-checkmark:before{content:\"\\e401\"}.bosch-ic-clipboard-list-parts:before{content:\"\\e402\"}.bosch-ic-clipboard-settings:before{content:\"\\e403\"}.bosch-ic-computer-tower-box:before{content:\"\\e404\"}.bosch-ic-connectivity-lock:before{content:\"\\e405\"}.bosch-ic-counter-current:before{content:\"\\e406\"}.bosch-ic-counter-target:before{content:\"\\e407\"}.bosch-ic-cube-checkmark:before{content:\"\\e408\"}.bosch-ic-cube-play:before{content:\"\\e409\"}.bosch-ic-timeline-settings:before{content:\"\\e40a\"}.bosch-ic-timeline-view:before{content:\"\\e40b\"}.bosch-ic-timeline:before{content:\"\\e40c\"}.bosch-ic-delivery-checkmark:before{content:\"\\e40d\"}.bosch-ic-desktop-application-3d-play:before{content:\"\\e40e\"}.bosch-ic-desktop-apps-download:before{content:\"\\e40f\"}.bosch-ic-desktop-dashboard:before{content:\"\\e410\"}.bosch-ic-desktop-graph-checkmark:before{content:\"\\e411\"}.bosch-ic-desktop-graph-search:before{content:\"\\e412\"}.bosch-ic-desktop-graph:before{content:\"\\e413\"}.bosch-ic-development-kit:before{content:\"\\e414\"}.bosch-ic-directory-cloud-settings:before{content:\"\\e415\"}.bosch-ic-directory-cloud:before{content:\"\\e416\"}.bosch-ic-directory-settings:before{content:\"\\e417\"}.bosch-ic-document-clock-cycle:before{content:\"\\e418\"}.bosch-ic-document-code-stack:before{content:\"\\e419\"}.bosch-ic-document-dat-edit:before{content:\"\\e41a\"}.bosch-ic-document-ohd-arrow-down:before{content:\"\\e41b\"}.bosch-ic-dolphin:before{content:\"\\e41c\"}.bosch-ic-energy-management:before{content:\"\\e41d\"}.bosch-ic-eraser:before{content:\"\\e41e\"}.bosch-ic-folder-broom:before{content:\"\\e41f\"}.bosch-ic-folder-check-broom:before{content:\"\\e420\"}.bosch-ic-folder-checkmark:before{content:\"\\e421\"}.bosch-ic-folder-oes-check:before{content:\"\\e422\"}.bosch-ic-folder-open-service:before{content:\"\\e423\"}.bosch-ic-folder-reload:before{content:\"\\e424\"}.bosch-ic-hand-command:before{content:\"\\e425\"}.bosch-ic-heating-start:before{content:\"\\e426\"}.bosch-ic-hierarchy-refresh:before{content:\"\\e427\"}.bosch-ic-hierarchy-search:before{content:\"\\e428\"}.bosch-ic-hierarchy-settings:before{content:\"\\e429\"}.bosch-ic-hierarchy:before{content:\"\\e42a\"}.bosch-ic-hmi-desktop-settings:before{content:\"\\e42b\"}.bosch-ic-human-machine-interface:before{content:\"\\e42c\"}.bosch-ic-keys-user-access:before{content:\"\\e42d\"}.bosch-ic-keys:before{content:\"\\e42e\"}.bosch-ic-label-edit:before{content:\"\\e42f\"}.bosch-ic-line-arrow-checkmark:before{content:\"\\e430\"}.bosch-ic-link-checkmark:before{content:\"\\e431\"}.bosch-ic-lock-closed-checkmark:before{content:\"\\e432\"}.bosch-ic-lock-open-checkmark:before{content:\"\\e433\"}.bosch-ic-material-search:before{content:\"\\e434\"}.bosch-ic-objects-search:before{content:\"\\e435\"}.bosch-ic-objects:before{content:\"\\e436\"}.bosch-ic-plan-a-to-b:before{content:\"\\e437\"}.bosch-ic-plan-chart:before{content:\"\\e438\"}.bosch-ic-print-send:before{content:\"\\e439\"}.bosch-ic-refresh-analysis:before{content:\"\\e43a\"}.bosch-ic-remote-desktop-add:before{content:\"\\e43b\"}.bosch-ic-rfid-checkmark:before{content:\"\\e43c\"}.bosch-ic-rfid-tag:before{content:\"\\e43d\"}.bosch-ic-sequence-checkmark:before{content:\"\\e43e\"}.bosch-ic-server-arrow-up:before{content:\"\\e43f\"}.bosch-ic-server-arrows:before{content:\"\\e440\"}.bosch-ic-server-settings:before{content:\"\\e441\"}.bosch-ic-server-share:before{content:\"\\e442\"}.bosch-ic-settings-editor:before{content:\"\\e443\"}.bosch-ic-survey-checkmark:before{content:\"\\e444\"}.bosch-ic-target-dart:before{content:\"\\e445\"}.bosch-ic-target-view:before{content:\"\\e446\"}.bosch-ic-target:before{content:\"\\e447\"}.bosch-ic-thickness-selection:before{content:\"\\e448\"}.bosch-ic-toolbox-checkmark:before{content:\"\\e449\"}.bosch-ic-toolbox:before{content:\"\\e44a\"}.bosch-ic-translate:before{content:\"\\e44b\"}.bosch-ic-tune-vertical-checkmark:before{content:\"\\e44c\"}.bosch-ic-wand-user-add:before{content:\"\\e44d\"}.bosch-ic-wand-user-settings:before{content:\"\\e44e\"}.bosch-ic-wand-user:before{content:\"\\e44f\"}.bosch-ic-window-globe:before{content:\"\\e450\"}.bosch-ic-wrench-change:before{content:\"\\e451\"}.bosch-ic-clock-24-7:before{content:\"\\e452\"}.bosch-ic-eagle:before{content:\"\\e453\"}.bosch-ic-fluorescent-light:before{content:\"\\e454\"}.bosch-ic-home-cloud-0:before{content:\"\\e455\"}.bosch-ic-home-cloud-1:before{content:\"\\e456\"}.bosch-ic-home-cloud-2:before{content:\"\\e457\"}.bosch-ic-home-cloud-3:before{content:\"\\e458\"}.bosch-ic-home-shadow:before{content:\"\\e459\"}.bosch-ic-motorcycle-sport:before{content:\"\\e45a\"}.bosch-ic-smartphone-connected:before{content:\"\\e45b\"}.bosch-ic-usb-stick:before{content:\"\\e45c\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-timeline-clock:before{content:\"\\e45e\"}.bosch-ic-alarm-fire:before{content:\"\\e45f\"}.bosch-ic-emergency-front:before{content:\"\\e460\"}.bosch-ic-transporter-front:before{content:\"\\e461\"}.bosch-ic-truck-front:before{content:\"\\e462\"}.bosch-ic-unpin-classic-disabled-light:before{content:\"\\e463\"}.bosch-ic-unpin-modern-disabled-light:before{content:\"\\e464\"}.bosch-ic-smartphone-ban:before{content:\"\\e465\"}.bosch-ic-helmet:before{content:\"\\e466\"}.bosch-ic-gate-closed:before{content:\"\\e467\"}.bosch-ic-fence:before{content:\"\\e468\"}.bosch-ic-desktop-apps:before{content:\"\\e469\"}.bosch-ic-candle:before{content:\"\\e46a\"}.bosch-ic-abort-frame:before{content:\"\\e886\"}.bosch-ic-arrow-down-frame:before{content:\"\\e887\"}.bosch-ic-arrow-left-down-frame:before{content:\"\\e888\"}.bosch-ic-arrow-left-frame:before{content:\"\\e889\"}.bosch-ic-arrow-left-up-frame:before{content:\"\\e88a\"}.bosch-ic-arrow-right-down-frame:before{content:\"\\e88b\"}.bosch-ic-arrow-right-frame:before{content:\"\\e88c\"}.bosch-ic-arrow-right-up-frame:before{content:\"\\e88d\"}.bosch-ic-arrow-up-frame:before{content:\"\\e88e\"}.bosch-ic-asiapacific-frame:before{content:\"\\e88f\"}.bosch-ic-cash-frame:before{content:\"\\e890\"}.bosch-ic-checkmark-frame:before{content:\"\\e891\"}.bosch-ic-copyright-frame:before{content:\"\\e892\"}.bosch-ic-download-frame:before{content:\"\\e893\"}.bosch-ic-europe-frame:before{content:\"\\e894\"}.bosch-ic-lock-closed-frame:before{content:\"\\e895\"}.bosch-ic-lock-open-frame:before{content:\"\\e896\"}.bosch-ic-my-brand-frame:before{content:\"\\e897\"}.bosch-ic-northamerica-frame:before{content:\"\\e898\"}.bosch-ic-problem-frame:before{content:\"\\e899\"}.bosch-ic-promotion-frame:before{content:\"\\e89a\"}.bosch-ic-question-frame:before{content:\"\\e89b\"}.bosch-ic-share-frame:before{content:\"\\e89c\"}.bosch-ic-southamerica-frame:before{content:\"\\e89d\"}.bosch-ic-start-play-frame:before{content:\"\\e89e\"}.bosch-ic-upload-frame:before{content:\"\\e89f\"}.bosch-ic-world-frame:before{content:\"\\e8a0\"}.bosch-ic-add-frame:before{content:\"\\e8a1\"}.bosch-ic-call-deny-frame:before{content:\"\\e8a2\"}.bosch-ic-call-frame:before{content:\"\\e8a3\"}.bosch-ic-fast-backward-frame:before{content:\"\\e8a4\"}.bosch-ic-fast-forward-frame:before{content:\"\\e8a5\"}.bosch-ic-skip-backward-frame:before{content:\"\\e8a6\"}.bosch-ic-skip-fast-backward-frame:before{content:\"\\e8a7\"}.bosch-ic-skip-fast-forward-frame:before{content:\"\\e8a8\"}.bosch-ic-skip-forward-frame:before{content:\"\\e8a9\"}.bosch-ic-subtract-frame:before{content:\"\\e8aa\"}.bosch-ic-facebook-frame:before{content:\"\\ea87\"}.bosch-ic-googleplus-frame:before{content:\"\\ea88\"}.bosch-ic-instagram-frame:before{content:\"\\ea89\"}.bosch-ic-lineapp-frame:before{content:\"\\ea8a\"}.bosch-ic-linkedin-frame:before{content:\"\\ea8b\"}.bosch-ic-pinterest-frame:before{content:\"\\ea8c\"}.bosch-ic-snapchat-frame:before{content:\"\\ea8d\"}.bosch-ic-tumblr-frame:before{content:\"\\ea8e\"}.bosch-ic-twitter-frame:before{content:\"\\ea8f\"}.bosch-ic-vimeo-frame:before{content:\"\\ea90\"}.bosch-ic-wechat-frame:before{content:\"\\ea91\"}.bosch-ic-weibo-frame:before{content:\"\\ea92\"}.bosch-ic-whatsapp-frame:before{content:\"\\ea93\"}.bosch-ic-xing-frame:before{content:\"\\ea94\"}.bosch-ic-youku-frame:before{content:\"\\ea95\"}.bosch-ic-youtube-frame:before{content:\"\\ea96\"}.bosch-ic-vk-frame:before{content:\"\\ea97\"}.bosch-ic-skype-frame:before{content:\"\\ea98\"}.bosch-ic-alert-error-filled:before{content:\"\\eb08\"}.bosch-ic-alert-error:before{content:\"\\eb09\"}.bosch-ic-alert-success-filled:before{content:\"\\eb0a\"}.bosch-ic-alert-success:before{content:\"\\eb0b\"}.bosch-ic-alert-warning-filled:before{content:\"\\eb0c\"}.bosch-ic-alert-warning:before{content:\"\\eb0d\"}.bosch-ic-chart-oee:before{content:\"\\e46b\"}.bosch-ic-cycletime-eighth:before{content:\"\\e46c\"}.bosch-ic-cycletime-quarter:before{content:\"\\e46d\"}.bosch-ic-gateway:before{content:\"\\e46e\"}.bosch-ic-heatmap:before{content:\"\\e46f\"}.bosch-ic-laptop-checklist:before{content:\"\\e470\"}.bosch-ic-molecule:before{content:\"\\e471\"}.bosch-ic-robot-settings:before{content:\"\\e472\"}.bosch-ic-sensor-cable:before{content:\"\\e473\"}.bosch-ic-tank-layers:before{content:\"\\e474\"}.bosch-ic-window-settings:before{content:\"\\e475\"}.bosch-ic-heatmap-independent:before{content:\"\\e476\"}.bosch-ic-dumble:before{content:\"\\e477\"}.bosch-ic-truck-wash:before{content:\"\\e478\"}.bosch-ic-smart-sensor:before{content:\"\\e479\"}.bosch-ic-audio-add:before{content:\"\\e47a\"}.bosch-ic-audio-checked:before{content:\"\\e47b\"}.bosch-ic-audio-play:before{content:\"\\e47c\"}.bosch-ic-audio-remove:before{content:\"\\e47d\"}.bosch-ic-image-gallery-checked:before{content:\"\\e47e\"}.bosch-ic-imagery-checked:before{content:\"\\e47f\"}.bosch-ic-image-gallery-remove:before{content:\"\\e480\"}.bosch-ic-recording:before{content:\"\\e481\"}.bosch-ic-stop-frame:before{content:\"\\e482\"}.bosch-ic-roof:before{content:\"\\e483\"}.bosch-ic-projection-interactive:before{content:\"\\e484\"}.bosch-ic-bridge:before{content:\"\\e485\"}.bosch-ic-box-arrows:before{content:\"\\e486\"}.bosch-ic-chart-line-fit:before{content:\"\\e487\"}.bosch-ic-chart-line-compress:before{content:\"\\e488\"}.bosch-ic-ciss:before{content:\"\\e489\"}.bosch-ic-scd:before{content:\"\\e48a\"}.bosch-ic-vci:before{content:\"\\e48b\"}.bosch-ic-access-controller:before{content:\"\\e48c\"}.bosch-ic-card-reader:before{content:\"\\e48d\"}.bosch-ic-table:before{content:\"\\e48e\"}.bosch-ic-replace:before{content:\"\\e48f\"}.bosch-ic-chart-topflop-organized-horizontal:before{content:\"\\e490\"}.bosch-ic-chart-topflop-random-horizontal:before{content:\"\\e491\"}.bosch-ic-chart-topflop-organized-vertical:before{content:\"\\e492\"}.bosch-ic-chart-topflop-random-vertical:before{content:\"\\e493\"}.bosch-ic-ois:before{content:\"\\e494\"}.bosch-ic-cloud-refresh-locked:before{content:\"\\e495\"}.bosch-ic-car-bluetooth:before{content:\"\\e496\"}.bosch-ic-camera-trainer:before{content:\"\\e497\"}.bosch-ic-magnifier-moving:before{content:\"\\e498\"}.bosch-ic-absolute-orientation:before{content:\"\\e499\"}.bosch-ic-accelerometer:before{content:\"\\e49a\"}.bosch-ic-imu:before{content:\"\\e49b\"}.bosch-ic-gyroscope:before{content:\"\\e49c\"}.bosch-ic-magneotmeter:before{content:\"\\e49d\"}.bosch-ic-battery-car-1:before{content:\"\\e49e\"}.bosch-ic-battery-car-2:before{content:\"\\e49f\"}.bosch-ic-battery-car-3:before{content:\"\\e4a0\"}.bosch-ic-battery-car-charging:before{content:\"\\e4a1\"}.bosch-ic-battery-car-defective:before{content:\"\\e4a2\"}.bosch-ic-truck-crane:before{content:\"\\e4a3\"}.bosch-ic-train-diesel:before{content:\"\\e4a4\"}.bosch-ic-snowmobile:before{content:\"\\e4a5\"}.bosch-ic-riding-mower:before{content:\"\\e4a6\"}.bosch-ic-motorcycle-small:before{content:\"\\e4a7\"}.bosch-ic-mining-truck:before{content:\"\\e4a8\"}.bosch-ic-lawnmower:before{content:\"\\e4a9\"}.bosch-ic-jetski:before{content:\"\\e4aa\"}.bosch-ic-golf-caddy:before{content:\"\\e4ab\"}.bosch-ic-generator-electric:before{content:\"\\e4ac\"}.bosch-ic-garbage-truck:before{content:\"\\e4ad\"}.bosch-ic-fork-lift-big:before{content:\"\\e4ae\"}.bosch-ic-engine-small:before{content:\"\\e4af\"}.bosch-ic-engine-big:before{content:\"\\e4b0\"}.bosch-ic-coach:before{content:\"\\e4b1\"}.bosch-ic-chainsaw:before{content:\"\\e4b2\"}.bosch-ic-car-van:before{content:\"\\e4b3\"}.bosch-ic-car-suv:before{content:\"\\e4b4\"}.bosch-ic-car-race:before{content:\"\\e4b5\"}.bosch-ic-car-old:before{content:\"\\e4b6\"}.bosch-ic-car-camping-van:before{content:\"\\e4b7\"}.bosch-ic-car-camper:before{content:\"\\e4b8\"}.bosch-ic-bus-side:before{content:\"\\e4b9\"}.bosch-ic-book-info:before{content:\"\\e4ba\"}.bosch-ic-gears:before{content:\"\\e4bb\"}.bosch-ic-shark:before{content:\"\\e4bc\"}.bosch-ic-mountaintop:before{content:\"\\e4bd\"}.bosch-ic-baby:before{content:\"\\e4be\"}.bosch-ic-coffee-bean:before{content:\"\\e4bf\"}.bosch-ic-coffee-beans:before{content:\"\\e4c0\"}.bosch-ic-skull:before{content:\"\\e4c1\"}.bosch-ic-stroller:before{content:\"\\e4c2\"}.bosch-ic-glasses:before{content:\"\\e4c3\"}.bosch-ic-detergent:before{content:\"\\e4c4\"}.bosch-ic-washing-machine:before{content:\"\\e4c5\"}.bosch-ic-sdcard:before{content:\"\\e4c6\"}.bosch-ic-water-connectivity:before{content:\"\\e4c7\"}.bosch-ic-server-connectivity:before{content:\"\\e4c8\"}.bosch-ic-presentation-speaker:before{content:\"\\e4c9\"}.bosch-ic-chart-bar-arrow:before{content:\"\\e4ca\"}.bosch-ic-book-connectivity:before{content:\"\\e4cb\"}.bosch-ic-route-focus:before{content:\"\\e4cc\"}.bosch-ic-bicycle-helmet:before{content:\"\\e4cd\"}.bosch-ic-cookie:before{content:\"\\e4ce\"}.bosch-ic-glass-wine:before{content:\"\\e4cf\"}.bosch-ic-cup-small:before{content:\"\\e4d0\"}.bosch-ic-cup-large:before{content:\"\\e4d1\"}.bosch-ic-bowl:before{content:\"\\e4d2\"}.bosch-ic-glass-small:before{content:\"\\e4d3\"}.bosch-ic-ice-bottle:before{content:\"\\e4d4\"}.bosch-ic-cup-plate-drop:before{content:\"\\e4d5\"}.bosch-ic-plate:before{content:\"\\e4d6\"}.bosch-ic-cup-plate:before{content:\"\\e4d7\"}.bosch-ic-bubbles:before{content:\"\\e4d8\"}.bosch-ic-button-start-stop:before{content:\"\\e4d9\"}.bosch-ic-elevator-freight-max:before{content:\"\\e4da\"}.bosch-ic-elevator-freight:before{content:\"\\e4db\"}.bosch-ic-elevator-persons:before{content:\"\\e4dc\"}.bosch-ic-embed:before{content:\"\\e4dd\"}.bosch-ic-female-male:before{content:\"\\e4de\"}.bosch-ic-fork-knife-spoon:before{content:\"\\e4df\"}.bosch-ic-global-entrepreneur:before{content:\"\\e4e0\"}.bosch-ic-arrows-left-right-checkmark:before{content:\"\\e4e1\"}.bosch-ic-automatic-mode:before{content:\"\\e4e2\"}.bosch-ic-babybottle:before{content:\"\\e4e3\"}.bosch-ic-barcode-scan:before{content:\"\\e4e4\"}.bosch-ic-barcode-scanner:before{content:\"\\e4e5\"}.bosch-ic-cash-frame-rupee:before{content:\"\\e4e6\"}.bosch-ic-chart-heat-curve:before{content:\"\\e4e7\"}.bosch-ic-compressor:before{content:\"\\e4e8\"}.bosch-ic-cubes:before{content:\"\\e4e9\"}.bosch-ic-highway:before{content:\"\\e4ea\"}.bosch-ic-industry-workers:before{content:\"\\e4eb\"}.bosch-ic-industry-workers-connected:before{content:\"\\e4ec\"}.bosch-ic-machine-station-control:before{content:\"\\e4ed\"}.bosch-ic-map-city:before{content:\"\\e4ee\"}.bosch-ic-ordernumber:before{content:\"\\e4ef\"}.bosch-ic-shirt-business:before{content:\"\\e4f0\"}.bosch-ic-shirt-casual:before{content:\"\\e4f1\"}.bosch-ic-suitcase:before{content:\"\\e4f2\"}.bosch-ic-weather-cloud-sun-sensor:before{content:\"\\e4f3\"}.bosch-ic-window-code:before{content:\"\\e4f4\"}.bosch-ic-wrench-warning:before{content:\"\\e4f5\"}.bosch-ic-arrow-circle-a:before{content:\"\\e4f6\"}.bosch-ic-arrows-agility:before{content:\"\\e4f7\"}.bosch-ic-arrows-collapse-object:before{content:\"\\e4f8\"}.bosch-ic-arrows-expand-object:before{content:\"\\e4f9\"}.bosch-ic-arrows-left-right:before{content:\"\\e4fa\"}.bosch-ic-fan-heat:before{content:\"\\e4fb\"}.bosch-ic-battery-car-arrows-circle:before{content:\"\\e4fc\"}.bosch-ic-battery-car-clock:before{content:\"\\e4fd\"}.bosch-ic-battery-vibration:before{content:\"\\e4fe\"}.bosch-ic-belt-elastic:before{content:\"\\e4ff\"}.bosch-ic-bicycle-e-arrow-away:before{content:\"\\e500\"}.bosch-ic-bicycle-e-arrow-towards:before{content:\"\\e501\"}.bosch-ic-brake-decoupled-power:before{content:\"\\e502\"}.bosch-ic-brake-i-booster:before{content:\"\\e503\"}.bosch-ic-brake-system-esp:before{content:\"\\e504\"}.bosch-ic-brake-system-integrated:before{content:\"\\e505\"}.bosch-ic-brake-vacuum-booster:before{content:\"\\e506\"}.bosch-ic-capacity:before{content:\"\\e507\"}.bosch-ic-car-beam-length:before{content:\"\\e508\"}.bosch-ic-car-smartphone-locator:before{content:\"\\e509\"}.bosch-ic-chart-pie:before{content:\"\\e50a\"}.bosch-ic-chess-jumper:before{content:\"\\e50b\"}.bosch-ic-clock-time-off:before{content:\"\\e50c\"}.bosch-ic-clock-time-on:before{content:\"\\e50d\"}.bosch-ic-coin:before{content:\"\\e50e\"}.bosch-ic-connection-off-settings:before{content:\"\\e50f\"}.bosch-ic-connection-on-settings:before{content:\"\\e510\"}.bosch-ic-control-unit:before{content:\"\\e511\"}.bosch-ic-customer:before{content:\"\\e512\"}.bosch-ic-engine:before{content:\"\\e513\"}.bosch-ic-filter-air:before{content:\"\\e514\"}.bosch-ic-filter-cabin-carbon:before{content:\"\\e515\"}.bosch-ic-filter-cabin:before{content:\"\\e516\"}.bosch-ic-filter-cooling-water:before{content:\"\\e517\"}.bosch-ic-filter-denox-cover:before{content:\"\\e518\"}.bosch-ic-filter-denox:before{content:\"\\e519\"}.bosch-ic-filter-element-compensation-sealing:before{content:\"\\e51a\"}.bosch-ic-filter-element-compensation:before{content:\"\\e51b\"}.bosch-ic-filter-element:before{content:\"\\e51c\"}.bosch-ic-filter-oil:before{content:\"\\e51d\"}.bosch-ic-filter-petrol:before{content:\"\\e51e\"}.bosch-ic-filter-prefilter:before{content:\"\\e51f\"}.bosch-ic-fireplace:before{content:\"\\e520\"}.bosch-ic-flash-plus:before{content:\"\\e521\"}.bosch-ic-globe-worldwideweb:before{content:\"\\e522\"}.bosch-ic-glow-control-unit:before{content:\"\\e523\"}.bosch-ic-glow-plug:before{content:\"\\e524\"}.bosch-ic-hand-cloth:before{content:\"\\e525\"}.bosch-ic-leaf-plus:before{content:\"\\e526\"}.bosch-ic-machine-warning:before{content:\"\\e527\"}.bosch-ic-measuring-jug:before{content:\"\\e528\"}.bosch-ic-plug-play:before{content:\"\\e529\"}.bosch-ic-refresh-checkmark:before{content:\"\\e52a\"}.bosch-ic-replacement-filament:before{content:\"\\e52b\"}.bosch-ic-screw:before{content:\"\\e52c\"}.bosch-ic-seat-belt:before{content:\"\\e52d\"}.bosch-ic-steering-wheel-hands-automated:before{content:\"\\e52e\"}.bosch-ic-sun-ice-parking:before{content:\"\\e52f\"}.bosch-ic-syringe-change-needle:before{content:\"\\e530\"}.bosch-ic-syringe-change:before{content:\"\\e531\"}.bosch-ic-syringe:before{content:\"\\e532\"}.bosch-ic-thread-diameter:before{content:\"\\e533\"}.bosch-ic-tool-extraction:before{content:\"\\e534\"}.bosch-ic-transporter-car-front:before{content:\"\\e535\"}.bosch-ic-truck-car-motorcycle:before{content:\"\\e536\"}.bosch-ic-truck-car:before{content:\"\\e537\"}.bosch-ic-truck-logistics-box:before{content:\"\\e538\"}.bosch-ic-truck-transporter:before{content:\"\\e539\"}.bosch-ic-watch-on-warning:before{content:\"\\e53a\"}.bosch-ic-wrench-calibrate:before{content:\"\\e53b\"}.bosch-ic-wrench-dummy:before{content:\"\\e53c\"}.bosch-ic-wrench-tightening-torque:before{content:\"\\e53d\"}.bosch-ic-wrench-width-across-flats:before{content:\"\\e53e\"}.bosch-ic-user-arm-bandage:before{content:\"\\e53f\"}.bosch-ic-user-eye-blind:before{content:\"\\e540\"}.bosch-ic-user-ear-deaf:before{content:\"\\e541\"}.bosch-ic-user-car-distracted:before{content:\"\\e542\"}.bosch-ic-user-mouth-mute:before{content:\"\\e543\"}.bosch-ic-user-ear-blocked:before{content:\"\\e544\"}.bosch-ic-user-bavarian:before{content:\"\\e545\"}.bosch-ic-user-ear-loud:before{content:\"\\e546\"}.bosch-ic-user-walk-crutch:before{content:\"\\e547\"}.bosch-ic-user-mouth-speechless:before{content:\"\\e548\"}.bosch-ic-user-arm-baby:before{content:\"\\e549\"}.bosch-ic-user-arm-missing:before{content:\"\\e54a\"}.bosch-ic-user-eye-eyepatch:before{content:\"\\e54b\"}.bosch-ic-user-strapped:before{content:\"\\e54c\"}.bosch-ic-cleaning-agent:before{content:\"\\e54d\"}.bosch-ic-cup-plate-dirty:before{content:\"\\e54e\"}.bosch-ic-cup-plate-private:before{content:\"\\e54f\"}.bosch-ic-rubbish-apple:before{content:\"\\e550\"}.bosch-ic-rubbish-fish:before{content:\"\\e551\"}.bosch-ic-rubbish-organic-bin:before{content:\"\\e552\"}.bosch-ic-rubbish-paper:before{content:\"\\e553\"}.bosch-ic-rubbish-paper-bin:before{content:\"\\e554\"}.bosch-ic-rubbish-residual-bin:before{content:\"\\e555\"}.bosch-ic-shower:before{content:\"\\e556\"}.bosch-ic-shower-female:before{content:\"\\e557\"}.bosch-ic-shower-male:before{content:\"\\e558\"}.bosch-ic-wardrobe-female:before{content:\"\\e559\"}.bosch-ic-wardrobe-male:before{content:\"\\e55a\"}.bosch-ic-car-top:before{content:\"\\e55b\"}.bosch-ic-label-info:before{content:\"\\e55c\"}.bosch-ic-perspective-view:before{content:\"\\e55d\"}.bosch-ic-road:before{content:\"\\e55e\"}.bosch-ic-screen-split-horizontal:before{content:\"\\e55f\"}.bosch-ic-screen-split-vertical:before{content:\"\\e560\"}.bosch-ic-screenshot-frame:before{content:\"\\e561\"}.bosch-ic-sensor-lidar:before{content:\"\\e562\"}.bosch-ic-tram:before{content:\"\\e563\"}.bosch-ic-video-360-degrees:before{content:\"\\e564\"}.bosch-ic-belt-groove-drives:before{content:\"\\e565\"}.bosch-ic-bottle-champagne:before{content:\"\\e566\"}.bosch-ic-box-falling:before{content:\"\\e567\"}.bosch-ic-box-falling-off-disabled-bold:before{content:\"\\e568\"}.bosch-ic-box-falling-off-disabled-light:before{content:\"\\e569\"}.bosch-ic-brake-pedal:before{content:\"\\e56a\"}.bosch-ic-car-front-light-inside:before{content:\"\\e56b\"}.bosch-ic-car-licence-plate-light:before{content:\"\\e56c\"}.bosch-ic-car-side-light-back:before{content:\"\\e56d\"}.bosch-ic-car-side-light-front:before{content:\"\\e56e\"}.bosch-ic-light-fog-warning:before{content:\"\\e56f\"}.bosch-ic-prism:before{content:\"\\e570\"}.bosch-ic-turnstile:before{content:\"\\e571\"}.bosch-ic-turnstile-open-in-and-outbound:before{content:\"\\e572\"}.bosch-ic-turnstile-open-inbound:before{content:\"\\e573\"}.bosch-ic-turnstile-open-outbound:before{content:\"\\e574\"}.bosch-ic-uwb:before{content:\"\\e575\"}.bosch-ic-heat-reduce-active:before{content:\"\\e576\"}.bosch-ic-heat-reduce-passive:before{content:\"\\e577\"}.bosch-ic-hourglass-add:before{content:\"\\e578\"}.bosch-ic-light-fog:before{content:\"\\e579\"}.bosch-ic-car-turn-signal:before{content:\"\\e57a\"}.bosch-ic-car-turn-signal-left:before{content:\"\\e57b\"}.bosch-ic-car-turn-signal-right:before{content:\"\\e57c\"}.bosch-ic-light-rear:before{content:\"\\e57d\"}.bosch-ic-car-top-turn-signal-front:before{content:\"\\e57e\"}.bosch-ic-car-top-turn-signal-back:before{content:\"\\e57f\"}.bosch-ic-ccu-cm:before{content:\"\\e580\"}.bosch-ic-lung:before{content:\"\\e581\"}.bosch-ic-head-speaking-voice:before{content:\"\\e582\"}.bosch-ic-car-motorcycle:before{content:\"\\e583\"}.bosch-ic-car-turn-signal-side:before{content:\"\\e584\"}.bosch-ic-car-light-parking:before{content:\"\\e585\"}.bosch-ic-phoenix:before{content:\"\\e586\"}.bosch-ic-sensor-air-quality:before{content:\"\\e587\"}.bosch-ic-weather-station:before{content:\"\\e588\"}.bosch-ic-watering-can:before{content:\"\\e589\"}.bosch-ic-propeller:before{content:\"\\e58a\"}.bosch-ic-temperature-fluctuating:before{content:\"\\e58b\"}.bosch-ic-stove-top:before{content:\"\\e58c\"}.bosch-ic-socket:before{content:\"\\e58d\"}.bosch-ic-siren:before{content:\"\\e58e\"}.bosch-ic-sensor-water:before{content:\"\\e58f\"}.bosch-ic-user-run-sensor-motion:before{content:\"\\e590\"}.bosch-ic-robot-lawnmower:before{content:\"\\e591\"}.bosch-ic-robot-cleaning:before{content:\"\\e592\"}.bosch-ic-oven:before{content:\"\\e593\"}.bosch-ic-loudspeaker:before{content:\"\\e594\"}.bosch-ic-keypad:before{content:\"\\e595\"}.bosch-ic-shape-circle-square:before{content:\"\\e596\"}.bosch-ic-garage:before{content:\"\\e597\"}.bosch-ic-fume-hood:before{content:\"\\e598\"}.bosch-ic-freezer:before{content:\"\\e599\"}.bosch-ic-food-processor:before{content:\"\\e59a\"}.bosch-ic-floorplan-rooms:before{content:\"\\e59b\"}.bosch-ic-dryer-tumble:before{content:\"\\e59c\"}.bosch-ic-dryer:before{content:\"\\e59d\"}.bosch-ic-door-window:before{content:\"\\e59e\"}.bosch-ic-door-lock:before{content:\"\\e59f\"}.bosch-ic-door-bell:before{content:\"\\e5a0\"}.bosch-ic-dishwasher:before{content:\"\\e5a1\"}.bosch-ic-dimmer:before{content:\"\\e5a2\"}.bosch-ic-detector-smoke:before{content:\"\\e5a3\"}.bosch-ic-detector-presence:before{content:\"\\e5a4\"}.bosch-ic-cooking:before{content:\"\\e5a5\"}.bosch-ic-coffee-maschine:before{content:\"\\e5a6\"}.bosch-ic-camera-indoor:before{content:\"\\e5a7\"}.bosch-ic-broom-clean:before{content:\"\\e5a8\"}.bosch-ic-blinds:before{content:\"\\e5a9\"}.bosch-ic-bicycle-e-flash:before{content:\"\\e5aa\"}.bosch-ic-finger-domino-trigger:before{content:\"\\e5ab\"}.bosch-ic-automation-start:before{content:\"\\e5ac\"}.bosch-ic-automation:before{content:\"\\e5ad\"}.bosch-ic-sensor-multifunctional:before{content:\"\\e5ae\"}.bosch-ic-plane-top:before{content:\"\\e5af\"}.bosch-ic-shape-square-overlapping:before{content:\"\\e5b0\"}.bosch-ic-square-overlapping:before{content:\"\\e5b1\"}.bosch-ic-battery-car-14v:before{content:\"\\e5b2\"}.bosch-ic-calibration:before{content:\"\\e5b3\"}.bosch-ic-structure-can-bus:before{content:\"\\e5b4\"}.bosch-ic-gears-direction-arrows:before{content:\"\\e5b5\"}.bosch-ic-car-limousine:before{content:\"\\e5b6\"}.bosch-ic-molecule-gas:before{content:\"\\e5b7\"}.bosch-ic-money-euro:before{content:\"\\e5b8\"}.bosch-ic-revolution-rpm:before{content:\"\\e5b9\"}.bosch-ic-rikscha-e:before{content:\"\\e5ba\"}.bosch-ic-sensor-circle:before{content:\"\\e5bb\"}.bosch-ic-speedometer-high:before{content:\"\\e5bc\"}.bosch-ic-steering-wheel-arrows:before{content:\"\\e5bd\"}.bosch-ic-tablet-checklist:before{content:\"\\e5be\"}.bosch-ic-petrol-charging-station-hybrid:before{content:\"\\e5bf\"}.bosch-ic-weather-cloud-sun-rain-snow:before{content:\"\\e5c0\"}.bosch-ic-light-half:before{content:\"\\e5c1\"}.bosch-ic-video-sensor:before{content:\"\\e5c2\"}.bosch-ic-binary-desktop:before{content:\"\\e5c3\"}.bosch-ic-truck-delivery:before{content:\"\\e5c4\"}.bosch-ic-van-delivery:before{content:\"\\e5c5\"}.bosch-ic-fullscreen-user:before{content:\"\\e5c6\"}.bosch-ic-user-helmet-motorcycle:before{content:\"\\e5c7\"}.bosch-ic-user-hand-disabled-light:before{content:\"\\e5c8\"}.bosch-ic-user-hand-disabled-bold:before{content:\"\\e5c9\"}.bosch-ic-link-connected:before{content:\"\\e5ca\"}.bosch-ic-gate-open:before{content:\"\\e5cb\"}.bosch-ic-box-checkmark:before{content:\"\\e5cc\"}.bosch-ic-box-place:before{content:\"\\e5cd\"}.bosch-ic-logistics-dock:before{content:\"\\e5ce\"}.bosch-ic-barcode-scan-logistics:before{content:\"\\e5cf\"}.bosch-ic-locator-tracking:before{content:\"\\e5d0\"}.bosch-ic-car-light-trunk:before{content:\"\\e5d1\"}.bosch-ic-car-light-glove-compartment:before{content:\"\\e5d2\"}.bosch-ic-car-light-dashboard:before{content:\"\\e5d3\"}.bosch-ic-fire-leaf:before{content:\"\\e5d4\"}.bosch-ic-laptop-hacker:before{content:\"\\e5d5\"}.bosch-ic-hands-drops:before{content:\"\\e5d6\"}.bosch-ic-team-disabled-bold:before{content:\"\\e5d7\"}.bosch-ic-team-disabled-light:before{content:\"\\e5d8\"}.bosch-ic-user-head-arm:before{content:\"\\e5d9\"}.bosch-ic-user-head-hand-disabled-bold:before{content:\"\\e5da\"}.bosch-ic-user-head-hand-disabled-light:before{content:\"\\e5db\"}.bosch-ic-welcome-disabled-bold:before{content:\"\\e5dc\"}.bosch-ic-welcome-disabled-light:before{content:\"\\e5dd\"}.bosch-ic-female-male-arrows:before{content:\"\\e5de\"}.bosch-ic-broccoli:before{content:\"\\e5df\"}.bosch-ic-chicken:before{content:\"\\e5e0\"}.bosch-ic-cow:before{content:\"\\e5e1\"}.bosch-ic-deer:before{content:\"\\e5e2\"}.bosch-ic-fish:before{content:\"\\e5e3\"}.bosch-ic-garlic:before{content:\"\\e5e4\"}.bosch-ic-hotpepper:before{content:\"\\e5e5\"}.bosch-ic-pig:before{content:\"\\e5e6\"}.bosch-ic-rabbit:before{content:\"\\e5e7\"}.bosch-ic-sheep:before{content:\"\\e5e8\"}.bosch-ic-shell:before{content:\"\\e5e9\"}.bosch-ic-shrimp:before{content:\"\\e5ea\"}.bosch-ic-car-seat:before{content:\"\\e5eb\"}.bosch-ic-car-seat-steering-wheel:before{content:\"\\e5ec\"}.bosch-ic-car-side-arrow-back:before{content:\"\\e5ed\"}.bosch-ic-car-side-arrow-front:before{content:\"\\e5ee\"}.bosch-ic-car-steering-left-disabled-bold:before{content:\"\\e5ef\"}.bosch-ic-car-steering-left-disabled-light:before{content:\"\\e5f0\"}.bosch-ic-car-steering-left-lle:before{content:\"\\e5f1\"}.bosch-ic-car-steering-left-lle-disabled-bold:before{content:\"\\e5f2\"}.bosch-ic-car-steering-left-lle-disabled-light:before{content:\"\\e5f3\"}.bosch-ic-car-steering-right-disabled-bold:before{content:\"\\e5f4\"}.bosch-ic-car-steering-right-disabled-light:before{content:\"\\e5f5\"}.bosch-ic-car-steering-right-rle:before{content:\"\\e5f6\"}.bosch-ic-car-steering-right-rle-disabled-bold:before{content:\"\\e5f7\"}.bosch-ic-car-steering-right-rle-disabled-light:before{content:\"\\e5f8\"}.bosch-ic-cars-traffic-left-right:before{content:\"\\e5f9\"}.bosch-ic-configuration-wrench:before{content:\"\\e5fa\"}.bosch-ic-hook:before{content:\"\\e5fb\"}.bosch-ic-hook-arrows:before{content:\"\\e5fc\"}.bosch-ic-hook-arrows-wiperblade:before{content:\"\\e5fd\"}.bosch-ic-hook-disabled-bold:before{content:\"\\e5fe\"}.bosch-ic-hook-disabled-light:before{content:\"\\e5ff\"}.bosch-ic-info-i-frame-add:before{content:\"\\e600\"}.bosch-ic-search-number:before{content:\"\\e601\"}.bosch-ic-search-number-add:before{content:\"\\e602\"}.bosch-ic-set:before{content:\"\\e603\"}.bosch-ic-speedometer-speed-high:before{content:\"\\e604\"}.bosch-ic-windshield-wiper-clean:before{content:\"\\e605\"}.bosch-ic-windshield-wiper-double:before{content:\"\\e606\"}.bosch-ic-wiper-adapter:before{content:\"\\e607\"}.bosch-ic-wiper-blade:before{content:\"\\e608\"}.bosch-ic-wiper-blade-arrow:before{content:\"\\e609\"}.bosch-ic-wiper-blade-eco:before{content:\"\\e60a\"}.bosch-ic-wiper-blade-mm:before{content:\"\\e60b\"}.bosch-ic-wiper-blade-mm-arrows:before{content:\"\\e60c\"}.bosch-ic-wiper-blade-spoiler:before{content:\"\\e60d\"}.bosch-ic-wiper-blade-twin:before{content:\"\\e60e\"}.bosch-ic-wiper-flatblade:before{content:\"\\e60f\"}.bosch-ic-wiper-flatblade-aeroeco:before{content:\"\\e610\"}.bosch-ic-wiper-flatblade-aerofit:before{content:\"\\e611\"}.bosch-ic-wiper-flatblade-aerotwin:before{content:\"\\e612\"}.bosch-ic-wiper-flatblade-arrow:before{content:\"\\e613\"}.bosch-ic-wiper-flatblade-mm:before{content:\"\\e614\"}.bosch-ic-wiper-flatblade-mm-arrows:before{content:\"\\e615\"}.bosch-ic-wiper-rubber:before{content:\"\\e616\"}.bosch-ic-address-control-system:before{content:\"\\e617\"}.bosch-ic-balcony:before{content:\"\\e618\"}.bosch-ic-battery-car-45-degree-arrow:before{content:\"\\e619\"}.bosch-ic-boiler-warning:before{content:\"\\e61a\"}.bosch-ic-book-wrench:before{content:\"\\e61b\"}.bosch-ic-car-driver-passenger-arrows:before{content:\"\\e61c\"}.bosch-ic-car-licence-plate:before{content:\"\\e61d\"}.bosch-ic-car-side-arrow-back-tires:before{content:\"\\e61e\"}.bosch-ic-car-side-arrow-front-tires:before{content:\"\\e61f\"}.bosch-ic-car-side-half-arrow-back:before{content:\"\\e620\"}.bosch-ic-car-steering-left-arrow:before{content:\"\\e621\"}.bosch-ic-car-steering-left-passenger-arrow:before{content:\"\\e622\"}.bosch-ic-car-steering-right-arrow:before{content:\"\\e623\"}.bosch-ic-car-steering-right-passenger-arrow:before{content:\"\\e624\"}.bosch-ic-chess-board-transparency:before{content:\"\\e625\"}.bosch-ic-cloud-locked:before{content:\"\\e626\"}.bosch-ic-conference-system:before{content:\"\\e627\"}.bosch-ic-cycletime-quarter-arrows-interval:before{content:\"\\e628\"}.bosch-ic-desktop-connectivity:before{content:\"\\e629\"}.bosch-ic-hierarchy-checkmark:before{content:\"\\e62a\"}.bosch-ic-home-office:before{content:\"\\e62b\"}.bosch-ic-home-semi-detached-house:before{content:\"\\e62c\"}.bosch-ic-kitchen:before{content:\"\\e62d\"}.bosch-ic-lambda-arrows-length:before{content:\"\\e62e\"}.bosch-ic-lambda-oxygen:before{content:\"\\e62f\"}.bosch-ic-lambda-sensor-cores:before{content:\"\\e630\"}.bosch-ic-panel-control:before{content:\"\\e631\"}.bosch-ic-prison:before{content:\"\\e632\"}.bosch-ic-prison-cell:before{content:\"\\e633\"}.bosch-ic-radio-broadcasting:before{content:\"\\e634\"}.bosch-ic-robot-body:before{content:\"\\e635\"}.bosch-ic-robot-body-arrow:before{content:\"\\e636\"}.bosch-ic-skyscraper-danger:before{content:\"\\e637\"}.bosch-ic-skyscraper-flat:before{content:\"\\e638\"}.bosch-ic-skyscraper-flat-floor-low:before{content:\"\\e639\"}.bosch-ic-skyscraper-flat-floor-middle:before{content:\"\\e63a\"}.bosch-ic-skyscraper-flat-floor-top:before{content:\"\\e63b\"}.bosch-ic-skyscraper-flat-semi-detached:before{content:\"\\e63c\"}.bosch-ic-skyscraper-locked:before{content:\"\\e63d\"}.bosch-ic-skyscraper-settings:before{content:\"\\e63e\"}.bosch-ic-team-signpost:before{content:\"\\e63f\"}.bosch-ic-toaster-hand-warning:before{content:\"\\e640\"}.bosch-ic-user-arrows-heart:before{content:\"\\e641\"}.bosch-ic-user-graduate-chat:before{content:\"\\e642\"}.bosch-ic-user-head-mask:before{content:\"\\e643\"}.bosch-ic-warehouse:before{content:\"\\e644\"}.bosch-ic-world-paperplane:before{content:\"\\e645\"}.bosch-ic-aperture:before{content:\"\\e646\"}.bosch-ic-hook-wiperblade-arrows:before{content:\"\\e647\"}.bosch-ic-image-remove:before{content:\"\\e648\"}.bosch-ic-image-checked:before{content:\"\\e649\"}.bosch-ic-user-setting:before{content:\"\\e45d\"}.bosch-ic-apartment-in-house-terraced:before{content:\"\\e64b\"}.bosch-ic-apartment-in-house:before{content:\"\\e64c\"}.bosch-ic-appliance-heat-pump-leaf-add:before{content:\"\\e64d\"}.bosch-ic-appliance-heat-pump-leaf-reload:before{content:\"\\e64e\"}.bosch-ic-appliance-heat-pump-oil-refresh:before{content:\"\\e64f\"}.bosch-ic-appliance-water-drop:before{content:\"\\e650\"}.bosch-ic-appliance-water-sun:before{content:\"\\e651\"}.bosch-ic-brake-pedal-thumb-up:before{content:\"\\e652\"}.bosch-ic-call-plus:before{content:\"\\e653\"}.bosch-ic-battery-car-3-plug:before{content:\"\\e654\"}.bosch-ic-cloud-share:before{content:\"\\e655\"}.bosch-ic-crankshaft:before{content:\"\\e656\"}.bosch-ic-detector-motion:before{content:\"\\e657\"}.bosch-ic-device-analog:before{content:\"\\e658\"}.bosch-ic-device-digital:before{content:\"\\e659\"}.bosch-ic-externallink-landscape:before{content:\"\\e65a\"}.bosch-ic-gears-interlocked-arrow:before{content:\"\\e65b\"}.bosch-ic-gearshift-reverse-gear:before{content:\"\\e65c\"}.bosch-ic-glove:before{content:\"\\e65d\"}.bosch-ic-hand-warranty:before{content:\"\\e65e\"}.bosch-ic-heating-floor-radiator:before{content:\"\\e65f\"}.bosch-ic-home-appliance-heat-pump-air-in:before{content:\"\\e660\"}.bosch-ic-home-appliance-heat-pump-air-out:before{content:\"\\e661\"}.bosch-ic-lightbulb-gear:before{content:\"\\e662\"}.bosch-ic-line-open-closed:before{content:\"\\e663\"}.bosch-ic-road-disabled-bold:before{content:\"\\e664\"}.bosch-ic-road-disabled-light:before{content:\"\\e665\"}.bosch-ic-shower-drop-1:before{content:\"\\e666\"}.bosch-ic-shower-drop-2:before{content:\"\\e667\"}.bosch-ic-shower-drop-3:before{content:\"\\e668\"}.bosch-ic-shower-drop-4:before{content:\"\\e669\"}.bosch-ic-sinus-curve:before{content:\"\\e66a\"}.bosch-ic-team-3:before{content:\"\\e66b\"}.bosch-ic-team-4:before{content:\"\\e66c\"}.bosch-ic-team-5:before{content:\"\\e66d\"}.bosch-ic-test-tube:before{content:\"\\e66e\"}.bosch-ic-truck-car-motorcycle-arrows:before{content:\"\\e66f\"}.bosch-ic-weather-cloud-dust:before{content:\"\\e670\"}.bosch-ic-windshield-wiper-double-parallel-left:before{content:\"\\e671\"}.bosch-ic-windshield-wiper-double-parallel-right:before{content:\"\\e672\"}.bosch-ic-world-pin:before{content:\"\\e673\"}.bosch-ic-sign-warning-slope:before{content:\"\\e674\"}.bosch-ic-facebook:before{content:\"\\ea99\"}.bosch-ic-google-frame:before{content:\"\\ea9a\"}.bosch-ic-google:before{content:\"\\ea9b\"}.bosch-ic-googleplus:before{content:\"\\ea9c\"}.bosch-ic-instagram:before{content:\"\\ea9d\"}.bosch-ic-lineapp:before{content:\"\\ea9e\"}.bosch-ic-linkedin:before{content:\"\\ea9f\"}.bosch-ic-pinterest:before{content:\"\\eaa0\"}.bosch-ic-skype:before{content:\"\\eaa1\"}.bosch-ic-snapchat:before{content:\"\\eaa2\"}.bosch-ic-tumblr:before{content:\"\\eaa3\"}.bosch-ic-twitter:before{content:\"\\eaa4\"}.bosch-ic-vimeo:before{content:\"\\eaa5\"}.bosch-ic-vk:before{content:\"\\eaa6\"}.bosch-ic-wechat:before{content:\"\\eaa7\"}.bosch-ic-weibo:before{content:\"\\eaa8\"}.bosch-ic-whatsapp:before{content:\"\\eaa9\"}.bosch-ic-xing:before{content:\"\\eaaa\"}.bosch-ic-youku:before{content:\"\\eaab\"}.bosch-ic-youtube:before{content:\"\\eaac\"}.bosch-ic-alert-info-filled:before{content:\"\\eb0e\"}.bosch-ic-alert-info:before{content:\"\\eb0f\"}.bosch-ic-info-i-frame:before{content:\"\\e8ab\"}.bosch-ic-stop-frame1:before{content:\"\\e8ac\"}.bosch-ic-arrow-up-frame-error1:before{content:\"\\e8ad\"}.bosch-ic-arrow-up-frame-check1:before{content:\"\\e8ae\"}.bosch-ic-arrow-left-end-frame1:before{content:\"\\e8af\"}.bosch-ic-arrow-right-end-frame1:before{content:\"\\e8b0\"}.bosch-ic-badge-checkmark:before{content:\"\\e3ee\"}.bosch-ic-warranty:before{content:\"\\e2b1\"}.bosch-ic-bond-laser:before{content:\"\\e675\"}.bosch-ic-bond-ribbon-thick:before{content:\"\\e676\"}.bosch-ic-bond-ribbon-thin:before{content:\"\\e677\"}.bosch-ic-bond-wire-thick:before{content:\"\\e678\"}.bosch-ic-bond-wire-thin:before{content:\"\\e679\"}.bosch-ic-clamp-connection-insulation:before{content:\"\\e67a\"}.bosch-ic-clamp-connection-stamping-grid:before{content:\"\\e67b\"}.bosch-ic-clinching:before{content:\"\\e67c\"}.bosch-ic-connector:before{content:\"\\e67d\"}.bosch-ic-connector-potting:before{content:\"\\e67e\"}.bosch-ic-element-damping:before{content:\"\\e67f\"}.bosch-ic-element-pressure-compensation:before{content:\"\\e680\"}.bosch-ic-flanging:before{content:\"\\e681\"}.bosch-ic-gap-pad:before{content:\"\\e682\"}.bosch-ic-heat-sink:before{content:\"\\e683\"}.bosch-ic-hook-clipsing:before{content:\"\\e684\"}.bosch-ic-interconnection-board-to-board:before{content:\"\\e685\"}.bosch-ic-labyrinth-geometry:before{content:\"\\e686\"}.bosch-ic-lacquering-pcb:before{content:\"\\e687\"}.bosch-ic-magnetic-rotor:before{content:\"\\e688\"}.bosch-ic-magnetic-switch:before{content:\"\\e689\"}.bosch-ic-pin-pressfit:before{content:\"\\e68a\"}.bosch-ic-power-contact:before{content:\"\\e68b\"}.bosch-ic-rivet:before{content:\"\\e68c\"}.bosch-ic-seal-ring:before{content:\"\\e68d\"}.bosch-ic-sealing-cipg:before{content:\"\\e68e\"}.bosch-ic-sealing-solid:before{content:\"\\e68f\"}.bosch-ic-sealing-tongue-groove:before{content:\"\\e690\"}.bosch-ic-shrinking-arrows-circle:before{content:\"\\e691\"}.bosch-ic-solder-selective:before{content:\"\\e692\"}.bosch-ic-spring-mechanical:before{content:\"\\e693\"}.bosch-ic-staking-hot-air:before{content:\"\\e694\"}.bosch-ic-stitching:before{content:\"\\e695\"}.bosch-ic-thermal-inlay:before{content:\"\\e696\"}.bosch-ic-thermal-via:before{content:\"\\e697\"}.bosch-ic-tube:before{content:\"\\e698\"}.bosch-ic-tube-gapfiller:before{content:\"\\e699\"}.bosch-ic-tube-gelling:before{content:\"\\e69a\"}.bosch-ic-tube-gluing:before{content:\"\\e69b\"}.bosch-ic-tube-thermal-adhesive:before{content:\"\\e69c\"}.bosch-ic-tube-underfill:before{content:\"\\e69d\"}.bosch-ic-welding-laser:before{content:\"\\e69e\"}.bosch-ic-welding-resistance:before{content:\"\\e69f\"}.bosch-ic-welding-ultrasonic:before{content:\"\\e6a0\"}.bosch-ic-air-condition-cassette:before{content:\"\\e6a1\"}.bosch-ic-air-condition-ceiling:before{content:\"\\e6a2\"}.bosch-ic-air-condition-convertible:before{content:\"\\e6a3\"}.bosch-ic-air-condition-duct:before{content:\"\\e6a4\"}.bosch-ic-air-condition-floor:before{content:\"\\e6a5\"}.bosch-ic-air-condition-outdoor:before{content:\"\\e6a6\"}.bosch-ic-air-condition-outdoor-hp:before{content:\"\\e6a7\"}.bosch-ic-air-condition-outdoor-hr:before{content:\"\\e6a8\"}.bosch-ic-air-condition-swing-horizontal:before{content:\"\\e6a9\"}.bosch-ic-air-condition-swing-vertical:before{content:\"\\e6aa\"}.bosch-ic-air-condition-wall:before{content:\"\\e6ab\"}.bosch-ic-angle-curvature-motorcycle:before{content:\"\\e6ac\"}.bosch-ic-appliance-hot:before{content:\"\\e6ad\"}.bosch-ic-appliance-water-hot:before{content:\"\\e6ae\"}.bosch-ic-appliances-hot:before{content:\"\\e6af\"}.bosch-ic-arrows-left-right-a:before{content:\"\\e6b0\"}.bosch-ic-arrows-left-right-exchange:before{content:\"\\e6b1\"}.bosch-ic-arrows-left-right-exchange-unit:before{content:\"\\e6b2\"}.bosch-ic-arrows-left-right-ice:before{content:\"\\e6b3\"}.bosch-ic-assembly-line-robot:before{content:\"\\e6b4\"}.bosch-ic-back-left-small:before{content:\"\\e6b5\"}.bosch-ic-bacteria:before{content:\"\\e6b6\"}.bosch-ic-battery-car-drop-leakage-proof:before{content:\"\\e6b7\"}.bosch-ic-battery-car-hourglass:before{content:\"\\e6b8\"}.bosch-ic-brake-disk-arrow:before{content:\"\\e6b9\"}.bosch-ic-brake-disk-compound:before{content:\"\\e6ba\"}.bosch-ic-building-religious-church:before{content:\"\\e6bb\"}.bosch-ic-calendar-sheet:before{content:\"\\e6bc\"}.bosch-ic-caliper:before{content:\"\\e6bd\"}.bosch-ic-car-middle-arrow:before{content:\"\\e6be\"}.bosch-ic-checkmark-bold:before{content:\"\\e6bf\"}.bosch-ic-close-small:before{content:\"\\e6c0\"}.bosch-ic-data-center:before{content:\"\\e6c1\"}.bosch-ic-dot:before{content:\"\\e6c2\"}.bosch-ic-down-small:before{content:\"\\e6c3\"}.bosch-ic-e-call:before{content:\"\\e6c4\"}.bosch-ic-fire-extinguisher:before{content:\"\\e6c5\"}.bosch-ic-fire-off-bold:before{content:\"\\e6c6\"}.bosch-ic-fire-off-light:before{content:\"\\e6c7\"}.bosch-ic-forward-right-small:before{content:\"\\e6c8\"}.bosch-ic-health-warning:before{content:\"\\e6c9\"}.bosch-ic-ignition-coil-pencil:before{content:\"\\e6ca\"}.bosch-ic-ignition-condensor:before{content:\"\\e6cb\"}.bosch-ic-ignition-contact-set:before{content:\"\\e6cc\"}.bosch-ic-ignition-distributor-cap:before{content:\"\\e6cd\"}.bosch-ic-ignition-distributor-rotor:before{content:\"\\e6ce\"}.bosch-ic-ignition-module:before{content:\"\\e6cf\"}.bosch-ic-less-minimize-bold:before{content:\"\\e6d0\"}.bosch-ic-leaf-plus-boost:before{content:\"\\e6d1\"}.bosch-ic-sensor-dynamic:before{content:\"\\e6d2\"}.bosch-ic-sensor-radar:before{content:\"\\e6d3\"}.bosch-ic-up-small:before{content:\"\\e6d4\"}.bosch-ic-water-ladder-off-bold:before{content:\"\\e6d5\"}.bosch-ic-water-ladder-off-light:before{content:\"\\e6d6\"}.bosch-ic-nose-desodorize:before{content:\"\\e6d7\"}.bosch-ic-nose-fine-dust:before{content:\"\\e6d8\"}.bosch-ic-smartcity-camera:before{content:\"\\e6d9\"}.bosch-ic-heating-underfloor:before{content:\"\\e6da\"}.bosch-ic-drill-bit:before{content:\"\\e6db\"}.bosch-ic-parameter-process:before{content:\"\\e6dc\"}.bosch-ic-car-side-plug-flash:before{content:\"\\e6dd\"}.bosch-ic-car-side-plug-engine-flash:before{content:\"\\e6de\"}.bosch-ic-car-side-engine-flash:before{content:\"\\e6df\"}.bosch-ic-car-side-hydrogen-flash:before{content:\"\\e6e0\"}.bosch-ic-car-side-engine:before{content:\"\\e6e1\"}.bosch-ic-tunnel:before{content:\"\\e6e2\"}.bosch-ic-stadium-sport:before{content:\"\\e6e3\"}.bosch-ic-refinery-oil:before{content:\"\\e6e4\"}.bosch-ic-power-plants:before{content:\"\\e6e5\"}.bosch-ic-hammer-chisel:before{content:\"\\e6e6\"}.bosch-ic-glass-cocktail:before{content:\"\\e6e7\"}.bosch-ic-traffic-light:before{content:\"\\e6e8\"}.bosch-ic-vehicle-off-road:before{content:\"\\e6e9\"}.bosch-ic-tablet-finger:before{content:\"\\e6ea\"}.bosch-ic-scooter-kick:before{content:\"\\e6eb\"}.bosch-ic-handlebar-motorcycle:before{content:\"\\e6ec\"}.bosch-ic-display-motorcycle:before{content:\"\\e6ed\"}.bosch-ic-case-label-product:before{content:\"\\e6ee\"}.bosch-ic-case-label:before{content:\"\\e6ef\"}.bosch-ic-product-falling:before{content:\"\\e6f0\"}.bosch-ic-product-checkmark:before{content:\"\\e6f1\"}.bosch-ic-product-broken:before{content:\"\\e6f2\"}.bosch-ic-lightbulb-halogen-rays-bright:before{content:\"\\e6f3\"}.bosch-ic-lightbulb-halogen-rays:before{content:\"\\e6f4\"}.bosch-ic-lamp-gas-discharge-rays:before{content:\"\\e6f5\"}.bosch-ic-home-lightbulb-off-disabled-bold:before{content:\"\\e6f6\"}.bosch-ic-home-lightbulb-off-disabled-light:before{content:\"\\e6f7\"}.bosch-ic-fork-lift-boxes:before{content:\"\\e6f8\"}.bosch-ic-harbour-ship-cargo:before{content:\"\\e6f9\"}.bosch-ic-rollercoaster:before{content:\"\\e6fa\"}.bosch-ic-sound-off-light:before{content:\"\\e6fb\"}.bosch-ic-sound-off-bold:before{content:\"\\e6fc\"}.bosch-ic-car-door:before{content:\"\\e6fd\"}.bosch-ic-car-door-tailgate:before{content:\"\\e6fe\"}.bosch-ic-car-door-split-rear:before{content:\"\\e6ff\"}.bosch-ic-car-door-rear:before{content:\"\\e700\"}.bosch-ic-drill-driver-cordless:before{content:\"\\e701\"}.bosch-ic-virus:before{content:\"\\e702\"}.bosch-ic-ice-temperature:before{content:\"\\e703\"}.bosch-ic-sensor-ultrasonic:before{content:\"\\e704\"}.bosch-ic-user-artificial-intelligence:before{content:\"\\e705\"}.bosch-ic-detector-leak:before{content:\"\\e706\"}.bosch-ic-car-seat-add:before{content:\"\\e707\"}.bosch-ic-car-shuttle-robo:before{content:\"\\e708\"}.bosch-ic-caravan:before{content:\"\\e709\"}.bosch-ic-user-water-swimming:before{content:\"\\e70a\"}.bosch-ic-square-add:before{content:\"\\e70b\"}.bosch-ic-map-zoom-in:before{content:\"\\e70c\"}.bosch-ic-map-arrow-pop-out:before{content:\"\\e70d\"}.bosch-ic-home-user-in-error:before{content:\"\\e70e\"}.bosch-ic-home-arrow-right-in-denied:before{content:\"\\e70f\"}.bosch-ic-home-arrow-right-in-check:before{content:\"\\e710\"}.bosch-ic-home-arrow-right-in:before{content:\"\\e711\"}.bosch-ic-home-arrow-left-out:before{content:\"\\e712\"}.bosch-ic-file-error:before{content:\"\\e713\"}.bosch-ic-dealer-details-missing:before{content:\"\\e714\"}.bosch-ic-dealer-details-check:before{content:\"\\e715\"}.bosch-ic-components-add:before{content:\"\\e716\"}.bosch-ic-machine-cell:before{content:\"\\e717\"}.bosch-ic-wheel-rim-clean:before{content:\"\\e718\"}.bosch-ic-gas-can:before{content:\"\\e719\"}.bosch-ic-steps-locator:before{content:\"\\e71a\"}.bosch-ic-panel-control-manual:before{content:\"\\e71b\"}.bosch-ic-socket-europe:before{content:\"\\e71c\"}.bosch-ic-tripod:before{content:\"\\e71d\"}.bosch-ic-document-dxf:before{content:\"\\e71e\"}.bosch-ic-document-dwg:before{content:\"\\e71f\"}.bosch-ic-hook-wiperblade:before{content:\"\\e720\"}.bosch-ic-traffic-cone:before{content:\"\\e721\"}.bosch-ic-folder-locked:before{content:\"\\e722\"}.bosch-ic-package-self-service:before{content:\"\\e723\"}.bosch-ic-drop-ice:before{content:\"\\e724\"}.bosch-ic-lamp-gas-discharge:before{content:\"\\e725\"}.bosch-ic-home-basement:before{content:\"\\e726\"}.bosch-ic-link-edit:before{content:\"\\e727\"}.bosch-ic-table-header-add:before{content:\"\\e728\"}.bosch-ic-table-column-add-after:before{content:\"\\e729\"}.bosch-ic-table-row-add-above:before{content:\"\\e72a\"}.bosch-ic-table-column-add-before:before{content:\"\\e72b\"}.bosch-ic-table-row-add-below:before{content:\"\\e72c\"}.bosch-ic-table-cells-split:before{content:\"\\e72d\"}.bosch-ic-table-cells-merge:before{content:\"\\e72e\"}.bosch-ic-table-row-delete:before{content:\"\\e72f\"}.bosch-ic-text-indent-arrow:before{content:\"\\e730\"}.bosch-ic-text-outdent-arrow:before{content:\"\\e731\"}.bosch-ic-car-side-flash:before{content:\"\\e732\"}.bosch-ic-battery-plug-flash:before{content:\"\\e733\"}.bosch-ic-car-chip:before{content:\"\\e734\"}.bosch-ic-car-cloud-upload:before{content:\"\\e735\"}.bosch-ic-circle-parking:before{content:\"\\e736\"}.bosch-ic-engine-battery-flash:before{content:\"\\e737\"}.bosch-ic-engine-battery-flash-plug:before{content:\"\\e738\"}.bosch-ic-engine-combustion:before{content:\"\\e739\"}.bosch-ic-user-head-scan:before{content:\"\\e73a\"}.bosch-ic-bluetooth-off-disabled-bold:before{content:\"\\e73b\"}.bosch-ic-bluetooth-off-disabled-light:before{content:\"\\e73c\"}.bosch-ic-communicator-off-disabled-bold:before{content:\"\\e73d\"}.bosch-ic-communicator-off-disabled-light:before{content:\"\\e73e\"}.bosch-ic-connectivity-off-disabled-bold:before{content:\"\\e73f\"}.bosch-ic-connectivity-off-disabled-light:before{content:\"\\e740\"}.bosch-ic-globe-off-disabled-bold:before{content:\"\\e741\"}.bosch-ic-globe-off-disabled-light:before{content:\"\\e742\"}.bosch-ic-lamp-led:before{content:\"\\e743\"}.bosch-ic-wifi-off-disabled-bold:before{content:\"\\e744\"}.bosch-ic-wifi-off-disabled-light:before{content:\"\\e745\"}.bosch-ic-wheel-vehicle:before{content:\"\\e746\"}.bosch-ic-ignition-cable:before{content:\"\\e747\"}.bosch-ic-ignition-coil:before{content:\"\\e748\"}.bosch-ic-ignition-distributor:before{content:\"\\e749\"}.bosch-ic-microphone-classic-off-disabled-light:before{content:\"\\e74a\"}.bosch-ic-microphone-classic-off-disabled-bold:before{content:\"\\e74b\"}.bosch-ic-cloud-refresh:before{content:\"\\e74c\"}.bosch-ic-ignition-coil-module:before{content:\"\\e74d\"}.bosch-ic-bluetooth-frame:before{content:\"\\e8b1\"}.bosch-ic-bluetooth-frame-error:before{content:\"\\e8b2\"}.bosch-ic-ece-ccc-frame:before{content:\"\\e8b3\"}.bosch-ic-ms-teams:before{content:\"\\eaad\"}.bosch-ic-ms-teams-frame:before{content:\"\\eaae\"}.bosch-ic-mold:before{content:\"\\f0000\"}.bosch-ic-battery-car-cranking-power-high:before{content:\"\\f0001\"}.bosch-ic-brake-disk-spray-can-coated:before{content:\"\\f0002\"}.bosch-ic-car-top-arrows-left-front-right-back:before{content:\"\\f0003\"}.bosch-ic-car-top-arrows-left-back-right-front:before{content:\"\\f0004\"}.bosch-ic-car-top-arrows-front:before{content:\"\\f0005\"}.bosch-ic-car-top-arrows-back-front-left-right:before{content:\"\\f0006\"}.bosch-ic-car-top-arrows-back:before{content:\"\\f0007\"}.bosch-ic-car-top-arrow-right:before{content:\"\\f0008\"}.bosch-ic-car-top-arrow-left:before{content:\"\\f0009\"}.bosch-ic-car-top-arrow-front-right:before{content:\"\\f000a\"}.bosch-ic-car-top-arrow-front-left:before{content:\"\\f000b\"}.bosch-ic-car-top-arrow-back-right:before{content:\"\\f000c\"}.bosch-ic-car-top-arrow-back-left:before{content:\"\\f000d\"}.bosch-ic-watermelon:before{content:\"\\f000e\"}.bosch-ic-wind-vane-moderate:before{content:\"\\f000f\"}.bosch-ic-radioactive:before{content:\"\\f0010\"}.bosch-ic-heating-underfloor-off-bold:before{content:\"\\f0011\"}.bosch-ic-heating-underfloor-off-light:before{content:\"\\f0012\"}.bosch-ic-goat:before{content:\"\\f0013\"}.bosch-ic-fire-hose:before{content:\"\\f0014\"}.bosch-ic-user-run-arrow:before{content:\"\\f0015\"}.bosch-ic-store:before{content:\"\\f0016\"}.bosch-ic-user-work-dig:before{content:\"\\f0017\"}.bosch-ic-car-top-arrow-angle-right:before{content:\"\\f0018\"}.bosch-ic-control-unit-ecu:before{content:\"\\f0019\"}.bosch-ic-cigarette:before{content:\"\\f001a\"}.bosch-ic-structure-line:before{content:\"\\f001b\"}.bosch-ic-product-database-table:before{content:\"\\f001c\"}.bosch-ic-bicycle-e-remote-led:before{content:\"\\f001d\"}.bosch-ic-bicycle-e-drive-unit:before{content:\"\\f001e\"}.bosch-ic-bicycle-e-drive-unit-individual-support:before{content:\"\\f001f\"}.bosch-ic-nm-arrow-circle:before{content:\"\\f0020\"}.bosch-ic-tree-path:before{content:\"\\f0021\"}.bosch-ic-bicycle-e-wheel-arrows:before{content:\"\\f0022\"}.bosch-ic-tune-vertical:before{content:\"\\f0023\"}.bosch-ic-sound-professional:before{content:\"\\f0024\"}.bosch-ic-plane-top-headset:before{content:\"\\f0025\"}.bosch-ic-desktop-user-headset:before{content:\"\\f0026\"}.bosch-ic-user-service:before{content:\"\\f0027\"}.bosch-ic-audio-commercial:before{content:\"\\f0028\"}.bosch-ic-loudspeaker-portable-note:before{content:\"\\f0029\"}.bosch-ic-audio-loudspeaker-public:before{content:\"\\f002a\"}.bosch-ic-items-lost-questionmark:before{content:\"\\f002b\"}.bosch-ic-hand-pull:before{content:\"\\f002c\"}.bosch-ic-steering-wheel-hand-horn:before{content:\"\\f002d\"}.bosch-ic-car-chassis:before{content:\"\\f002e\"}.bosch-ic-steering-system:before{content:\"\\f002f\"}.bosch-ic-wheel-suspension:before{content:\"\\f0030\"}.bosch-ic-car-body-side:before{content:\"\\f0031\"}.bosch-ic-address-consumer-data-upright:before{content:\"\\f0032\"}.bosch-ic-weather-wind-rain:before{content:\"\\f0033\"}.bosch-ic-weather-rain-warning:before{content:\"\\f0034\"}.bosch-ic-user-check:before{content:\"\\f0035\"}.bosch-ic-user-warning:before{content:\"\\f0036\"}.bosch-ic-black-white-stars-circle:before{content:\"\\f0037\"}.bosch-ic-team-warning:before{content:\"\\f0038\"}.bosch-ic-black-white-circle:before{content:\"\\f0039\"}.bosch-ic-suitcase-small:before{content:\"\\f003a\"}.bosch-ic-suitcase-small-remove:before{content:\"\\f003b\"}.bosch-ic-home-arrow-out:before{content:\"\\f003c\"}.bosch-ic-home-arrow-in:before{content:\"\\f003d\"}.bosch-ic-document-copy-check:before{content:\"\\f003e\"}.bosch-ic-ring-infrared:before{content:\"\\f003f\"}.bosch-ic-spirit-level:before{content:\"\\f0040\"}.bosch-ic-drill-driver-check:before{content:\"\\f0041\"}.bosch-ic-desktop-led:before{content:\"\\f0042\"}.bosch-ic-desktop-black-white:before{content:\"\\f0043\"}.bosch-ic-desktop-warning:before{content:\"\\f0044\"}.bosch-ic-fire-wall:before{content:\"\\f0045\"}.bosch-ic-onroad-car-top:before{content:\"\\f0046\"}.bosch-ic-onroad-suv-top:before{content:\"\\f0047\"}.bosch-ic-onroad-truck-top:before{content:\"\\f0048\"}.bosch-ic-weather-rain-ip44:before{content:\"\\f0049\"}.bosch-ic-weather-rain-ip65:before{content:\"\\f004a\"}.bosch-ic-weather-rain-ip66:before{content:\"\\f004b\"}.bosch-ic-weather-rain-ip68:before{content:\"\\f004c\"}.bosch-ic-weather-rain-ip67:before{content:\"\\f004d\"}.bosch-ic-user-masking:before{content:\"\\f004e\"}.bosch-ic-address-consumer-data-dual:before{content:\"\\f004f\"}.bosch-ic-user-framed:before{content:\"\\f0050\"}.bosch-ic-user-counting-1:before{content:\"\\f0051\"}.bosch-ic-square-kx-uhd:before{content:\"\\f0052\"}.bosch-ic-square-qvga:before{content:\"\\f0053\"}.bosch-ic-square-vga:before{content:\"\\f0054\"}.bosch-ic-square-xmp:before{content:\"\\f0055\"}.bosch-ic-square-xxxp:before{content:\"\\f0056\"}.bosch-ic-angle-arrows-across:before{content:\"\\f0057\"}.bosch-ic-angle:before{content:\"\\f0058\"}.bosch-ic-angle-arrow-up:before{content:\"\\f0059\"}.bosch-ic-angle-view-wide:before{content:\"\\f005a\"}.bosch-ic-chat-language-selection:before{content:\"\\f005b\"}.bosch-ic-smartphone-app:before{content:\"\\f005c\"}.bosch-ic-commissioning-remote:before{content:\"\\f005d\"}.bosch-ic-user-run-detection-pir:before{content:\"\\f005e\"}.bosch-ic-field-in-motion:before{content:\"\\f005f\"}.bosch-ic-building-integration-system:before{content:\"\\f0060\"}.bosch-ic-desktop-mamangement:before{content:\"\\f0061\"}.bosch-ic-desktop-management-camera:before{content:\"\\f0062\"}.bosch-ic-screwdriver-check:before{content:\"\\f0063\"}.bosch-ic-screwdriver:before{content:\"\\f0064\"}.bosch-ic-desktop-hand-touch:before{content:\"\\f0065\"}.bosch-ic-user-robot-head:before{content:\"\\f0066\"}.bosch-ic-gear-circles-elements:before{content:\"\\f0067\"}.bosch-ic-gears-3:before{content:\"\\f0068\"}.bosch-ic-door-address-consumer-data:before{content:\"\\f0069\"}.bosch-ic-lock-closed-128bit:before{content:\"\\f006a\"}.bosch-ic-lock-closed-connected-wps:before{content:\"\\f006b\"}.bosch-ic-battery-arrow-right:before{content:\"\\f006c\"}.bosch-ic-connector-bnc:before{content:\"\\f006d\"}.bosch-ic-tower-connection-cdma:before{content:\"\\f006e\"}.bosch-ic-tower-connection-gprs:before{content:\"\\f006f\"}.bosch-ic-tritech:before{content:\"\\f0070\"}.bosch-ic-quad-sensor-two:before{content:\"\\f0071\"}.bosch-ic-dynamic-range-hdr:before{content:\"\\f0072\"}.bosch-ic-dynamic-range-wdr:before{content:\"\\f0073\"}.bosch-ic-battery-thumb-up-arrow-right:before{content:\"\\f0074\"}.bosch-ic-screwdriver-thumb-up:before{content:\"\\f0075\"}.bosch-ic-thumb-up-arrow-up:before{content:\"\\f0076\"}.bosch-ic-core-data-sql:before{content:\"\\f0077\"}.bosch-ic-frequency-869:before{content:\"\\f0078\"}.bosch-ic-coins:before{content:\"\\f0079\"}.bosch-ic-frames-60:before{content:\"\\f007a\"}.bosch-ic-hand-card:before{content:\"\\f007b\"}.bosch-ic-microphone-plugabble:before{content:\"\\f007c\"}.bosch-ic-video-h264:before{content:\"\\f007d\"}.bosch-ic-video-hd:before{content:\"\\f007e\"}.bosch-ic-video-mpeg:before{content:\"\\f007f\"}.bosch-ic-video-hd-4k:before{content:\"\\f0080\"}.bosch-ic-video-hd-720:before{content:\"\\f0081\"}.bosch-ic-lasso:before{content:\"\\f0082\"}.bosch-ic-lasso-remove:before{content:\"\\f0083\"}.bosch-ic-battery-car-3-clock:before{content:\"\\f0084\"}.bosch-ic-battery-car-agm:before{content:\"\\f0085\"}.bosch-ic-battery-car-flash:before{content:\"\\f0086\"}.bosch-ic-filter-binary:before{content:\"\\f0087\"}.bosch-ic-speedometer-bitrate-reduce:before{content:\"\\f0088\"}.bosch-ic-voting:before{content:\"\\f0089\"}.bosch-ic-voting-dual:before{content:\"\\f008a\"}.bosch-ic-sound-pressure-level-90:before{content:\"\\f008b\"}.bosch-ic-sound-pressure-level-88:before{content:\"\\f008c\"}.bosch-ic-sound-pressure-level-80:before{content:\"\\f008d\"}.bosch-ic-power-rated-20:before{content:\"\\f008e\"}.bosch-ic-labyrinth-geometry-integration:before{content:\"\\f008f\"}.bosch-ic-head-speaking-voice-2:before{content:\"\\f0090\"}.bosch-ic-volume-microphone:before{content:\"\\f0091\"}.bosch-ic-user-walk-route:before{content:\"\\f0092\"}.bosch-ic-user-walk-line-crossing:before{content:\"\\f0093\"}.bosch-ic-user-walk-line-count:before{content:\"\\f0094\"}.bosch-ic-fullscreen-arrows-circle:before{content:\"\\f0095\"}.bosch-ic-field-arrows-counter-flow:before{content:\"\\f0096\"}.bosch-ic-field-arrows-directed-flow:before{content:\"\\f0097\"}.bosch-ic-user-birds-eye-count-12:before{content:\"\\f0098\"}.bosch-ic-field-user-count-12:before{content:\"\\f0099\"}.bosch-ic-field-user-falling:before{content:\"\\f009a\"}.bosch-ic-field-user-loitering-arrow:before{content:\"\\f009b\"}.bosch-ic-field-user-similarity:before{content:\"\\f009c\"}.bosch-ic-field-user-walk:before{content:\"\\f009d\"}.bosch-ic-field-user-walk-arrow-behind:before{content:\"\\f009e\"}.bosch-ic-field-user-walk-arrow-in-front:before{content:\"\\f009f\"}.bosch-ic-field-users:before{content:\"\\f00a0\"}.bosch-ic-shock-resistant:before{content:\"\\f00a1\"}.bosch-ic-bus-sdi:before{content:\"\\f00a2\"}.bosch-ic-bus-sdi2:before{content:\"\\f00a3\"}.bosch-ic-folder-ip-net:before{content:\"\\f00a4\"}.bosch-ic-globe-iscsi:before{content:\"\\f00a5\"}.bosch-ic-offline-www:before{content:\"\\f00a6\"}.bosch-ic-opc:before{content:\"\\f00a7\"}.bosch-ic-osdp:before{content:\"\\f00a8\"}.bosch-ic-settings-connected-ptrz:before{content:\"\\f00a9\"}.bosch-ic-raid:before{content:\"\\f00aa\"}.bosch-ic-lsn:before{content:\"\\f00ab\"}.bosch-ic-pointbus:before{content:\"\\f00ac\"}.bosch-ic-male-count-12:before{content:\"\\f00ad\"}.bosch-ic-circle-point-record:before{content:\"\\f00ae\"}.bosch-ic-plug-poe:before{content:\"\\f00af\"}.bosch-ic-ir-distribution:before{content:\"\\f00b0\"}.bosch-ic-ir-range:before{content:\"\\f00b1\"}.bosch-ic-connectivity-it:before{content:\"\\f00b2\"}.bosch-ic-music-note-plus:before{content:\"\\f00b3\"}.bosch-ic-music-note-plus-two:before{content:\"\\f00b4\"}.bosch-ic-chat-language-chn:before{content:\"\\f00b5\"}.bosch-ic-microphone-high-directive:before{content:\"\\f00b6\"}.bosch-ic-document-copy-rg-gf:before{content:\"\\f00b7\"}.bosch-ic-document-copy-cdq:before{content:\"\\f00b8\"}.bosch-ic-document-copy-cgp:before{content:\"\\f00b9\"}.bosch-ic-folder-add:before{content:\"\\f00ba\"}.bosch-ic-user-tie:before{content:\"\\f00bb\"}.bosch-ic-pill:before{content:\"\\f00bc\"}.bosch-ic-sonotrode:before{content:\"\\f00bd\"}.bosch-ic-user-thumb-up:before{content:\"\\f00be\"}.bosch-ic-architecture:before{content:\"\\f00bf\"}.bosch-ic-clamping-area-arrows:before{content:\"\\f00c0\"}.bosch-ic-clamping-rib:before{content:\"\\f00c1\"}.bosch-ic-connection-plug-in:before{content:\"\\f00c2\"}.bosch-ic-connection-plug-in-electrical:before{content:\"\\f00c3\"}.bosch-ic-connection-plug-in-socket:before{content:\"\\f00c4\"}.bosch-ic-connector-shield-emc:before{content:\"\\f00c5\"}.bosch-ic-foil-flex:before{content:\"\\f00c6\"}.bosch-ic-grid-fan:before{content:\"\\f00c7\"}.bosch-ic-screw-nut:before{content:\"\\f00c8\"}.bosch-ic-sealing-groove:before{content:\"\\f00c9\"}.bosch-ic-sealing-tongue-groove-flash:before{content:\"\\f00ca\"}.bosch-ic-solder:before{content:\"\\f00cb\"}.bosch-ic-solder-selective-ssl:before{content:\"\\f00cc\"}.bosch-ic-solder-selective-thrs:before{content:\"\\f00cd\"}.bosch-ic-tuner-box:before{content:\"\\f00ce\"}.bosch-ic-water-cooling-arrow-up:before{content:\"\\f00cf\"}.bosch-ic-generic-device-connected:before{content:\"\\f00d0\"}.bosch-ic-home-sun-flash:before{content:\"\\f00d1\"}.bosch-ic-home-sun-wrench:before{content:\"\\f00d2\"}.bosch-ic-temperature-lock:before{content:\"\\f00d3\"}.bosch-ic-solder-method-smd:before{content:\"\\f00d4\"}.bosch-ic-pins:before{content:\"\\f00d5\"}.bosch-ic-connector-housing:before{content:\"\\f00d6\"}.bosch-ic-antenna-cover-arrow-up:before{content:\"\\f00d7\"}.bosch-ic-angle-open-horizontal-130:before{content:\"\\f00d8\"}.bosch-ic-angle-open-vertical-130:before{content:\"\\f00d9\"}.bosch-ic-angle-open-vertical-90:before{content:\"\\f00da\"}.bosch-ic-angle-open-horizontal-145:before{content:\"\\f00db\"}.bosch-ic-steering-system-steer-by-wire:before{content:\"\\f00dc\"}.bosch-ic-tune-vertical-music-note:before{content:\"\\f00dd\"}.bosch-ic-hammer-movement:before{content:\"\\f00de\"}.bosch-ic-drill-drive-percent-35:before{content:\"\\f00df\"}.bosch-ic-country-switzerland:before{content:\"\\f00e0\"}.bosch-ic-speedlimit-revoking:before{content:\"\\f00e1\"}.bosch-ic-speedlimit-80-revoking:before{content:\"\\f00e2\"}.bosch-ic-speedlimit-80-dirty:before{content:\"\\f00e3\"}.bosch-ic-battery-car-mf:before{content:\"\\f00e4\"}.bosch-ic-access-controller-2:before{content:\"\\f00e5\"}.bosch-ic-access-controller-arrow-up:before{content:\"\\f00e6\"}.bosch-ic-access-controller-points-set:before{content:\"\\f00e7\"}.bosch-ic-analog-value:before{content:\"\\f00e8\"}.bosch-ic-analog-value-arrow-in:before{content:\"\\f00e9\"}.bosch-ic-analog-value-arrow-out:before{content:\"\\f00ea\"}.bosch-ic-binary-value:before{content:\"\\f00eb\"}.bosch-ic-binary-value-arrow-in:before{content:\"\\f00ec\"}.bosch-ic-binary-value-arrow-out:before{content:\"\\f00ed\"}.bosch-ic-calendar-date-single:before{content:\"\\f00ee\"}.bosch-ic-calendar-date-weekend:before{content:\"\\f00ef\"}.bosch-ic-calendar-view:before{content:\"\\f00f0\"}.bosch-ic-chart-average:before{content:\"\\f00f1\"}.bosch-ic-chart-line-arrows-up:before{content:\"\\f00f2\"}.bosch-ic-configuration-points-set:before{content:\"\\f00f3\"}.bosch-ic-data-points-set:before{content:\"\\f00f4\"}.bosch-ic-document-points-set:before{content:\"\\f00f5\"}.bosch-ic-multistate-value:before{content:\"\\f00f6\"}.bosch-ic-multistate-value-arrow-in:before{content:\"\\f00f7\"}.bosch-ic-multistate-value-arrow-out:before{content:\"\\f00f8\"}.bosch-ic-structure-bacnet:before{content:\"\\f00f9\"}.bosch-ic-structure-sbus:before{content:\"\\f00fa\"}.bosch-ic-team-arrows-in:before{content:\"\\f00fb\"}.bosch-ic-import-points-set:before{content:\"\\f00fc\"}.bosch-ic-image-points-set:before{content:\"\\f00fd\"}.bosch-ic-circuit-hydraulic:before{content:\"\\f00fe\"}.bosch-ic-structure-area:before{content:\"\\f00ff\"}.bosch-ic-structure-calculate:before{content:\"\\f0100\"}.bosch-ic-structure-clipboard:before{content:\"\\f0101\"}.bosch-ic-structure-collection:before{content:\"\\f0102\"}.bosch-ic-structure-components:before{content:\"\\f0103\"}.bosch-ic-structure-device:before{content:\"\\f0104\"}.bosch-ic-structure-dragdrop:before{content:\"\\f0105\"}.bosch-ic-structure-floor:before{content:\"\\f0106\"}.bosch-ic-structure-organization:before{content:\"\\f0107\"}.bosch-ic-structure-questionmark:before{content:\"\\f0108\"}.bosch-ic-structure-settings:before{content:\"\\f0109\"}.bosch-ic-structure-skyscraper:before{content:\"\\f010a\"}.bosch-ic-structure-window-console:before{content:\"\\f010b\"}.bosch-ic-structure-wrench:before{content:\"\\f010c\"}.bosch-ic-car-pickup:before{content:\"\\f010d\"}.bosch-ic-desktop-team:before{content:\"\\f010e\"}.bosch-ic-keyboard-arrow-enter-return:before{content:\"\\f010f\"}.bosch-ic-keyboard-arrow-left:before{content:\"\\f0110\"}.bosch-ic-keyboard-arrow-right-tab:before{content:\"\\f0111\"}.bosch-ic-keyboard-capslock:before{content:\"\\f0112\"}.bosch-ic-keyboard-shift:before{content:\"\\f0113\"}.bosch-ic-keyboard-space:before{content:\"\\f0114\"}.bosch-ic-bluetooth-le-off-disabled-light:before{content:\"\\f0115\"}.bosch-ic-structure-label-edit:before{content:\"\\f0116\"}.bosch-ic-structure-server:before{content:\"\\f0117\"}.bosch-ic-truck-box:before{content:\"\\f0118\"}.bosch-ic-imagery-off-disabled-light:before{content:\"\\f0119\"}.bosch-ic-imagery-off-disabled-bold:before{content:\"\\f011a\"}.bosch-ic-plan-floorplan-edit:before{content:\"\\f011b\"}.bosch-ic-desktop-building-ruler-pen:before{content:\"\\f011c\"}.bosch-ic-conference-system-connected:before{content:\"\\f011d\"}.bosch-ic-receiver-pocket-headphones:before{content:\"\\f011e\"}.bosch-ic-conference-system-microphone:before{content:\"\\f011f\"}.bosch-ic-user-carry-box:before{content:\"\\f0120\"}.bosch-ic-user-handcart:before{content:\"\\f0121\"}.bosch-ic-user-carry-box-2:before{content:\"\\f0122\"}.bosch-ic-locator-error:before{content:\"\\f0123\"}.bosch-ic-locator-offline:before{content:\"\\f0124\"}.bosch-ic-locator-refresh:before{content:\"\\f0125\"}.bosch-ic-chart-bar-plug-arrow-up:before{content:\"\\f0126\"}.bosch-ic-clipboard-search-leaf:before{content:\"\\f0127\"}.bosch-ic-handover-leaf:before{content:\"\\f0128\"}.bosch-ic-home-sun-plug:before{content:\"\\f0129\"}.bosch-ic-lightbulb-plug-leaf:before{content:\"\\f012a\"}.bosch-ic-wheel-water-plug:before{content:\"\\f012b\"}.bosch-ic-windmill-plug:before{content:\"\\f012c\"}.bosch-ic-windmill-plug-sun-water:before{content:\"\\f012d\"}.bosch-ic-scale-co2:before{content:\"\\f012e\"}.bosch-ic-certificat-co2:before{content:\"\\f012f\"}.bosch-ic-car-crash:before{content:\"\\f0130\"}.bosch-ic-headphones:before{content:\"\\f0131\"}.bosch-ic-music-note:before{content:\"\\f0132\"}.bosch-ic-clef-treble:before{content:\"\\f0133\"}.bosch-ic-clef-bass:before{content:\"\\f0134\"}.bosch-ic-volume:before{content:\"\\f0135\"}.bosch-ic-volume-increase:before{content:\"\\f0136\"}.bosch-ic-head-speach-speaking:before{content:\"\\f0137\"}.bosch-ic-head-speaking-request:before{content:\"\\f0138\"}.bosch-ic-tetragon-priority:before{content:\"\\f0139\"}.bosch-ic-head-respond-speaking:before{content:\"\\f013a\"}.bosch-ic-head-respond-request:before{content:\"\\f013b\"}.bosch-ic-hand-stop:before{content:\"\\f013c\"}.bosch-ic-layers-stack:before{content:\"\\f013d\"}.bosch-ic-puzzle-binary:before{content:\"\\f013e\"}.bosch-ic-sdcard-micro:before{content:\"\\f013f\"}.bosch-ic-reader-card:before{content:\"\\f0140\"}.bosch-ic-reader-user-recognition-face:before{content:\"\\f0141\"}.bosch-ic-detector-point-type:before{content:\"\\f0142\"}.bosch-ic-manual-call-point:before{content:\"\\f0143\"}.bosch-ic-gun-magnifier:before{content:\"\\f0144\"}.bosch-ic-interface-module:before{content:\"\\f0145\"}.bosch-ic-fire-detector-automatic:before{content:\"\\f0146\"}.bosch-ic-fire-brigade-emergency:before{content:\"\\f0147\"}.bosch-ic-volume-siren-device:before{content:\"\\f0148\"}.bosch-ic-user-reception:before{content:\"\\f0149\"}.bosch-ic-shredder-machine:before{content:\"\\f014a\"}.bosch-ic-badge-clean:before{content:\"\\f014b\"}.bosch-ic-parking-assistance:before{content:\"\\f014c\"}.bosch-ic-laptop-questionmark:before{content:\"\\f014d\"}.bosch-ic-security-lock:before{content:\"\\f014e\"}.bosch-ic-battery-car-12v:before{content:\"\\f014f\"}.bosch-ic-car-far-propagation:before{content:\"\\f0150\"}.bosch-ic-car-wide-propagation:before{content:\"\\f0151\"}.bosch-ic-ear-sound:before{content:\"\\f0152\"}.bosch-ic-umbrella-drops:before{content:\"\\f0153\"}.bosch-ic-horn-arrows-in:before{content:\"\\f0154\"}.bosch-ic-video-bell:before{content:\"\\f0155\"}.bosch-ic-steering-wheel-arrows-up:before{content:\"\\f0156\"}.bosch-ic-circle-full:before{content:\"\\f0157\"}.bosch-ic-circle-half:before{content:\"\\f0158\"}.bosch-ic-circle-quarter:before{content:\"\\f0159\"}.bosch-ic-circle-quarter-three:before{content:\"\\f015a\"}.bosch-ic-appliance-flame-arrow:before{content:\"\\f015b\"}.bosch-ic-appliance-flame-arrow-bold:before{content:\"\\f015c\"}.bosch-ic-appliance-flame-disabled-bold:before{content:\"\\f015d\"}.bosch-ic-appliance-flame-disabled-light:before{content:\"\\f015e\"}.bosch-ic-appliance-flame-plus:before{content:\"\\f015f\"}.bosch-ic-appliance-flame-plus-bold:before{content:\"\\f0160\"}.bosch-ic-flash-minus-bold:before{content:\"\\f0161\"}.bosch-ic-flash-minus:before{content:\"\\f0162\"}.bosch-ic-flash-off:before{content:\"\\f0163\"}.bosch-ic-flash-plus-bold:before{content:\"\\f0164\"}.bosch-ic-flash-smart-grid:before{content:\"\\f0165\"}.bosch-ic-signal-high:before{content:\"\\f0166\"}.bosch-ic-signal-low:before{content:\"\\f0167\"}.bosch-ic-signal-very-low:before{content:\"\\f0168\"}.bosch-ic-5g:before{content:\"\\f0169\"}.bosch-ic-sun-photovoltaic:before{content:\"\\f016a\"}.bosch-ic-inverter:before{content:\"\\f016b\"}.bosch-ic-car-arrow-down:before{content:\"\\f016c\"}.bosch-ic-car-distance:before{content:\"\\f016d\"}.bosch-ic-car-sensor:before{content:\"\\f016e\"}.bosch-ic-car-sensor-environment:before{content:\"\\f016f\"}.bosch-ic-car-sensor-front:before{content:\"\\f0170\"}.bosch-ic-streets-locator:before{content:\"\\f0171\"}.bosch-ic-truck-distance:before{content:\"\\f0172\"}.bosch-ic-warning-user-walk:before{content:\"\\f0173\"}.bosch-ic-parking-check:before{content:\"\\f0174\"}.bosch-ic-dremel:before{content:\"\\f0175\"}.bosch-ic-allergens:before{content:\"\\f0176\"}.bosch-ic-money-currency-coins:before{content:\"\\f0177\"}.bosch-ic-hydrogen-small:before{content:\"\\f0178\"}.bosch-ic-hydrogen-large:before{content:\"\\f0179\"}.bosch-ic-shoppingcart-remove:before{content:\"\\f017a\"}.bosch-ic-chimney-sweeper-disabled-light:before{content:\"\\f017b\"}.bosch-ic-chimney-sweeper-disabled-bold:before{content:\"\\f017c\"}.bosch-ic-car-waves:before{content:\"\\f017d\"}.bosch-ic-360-degrees-arrow:before{content:\"\\f017e\"}.bosch-ic-clock-info:before{content:\"\\f017f\"}.bosch-ic-truck-front-disabled-bold:before{content:\"\\f0180\"}.bosch-ic-truck-front-disabled-light:before{content:\"\\f0181\"}.bosch-ic-box-closed-list:before{content:\"\\f0182\"}.bosch-ic-battery-car-exchange:before{content:\"\\f0183\"}.bosch-ic-guitar:before{content:\"\\f0184\"}.bosch-ic-actuator-by-wire:before{content:\"\\f0185\"}.bosch-ic-guitar-electric:before{content:\"\\f0186\"}.bosch-ic-format-align-center:before{content:\"\\f0187\"}.bosch-ic-format-align-justify:before{content:\"\\f0188\"}.bosch-ic-format-align-right:before{content:\"\\f0189\"}.bosch-ic-format-checklist:before{content:\"\\f018a\"}.bosch-ic-format-italic:before{content:\"\\f018b\"}.bosch-ic-format-ordered-list:before{content:\"\\f018c\"}.bosch-ic-format-strike:before{content:\"\\f018d\"}.bosch-ic-format-underline:before{content:\"\\f018e\"}.bosch-ic-format-align-left:before{content:\"\\f018f\"}.bosch-ic-format-fill-drop:before{content:\"\\f0190\"}.bosch-ic-format-color-a:before{content:\"\\f0191\"}.bosch-ic-format-bold-b:before{content:\"\\f0192\"}.bosch-ic-fly:before{content:\"\\f0193\"}.bosch-ic-paw:before{content:\"\\f0194\"}.bosch-ic-desktop-video-response-fast:before{content:\"\\f0195\"}.bosch-ic-user-walk-weather-rain:before{content:\"\\f0196\"}.bosch-ic-brake-pedal-plug:before{content:\"\\f0197\"}.bosch-ic-detector-motion-range-down-view:before{content:\"\\f0198\"}.bosch-ic-detector-motion-range-long:before{content:\"\\f0199\"}.bosch-ic-monitor-tiled:before{content:\"\\f019a\"}.bosch-ic-radar-doppler:before{content:\"\\f019b\"}.bosch-ic-radioactive-disabled-light:before{content:\"\\f019c\"}.bosch-ic-radioactive-disabled-bold:before{content:\"\\f019d\"}.bosch-ic-bug-disabled-light:before{content:\"\\f019e\"}.bosch-ic-bug-disabled-bold:before{content:\"\\f019f\"}.bosch-ic-fly-disabled-light:before{content:\"\\f01a0\"}.bosch-ic-fly-disabled-bold:before{content:\"\\f01a1\"}.bosch-ic-puzzle-2:before{content:\"\\f01a2\"}.bosch-ic-camera-lenses-multi-images:before{content:\"\\f01a3\"}.bosch-ic-detector-spray:before{content:\"\\f01a4\"}.bosch-ic-weather-rain-ip56:before{content:\"\\f01a5\"}.bosch-ic-document-log-start:before{content:\"\\f01a6\"}.bosch-ic-document-log-stop:before{content:\"\\f01a7\"}.bosch-ic-document-3d-pdf:before{content:\"\\f01a8\"}.bosch-ic-document-ifc:before{content:\"\\f01a9\"}.bosch-ic-document-rfa:before{content:\"\\f01aa\"}.bosch-ic-document-stp:before{content:\"\\f01ab\"}.bosch-ic-recycle-arrows-pap20-frame:before{content:\"\\e8b4\"}.bosch-ic-umbrella-drops-frame:before{content:\"\\e8b5\"}.bosch-ic-arrows-this-side-up-frame:before{content:\"\\e8b6\"}.bosch-ic-glass-wine-frame:before{content:\"\\e8b7\"}.bosch-ic-pb-frame:before{content:\"\\e8b8\"}.bosch-ic-pb-frame-disabled-bold:before{content:\"\\e8b9\"}.bosch-ic-pb-frame-disabled-light:before{content:\"\\e8ba\"}.bosch-ic-mode-frame:before{content:\"\\e8bb\"}.bosch-ic-uv-frame-disabled-bold:before{content:\"\\e8bc\"}.bosch-ic-uv-frame-disabled-light:before{content:\"\\e8bd\"}.bosch-ic-fan-frame:before{content:\"\\e8be\"}.bosch-ic-flash-frame:before{content:\"\\e8bf\"}.bosch-ic-kakao:before{content:\"\\eaaf\"}.bosch-ic-kakao-frame:before{content:\"\\eab0\"}.bosch-ic-naver:before{content:\"\\eab1\"}.bosch-ic-naver-frame:before{content:\"\\eab2\"}.bosch-ic-tencent:before{content:\"\\eab3\"}.bosch-ic-tencent-frame:before{content:\"\\eab4\"}.bosch-ic-wordpress:before{content:\"\\eab5\"}.bosch-ic-wordpress-frame:before{content:\"\\eab6\"}.bosch-ic-mini-program-frame:before{content:\"\\eab7\"}.bosch-ic-mini-program:before{content:\"\\eab8\"}.bosch-ic-tiktok:before{content:\"\\eab9\"}.bosch-ic-tiktok-frame:before{content:\"\\eaba\"}.bosch-ic-glassdoor:before{content:\"\\eabb\"}.bosch-ic-glassdoor-frame:before{content:\"\\eabc\"}.bosch-ic-git-hub:before{content:\"\\eabd\"}.bosch-ic-git-hub-frame:before{content:\"\\eabe\"}.bosch-ic-online-www:before{content:\"\\f01ac\"}.bosch-ic-bluetooth-connected:before{content:\"\\f01ad\"}.bosch-ic-connected-tools:before{content:\"\\f01ae\"}.bosch-ic-badge-arrow-kickback:before{content:\"\\f01af\"}.bosch-ic-copy-paste:before{content:\"\\f01b0\"}.bosch-ic-volleyball:before{content:\"\\f01b1\"}.bosch-ic-car-top-trailer:before{content:\"\\f01b2\"}.bosch-ic-mountain:before{content:\"\\f01b3\"}.bosch-ic-battery-car-oil-lamp:before{content:\"\\f01b4\"}.bosch-ic-battery-car-efb:before{content:\"\\f01b5\"}.bosch-ic-battery-car-ams:before{content:\"\\f01b6\"}.bosch-ic-battery-car-ag:before{content:\"\\f01b7\"}.bosch-ic-display-fuel-saving:before{content:\"\\f01b8\"}.bosch-ic-circle-parking-arrows:before{content:\"\\f01b9\"}.bosch-ic-unicorn-head:before{content:\"\\f01ba\"}.bosch-ic-car-parking-automated:before{content:\"\\f01bb\"}.bosch-ic-lunchboard:before{content:\"\\f01bc\"}.bosch-ic-tetrapack:before{content:\"\\f01bd\"}.bosch-ic-recycle-arrows:before{content:\"\\f01be\"}.bosch-ic-stairs-arrow-up:before{content:\"\\f01bf\"}.bosch-ic-stairs-arrow-down:before{content:\"\\f01c0\"}.bosch-ic-industry-car-arrow:before{content:\"\\f01c1\"}.bosch-ic-sensor-throttle-position:before{content:\"\\f01c2\"}.bosch-ic-sensor-knock:before{content:\"\\f01c3\"}.bosch-ic-sensor-air-mass-meter:before{content:\"\\f01c4\"}.bosch-ic-sensor-pressure:before{content:\"\\f01c5\"}.bosch-ic-sensor-temperature:before{content:\"\\f01c6\"}.bosch-ic-sensor-speed-rpm:before{content:\"\\f01c7\"}.bosch-ic-desktop-user-software-clock:before{content:\"\\f01c8\"}.bosch-ic-book-service-wrench-add:before{content:\"\\f01c9\"}.bosch-ic-book-service-wrench:before{content:\"\\f01ca\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-light:before{content:\"\\f01cb\"}.bosch-ic-bicycle-e-drive-unit-individual-support-off-bold:before{content:\"\\f01cc\"}.bosch-ic-circle-parking-disabled-bold:before{content:\"\\f01cd\"}.bosch-ic-circle-parking-disabled-light:before{content:\"\\f01ce\"}.bosch-ic-interval-30sec:before{content:\"\\f01cf\"}.bosch-ic-truck-front-light-inside:before{content:\"\\f01d0\"}.bosch-ic-software-tex:before{content:\"\\f01d1\"}.bosch-ic-fire-arrow-circle:before{content:\"\\f01d2\"}.bosch-ic-heating-underfloor-circle-global-off:before{content:\"\\f01d3\"}.bosch-ic-heating-radiator-circle-global-off:before{content:\"\\f01d4\"}.bosch-ic-faucet-circle-global-off:before{content:\"\\f01d5\"}.bosch-ic-moon-stars-starlight:before{content:\"\\f01d6\"}.bosch-ic-tube-thumb-up:before{content:\"\\f01d7\"}.bosch-ic-detector-microwave:before{content:\"\\f01d8\"}.bosch-ic-user-headset-thumb-up:before{content:\"\\f01d9\"}.bosch-ic-cloud-battery:before{content:\"\\f01da\"}.bosch-ic-temperature-arrow-up-increase:before{content:\"\\f01db\"}.bosch-ic-temperature-arrow-down-decrease:before{content:\"\\f01dc\"}.bosch-ic-moneybag-dollar-chart-double:before{content:\"\\f01dd\"}.bosch-ic-locator-remove:before{content:\"\\f01de\"}.bosch-ic-square-remove:before{content:\"\\f01df\"}.bosch-ic-sun-hdr:before{content:\"\\f01e0\"}.bosch-ic-moon-stars-circle:before{content:\"\\f01e1\"}.bosch-ic-temperature-user-detection:before{content:\"\\f01e2\"}.bosch-ic-wrench-thumb-up:before{content:\"\\f01e3\"}.bosch-ic-ring-infrared-ir:before{content:\"\\f01e4\"}.bosch-ic-play-back-arrow:before{content:\"\\f01e5\"}.bosch-ic-ignition-off:before{content:\"\\f01e6\"}.bosch-ic-arrow-up-down:before{content:\"\\f01e7\"}.bosch-ic-arrow-left-right:before{content:\"\\f01e8\"}.bosch-ic-number-0-bold:before{content:\"\\f01e9\"}.bosch-ic-number-1-bold:before{content:\"\\f01ea\"}.bosch-ic-number-2-bold:before{content:\"\\f01eb\"}.bosch-ic-number-3-bold:before{content:\"\\f01ec\"}.bosch-ic-number-4-bold:before{content:\"\\f01ed\"}.bosch-ic-number-5-bold:before{content:\"\\f01ee\"}.bosch-ic-number-6-bold:before{content:\"\\f01ef\"}.bosch-ic-number-7-bold:before{content:\"\\f01f0\"}.bosch-ic-number-8-bold:before{content:\"\\f01f1\"}.bosch-ic-number-9-bold:before{content:\"\\f01f2\"}.bosch-ic-e-charging-plug-dc:before{content:\"\\f01f3\"}.bosch-ic-e-charging-plug-ac-dc:before{content:\"\\f01f4\"}.bosch-ic-angle-view-wide-monitor:before{content:\"\\f01f5\"}.bosch-ic-settings-range-ptrz:before{content:\"\\f01f6\"}.bosch-ic-puzzle-video-integration:before{content:\"\\f01f7\"}.bosch-ic-number-0-regular:before{content:\"\\f01f8\"}.bosch-ic-number-1-regular:before{content:\"\\f01f9\"}.bosch-ic-number-2-regular:before{content:\"\\f01fa\"}.bosch-ic-number-3-regular:before{content:\"\\f01fb\"}.bosch-ic-number-4-regular:before{content:\"\\f01fc\"}.bosch-ic-number-5-regular:before{content:\"\\f01fd\"}.bosch-ic-number-6-regular:before{content:\"\\f01fe\"}.bosch-ic-number-7-regular:before{content:\"\\f01ff\"}.bosch-ic-number-8-regular:before{content:\"\\f0200\"}.bosch-ic-number-9-regular:before{content:\"\\f0201\"}.bosch-ic-import-beep:before{content:\"\\f0202\"}.bosch-ic-file-questionmark:before{content:\"\\f0203\"}.bosch-ic-file-dollar:before{content:\"\\f0204\"}.bosch-ic-document-reporting:before{content:\"\\f0205\"}.bosch-ic-box-questionmark1:before{content:\"\\f0206\"}.bosch-ic-chat-faq:before{content:\"\\f0207\"}.bosch-ic-wishlist-filled:before{content:\"\\f0208\"}.bosch-ic-wishlist-half-filled:before{content:\"\\f0209\"}.bosch-ic-glove-work:before{content:\"\\f020a\"}.bosch-ic-lightbulb-halogen-hand-disabled-bold:before{content:\"\\f020b\"}.bosch-ic-lightbulb-halogen-hand-disabled-light:before{content:\"\\f020c\"}.bosch-ic-lightbulb-halogen-cracked-disabled-bold:before{content:\"\\f020d\"}.bosch-ic-lightbulb-halogen-cracked-disabled-light:before{content:\"\\f020e\"}.bosch-ic-lightbulb-halogen-cracked:before{content:\"\\f020f\"}.bosch-ic-drops:before{content:\"\\f0210\"}.bosch-ic-drops-disabled-bold:before{content:\"\\f0211\"}.bosch-ic-drops-disabled-light:before{content:\"\\f0212\"}.bosch-ic-temperature-heat:before{content:\"\\f0213\"}.bosch-ic-allen-key:before{content:\"\\f0214\"}.bosch-ic-aperture-closed:before{content:\"\\f0215\"}.bosch-ic-appliance-heat-pump-leaf-connected:before{content:\"\\f0216\"}.bosch-ic-badge-flash:before{content:\"\\f0217\"}.bosch-ic-battery-car-1-plug-warning:before{content:\"\\f0218\"}.bosch-ic-battery-car-3-automatic-mode:before{content:\"\\f0219\"}.bosch-ic-battery-car-3-ice:before{content:\"\\f021a\"}.bosch-ic-battery-car-memory-function:before{content:\"\\f021b\"}.bosch-ic-battery-polarity-reverse-warning:before{content:\"\\f021c\"}.bosch-ic-book-questionmark:before{content:\"\\f021d\"}.bosch-ic-book-service-wrench-refresh:before{content:\"\\f021e\"}.bosch-ic-box-arrow-up:before{content:\"\\f021f\"}.bosch-ic-box-arrows-up-down:before{content:\"\\f0220\"}.bosch-ic-camera-optic-sensor:before{content:\"\\f0221\"}.bosch-ic-car-cloud-software:before{content:\"\\f0222\"}.bosch-ic-car-gear-technology:before{content:\"\\f0223\"}.bosch-ic-car-street:before{content:\"\\f0224\"}.bosch-ic-chip-sensor:before{content:\"\\f0225\"}.bosch-ic-circle-blank:before{content:\"\\f0226\"}.bosch-ic-detector-point-type-replace:before{content:\"\\f0227\"}.bosch-ic-document-x:before{content:\"\\f0228\"}.bosch-ic-drag-handle-arrows-up-down:before{content:\"\\f0229\"}.bosch-ic-electrolyzer:before{content:\"\\f022a\"}.bosch-ic-exhaust-pipe:before{content:\"\\f022b\"}.bosch-ic-flower-focus-near:before{content:\"\\f022c\"}.bosch-ic-fuel-pump-electric:before{content:\"\\f022d\"}.bosch-ic-gear-dollar:before{content:\"\\f022e\"}.bosch-ic-gloves-work:before{content:\"\\f022f\"}.bosch-ic-handlewithcare-globe:before{content:\"\\f0230\"}.bosch-ic-handlewithcare-user:before{content:\"\\f0231\"}.bosch-ic-hard-disk:before{content:\"\\f0232\"}.bosch-ic-hierarchy-child:before{content:\"\\f0233\"}.bosch-ic-hierarchy-parent:before{content:\"\\f0234\"}.bosch-ic-hydraulic-connection-leaking-drops:before{content:\"\\f0235\"}.bosch-ic-hydraulic-connection-off:before{content:\"\\f0236\"}.bosch-ic-hydraulic-connection-on:before{content:\"\\f0237\"}.bosch-ic-light-beam:before{content:\"\\f0238\"}.bosch-ic-map5000:before{content:\"\\f0239\"}.bosch-ic-module-anti-blocking-system:before{content:\"\\f023a\"}.bosch-ic-module-ecg:before{content:\"\\f023b\"}.bosch-ic-module-ecg-suction:before{content:\"\\f023c\"}.bosch-ic-module-navigation:before{content:\"\\f023d\"}.bosch-ic-power-supply-unit-plug:before{content:\"\\f023e\"}.bosch-ic-security-street:before{content:\"\\f023f\"}.bosch-ic-server-single:before{content:\"\\f0240\"}.bosch-ic-skip-backward:before{content:\"\\f0241\"}.bosch-ic-skip-forward:before{content:\"\\f0242\"}.bosch-ic-speedometer-instrument-cluster:before{content:\"\\f0243\"}.bosch-ic-system-radar-connected:before{content:\"\\f0244\"}.bosch-ic-truck-dump:before{content:\"\\f0245\"}.bosch-ic-truck-loader:before{content:\"\\f0246\"}.bosch-ic-user-head-twin-digital:before{content:\"\\f0247\"}.bosch-ic-wind-vane-strong:before{content:\"\\f0248\"}.bosch-ic-wind-vane-weak:before{content:\"\\f0249\"}.bosch-ic-windshield-crack:before{content:\"\\f024a\"}.bosch-ic-common-rail:before{content:\"\\f024b\"}.bosch-ic-common-rail-injector-cri1-cri2:before{content:\"\\f024c\"}.bosch-ic-common-rail-injector-crii3:before{content:\"\\f024d\"}.bosch-ic-common-rail-injector-cril3:before{content:\"\\f024e\"}.bosch-ic-common-rail-injector-crin4-2:before{content:\"\\f024f\"}.bosch-ic-common-rail-injector-crin123:before{content:\"\\f0250\"}.bosch-ic-conventional-pump-a:before{content:\"\\f0251\"}.bosch-ic-conventional-pump-b:before{content:\"\\f0252\"}.bosch-ic-edc-inline-injection-pump-pes6p:before{content:\"\\f0253\"}.bosch-ic-edc-inline-injection-pump-pes6h:before{content:\"\\f0254\"}.bosch-ic-nozzle-holder:before{content:\"\\f0255\"}.bosch-ic-unit-injector:before{content:\"\\f0256\"}.bosch-ic-qr-code-scan:before{content:\"\\f0257\"}.bosch-ic-car-mechanic-plug:before{content:\"\\f0258\"}.bosch-ic-copy-machine:before{content:\"\\f0259\"}.bosch-ic-dispenser-paper-towel:before{content:\"\\f025a\"}.bosch-ic-hand-drops-soap:before{content:\"\\f025b\"}.bosch-ic-door-automatic:before{content:\"\\f025c\"}.bosch-ic-door-automatic-slide:before{content:\"\\f025d\"}.bosch-ic-door-pull:before{content:\"\\f025e\"}.bosch-ic-door-push:before{content:\"\\f025f\"}.bosch-ic-floor-load-max:before{content:\"\\f0260\"}.bosch-ic-lockers:before{content:\"\\f0261\"}.bosch-ic-rubbish-trash-bin:before{content:\"\\f0262\"}.bosch-ic-rubbish-waste-bin:before{content:\"\\f0263\"}.bosch-ic-rubbish-waste-container:before{content:\"\\f0264\"}.bosch-ic-sugar:before{content:\"\\f0265\"}.bosch-ic-rubbish-glass-bin:before{content:\"\\f0266\"}.bosch-ic-car-seat-steering-wheel-stowable:before{content:\"\\f0267\"}.bosch-ic-car-seat-steering-wheel-standard:before{content:\"\\f0268\"}.bosch-ic-car-seat-steering-wheel-extended:before{content:\"\\f0269\"}.bosch-ic-device-bridge:before{content:\"\\f026a\"}.bosch-ic-screw-cross-slot:before{content:\"\\f026b\"}.bosch-ic-screw-hexagon-head:before{content:\"\\f026c\"}.bosch-ic-screw-hexagon-internal:before{content:\"\\f026d\"}.bosch-ic-screw-slotted:before{content:\"\\f026e\"}.bosch-ic-screw-torx:before{content:\"\\f026f\"}.bosch-ic-tape:before{content:\"\\f0270\"}.bosch-ic-tape-teflon:before{content:\"\\f0271\"}.bosch-ic-security-fire:before{content:\"\\f0272\"}.bosch-ic-led-fault:before{content:\"\\f0273\"}.bosch-ic-led-alarm:before{content:\"\\f0274\"}.bosch-ic-firepanel:before{content:\"\\f0275\"}.bosch-ic-fire-output-disabled:before{content:\"\\f0276\"}.bosch-ic-user-fire-alarm:before{content:\"\\f0277\"}.bosch-ic-car-desktop-graph:before{content:\"\\f0278\"}.bosch-ic-desktop-management-statistics:before{content:\"\\f0279\"}.bosch-ic-document-key:before{content:\"\\f027a\"}.bosch-ic-options-vertical:before{content:\"\\f027b\"}.bosch-ic-euro-arrows:before{content:\"\\f027c\"}.bosch-ic-monkey:before{content:\"\\f027d\"}.bosch-ic-appliance-resistance:before{content:\"\\f027e\"}.bosch-ic-pipe-cutter:before{content:\"\\f027f\"}.bosch-ic-flash-arrow:before{content:\"\\f0280\"}.bosch-ic-flash-arrow-disabled-light:before{content:\"\\f0281\"}.bosch-ic-heating-ceiling:before{content:\"\\f0282\"}.bosch-ic-heating-ceiling-off-light:before{content:\"\\f0283\"}.bosch-ic-heating-ceiling-off-bold:before{content:\"\\f0284\"}.bosch-ic-battery-analysis:before{content:\"\\f0285\"}.bosch-ic-battery-ice:before{content:\"\\f0286\"}.bosch-ic-battery-refresh:before{content:\"\\f0287\"}.bosch-ic-device-ids:before{content:\"\\f0288\"}.bosch-ic-device-thermostat-bcc50-72:before{content:\"\\f0289\"}.bosch-ic-device-thermostat-bcc100-72:before{content:\"\\f028a\"}.bosch-ic-silos-gas-natural:before{content:\"\\f028b\"}.bosch-ic-silos-gas-leaf-biogas:before{content:\"\\f028c\"}.bosch-ic-circle-decentralized:before{content:\"\\f028d\"}.bosch-ic-molecules-h2:before{content:\"\\f028e\"}.bosch-ic-megawatt-range-arrow-scalable:before{content:\"\\f028f\"}.bosch-ic-fan-arrows:before{content:\"\\f0290\"}.bosch-ic-recycle-arrows-pvc3:before{content:\"\\f0291\"}.bosch-ic-recycle-arrows-ps6:before{content:\"\\f0292\"}.bosch-ic-recycle-arrows-pp5:before{content:\"\\f0293\"}.bosch-ic-recycle-arrows-pet1:before{content:\"\\f0294\"}.bosch-ic-recycle-arrows-pap22:before{content:\"\\f0295\"}.bosch-ic-recycle-arrows-pap21:before{content:\"\\f0296\"}.bosch-ic-recycle-arrows-pap20:before{content:\"\\f0297\"}.bosch-ic-recycle-arrows-oth7:before{content:\"\\f0298\"}.bosch-ic-recycle-arrows-ldpe4:before{content:\"\\f0299\"}.bosch-ic-recycle-arrows-hdpe2:before{content:\"\\f029a\"}.bosch-ic-car-petrol-plug:before{content:\"\\f029b\"}.bosch-ic-user-diverse:before{content:\"\\f029c\"}.bosch-ic-fire-emergency-run-map:before{content:\"\\f029d\"}.bosch-ic-automatic-mode-fan:before{content:\"\\f029e\"}.bosch-ic-air-condition-swing-pac:before{content:\"\\f029f\"}.bosch-ic-user-fire-alarm-check:before{content:\"\\f02a0\"}.bosch-ic-handlewithcare-team:before{content:\"\\f02a1\"}.bosch-ic-settings-user-dollar:before{content:\"\\f02a2\"}.bosch-ic-skyscraper-connected:before{content:\"\\f02a3\"}.bosch-ic-security-ac:before{content:\"\\f02a4\"}.bosch-ic-windshield-glare-sun:before{content:\"\\f02a5\"}.bosch-ic-user-flag:before{content:\"\\f02a6\"}.bosch-ic-hammer-legal:before{content:\"\\f02a7\"}.bosch-ic-handover-star:before{content:\"\\f02a8\"}.bosch-ic-laptop-wrench:before{content:\"\\f02a9\"}/*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2017 Robert Bosch GmbH Copyright (C) 2017 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n *//*!\n  *  Copyright (C) 2019 Robert Bosch GmbH Copyright (C) 2019 Robert Bosch Manufacturing Solutions GmbH, Germany. All rights reserved.\n */.Bosch-IC-find-part:before{content:\"\\e38f\"}.Bosch-IC-next-station:before{content:\"\\e8a9\"}.Bosch-Ic-HMI:before{content:\"\\e293\"}.Bosch-Ic-add-outline:before{content:\"\\e8a1\"}.Bosch-Ic-apas:before{content:\"\\e224\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-Ic-cash:before{content:\"\\e890\"}.Bosch-Ic-pinterest:before{content:\"\\ea8c\"}.Bosch-Ic-problem:before{content:\"\\e899\"}.Bosch-Ic-promotion:before{content:\"\\e89a\"}.Bosch-Ic-arrow-diagonal:before{content:\"\\e095\"}.Bosch-Ic-arrow-down-filled:before{content:\"\\e887\"}.Bosch-Ic-arrow-down-circle:before{content:\"\\e887\"}.Bosch-Ic-arrow-end-left:before{content:\"\\e0a1\"}.Bosch-Ic-arrow-end-right:before{content:\"\\e182\"}.Bosch-Ic-arrow-left-down-filled:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-filled:before{content:\"\\e889\"}.Bosch-Ic-arrow-left-up-filled:before{content:\"\\e88a\"}.Bosch-Ic-arrow-right-down-filled:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-filled:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-up-filled:before{content:\"\\e88d\"}.Bosch-Ic-arrow-up-filled:before{content:\"\\e88e\"}.Bosch-Ic-batch:before{content:\"\\e384\"}.Bosch-Ic-calculator:before{content:\"\\e0c7\"}.Bosch-Ic-calendar-delete:before{content:\"\\e385\"}.Bosch-Ic-call-bluetooth:before{content:\"\\e386\"}.Bosch-Ic-call-deny-outline:before{content:\"\\e8a2\"}.Bosch-Ic-call-deny:before{content:\"\\e387\"}.Bosch-Ic-call-pick-up-outline:before{content:\"\\e8a3\"}.Bosch-Ic-call-pick-up:before{content:\"\\e0cb\"}.Bosch-Ic-call-settings:before{content:\"\\e388\"}.Bosch-Ic-clear-all-annotations:before{content:\"\\e0ec\"}.Bosch-Ic-connection:before{content:\"\\e107\"}.Bosch-Ic-core-data:before{content:\"\\e38b\"}.Bosch-Ic-curve-scaling-view:before{content:\"\\e2bb\"}.Bosch-Ic-dashboard:before{content:\"\\e190\"}.Bosch-Ic-delete-material-manually:before{content:\"\\e394\"}.Bosch-Ic-dmc-code:before{content:\"\\e38e\"}.Bosch-Ic-document-to-svg:before{content:\"\\e34e\"}.Bosch-Ic-emergency-order:before{content:\"\\e12f\"}.Bosch-Ic-fast-backwards:before{content:\"\\e8a4\"}.Bosch-Ic-fast-forwards:before{content:\"\\e8a5\"}.Bosch-Ic-favorite:before{content:\"\\e2c5\"}.Bosch-Ic-go-to:before{content:\"\\e1ca\"}.Bosch-Ic-gps:before{content:\"\\e1c6\"}.Bosch-Ic-home-filled:before{content:\"\\e19d\"}.Bosch-Ic-input:before{content:\"\\e1ca\"}.Bosch-Ic-interface:before{content:\"\\e391\"}.Bosch-Ic-list:before{content:\"\\e1c1\"}.Bosch-Ic-malfunction-history:before{content:\"\\e389\"}.Bosch-Ic-malfunction-order:before{content:\"\\e38a\"}.Bosch-Ic-material-list:before{content:\"\\e393\"}.Bosch-Ic-material:before{content:\"\\e395\"}.Bosch-Ic-maximize-window:before{content:\"\\e269\"}.Bosch-Ic-menu-tree:before{content:\"\\e396\"}.Bosch-Ic-minimize-window:before{content:\"\\e1ba\"}.Bosch-Ic-notification-hint:before{content:\"\\e899\"}.Bosch-Ic-notification-question:before{content:\"\\e89b\"}.Bosch-Ic-notification-info:before{content:\"\\eb0f\"}.Bosch-Ic-notification-error:before{content:\"\\eb08\"}.Bosch-Ic-notification-error-outline:before{content:\"\\eb09\"}.Bosch-Ic-notification-outline:before{content:\"\\eb0d\"}.Bosch-Ic-notification-success:before{content:\"\\eb0a\"}.Bosch-Ic-notification-success-outline:before{content:\"\\eb0b\"}.Bosch-Ic-notification-warning:before{content:\"\\eb0c\"}.Bosch-Ic-notification:before{content:\"\\e1f5\"}.Bosch-Ic-output:before{content:\"\\e1cc\"}.Bosch-Ic-paste:before{content:\"\\e0ed\"}.Bosch-Ic-qr-code:before{content:\"\\e397\"}.Bosch-Ic-remote-cursor:before{content:\"\\e38c\"}.Bosch-Ic-scale:before{content:\"\\e167\"}.Bosch-Ic-search-chart:before{content:\"\\e383\"}.Bosch-Ic-search-web:before{content:\"\\e390\"}.Bosch-Ic-search-window:before{content:\"\\e39b\"}.Bosch-Ic-shape-ellipse:before{content:\"\\e3d1\"}.Bosch-Ic-shape-rectangle:before{content:\"\\e269\"}.Bosch-Ic-show-more-vertical:before{content:\"\\e1fb\"}.Bosch-Ic-signal:before{content:\"\\e249\"}.Bosch-Ic-skip-back:before{content:\"\\e8a6\"}.Bosch-Ic-skip-backward:before{content:\"\\e8a7\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-Ic-skip-for:before{content:\"\\e8a9\"}.Bosch-Ic-skip-forward:before{content:\"\\e8a8\"}.Bosch-Ic-sort-alphabetically:before{content:\"\\e398\"}.Bosch-Ic-switch-language:before{content:\"\\e18d\"}.Bosch-Ic-snyc-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-disabled:before{content:\"\\e3d8\"}.Bosch-Ic-sync-problem:before{content:\"\\e37f\"}.Bosch-Ic-sync:before{content:\"\\e21e\"}.Bosch-Ic-test:before{content:\"\\e105\"}.Bosch-Ic-to-order:before{content:\"\\e127\"}.Bosch-Ic-to-raw:before{content:\"\\e126\"}.Bosch-Ic-transport-orders:before{content:\"\\e0ee\"}.Bosch-Ic-unpin:before{content:\"\\e463\"}.Bosch-Ic-user-filled:before{content:\"\\e29b\"}.Bosch-Ic-video-play:before{content:\"\\e20b\"}.Bosch-Ic-weblink:before{content:\"\\e1be\"}.Bosch-Ic-warning-outline:before{content:\"\\eb0d\"}.Bosch-Ic-add-material-manually:before{content:\"\\e392\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-IC-3d-print:before{content:\"\\e085\"}.Bosch-Ic-tours:before{content:\"\\e086\"}.Bosch-Ic-add:before{content:\"\\e087\"}.Bosch-Ic-adress-consumer:before{content:\"\\e088\"}.Bosch-Ic-adressbook:before{content:\"\\e089\"}.Bosch-IC-agility:before{content:\"\\e08a\"}.Bosch-Ic-clock-stop:before{content:\"\\e08b\"}.Bosch-IC-ambulance:before{content:\"\\e08c\"}.Bosch-Ic-analysis:before{content:\"\\e08d\"}.Bosch-Ic-anchor:before{content:\"\\e08e\"}.Bosch-Ic-arrow-down:before{content:\"\\e08f\"}.Bosch-Ic-arrow-left:before{content:\"\\e090\"}.Bosch-Ic-arrow-left-down:before{content:\"\\e091\"}.Bosch-Ic-arrow-left-up:before{content:\"\\e092\"}.Bosch-Ic-arrow-right:before{content:\"\\e093\"}.Bosch-Ic-arrow-right-down:before{content:\"\\e094\"}.Bosch-Ic-arrow-right-up:before{content:\"\\e095\"}.Bosch-IC-arrow-turn-left:before{content:\"\\e096\"}.Bosch-IC-arrow-turn-right:before{content:\"\\e097\"}.Bosch-Ic-arrow-up:before{content:\"\\e098\"}.Bosch-IC-arrows:before{content:\"\\e099\"}.Bosch-IC-arrows-x-y-z:before{content:\"\\e09a\"}.Bosch-IC-artificial-intelligence:before{content:\"\\e09b\"}.Bosch-IC-assembly-line:before{content:\"\\e09c\"}.Bosch-IC-atom:before{content:\"\\e09d\"}.Bosch-IC-augmented-reality:before{content:\"\\e09e\"}.Bosch-Ic-autobod:before{content:\"\\e09f\"}.Bosch-Ic-arrow-expand-left:before{content:\"\\e0a0\"}.Bosch-Ic-arrow-expand-double-left:before{content:\"\\e0a1\"}.Bosch-IC-bar-chart:before{content:\"\\e0a2\"}.Bosch-IC-bar-chart-double:before{content:\"\\e0a3\"}.Bosch-Ic-battery-empty:before{content:\"\\e0a4\"}.Bosch-Ic-battery-25:before{content:\"\\e0a5\"}.Bosch-Ic-battery-50:before{content:\"\\e0a6\"}.Bosch-Ic-battery-75:before{content:\"\\e0a7\"}.Bosch-Ic-battery-100:before{content:\"\\e0a8\"}.Bosch-Ic-battery-error:before{content:\"\\e0a9\"}.Bosch-IC-battery_Li-Ion:before{content:\"\\e0aa\"}.Bosch-Ic-battery-charging:before{content:\"\\e0ab\"}.Bosch-Ic-help:before{content:\"\\e89b\"}.Bosch-IC-bed:before{content:\"\\e0ac\"}.Bosch-IC-bicycle:before{content:\"\\e0ad\"}.Bosch-IC-bicycle-e:before{content:\"\\e0ae\"}.Bosch-IC-blog:before{content:\"\\e0af\"}.Bosch-IC-bluetooth:before{content:\"\\e0b0\"}.Bosch-IC-bluetooth-LE:before{content:\"\\e0b1\"}.Bosch-IC-board-speaker:before{content:\"\\e0b2\"}.Bosch-IC-boiler-connected:before{content:\"\\e0b3\"}.Bosch-IC-book:before{content:\"\\e0b4\"}.Bosch-Ic-bookmark:before{content:\"\\e0b5\"}.Bosch-Ic-bookmark-add:before{content:\"\\e0b6\"}.Bosch-Ic-bookmark-check:before{content:\"\\e0b7\"}.Bosch-Ic-bookmark-delete:before{content:\"\\e0b8\"}.Bosch-IC-books:before{content:\"\\e0b9\"}.Bosch-IC-bookshelf:before{content:\"\\e0ba\"}.Bosch-Ic-box:before{content:\"\\e0bb\"}.Bosch-IC-box-closed:before{content:\"\\e0bc\"}.Bosch-IC-box-delivery:before{content:\"\\e0bd\"}.Bosch-IC-box-delivery-connected:before{content:\"\\e0be\"}.Bosch-IC-brake-disk:before{content:\"\\e0bf\"}.Bosch-IC-briefcase:before{content:\"\\e0c0\"}.Bosch-IC-broom:before{content:\"\\e0c1\"}.Bosch-IC-broom-cloud-dirt:before{content:\"\\e0c2\"}.Bosch-Ic-brush:before{content:\"\\e0c3\"}.Bosch-Ic-bug:before{content:\"\\e0c4\"}.Bosch-IC-building:before{content:\"\\e0c5\"}.Bosch-IC-cactus:before{content:\"\\e0c6\"}.Bosch-IC-calculate:before{content:\"\\e0c7\"}.Bosch-Ic-calendar:before{content:\"\\e0c8\"}.Bosch-Ic-calendar-add:before{content:\"\\e0c9\"}.Bosch-IC-calendar-settings:before{content:\"\\e0ca\"}.Bosch-Ic-call:before{content:\"\\e0cb\"}.Bosch-IC-call-add:before{content:\"\\e0cc\"}.Bosch-IC-call-remove:before{content:\"\\e0cd\"}.Bosch-IC-call-sos:before{content:\"\\e0ce\"}.Bosch-IC-call-team:before{content:\"\\e0cf\"}.Bosch-Ic-call-wifi:before{content:\"\\e0d0\"}.Bosch-Ic-camera:before{content:\"\\e0d1\"}.Bosch-Ic-camera-switch:before{content:\"\\e0d2\"}.Bosch-IC-car:before{content:\"\\e0d3\"}.Bosch-IC-car-clock:before{content:\"\\e0d4\"}.Bosch-IC-car-connected:before{content:\"\\e0d5\"}.Bosch-IC-car-locator:before{content:\"\\e0d6\"}.Bosch-IC-car-mechanic:before{content:\"\\e0d7\"}.Bosch-IC-car-rental:before{content:\"\\e0d8\"}.Bosch-IC-car-seat-connected:before{content:\"\\e0d9\"}.Bosch-IC-car-side:before{content:\"\\e0da\"}.Bosch-IC-car-side-user:before{content:\"\\e0db\"}.Bosch-Ic-milkrun-cart:before{content:\"\\e0dc\"}.Bosch-Ic-certificate:before{content:\"\\e0dd\"}.Bosch-IC-certificate-pki:before{content:\"\\e0de\"}.Bosch-IC-change:before{content:\"\\e0df\"}.Bosch-Ic-chart:before{content:\"\\e0e0\"}.Bosch-IC-chart-check:before{content:\"\\e0e1\"}.Bosch-IC-chart-dummy:before{content:\"\\e0e2\"}.Bosch-Ic-diagram:before{content:\"\\e0e3\"}.Bosch-Ic-chat:before{content:\"\\e0e4\"}.Bosch-IC-chat-add:before{content:\"\\e0e5\"}.Bosch-IC-chat-question-answer:before{content:\"\\e0e6\"}.Bosch-IC-chat-remove:before{content:\"\\e0e7\"}.Bosch-Ic-checklist:before{content:\"\\e0e8\"}.Bosch-Ic-checkmark:before{content:\"\\e0e9\"}.Bosch-Ic-chip:before{content:\"\\e0ea\"}.Bosch-IC-circle-segment:before{content:\"\\e0eb\"}.Bosch-IC-clear-all:before{content:\"\\e0ec\"}.Bosch-Ic-clipboard:before{content:\"\\e0ed\"}.Bosch-Ic-clipboard-list:before{content:\"\\e0ee\"}.Bosch-Ic-clock:before{content:\"\\e0ef\"}.Bosch-Ic-abort:before,.bosch-ic-abort:before{content:\"\\e0f0\"}.Bosch-IC-close-all:before{content:\"\\e0f1\"}.Bosch-Ic-cloud:before{content:\"\\e0f2\"}.Bosch-IC-cloud-co2:before{content:\"\\e0f3\"}.Bosch-Ic-cloud-download:before{content:\"\\e0f4\"}.Bosch-IC-cloud-nox:before{content:\"\\e0f5\"}.Bosch-Ic-cloud-upload:before{content:\"\\e0f6\"}.Bosch-IC-club-arm-chair:before{content:\"\\e0f7\"}.Bosch-IC-coffee-break:before{content:\"\\e0f8\"}.Bosch-IC-coin-currency:before{content:\"\\e0f9\"}.Bosch-Ic-currency:before{content:\"\\e0fa\"}.Bosch-Ic-color-picker:before{content:\"\\e0fb\"}.Bosch-IC-colors:before{content:\"\\e0fc\"}.Bosch-Ic-command:before{content:\"\\e0fd\"}.Bosch-IC-communicator:before{content:\"\\e0fe\"}.Bosch-Ic-compare:before{content:\"\\e0ff\"}.Bosch-Ic-components:before{content:\"\\e100\"}.Bosch-Ic-components-available:before{content:\"\\e101\"}.Bosch-Ic-components-reservation:before{content:\"\\e102\"}.Bosch-IC-concierge-bell:before{content:\"\\e103\"}.Bosch-IC-configuration:before{content:\"\\e104\"}.Bosch-Ic-confirmation:before{content:\"\\e105\"}.Bosch-Ic-unconnected:before{content:\"\\e106\"}.Bosch-Ic-connected:before{content:\"\\e107\"}.Bosch-IC-connectivity:before{content:\"\\e108\"}.Bosch-Ic-builder:before{content:\"\\e109\"}.Bosch-IC-consumer-goods:before{content:\"\\e10a\"}.Bosch-IC-contract:before{content:\"\\e10b\"}.Bosch-Ic-copy:before{content:\"\\e10c\"}.Bosch-IC-cube:before{content:\"\\e10d\"}.Bosch-IC-cube-arrows:before{content:\"\\e10e\"}.Bosch-Ic-StarterKit:before{content:\"\\e10f\"}.Bosch-IC-cube-shock:before{content:\"\\e110\"}.Bosch-IC-cube-stacked:before{content:\"\\e111\"}.Bosch-Ic-customerservice:before{content:\"\\e112\"}.Bosch-IC-cutlery:before{content:\"\\e113\"}.Bosch-IC-damper:before{content:\"\\e114\"}.Bosch-IC-danger-flash:before{content:\"\\e115\"}.Bosch-IC-data-loss:before{content:\"\\e116\"}.Bosch-Ic-dealer:before{content:\"\\e117\"}.Bosch-Ic-delete:before{content:\"\\e118\"}.Bosch-Ic-delivery:before{content:\"\\e119\"}.Bosch-IC-denied:before{content:\"\\e11a\"}.Bosch-Ic-desktop:before{content:\"\\e11b\"}.Bosch-IC-desktop-application-2d:before{content:\"\\e11c\"}.Bosch-IC-desktop-application-3d:before{content:\"\\e11d\"}.Bosch-IC-development:before{content:\"\\e11e\"}.Bosch-Ic-devices:before{content:\"\\e11f\"}.Bosch-IC-devices-mobile:before{content:\"\\e120\"}.Bosch-Ic-directory:before{content:\"\\e121\"}.Bosch-Ic-dna:before{content:\"\\e122\"}.Bosch-Ic-do-not-disturb:before{content:\"\\e123\"}.Bosch-Ic-document:before{content:\"\\e124\"}.Bosch-Ic-document-add:before{content:\"\\e125\"}.Bosch-IC-document-arrow-down:before{content:\"\\e126\"}.Bosch-IC-document-arrow-up:before{content:\"\\e127\"}.Bosch-Ic-document-audio:before{content:\"\\e128\"}.Bosch-IC-document-check:before{content:\"\\e129\"}.Bosch-Ic-document-to-cloud:before{content:\"\\e12a\"}.Bosch-Ic-document-copy:before{content:\"\\e12b\"}.Bosch-Ic-document-delete:before{content:\"\\e12c\"}.Bosch-Ic-document-to-doc:before{content:\"\\e12d\"}.Bosch-Ic-document-edit:before{content:\"\\e12e\"}.Bosch-Ic-document-error:before{content:\"\\e12f\"}.Bosch-Ic-document-locked:before{content:\"\\e130\"}.Bosch-Ic-log-file:before{content:\"\\e131\"}.Bosch-Ic-document-to-one:before{content:\"\\e132\"}.Bosch-Ic-document-to-pdf:before{content:\"\\e133\"}.Bosch-Ic-document-plain:before{content:\"\\e134\"}.Bosch-Ic-document-plain-add:before{content:\"\\e135\"}.Bosch-Ic-document-plain-delete:before{content:\"\\e136\"}.Bosch-Ic-document-to-ppt:before{content:\"\\e137\"}.Bosch-Ic-document-to-pub:before{content:\"\\e138\"}.Bosch-Ic-document-to-rtf:before{content:\"\\e139\"}.Bosch-Ic-document-save-to:before{content:\"\\e13a\"}.Bosch-Ic-search-document:before{content:\"\\e13b\"}.Bosch-Ic-document-settings:before{content:\"\\e13c\"}.Bosch-Ic-document-test:before{content:\"\\e13d\"}.Bosch-Ic-document-text:before{content:\"\\e13e\"}.Bosch-Ic-document-to-txt:before{content:\"\\e13f\"}.Bosch-Ic-document-to-vsd:before{content:\"\\e140\"}.Bosch-Ic-document-to-xls:before{content:\"\\e141\"}.Bosch-Ic-document-to-xml:before{content:\"\\e142\"}.Bosch-Ic-document-to-zip:before{content:\"\\e143\"}.Bosch-IC-dome:before{content:\"\\e144\"}.Bosch-IC-door:before{content:\"\\e145\"}.Bosch-IC-door-sensor:before{content:\"\\e146\"}.Bosch-Ic-arrow-expand-down:before{content:\"\\e147\"}.Bosch-Ic-arrow-expand-double-down:before{content:\"\\e148\"}.Bosch-Ic-arrow-left-circle:before{content:\"\\e889\"}.Bosch-Ic-arrow-right-circle:before{content:\"\\e88c\"}.Bosch-Ic-arrow-right-down-circle:before{content:\"\\e88b\"}.Bosch-Ic-arrow-right-up-circle:before{content:\"\\e88d\"}.Bosch-Ic-arrow-left-down-circle:before{content:\"\\e888\"}.Bosch-Ic-arrow-left-up-circle:before{content:\"\\e88a\"}.Bosch-Ic-arrow-up-circle:before{content:\"\\e88e\"}.Bosch-Ic-download:before{content:\"\\e149\"}.Bosch-IC-drag-handle:before{content:\"\\e14a\"}.Bosch-IC-dragdrop:before{content:\"\\e14b\"}.Bosch-IC-drop:before{content:\"\\e14c\"}.Bosch-IC-e-Charging:before{content:\"\\e14d\"}.Bosch-IC-e-mobility:before{content:\"\\e14e\"}.Bosch-IC-ear-off:before{content:\"\\e14f\"}.Bosch-IC-ear-on:before{content:\"\\e150\"}.Bosch-IC-eco-system:before{content:\"\\e151\"}.Bosch-Ic-edit:before{content:\"\\e152\"}.Bosch-IC-education:before{content:\"\\e153\"}.Bosch-IC-efficiency:before{content:\"\\e154\"}.Bosch-IC-elevator:before{content:\"\\e155\"}.Bosch-IC-elevator-alarm:before{content:\"\\e156\"}.Bosch-IC-elevator-cloud:before{content:\"\\e157\"}.Bosch-IC-elevator-headset:before{content:\"\\e158\"}.Bosch-IC-elevator-service:before{content:\"\\e159\"}.Bosch-IC-emergency-exit:before{content:\"\\e15a\"}.Bosch-IC-emoji-happy:before{content:\"\\e15b\"}.Bosch-IC-emoji-neutral:before{content:\"\\e15c\"}.Bosch-IC-emoji-sad:before{content:\"\\e15d\"}.Bosch-IC-emoji-super-happy:before{content:\"\\e15e\"}.Bosch-IC-emoji-very-sad:before{content:\"\\e15f\"}.Bosch-IC-EU-energy-label:before{content:\"\\e160\"}.Bosch-IC-excavator:before{content:\"\\e161\"}.Bosch-Ic-exit:before{content:\"\\e163\"}.Bosch-IC-expansion-arrows:before{content:\"\\e164\"}.Bosch-IC-explosion:before{content:\"\\e165\"}.Bosch-Ic-export:before{content:\"\\e166\"}.Bosch-IC-externallink:before{content:\"\\e167\"}.Bosch-IC-fast-forward:before{content:\"\\e168\"}.Bosch-IC-faucet:before{content:\"\\e169\"}.Bosch-IC-favorites:before{content:\"\\e16a\"}.Bosch-IC-fax:before{content:\"\\e16b\"}.Bosch-IC-female:before{content:\"\\e16c\"}.Bosch-IC-film:before{content:\"\\e16d\"}.Bosch-Ic-filter:before{content:\"\\e16e\"}.Bosch-IC-filter-success:before{content:\"\\e16f\"}.Bosch-IC-fingerprint:before{content:\"\\e170\"}.Bosch-IC-fire:before{content:\"\\e171\"}.Bosch-IC-fire-emergency:before{content:\"\\e172\"}.Bosch-IC-fireworks:before{content:\"\\e173\"}.Bosch-Ic-first-aid:before{content:\"\\e174\"}.Bosch-IC-first-aid-cross:before{content:\"\\e175\"}.Bosch-IC-fit-to-sceen:before{content:\"\\e176\"}.Bosch-Ic-flag:before{content:\"\\e177\"}.Bosch-Ic-danger-flash-outline:before{content:\"\\e178\"}.Bosch-IC-flask:before{content:\"\\e179\"}.Bosch-Ic-flexpress:before{content:\"\\e17a\"}.Bosch-Ic-twitter:before{content:\"\\ea8f\"}.Bosch-Ic-folder:before{content:\"\\e17b\"}.Bosch-Ic-download-from-folder:before{content:\"\\e17c\"}.Bosch-Ic-download-frame:before{content:\"\\e893\"}.Bosch-IC-folder-open:before{content:\"\\e17d\"}.Bosch-Ic-upload-to-folder:before{content:\"\\e17e\"}.Bosch-Ic-fork-lift:before{content:\"\\e17f\"}.Bosch-IC-fork-lift-locator:before{content:\"\\e180\"}.Bosch-Ic-arrow-expand-right:before{content:\"\\e181\"}.Bosch-Ic-arrow-expand-double-right:before{content:\"\\e182\"}.Bosch-IC-full-empty:before{content:\"\\e183\"}.Bosch-Ic-fullscreen:before{content:\"\\e184\"}.Bosch-IC-fullscreen-arrows:before{content:\"\\e185\"}.Bosch-Ic-fullscreen-exit:before{content:\"\\e186\"}.Bosch-IC-g-arrow-down:before{content:\"\\e187\"}.Bosch-IC-g-arrow-up:before{content:\"\\e188\"}.Bosch-IC-generator:before{content:\"\\e189\"}.Bosch-IC-gift:before{content:\"\\e18a\"}.Bosch-IC-glance:before{content:\"\\e18b\"}.Bosch-IC-glas-plate:before{content:\"\\e18c\"}.Bosch-IC-globe:before{content:\"\\e18d\"}.Bosch-IC-globe-arrow:before{content:\"\\e18e\"}.Bosch-IC-glossary:before{content:\"\\e18f\"}.Bosch-Ic-grid:before{content:\"\\e190\"}.Bosch-IC-hammer:before{content:\"\\e191\"}.Bosch-Ic-hand:before{content:\"\\e192\"}.Bosch-IC-hand-motion:before{content:\"\\e193\"}.Bosch-IC-hand-motion-off:before{content:\"\\e194\"}.Bosch-IC-handlewithcare:before{content:\"\\e195\"}.Bosch-Ic-add-material-manually:before{content:\"\\e196\"}.Bosch-Ic-handover-check:before{content:\"\\e197\"}.Bosch-Ic-remove-from-handover:before{content:\"\\e198\"}.Bosch-IC-hanger:before{content:\"\\e199\"}.Bosch-Ic-health:before{content:\"\\e19a\"}.Bosch-IC-heart:before{content:\"\\e19b\"}.Bosch-Ic-history:before{content:\"\\e19c\"}.Bosch-Ic-home-outline:before{content:\"\\e19d\"}.Bosch-IC-home-locator:before{content:\"\\e19e\"}.Bosch-Ic-sandclock:before{content:\"\\e19f\"}.Bosch-IC-hydrant:before{content:\"\\e1a0\"}.Bosch-IC-ice:before{content:\"\\e1a1\"}.Bosch-Ic-image:before{content:\"\\e1a2\"}.Bosch-IC-imagery-add:before{content:\"\\e1a3\"}.Bosch-IC-imagery-remove:before{content:\"\\e1a4\"}.Bosch-Ic-import:before{content:\"\\e1a5\"}.Bosch-Ic-imprint:before{content:\"\\e1a6\"}.Bosch-IC-impulse:before{content:\"\\e1a7\"}.Bosch-Ic-fabric:before{content:\"\\e1a8\"}.Bosch-IC-industry-clock:before{content:\"\\e1a9\"}.Bosch-IC-industry-connected:before{content:\"\\e1aa\"}.Bosch-Ic-information:before{content:\"\\e1ac\"}.Bosch-Ic-information-tooltip:before{content:\"\\e1ac\"}.Bosch-IC-interval:before{content:\"\\e1ad\"}.Bosch-IC-it-device:before{content:\"\\e1ae\"}.Bosch-IC-jewel:before{content:\"\\e1af\"}.Bosch-IC-keyboard:before{content:\"\\e1b0\"}.Bosch-IC-label:before{content:\"\\e1b1\"}.Bosch-Ic-laptop-01:before{content:\"\\e1b2\"}.Bosch-IC-laser:before{content:\"\\e1b3\"}.Bosch-IC-layout:before{content:\"\\e1b4\"}.Bosch-IC-leaf:before{content:\"\\e1b5\"}.Bosch-IC-led:before{content:\"\\e1b6\"}.Bosch-Ic-world:before{content:\"\\e8a0\"}.Bosch-Ic-facebook:before{content:\"\\ea87\"}.Bosch-Ic-checkmark-outline:before{content:\"\\e891\"}.Bosch-Ic-youtube:before{content:\"\\ea96\"}.Bosch-Ic-share:before{content:\"\\e242\"}.Bosch-Ic-user:before{content:\"\\e29b\"}.Bosch-Ic-upload_outline:before{content:\"\\e89f\"}.Bosch-Ic-download_outline:before{content:\"\\e893\"}.Bosch-Ic-wishlist-add:before{content:\"\\e3d3\"}.Bosch-Ic-youtube-frame:before{content:\"\\ea96\"}.Bosch-Ic-less-minimize:before{content:\"\\e1ba\"}.Bosch-IC-lightbulb:before{content:\"\\e1bb\"}.bosch-Ic-innovation:before{content:\"\\e1bc\"}.Bosch-IC-line-chart:before{content:\"\\e1bd\"}.Bosch-Ic-link:before{content:\"\\e1be\"}.Bosch-IC-link-broken:before{content:\"\\e1bf\"}.Bosch-Ic-list-view:before{content:\"\\e1c0\"}.Bosch-Ic-menu:before{content:\"\\e1c1\"}.Bosch-IC-livechat:before{content:\"\\e1c2\"}.Bosch-Ic-location-pin:before{content:\"\\e1c3\"}.Bosch-IC-locator-ip-connected:before{content:\"\\e1c4\"}.Bosch-IC-locator-ip-disconnected:before{content:\"\\e1c5\"}.Bosch-IC-locator-spot:before{content:\"\\e1c6\"}.Bosch-IC-locator-spot-check:before{content:\"\\e1c7\"}.Bosch-Ic-lock-closed:before{content:\"\\e1c8\"}.Bosch-Ic-lock-open:before{content:\"\\e1c9\"}.Bosch-Ic-log-in:before{content:\"\\e1ca\"}.Bosch-Ic-logistics:before{content:\"\\e1cb\"}.Bosch-Ic-log-out:before{content:\"\\e1cc\"}.Bosch-IC-machine:before{content:\"\\e1cd\"}.Bosch-Ic-magnet:before{content:\"\\e1ce\"}.Bosch-Ic-mail:before{content:\"\\e1cf\"}.Bosch-Ic-mail-forward:before{content:\"\\e1d0\"}.Bosch-Ic-mail-open:before{content:\"\\e1d1\"}.Bosch-Ic-mail-out:before{content:\"\\e1d2\"}.Bosch-Ic-mail-answer:before{content:\"\\e1d3\"}.Bosch-IC-male:before{content:\"\\e1d4\"}.Bosch-Ic-map:before{content:\"\\e1d5\"}.Bosch-Ic-add-material:before{content:\"\\e1d6\"}.Bosch-IC-material-check:before{content:\"\\e1d7\"}.Bosch-Ic-remove-material:before{content:\"\\e1d8\"}.Bosch-IC-maximize:before{content:\"\\e1d9\"}.Bosch-IC-mechanic:before{content:\"\\e1da\"}.Bosch-IC-megaphone:before{content:\"\\e1db\"}.Bosch-Ic-message:before{content:\"\\e1dc\"}.Bosch-IC-mic:before{content:\"\\e1dd\"}.Bosch-Ic-microphone:before{content:\"\\e1de\"}.Bosch-IC-microphone-classic:before{content:\"\\e1df\"}.Bosch-IC-minimize:before{content:\"\\e1e0\"}.Bosch-IC-money-currency:before{content:\"\\e1e1\"}.Bosch-IC-money-dollar:before{content:\"\\e1e2\"}.Bosch-IC-moneybag-currency:before{content:\"\\e1e3\"}.Bosch-IC-moneybag-dollar:before{content:\"\\e1e4\"}.Bosch-IC-monument:before{content:\"\\e1e5\"}.Bosch-IC-moon:before{content:\"\\e1e6\"}.Bosch-IC-mouse:before{content:\"\\e1ea\"}.Bosch-IC-mouse-left:before{content:\"\\e1eb\"}.Bosch-IC-mouse-right:before{content:\"\\e1ec\"}.Bosch-IC-mouth:before{content:\"\\e1ed\"}.Bosch-Ic-my-product:before{content:\"\\e1ee\"}.Bosch-IC-new-way-work:before{content:\"\\e1ef\"}.Bosch-Ic-mail-sent:before{content:\"\\e1f0\"}.Bosch-Ic-news:before{content:\"\\e1f1\"}.Bosch-IC-nose:before{content:\"\\e1f2\"}.Bosch-Ic-notes:before{content:\"\\e1f3\"}.Bosch-Ic-notes-edit:before{content:\"\\e1f4\"}.Bosch-Ic-alarm:before{content:\"\\e1f5\"}.Bosch-IC-notification-active:before{content:\"\\e1f6\"}.Bosch-IC-notification-add:before{content:\"\\e1f7\"}.Bosch-IC-notification-remove:before{content:\"\\e1f8\"}.Bosch-Ic-oil-trailer:before{content:\"\\e1f9\"}.Bosch-IC-omnichannel:before{content:\"\\e1fa\"}.Bosch-Ic-show-more-horizontal:before{content:\"\\e1fb\"}.Bosch-IC-origami-boat:before{content:\"\\e1fc\"}.Bosch-Ic-palette-trailer:before{content:\"\\e1fd\"}.Bosch-IC-pallete-car-connected:before{content:\"\\e1fe\"}.Bosch-IC-panel:before{content:\"\\e1ff\"}.Bosch-Ic-paperclip:before{content:\"\\e200\"}.Bosch-IC-paperplane:before{content:\"\\e201\"}.Bosch-IC-parking:before{content:\"\\e202\"}.Bosch-Ic-pause:before{content:\"\\e203\"}.Bosch-Ic-creditcard:before{content:\"\\e204\"}.Bosch-IC-people:before{content:\"\\e205\"}.Bosch-IC-petrol-station:before{content:\"\\e206\"}.Bosch-IC-piggybank:before{content:\"\\e207\"}.Bosch-Ic-pin:before{content:\"\\e208\"}.Bosch-IC-pin-modern:before{content:\"\\e209\"}.Bosch-IC-plane-side:before{content:\"\\e20a\"}.Bosch-Ic-play:before{content:\"\\e20b\"}.Bosch-IC-plug-12V:before{content:\"\\e20c\"}.Bosch-IC-podium-speaker:before{content:\"\\e20d\"}.Bosch-IC-police:before{content:\"\\e20e\"}.Bosch-IC-poop:before{content:\"\\e20f\"}.Bosch-IC-post-it:before{content:\"\\e210\"}.Bosch-IC-postal-code:before{content:\"\\e211\"}.Bosch-IC-power:before{content:\"\\e212\"}.Bosch-Ic-print:before{content:\"\\e213\"}.Bosch-IC-prototyping:before{content:\"\\e214\"}.Bosch-IC-puzzle:before{content:\"\\e215\"}.Bosch-IC-quad:before{content:\"\\e216\"}.Bosch-IC-question:before{content:\"\\e217\"}.Bosch-IC-radar:before{content:\"\\e218\"}.Bosch-IC-radio:before{content:\"\\e219\"}.Bosch-Ic-radio:before{content:\"\\e21a\"}.Bosch-IC-redirect:before{content:\"\\e21b\"}.Bosch-Ic-redo:before{content:\"\\e21c\"}.Bosch-IC-reference:before{content:\"\\e21d\"}.Bosch-Ic-refresh:before{content:\"\\e21e\"}.Bosch-IC-refresh-cloud:before{content:\"\\e21f\"}.Bosch-Ic-registration:before{content:\"\\e220\"}.Bosch-IC-remote:before{content:\"\\e221\"}.Bosch-Ic-reset:before{content:\"\\e222\"}.Bosch-IC-resolution:before{content:\"\\e223\"}.Bosch-Ic-robot-automation:before{content:\"\\e224\"}.Bosch-IC-robot-connected:before{content:\"\\e225\"}.Bosch-IC-robothead:before{content:\"\\e226\"}.Bosch-IC-rocket:before{content:\"\\e227\"}.Bosch-Ic-rotation:before{content:\"\\e228\"}.Bosch-Ic-rotate-360-degrees:before{content:\"\\e229\"}.Bosch-Ic-rotate-clockwise:before{content:\"\\e22a\"}.Bosch-Ic-rotate-counterclockwise:before{content:\"\\e22b\"}.Bosch-IC-rotation-y-down:before{content:\"\\e22c\"}.Bosch-IC-rotation-y-up:before{content:\"\\e22d\"}.Bosch-Ic-navigation:before{content:\"\\e22e\"}.Bosch-Ic-measure:before{content:\"\\e22f\"}.Bosch-IC-ruler-pen:before{content:\"\\e230\"}.Bosch-Ic-satelite:before{content:\"\\e231\"}.Bosch-Ic-save:before{content:\"\\e232\"}.Bosch-IC-scale:before{content:\"\\e233\"}.Bosch-Ic-cut:before{content:\"\\e234\"}.Bosch-Ic-search:before{content:\"\\e235\"}.Bosch-IC-security:before{content:\"\\e236\"}.Bosch-IC-security-check:before{content:\"\\e237\"}.Bosch-IC-security-user:before{content:\"\\e238\"}.Bosch-Ic-sensor:before{content:\"\\e239\"}.Bosch-Ic-server:before{content:\"\\e23a\"}.Bosch-IC-server-expansion:before{content:\"\\e23b\"}.Bosch-IC-server-rate:before{content:\"\\e23c\"}.Bosch-IC-service-agent:before{content:\"\\e23d\"}.Bosch-IC-service-time:before{content:\"\\e23e\"}.Bosch-Ic-settings:before{content:\"\\e23f\"}.Bosch-IC-settings-arrows:before{content:\"\\e240\"}.Bosch-IC-settings-connected:before{content:\"\\e241\"}.Bosch-Ic-share-outline:before{content:\"\\e242\"}.Bosch-IC-shield-stop:before{content:\"\\e243\"}.Bosch-IC-ship:before{content:\"\\e244\"}.Bosch-IC-ship-side:before{content:\"\\e245\"}.Bosch-Ic-shoppingcart:before{content:\"\\e246\"}.Bosch-IC-shoppingcart-add:before{content:\"\\e247\"}.Bosch-IC-shoppingcart-switch:before{content:\"\\e248\"}.Bosch-IC-signal-full:before{content:\"\\e249\"}.Bosch-IC-signal-half:before{content:\"\\e24a\"}.Bosch-IC-signal-lost:before{content:\"\\e24b\"}.Bosch-IC-signpost:before{content:\"\\e24c\"}.Bosch-IC-simcard:before{content:\"\\e24d\"}.Bosch-IC-skyscraper:before{content:\"\\e24e\"}.Bosch-IC-smartcity:before{content:\"\\e24f\"}.Bosch-IC-smarthome:before{content:\"\\e250\"}.Bosch-Ic-phone:before{content:\"\\e251\"}.Bosch-IC-smartphone-acoustic:before{content:\"\\e252\"}.Bosch-IC-smartphone-arrow-right:before{content:\"\\e253\"}.Bosch-IC-smartphone-arrows-x-y-z:before{content:\"\\e254\"}.Bosch-IC-smartphone-clock:before{content:\"\\e255\"}.Bosch-IC-smartphone-cloud:before{content:\"\\e256\"}.Bosch-IC-smartphone-loading-wheel:before{content:\"\\e257\"}.Bosch-IC-smartphone-smashed:before{content:\"\\e258\"}.Bosch-IC-smartphone-vibration:before{content:\"\\e259\"}.Bosch-IC-smartphone-z:before{content:\"\\e25a\"}.Bosch-Ic-spareparts:before{content:\"\\e25c\"}.Bosch-IC-spark-plug:before{content:\"\\e25d\"}.Bosch-IC-speaker-acoustic:before{content:\"\\e25e\"}.Bosch-IC-speedometer:before{content:\"\\e25f\"}.Bosch-IC-stadium:before{content:\"\\e260\"}.Bosch-IC-stairs:before{content:\"\\e261\"}.Bosch-IC-stamp:before{content:\"\\e262\"}.Bosch-IC-standby:before{content:\"\\e263\"}.Bosch-IC-steering-wheel:before{content:\"\\e264\"}.Bosch-IC-steering-wheel-connected:before{content:\"\\e265\"}.Bosch-IC-steering-wheel-hands:before{content:\"\\e266\"}.Bosch-IC-steps:before{content:\"\\e267\"}.Bosch-IC-stethoscope:before{content:\"\\e268\"}.Bosch-Ic-stop:before{content:\"\\e269\"}.Bosch-IC-stopwatch:before{content:\"\\e26a\"}.Bosch-IC-storage-local:before{content:\"\\e26b\"}.Bosch-IC-structure:before{content:\"\\e26c\"}.Bosch-IC-subtitles:before{content:\"\\e26d\"}.Bosch-Ic-summary:before{content:\"\\e26f\"}.Bosch-IC-sun:before{content:\"\\e270\"}.Bosch-IC-sun-half-filled:before{content:\"\\e271\"}.Bosch-IC-sunshade-table:before{content:\"\\e272\"}.Bosch-IC-surveillance:before{content:\"\\e273\"}.Bosch-IC-table-chairs:before{content:\"\\e274\"}.Bosch-Ic-tablet:before{content:\"\\e275\"}.Bosch-IC-team:before{content:\"\\e276\"}.Bosch-IC-team-lightbulb:before{content:\"\\e277\"}.Bosch-IC-technical-service:before{content:\"\\e278\"}.Bosch-Ic-temperature:before{content:\"\\e279\"}.Bosch-IC-temperature-high:before{content:\"\\e27a\"}.Bosch-IC-temperature-low:before{content:\"\\e27b\"}.Bosch-IC-theater:before{content:\"\\e27c\"}.Bosch-Ic-dislike:before{content:\"\\e27d\"}.Bosch-Ic-like:before{content:\"\\e27e\"}.Bosch-Ic-ticket:before{content:\"\\e27f\"}.Bosch-IC-touch:before{content:\"\\e280\"}.Bosch-IC-touch-sos:before{content:\"\\e281\"}.Bosch-IC-towing-truck:before{content:\"\\e282\"}.Bosch-IC-train:before{content:\"\\e283\"}.Bosch-IC-train-side:before{content:\"\\e284\"}.Bosch-Ic-transport-movements:before{content:\"\\e285\"}.Bosch-IC-transscript:before{content:\"\\e286\"}.Bosch-IC-transscript-off:before{content:\"\\e287\"}.Bosch-IC-transscript-on:before{content:\"\\e288\"}.Bosch-IC-tree:before{content:\"\\e289\"}.Bosch-IC-tricycle:before{content:\"\\e28a\"}.Bosch-Ic-default-trailer:before{content:\"\\e28b\"}.Bosch-Ic-taxi-trailer:before{content:\"\\e28c\"}.Bosch-Ic-shelve-trailer:before{content:\"\\e28d\"}.Bosch-IC-trophy:before{content:\"\\e28e\"}.Bosch-IC-truck:before{content:\"\\e28f\"}.Bosch-IC-tuktuk:before{content:\"\\e290\"}.Bosch-Ic-tune:before{content:\"\\e291\"}.Bosch-IC-typography:before{content:\"\\e292\"}.Bosch-IC-ui:before{content:\"\\e293\"}.Bosch-IC-umbrella:before{content:\"\\e294\"}.Bosch-Ic-undo:before{content:\"\\e295\"}.Bosch-Ic-arrow-expand-up:before{content:\"\\e296\"}.Bosch-Ic-arrow-expand-double-up:before{content:\"\\e297\"}.Bosch-Ic-sort:before{content:\"\\e298\"}.Bosch-Ic-upload:before{content:\"\\e299\"}.Bosch-IC-usb:before{content:\"\\e29a\"}.Bosch-Ic-user-outline:before{content:\"\\e29b\"}.Bosch-IC-user-advanced:before{content:\"\\e29c\"}.Bosch-IC-user-beginner:before{content:\"\\e29d\"}.Bosch-IC-user-desktop:before{content:\"\\e29e\"}.Bosch-IC-user-down:before{content:\"\\e29f\"}.Bosch-IC-user-falling:before{content:\"\\e2a0\"}.Bosch-IC-user-hand:before{content:\"\\e2a1\"}.Bosch-IC-user-hand-stop:before{content:\"\\e2a2\"}.Bosch-IC-user-professional:before{content:\"\\e2a3\"}.Bosch-IC-user-run:before{content:\"\\e2a4\"}.Bosch-IC-user-share:before{content:\"\\e2a5\"}.Bosch-IC-user-voice:before{content:\"\\e2a6\"}.Bosch-IC-user-walk:before{content:\"\\e2a7\"}.Bosch-Ic-videocam:before{content:\"\\e2a8\"}.Bosch-Ic-videocam-disabled:before{content:\"\\e2a9\"}.Bosch-IC-video-record:before{content:\"\\e2aa\"}.Bosch-IC-virtual-reality:before{content:\"\\e2ab\"}.Bosch-Ic-volume-disabled:before{content:\"\\e2ac\"}.Bosch-Ic-volume-high:before{content:\"\\e2ad\"}.Bosch-Ic-volume-low:before{content:\"\\e2ae\"}.Bosch-Ic-volume-off:before{content:\"\\e2af\"}.Bosch-Ic-magic-wand:before{content:\"\\e2b0\"}.Bosch-Ic-hidden-project:before{content:\"\\e2b2\"}.Bosch-Ic-show:before{content:\"\\e2b3\"}.Bosch-IC-water-shower:before{content:\"\\e2b4\"}.Bosch-IC-welcome:before{content:\"\\e2b5\"}.Bosch-IC-wellness:before{content:\"\\e2b6\"}.Bosch-IC-wheelchair:before{content:\"\\e2b7\"}.Bosch-Ic-alert:before{content:\"\\e2b8\"}.Bosch-IC-wide-angled-arrow:before{content:\"\\e2b9\"}.Bosch-Ic-wifi:before{content:\"\\e2ba\"}.Bosch-Ic-window-analysis:before{content:\"\\e2bb\"}.Bosch-Ic-window-browser:before{content:\"\\e2bc\"}.Bosch-Ic-window-console:before{content:\"\\e2bd\"}.Bosch-Ic-gaussian-view:before{content:\"\\e2be\"}.Bosch-Ic-window-info:before{content:\"\\e2bf\"}.Bosch-IC-window-new:before{content:\"\\e2c0\"}.Bosch-IC-window-resize:before{content:\"\\e2c1\"}.Bosch-Ic-general-scaling-view:before{content:\"\\e2c2\"}.Bosch-Ic-restore-window:before{content:\"\\e2c3\"}.Bosch-Ic-window-statistic:before{content:\"\\e2c4\"}.Bosch-Ic-wishlist:before{content:\"\\e2c5\"}.Bosch-Ic-work-orders:before{content:\"\\e2c6\"}.Bosch-IC-worldwideweb:before{content:\"\\e2c7\"}.Bosch-Ic-wrench:before{content:\"\\e2c8\"}.Bosch-Ic-zoom-in:before{content:\"\\e2c9\"}.Bosch-Ic-zoom-out:before{content:\"\\e2ca\"}:host(.hidden){display:none;visibility:hidden}:host(.hidden-page-size-selector){justify-content:flex-end}:host(.hidden-page-size-selector) .mdc-select{display:none}:host{display:flex;flex-flow:row wrap;justify-content:space-between;align-items:center;margin-top:24px}:host .mdc-select{border-radius:0}:host .mdc-select:not(.mdc-select--disabled) .mdc-floating-label{color:#000000}:host .mdc-select.hidden{visibility:hidden}:host .mdc-select>.mdc-select__anchor{border-radius:0}:host .mdc-select div.bci-core-paginator-dropdown{background-color:#e0e2e5;width:192px;height:48px}:host .mdc-select div.bci-core-paginator-dropdown:hover{background-color:#c1c7cc}:host .mdc-select div.bci-core-paginator-dropdown:hover{background-color:#a4abb3}:host .mdc-select div.bci-core-paginator-dropdown>.mdc-select__dropdown-icon.bosch-ic.bosch-ic-down{height:20px;width:20px}:host .mdc-select div.bci-core-paginator-dropdown>.mdc-floating-label{font-family:var(--bci-global-font-family);font-size:12px;font-weight:500;line-height:18px;transform:translateY(-106%) scale(1);will-change:inherit}:host .mdc-select div.bci-core-paginator-dropdown>.mdc-select__selected-text{font-family:var(--bci-global-font-family);font-size:16px;font-weight:400;line-height:24px}:host .mdc-select .mdc-select__dropdown-icon{transform:rotate(0);transition:transform ease-in-out 200ms}:host .mdc-select.mdc-select--activated .mdc-select__dropdown-icon{transform:rotate(-180deg)}:host .mdc-select div.bci-core-paginator-dropdown-select{width:192px;background:white;border-radius:0}:host .mdc-select div.bci-core-paginator-dropdown-select>.mdc-list{padding:0}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item{height:48px}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item:focus{background-color:#007bc0;color:#ffffff}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item.mdc-list-item--selected{background-color:#007bc0;color:#ffffff}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item.mdc-list-item--selected:hover{background-color:#00629a;color:#ffffff}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item.mdc-list-item--selected:active{background-color:#004975;color:#ffffff}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item:hover{background-color:#e0e2e5;color:#000000}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item:active{background-color:#c1c7cc}:host .mdc-select div.bci-core-paginator-dropdown-select .mdc-list-item .mdc-list-item__text{margin:auto 0}:host .bci-core-paginator-pages-container.hidden{visibility:hidden}@media (min-width: 480px){:host .bci-core-paginator-pages-container{display:flex;flex-flow:row nowrap;justify-content:flex-end;min-width:auto;height:32px}:host .bci-core-paginator-pages-container *,:host .bci-core-paginator-pages-container *:hover,:host .bci-core-paginator-pages-container *:active,:host .bci-core-paginator-pages-container *:focus{margin:0;padding:0;line-height:1.5;color:#000000;border:none;background:none;text-decoration:none}:host .bci-core-paginator-pages-container>*,:host .bci-core-paginator-pages-container>*:hover,:host .bci-core-paginator-pages-container>*:active,:host .bci-core-paginator-pages-container>*:focus{cursor:pointer;font-size:16px;display:inline-block;min-width:32px;width:32px;height:32px;line-height:32px;border-radius:50%;text-align:center;margin-right:8px;color:#000000;background-color:transparent}:host .bci-core-paginator-pages-container>*:first-child,:host .bci-core-paginator-pages-container>*:last-child,:host .bci-core-paginator-pages-container>*:hover:first-child,:host .bci-core-paginator-pages-container>*:hover:last-child,:host .bci-core-paginator-pages-container>*:active:first-child,:host .bci-core-paginator-pages-container>*:active:last-child,:host .bci-core-paginator-pages-container>*:focus:first-child,:host .bci-core-paginator-pages-container>*:focus:last-child{height:24px;line-height:1;width:24px;min-width:24px;color:#000000;background-color:transparent}:host .bci-core-paginator-pages-container>*:first-child:hover,:host .bci-core-paginator-pages-container>*:last-child:hover,:host .bci-core-paginator-pages-container>*:hover:first-child:hover,:host .bci-core-paginator-pages-container>*:hover:last-child:hover,:host .bci-core-paginator-pages-container>*:active:first-child:hover,:host .bci-core-paginator-pages-container>*:active:last-child:hover,:host .bci-core-paginator-pages-container>*:focus:first-child:hover,:host .bci-core-paginator-pages-container>*:focus:last-child:hover{color:#007bc0}:host .bci-core-paginator-pages-container>*:first-child:active,:host .bci-core-paginator-pages-container>*:last-child:active,:host .bci-core-paginator-pages-container>*:hover:first-child:active,:host .bci-core-paginator-pages-container>*:hover:last-child:active,:host .bci-core-paginator-pages-container>*:active:first-child:active,:host .bci-core-paginator-pages-container>*:active:last-child:active,:host .bci-core-paginator-pages-container>*:focus:first-child:active,:host .bci-core-paginator-pages-container>*:focus:last-child:active{color:#00629a}:host .bci-core-paginator-pages-container>*:first-child:disabled,:host .bci-core-paginator-pages-container>*:first-child.disabled,:host .bci-core-paginator-pages-container>*:last-child:disabled,:host .bci-core-paginator-pages-container>*:last-child.disabled,:host .bci-core-paginator-pages-container>*:hover:first-child:disabled,:host .bci-core-paginator-pages-container>*:hover:first-child.disabled,:host .bci-core-paginator-pages-container>*:hover:last-child:disabled,:host .bci-core-paginator-pages-container>*:hover:last-child.disabled,:host .bci-core-paginator-pages-container>*:active:first-child:disabled,:host .bci-core-paginator-pages-container>*:active:first-child.disabled,:host .bci-core-paginator-pages-container>*:active:last-child:disabled,:host .bci-core-paginator-pages-container>*:active:last-child.disabled,:host .bci-core-paginator-pages-container>*:focus:first-child:disabled,:host .bci-core-paginator-pages-container>*:focus:first-child.disabled,:host .bci-core-paginator-pages-container>*:focus:last-child:disabled,:host .bci-core-paginator-pages-container>*:focus:last-child.disabled{color:#c1c7cc;cursor:default}:host .bci-core-paginator-pages-container>*:first-child::after,:host .bci-core-paginator-pages-container>*:hover:first-child::after,:host .bci-core-paginator-pages-container>*:active:first-child::after,:host .bci-core-paginator-pages-container>*:focus:first-child::after{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;content:\"\\e0a0\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}:host .bci-core-paginator-pages-container>*:last-child,:host .bci-core-paginator-pages-container>*:hover:last-child,:host .bci-core-paginator-pages-container>*:active:last-child,:host .bci-core-paginator-pages-container>*:focus:last-child{margin-right:0px}:host .bci-core-paginator-pages-container>*:last-child::after,:host .bci-core-paginator-pages-container>*:hover:last-child::after,:host .bci-core-paginator-pages-container>*:active:last-child::after,:host .bci-core-paginator-pages-container>*:focus:last-child::after{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;content:\"\\e181\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}:host .bci-core-paginator-pages-container>*:hover{background-color:#e0e2e5}:host .bci-core-paginator-pages-container>*:hover *{background-color:#e0e2e5}:host .bci-core-paginator-pages-container>*:active{background-color:#c1c7cc}:host .bci-core-paginator-pages-container>*:active *{background-color:#c1c7cc}:host .bci-core-paginator-pages-container>*:not(:last-child):not(:first-child):disabled,:host .bci-core-paginator-pages-container *:not(:last-child):not(:first-child).disabled{cursor:default;color:#c1c7cc;background-color:#ffffff}:host .bci-core-paginator-pages-container>*:not(:last-child):not(:first-child):disabled *,:host .bci-core-paginator-pages-container *:not(:last-child):not(:first-child).disabled *{cursor:default;color:#c1c7cc;background-color:#ffffff}:host .bci-core-paginator-pages-container>*:first-child:disabled,:host .bci-core-paginator-pages-container *:last-child:disabled *:first-child.disabled,:host .bci-core-paginator-pages-container *:last-child.disabled{color:#c1c7cc;background-color:transparent;cursor:default}:host .bci-core-paginator-pages-container>*.selected{color:#ffffff;background-color:#007bc0;cursor:default}:host .bci-core-paginator-pages-container>*.selected:hover{background-color:#00629a}:host .bci-core-paginator-pages-container>*.selected:active{background-color:#004975}:host .bci-core-paginator-pages-container>*.selected:disabled,:host .bci-core-paginator-pages-container>*.selected.disabled{background-color:#c1c7cc;color:#8a9097}:host .bci-core-paginator-pages-container>*.wide{width:fit-content;max-width:fit-content;border-radius:25px;padding:0 8px}:host .bci-core-paginator-pages-container>*.ellipsis{color:#000000;cursor:default}:host .bci-core-paginator-pages-container>*.ellipsis:hover{color:#000000;background-color:transparent}:host .bci-core-paginator-pages-container>*.mobile{display:none}:host .mdc-select+.bci-core-paginator-pages-container{height:48px;align-items:center}}@media (max-width: 480px){:host{flex-flow:column nowrap;align-items:center}:host .mdc-select{width:100%}:host .mdc-select div.bci-core-paginator-dropdown{width:100%;margin-bottom:15px}:host div.bci-core-paginator-pages-container{display:flex;flex-flow:row nowrap;width:100%;justify-content:space-between}:host div.bci-core-paginator-pages-container *,:host div.bci-core-paginator-pages-container *:hover,:host div.bci-core-paginator-pages-container *:active,:host div.bci-core-paginator-pages-container *:focus{margin:0;padding:0;line-height:1.5;color:#000000;border:none;background:none;text-decoration:none}:host div.bci-core-paginator-pages-container>*,:host div.bci-core-paginator-pages-container>*:hover,:host div.bci-core-paginator-pages-container>*:active,:host div.bci-core-paginator-pages-container>*:focus{cursor:pointer;font-size:16px;display:inline-block;min-width:32px;width:32px;height:32px;line-height:32px;border-radius:50%;text-align:center;margin-right:8px;color:#000000;background-color:transparent;display:none}:host div.bci-core-paginator-pages-container>*:first-child,:host div.bci-core-paginator-pages-container>*:last-child,:host div.bci-core-paginator-pages-container>*:hover:first-child,:host div.bci-core-paginator-pages-container>*:hover:last-child,:host div.bci-core-paginator-pages-container>*:active:first-child,:host div.bci-core-paginator-pages-container>*:active:last-child,:host div.bci-core-paginator-pages-container>*:focus:first-child,:host div.bci-core-paginator-pages-container>*:focus:last-child{height:24px;line-height:1;width:24px;min-width:24px;color:#000000;background-color:transparent}:host div.bci-core-paginator-pages-container>*:first-child:hover,:host div.bci-core-paginator-pages-container>*:last-child:hover,:host div.bci-core-paginator-pages-container>*:hover:first-child:hover,:host div.bci-core-paginator-pages-container>*:hover:last-child:hover,:host div.bci-core-paginator-pages-container>*:active:first-child:hover,:host div.bci-core-paginator-pages-container>*:active:last-child:hover,:host div.bci-core-paginator-pages-container>*:focus:first-child:hover,:host div.bci-core-paginator-pages-container>*:focus:last-child:hover{color:#007bc0}:host div.bci-core-paginator-pages-container>*:first-child:active,:host div.bci-core-paginator-pages-container>*:last-child:active,:host div.bci-core-paginator-pages-container>*:hover:first-child:active,:host div.bci-core-paginator-pages-container>*:hover:last-child:active,:host div.bci-core-paginator-pages-container>*:active:first-child:active,:host div.bci-core-paginator-pages-container>*:active:last-child:active,:host div.bci-core-paginator-pages-container>*:focus:first-child:active,:host div.bci-core-paginator-pages-container>*:focus:last-child:active{color:#00629a}:host div.bci-core-paginator-pages-container>*:first-child:disabled,:host div.bci-core-paginator-pages-container>*:first-child.disabled,:host div.bci-core-paginator-pages-container>*:last-child:disabled,:host div.bci-core-paginator-pages-container>*:last-child.disabled,:host div.bci-core-paginator-pages-container>*:hover:first-child:disabled,:host div.bci-core-paginator-pages-container>*:hover:first-child.disabled,:host div.bci-core-paginator-pages-container>*:hover:last-child:disabled,:host div.bci-core-paginator-pages-container>*:hover:last-child.disabled,:host div.bci-core-paginator-pages-container>*:active:first-child:disabled,:host div.bci-core-paginator-pages-container>*:active:first-child.disabled,:host div.bci-core-paginator-pages-container>*:active:last-child:disabled,:host div.bci-core-paginator-pages-container>*:active:last-child.disabled,:host div.bci-core-paginator-pages-container>*:focus:first-child:disabled,:host div.bci-core-paginator-pages-container>*:focus:first-child.disabled,:host div.bci-core-paginator-pages-container>*:focus:last-child:disabled,:host div.bci-core-paginator-pages-container>*:focus:last-child.disabled{color:#c1c7cc;cursor:default}:host div.bci-core-paginator-pages-container>*:first-child::after,:host div.bci-core-paginator-pages-container>*:hover:first-child::after,:host div.bci-core-paginator-pages-container>*:active:first-child::after,:host div.bci-core-paginator-pages-container>*:focus:first-child::after{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;content:\"\\e0a0\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}:host div.bci-core-paginator-pages-container>*:last-child,:host div.bci-core-paginator-pages-container>*:hover:last-child,:host div.bci-core-paginator-pages-container>*:active:last-child,:host div.bci-core-paginator-pages-container>*:focus:last-child{margin-right:0px}:host div.bci-core-paginator-pages-container>*:last-child::after,:host div.bci-core-paginator-pages-container>*:hover:last-child::after,:host div.bci-core-paginator-pages-container>*:active:last-child::after,:host div.bci-core-paginator-pages-container>*:focus:last-child::after{font-family:\"Bosch-Ic\";font-style:normal;font-weight:normal;font-size:24px;content:\"\\e181\";-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}:host div.bci-core-paginator-pages-container>*:disabled,:host div.bci-core-paginator-pages-container *.disabled{cursor:default;color:#c1c7cc;background-color:#ffffff}:host div.bci-core-paginator-pages-container>*:disabled *,:host div.bci-core-paginator-pages-container *.disabled *{cursor:default;color:#c1c7cc;background-color:#ffffff}:host div.bci-core-paginator-pages-container>*.wide{width:fit-content;max-width:fit-content;border-radius:25px;padding:0 8px}:host div.bci-core-paginator-pages-container>*.mobile,:host div.bci-core-paginator-pages-container *:first-child,:host div.bci-core-paginator-pages-container *:last-child{display:inline}}";

const Paginator = /*@__PURE__*/ proxyCustomElement(class extends H {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.page = createEvent(this, "page", 7);
    /** Number of items to display on a page. By default set to 10. */
    this.pageSize = 10;
    /** The set of provided page size options to display to the user. */
    this.pageSizeOptions = '[]';
    /** The length of the total number of items that are being paginated. Defaulted to 0. */
    this.length = 0;
    /** The zero-based page index of the displayed list of items. Defaulted to 0. */
    this.pageIndex = 0;
    /** If page size option is not needed */
    this.showPageSizeSelector = false;
    /** If page index calculation is not needed then set to true */
    this.pageIndexCalculationOff = false;
    /** Set to true when used inside a dialog container to avoid overflow */
    this.insideDialogContainer = false;
    this.paginatorInput = {
      length: 0,
      pageIndex: 0,
      pageSize: 10,
      pageSizeOptions: [],
      showPageSizeSelector: false,
    };
    this.pages = [];
    this.pagesNumber = 0;
    this.previousPageIndex = 0;
    this.previousPageSize = 0;
    this.isFirstLoad = true;
    this.isPageSizeUpdate = false;
  }
  watchLengthHandler(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { length: newValue });
    this.recalculatePages();
  }
  watchPageSizeOptionsHandler(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageSizeOptions: typeof newValue === 'string' ? JSON.parse(newValue) : newValue });
    this.recalculatePages();
  }
  watchPageIndex(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageIndex: newValue });
  }
  watchShowPageSizeSelector(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { showPageSizeSelector: newValue });
  }
  watchPageSize(newValue) {
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageSize: newValue });
    this.recalculatePages();
  }
  clickHandler(ev) {
    const currentTarget = ev.composedPath()[0];
    if (!currentTarget || currentTarget.offsetParent === this.menuElement) {
      return;
    }
    if (this.selectElement !== undefined && this.selectElement.className.includes('mdc-select--activated') && this.mdcMenu !== undefined) {
      this.mdcMenu.open = false;
    }
  }
  async componentWillLoad() {
    await setLocale();
    this.paginatorInput = {
      length: this.length,
      pageIndex: this.pageIndex,
      pageSize: this.pageSize,
      pageSizeOptions: typeof this.pageSizeOptions === 'string' ? JSON.parse(this.pageSizeOptions) : this.pageSizeOptions,
      showPageSizeSelector: this.showPageSizeSelector,
    };
    this.previousPageIndex = this.paginatorInput.pageIndex;
    this.previousPageSize = this.paginatorInput.pageSize;
    this.recalculatePages();
    this.isFirstLoad = false;
  }
  componentDidLoad() {
    if (this.selectElement !== undefined && this.menuElement !== undefined) {
      this.mdcSelect = new MDCSelect(this.selectElement);
      this.mdcMenu = new MDCMenu(this.menuElement);
    }
  }
  disconnectedCallback() {
    if (this.mdcSelect === undefined) {
      return;
    }
    this.mdcSelect.destroy();
    if (this.mdcMenu === undefined) {
      return;
    }
    this.mdcMenu.destroy();
  }
  render() {
    const hidePageSizeSelector = this.shouldHidePageSizeSelector();
    const hidePageIndexSelector = this.shouldHidePageIndexSelector();
    return (h(Host, { class: { 'hidden-page-size-selector': hidePageSizeSelector, hidden: hidePageSizeSelector && hidePageIndexSelector } }, this.renderPageSizeSelector(hidePageSizeSelector), this.renderPageIndexSelector(hidePageIndexSelector)));
  }
  shouldHidePageSizeSelector() {
    return (!this.paginatorInput.showPageSizeSelector ||
      this.paginatorInput.pageSizeOptions.length === 0 ||
      this.paginatorInput.length <= Math.min(...this.paginatorInput.pageSizeOptions));
  }
  shouldHidePageIndexSelector() {
    return this.paginatorInput.length <= this.paginatorInput.pageSize;
  }
  renderPageIndexSelector(hidden) {
    return (h("div", { class: { 'bci-core-paginator-pages-container': true, hidden: hidden } }, h("span", { onClick: () => this.previousPage(), "data-test": "webcore.webcomponents.paginator.component.bci-paginator.page-prev", class: `bci-core-paginator-page-prev ${this.isAtFirstPage() ? 'disabled' : ''}` }), this.positionBasedRender(), this.renderMobilePages(), h("span", { onClick: () => this.nextPage(), "data-test": "webcore.webcomponents.paginator.component.bci-paginator.page-next", class: `bci-core-paginator-page-next ${this.isAtLastPage() ? 'disabled' : ''}` })));
  }
  updatePageSize(newVal) {
    this.previousPageSize = this.paginatorInput.pageSize;
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageSize: parseFloat(newVal) });
    this.isPageSizeUpdate = true;
    this.recalculatePages();
  }
  updatePageIndex(newVal) {
    if (newVal < 0 || newVal > this.pagesNumber - 1) {
      return;
    }
    this.previousPageIndex = this.paginatorInput.pageIndex;
    this.paginatorInput = Object.assign(Object.assign({}, this.paginatorInput), { pageIndex: newVal });
    this.positionBasedRender();
    this.emitPageEvent();
  }
  previousPage() {
    this.updatePageIndex(this.paginatorInput.pageIndex - 1);
  }
  nextPage() {
    this.updatePageIndex(this.paginatorInput.pageIndex + 1);
  }
  recalculatePages() {
    let pageIndex;
    this.pagesNumber = Math.ceil(this.paginatorInput.length / this.paginatorInput.pageSize);
    if (!this.pageIndexCalculationOff) {
      const indexOfFirstItemOnCurrentPage = this.paginatorInput.pageIndex * this.previousPageSize;
      const newPageIndex = Math.floor(indexOfFirstItemOnCurrentPage / this.paginatorInput.pageSize);
      pageIndex = newPageIndex < this.pagesNumber ? newPageIndex : this.pagesNumber - 1;
    }
    else {
      pageIndex = this.isPageSizeUpdate ? 0 : this.pageIndex;
    }
    this.updatePageIndex(pageIndex);
    this.pages = [...Array(this.pagesNumber)].map((_, i) => i + 1);
    this.isPageSizeUpdate = false;
    this.previousPageSize = this.paginatorInput.pageSize;
  }
  isAtFirstPage() {
    return this.paginatorInput.pageIndex === 0;
  }
  isAtLastPage() {
    return this.paginatorInput.pageIndex === this.pagesNumber - 1;
  }
  emitPageEvent() {
    if (!this.isFirstLoad) {
      this.page.emit({
        length: this.paginatorInput.length,
        pageIndex: this.paginatorInput.pageIndex,
        pageSize: this.paginatorInput.pageSize,
        previousPageIndex: this.previousPageIndex,
      });
    }
  }
  getSelectedMenuItemClass(option) {
    return this.paginatorInput.pageSize === option ? ' mdc-list-item--selected' : '';
  }
  getPageButtonClass(index, page) {
    let buttonClass = 'bci-core-paginator-page-button ';
    buttonClass += this.paginatorInput.pageIndex === index ? ' selected' : '';
    buttonClass += page.toString().length > 2 ? ' wide' : '';
    return buttonClass;
  }
  getEllipsesElement() {
    return h("span", { class: "ellipsis" }, "...");
  }
  getPageButtonElement(page, index) {
    return (h("span", { key: index, onClick: () => this.updatePageIndex(index), "data-test": "webcore.webcomponents.paginator.component.bci-paginator.page-button", class: `${this.getPageButtonClass(index, page)}` }, page));
  }
  getPredicateBasedRender(acc, page, index, pagePredicate, ellipsisPredicate) {
    if (pagePredicate) {
      return acc.concat(this.getPageButtonElement(page, index));
    }
    else if (ellipsisPredicate) {
      return acc.concat(this.getEllipsesElement());
    }
    else {
      return acc;
    }
  }
  positionBasedRender() {
    if (this.paginatorInput.pageIndex < 4) {
      return this.pages.reduce((acc, page, index) => this.getPredicateBasedRender(acc, page, index, index < 5 || index === this.pages.length - 1, index === 5), []);
    }
    else if (this.paginatorInput.pageIndex > this.pagesNumber - 5) {
      return this.pages.reduce((acc, page, index) => this.getPredicateBasedRender(acc, page, index, index === 0 || index > this.pages.length - 6, index === 1), []);
    }
    else {
      return this.pages.reduce((acc, page, index) => this.getPredicateBasedRender(acc, page, index, index === 0 ||
        index === this.pages.length - 1 ||
        (index >= this.paginatorInput.pageIndex - 1 && index <= this.paginatorInput.pageIndex + 1), index === 1 || index === this.pages.length - 2), []);
    }
  }
  renderMobilePages() {
    return (h("span", { class: "mobile" }, this.paginatorInput.pageIndex + 1, "/", this.pagesNumber));
  }
  renderPageSizeSelector(hidden) {
    return (h("div", { class: { 'mdc-select': true, 'mdc-select--filled': true, hidden: hidden }, ref: (el) => (this.selectElement = el) }, h("div", { class: "mdc-select__anchor bci-core-paginator-dropdown" }, h("span", { class: "mdc-select__ripple" }), h("span", { class: "mdc-floating-label mdc-floating-label--float-above" }, translate('listItemsPerPage')), h("span", { class: "mdc-select__selected-text" }), h("span", { class: "mdc-select__dropdown-icon bosch-ic bosch-ic-down" })), h("div", { class: {
        'mdc-select__menu': true,
        'mdc-menu': true,
        'mdc-menu-surface': true,
        'mdc-menu-surface--fixed': this.insideDialogContainer,
        'mdc-menu-surface--fullwidth': !this.insideDialogContainer,
        'bci-core-paginator-dropdown-select': true,
      }, ref: (el) => (this.menuElement = el) }, h("ul", { class: "mdc-list" }, this.paginatorInput.pageSizeOptions.map((option, index) => (h("li", { "data-test": "webcore.webcomponents.paginator.component.bci-paginator.pagesize-option", "data-value": option, key: index, "aria-selected": "false", role: "option", onClick: () => this.updatePageSize(option), class: `mdc-list-item ${this.getSelectedMenuItemClass(option)}` }, h("span", { class: "mdc-list-item__text" }, option))))))));
  }
  static get watchers() { return {
    "length": ["watchLengthHandler"],
    "pageSizeOptions": ["watchPageSizeOptionsHandler"],
    "pageIndex": ["watchPageIndex"],
    "showPageSizeSelector": ["watchShowPageSizeSelector"],
    "pageSize": ["watchPageSize"]
  }; }
  static get style() { return paginatorComponentCss; }
}, [1, "bci-paginator", {
    "pageSize": [2, "page-size"],
    "pageSizeOptions": [1, "page-size-options"],
    "length": [2],
    "pageIndex": [2, "page-index"],
    "showPageSizeSelector": [4, "show-page-size-selector"],
    "pageIndexCalculationOff": [4, "page-index-calculation-off"],
    "insideDialogContainer": [4, "inside-dialog-container"],
    "paginatorInput": [32],
    "pages": [32]
  }, [[17, "mousedown", "clickHandler"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["bci-paginator"];
  components.forEach(tagName => { switch (tagName) {
    case "bci-paginator":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, Paginator);
      }
      break;
  } });
}

const BciPaginator = Paginator;
const defineCustomElement = defineCustomElement$1;

export { BciPaginator, defineCustomElement };

